/*global define */
/**
 * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * Implements a breadcrumbs widget providing for display of a Breadcrumbs
 * navigation control with popup menus.
 * @author Shailesh Zirpe
 */

/**
 * The oj-lux-breadcrumbs CCA
 * @module oj-lux-breadcrumbs
 * @jqbase
 * @requires jquery
 * @requires underscore
 * @requires luxhierarchy
 * @requires koWidget
 * @requires ojs/ojcore
 * @requires ojs/ojbutton
 * @requires ojs/ojmenu
 */

define(['require',
    'ojs/ojcore',
    'jquery',
    'underscore',
    'knockout',
    'koWidget',
    'luxhierarchy',
    'ojs/ojbutton',
    'ojs/ojmenu',
    'jqueryui-amd/widget'
],

function(myLocalRequire, oj, $, _, ko, koWidget, luxhierarchy) {
    'use strict';
    $.widget('GBU.luxBreadcrumbs', $.GBU.koWidget,
        /** @lends module:oj-lux-breadcrumbs */
        {
            version: '5.1.0',

            /**
                 * Options
                 * <dl>
                 *     <dt>data</dt>
                 *     <dd>
                 *         The data element containing the hierarchy to render, expects an <code>luxhierarchynode</code>. Default is empty object.
                 *     </dd>
                 *     <dt>select</dt>
                 *     <dd>
                 *         Callback to use when a select event occurs. Signature should be <code>function (event, data)</code> </br>
                 *         Where data is an object that includes the properties action (the action string) and id (the node ID) </br>
                 *         Default is null.
                 *     </dd>
                 *     <dt>elementId</dt>
                 *     <dd>
                 *         The id for the top level DOM element for the control. By default the id is left as-is.
                 *     </dd>
                 *     <dt>fixToTop</dt>
                 *     <dd>
                 *         Use of 'fixToTop' option is deprecated.
                 *         Boolean value indicating whether the CCA should be fixed to the top of the screen. </br> Default is false
                 *     </dd>
                 *     <dt>navigateToCurrentNode</dt>
                 *     <dd>
                 *         Boolean value indicating whether to display the current node in its parent's breadcrumb menu. Default is false.
                 *     </dd>
                 *     <dt>disableCurrentNode</dt>
                 *     <dd>
                 *         Boolean value indicating whether to disable the current node. Default is false.
                 *     </dd>
                 * </dl>
                 */
            options: {
                data:                  {},
                elementId:             '',
                select:                null,
                showAdditionalActions: true,
                showChildren:          true,
                fixToTop:              false,
                navigateToCurrentNode: false,
                disableCurrentNode:    false
            },

            _template: 'text!luxbreadcrumbs_template',

            _vm: function() {
                var widgetObj = this;
                if (widgetObj.options.elementId !== '') {
                    widgetObj.element.prop('id', widgetObj.options.elementId);
                }
                if (widgetObj.options.fixToTop){
                    widgetObj.element.addClass('navbar-fixed-top');
                }
                var VM_Constructor_Breadcrumbs = function() {
                    var vmObj = this;
                    vmObj.fixToTop = ko.observable(widgetObj.options.fixToTop);
                    vmObj.observableData = ko.observable(widgetObj.options.data);
                    vmObj.nodeList = ko.computed(function() {
                        var nodeList;
                        if (vmObj.observableData().constructor === luxhierarchy){
                            nodeList = vmObj.observableData().listParents();
                            nodeList.push(vmObj.observableData());
                        }
                        return nodeList;
                    });
                };
                return new VM_Constructor_Breadcrumbs();
            },

            _vm_prototype: {
                selectNode_topLevelButton: function (data, event) {
                    var vmObj = this;
                    vmObj.selectNode(event, data);
                },
                selectNode_menu: function (ui, event, vm) {
                    var vmObj = this;
                    var domNode = ui.item.children('a');
                    var action = $(domNode).attr('action');
                    var data = vm.nodeList()[0].findNodeByAction(action);
                    vmObj.selectNode(event, data);
                },
                selectNode: function(event, node) {
                    /*
                         * from template bind the view-model's object to this method using :  click: $parent.selectNode.bind($parents[0])
                         * one can also grab the vmObj using -> var vmObj = widgetObj.options.pvtObservables;
                         */
                    var vmObj = this;

                    vmObj.widgetObj.options.select('event', node); // call the user defined callback
                    var eventData = {
                        option:         'data',
                        previousValue:  vmObj.widgetObj.options.data,
                        value:          node,
                        optionMetadata: {'uuid': vmObj.widgetObj.uuid}
                    };
                        /*
                         * ko observable setter ; Writing back to observable only if it's an observable, it's not your responsibility otherwise
                         */
                    vmObj.observableData(node);
                    /**
                         * This is a standard event triggered when a user interaction with the CCA results in the change of CCA state.
                         * When user selects any item in the Breadcrumbs, either by clicking on it or by using keyboard, results in
                         * setting the current 'data' to the newly selected item the luxoptionchange event is triggered.
                         *
                         * @event module:oj-lux-breadcrumbs#luxoptionchange
                         * @param {jqEvent} event
                         * @param {object} data
                         * @param {Boolean} data.option name of the option that has just changed, which is 'data'
                         * @param {string} data.value The newly selected node
                         * @param {string} data.previousValue Previous node value
                         * @param {string} data.optionMetadata empty object
                         *
                         * @example
                         * <caption>  Bind an event listener to the event:</caption>
                         * $(".selector").on("luxoptionchange", function(event, data) { console.log('luxoptionchange'); });
                         */
                    vmObj.widgetObj.element.trigger('luxoptionchange', eventData);
                },
                renderOrNot: function(action, child) {
                    var vmObj = this; //var vmObj = widgetObj.options.pvtObservables;
                    if (!vmObj.widgetObj.options.navigateToCurrentNode) { // new option added for navigating to current child node
                        var nodeList = vmObj.nodeList();
                        if (_.contains(nodeList, child)) {
                            return false;
                        }
                    }
                    return true;
                },
                disableOrNot: function(data) {
                    if (data.disabled){
                        return true;
                    }
                    return false;
                },
                disableCurrentNode: function(data) {
                    var vmObj = this;
                    if ((data.disabled) || (vmObj.widgetObj.options.disableCurrentNode && data === vmObj.observableData())) {
                        return true;
                    }
                    return false;
                },
                createMenuId: function(str, hash) {
                    var vmObj = this;
                    /*
                         * menu id defined in the button has '#' prepended, <ul> doesn't
                         * make sure you add 'uniqueId', because although each component has it's own VM, here your logic is HTML based not VM based.
                         */
                    var id = ((hash === '#') ? '#' : '') + str.replace(/\//gi, '-') + '-menu' + vmObj.widgetObj.uuid;
                    return id;
                }
            },

            _setOption: function(key, value) {
                var widgetObj = this;
                var vmObj = widgetObj.options.pvtObservables;
                switch (key) {
                case 'data':
                    if (ko.isObservable(vmObj.observableData)) {
                        vmObj.observableData(value); //ko observable setter
                    }
                    break;

                default:
                    break;
                }
            }

        });

});
