/*
 * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * helper utility for luxDashboard
 * @author Shailesh Zirpe
 */
define(['knockout'], function(ko) {

    function isIE9OrOlder() {
        if (navigator !== undefined && navigator.userAgent) {
            var ver = /MSIE ([0-9]{1,}[\.0-9]{0,})/.exec(navigator.userAgent);
            return ver && parseFloat(ver[1]) <= 9;
        }
        return false;
    }

    var tileFlipHelper = {

        ie9OrOlder: isIE9OrOlder(),

        //Flip a tile by 180 degrees.
        flipPos180: function(tile) {
            var content = tile.find('.lux-tile-content');
            var f = function() {
                tileFlipHelper.flipMidPos180(tile, content);
            };
            tile[0]._flipFunc = f;
            tile[0].addEventListener('transitionend', f, false);
            tile[0].addEventListener('webkittransitionend', f, false);
            //Transition the first part of the flip to 90 degrees.
            content.addClass('lux-tile-flip-transition-ease-in');
            content.addClass('lux-tile-flip-pos90');

            //IE9 does not support transitions, so call the transitionend function
            //directly.
            if (tileFlipHelper.ie9OrOlder) {
                f();
            }
        },

        //Flip a tile by -180 degrees.
        flipNeg180: function(tile) {
            var content = tile.find('.lux-tile-content');
            var f = function() {
                tileFlipHelper.flipMidNeg180(tile, content);
            };
            tile[0]._flipFunc = f;
            tile[0].addEventListener('transitionend', f, false);
            tile[0].addEventListener('webkittransitionend', f, false);
            //Transition the first part of the flip to -90 degrees.
            content.addClass('lux-tile-flip-transition-ease-in');
            content.addClass('lux-tile-flip-neg90');

            //IE9 does not support transitions, so call the transitionend function
            //directly.
            if (tileFlipHelper.ie9OrOlder) {
                f();
            }
        },

        //Called after the first part of the flip to 90 degrees is complete.
        flipMidPos180: function(tile, content) {
            var context = ko.contextFor(tile[0]);
            var data = context.$data;

            tile[0].removeEventListener('transitionend', tile[0]._flipFunc, false);
            tile[0].removeEventListener('webkittransitionend', tile[0]._flipFunc, false);
            tile[0]._flipFunc = null;

            //Toggle the flipped flag in the data so that the content of the tile
            //changes while it is perpendicular to the user.

            data.flipped(!data.flipped());

            content.removeClass('lux-tile-flip-transition-ease-in');
            content.removeClass('lux-tile-flip-pos90');
            //Rotate the new content so that it appears to flip in on the reverse
            //side of the tile when we rotate the tile back to 0 degrees.
            content.addClass('lux-tile-flip-neg90');
            //Process the second part of the flip in a timeout so that the rotation
            //above applies immediately, not as part of the next transition.
            setTimeout(function() {
                tileFlipHelper.flip0(tile, content);
            }, 0);
        },

        //Called after the first part of the flip to -90 degrees is complete.
        flipMidNeg180: function(tile, content) {
            var context = ko.contextFor(tile[0]);
            var data = context.$data;

            tile[0].removeEventListener('transitionend', tile[0]._flipFunc, false);
            tile[0].removeEventListener('webkittransitionend', tile[0]._flipFunc, false);
            tile[0]._flipFunc = null;

            //Toggle the flipped flag in the data so that the content of the tile
            //changes while it is perpendicular to the user.

            data.flipped(!data.flipped());

            content.removeClass('lux-tile-flip-transition-ease-in');
            content.removeClass('lux-tile-flip-neg90');
            //Rotate the new content so that it appears to flip in on the reverse
            //side of the tile when we rotate the tile back to 0 degrees.
            content.addClass('lux-tile-flip-pos90');
            //Process the second part of the flip in a timeout so that the rotation
            //above applies immediately, not as part of the next transition.
            setTimeout(function() {
                tileFlipHelper.flip0(tile, content);
            }, 0);
        },

        //Flip a tile to 0 degrees.
        flip0: function(tile, content) {
            var f = function() {
                tileFlipHelper.demoPostFlip0(tile, content);
            };
            tile[0]._flipFunc = f;
            tile[0].addEventListener('transitionend', f, false);
            tile[0].addEventListener('webkittransitionend', f, false);
            content.removeClass('lux-tile-flip-pos90');
            content.removeClass('lux-tile-flip-neg90');
            //Transition the rotation to 0 degrees.
            content.addClass('lux-tile-flip-transition-ease-out');
            content.addClass('lux-tile-flip-0');

            //IE9 does not support transitions, so call the transitionend function
            //directly.
            if (tileFlipHelper.ie9OrOlder) {
                f();
            }
        },

        //Called after the second part of the flip back to 0 degrees is complete.
        demoPostFlip0: function(tile, content) {
            tile[0].removeEventListener('transitionend', tile[0]._flipFunc, false);
            tile[0].removeEventListener('webkittransitionend', tile[0]._flipFunc, false);
            tile[0]._flipFunc = null;

            content.removeClass('lux-tile-flip-transition-ease-out');
            content.removeClass('lux-tile-flip-0');
            tile.removeClass('lux-tile-flipping');
        },

        isRtl: function() {
            return (document.documentElement.dir === 'rtl');
        },

        getContentCss: function(data) {
            return data.flipped() ? 'oj-panel-alt2' : '';
        }

    };

    return tileFlipHelper;

});
