/**
* @module jQueryUIWidget
*/

/**
* option( optionName )Returns: Object </br>
Gets the value currently associated with the specified optionName. </br>
optionName  Type: String  The name of the option to get. </br>

option()Returns: PlainObject </br>
Gets an object containing key/value pairs representing the current widget options hash. </br>
This method does not accept any arguments. </br>

option( optionName, value ) </br>
Sets the value of the widget option associated with the specified optionName. </br>
optionName Type: String  The name of the option to set. </br>
value Type: Object  A value to set for the option.  </br>

option( options ) </br>
Sets one or more options for the widget. </br>
options Type: Object A map of option-value pairs to set. </br>

* @method module:jQueryUIWidget#option

* @example
<caption>Invoke the method:</caption>
var isDisabled = $( ".selector" ).widget( "option", "disabled" );

* @example
<caption>Invoke the method:</caption>
var options = $( ".selector" ).widget( "option" );

* @example
<caption>Invoke the method:</caption>
$( ".selector" ).widget( "option", "disabled", true );

* @example
<caption>Invoke the method:</caption>
$( ".selector" ).widget( "option", { disabled: true } );
*/

/**
* Removes the widget functionality completely. This will return the element back to its pre-init state. </br>
* This method does not accept any arguments.
* @method module:jQueryUIWidget#destroy
*/

/**
* Disables the widget. </br>
* This method does not accept any arguments.
* @method module:jQueryUIWidget#disable
*/

/**
* Enables the widget. </br>
* This method does not accept any arguments.
* @method module:jQueryUIWidget#enable
*/

/**
* Returns a jQuery object containing the original element or other relevant generated element. </br>
* This method does not accept any arguments.
* @method module:jQueryUIWidget#widget
*/

/**
* Triggered when the widget is created.
*
* @event module:jQueryUIWidget#create
* @param {event} event
* @param {UI} object The ui object is empty but included for consistency with other events.
*
* @example
* <caption>Initialize the widget with the create callback specified:</caption>
* $( ".selector" ).widget({
*   create: function( event, ui ) {}
* });
*
* @example
* <caption>Bind an event listener to the widgetcreate event:</caption>
* $( ".selector" ).on( "widgetcreate", function( event, ui ) {} );
*/

define(function() {
    var obj = /** @lends module:jQueryUIWidget */{
        version: '0.5',
        /**
* Options
* <dl>
*     <dt>disabled</dt>
*       <dd>
            * Disables the widget if set to true.
            * Type: Boolean,
            * Default: false
            * @example
            * <caption>Initialize the widget with the disabled option specified:</caption>
            * $( ".selector" ).widget({ disabled: true });
            * @example
            * <caption>Get or set the disabled option, after initialization:</caption>
            * // getter
            * var disabled = $( ".selector" ).widget( "option", "disabled" );
            * // setter
            * $( ".selector" ).widget( "option", "disabled", true );
*     </dd>
* </dl>
*/ options: {}
    };

});