define([
    'gridapi.jquery',
    'gridapi.lodash',
    'ganttapi.gantt/locale',
    'gridapi.logger',
    'gridapi.jqueryui',
    'gridapi.oui',
    'ganttapi.gantt/config/barPositioner',
    'ganttapi.gantt/enums/barHeight',
    'ganttapi.gantt/enums/barColor',
    'ganttapi.gantt/enums/barType',
    'ganttapi.gantt/enums/labelHeight',
    'ganttapi.gantt/enums/barProgressStyle',
    'ganttapi.gantt/enums/barThemes',
    'markup!ganttapi.gantt/config/templates/bar-config.htm:.htm-layout',
    'markup!ganttapi.gantt/config/templates/bar-config.htm:.htm-label-options',
    'markup!ganttapi.gantt/config/templates/bar-config.htm:.htm-bar-options',
    'markup!ganttapi.gantt/config/templates/bar-config.htm:.htm-theme-selector-value',
    'markup!ganttapi.gantt/config/templates/bar-config.htm:.htm-preview-row',
    'markup!ganttapi.gantt/config/templates/bar-config.htm:.htm-preview-bar'
], function ($, _, locale, logger, jqui, oui, BarPositioner, BAR_HEIGHT, BAR_COLOR, BAR_TYPE, LABEL_HEIGHT, BAR_PROGRESS_STYLE, BAR_THEMES, layoutMarkup, labelOptionsMarkup, barOptionsMarkup, themeValue, previewRowTemplate, previewBarTemplate) {
    'use strict';

    var getEnumOptions = function (ENUM) {
        return _.map(ENUM.order, function (mapee) {
            if (ENUM[mapee])
                return ENUM[mapee];
            else
                return mapee;
        });
    };

    var configuratorView = function (widget) {
        this.options = widget.options;
        this.widget = widget;
        this.config = this.widget.config;
    };

    _.assign(configuratorView.prototype, {
        init : function () {
            this.$element = this.widget.element;
            this.$modal = this.$element.parents('.modal');
            this.$layout = $(layoutMarkup({ locale : this.options.locale }));
            this.$barOptions = $(barOptionsMarkup({ locale : this.options.locale }));
            this.$labelOptions = $(labelOptionsMarkup({ locale : this.options.locale }));
            this.$previewPanel = this.$layout.find('.preview-panel');
            this.$configurePanel = this.$layout.find('.configure-panel');

            this.$layout.find('.options-panel').append(this.$barOptions).append(this.$labelOptions);

            this.$element.append(this.$layout);
            this.positioner = new BarPositioner(this);
            this.positioner.init();
            this.initToolbar();
            this.initBarOptions();
            this.initLabelOptions();
            this.config = this.widget.config;
            this.timeouts = {};

            this.$configurePanel.addClass('show-progress show-critical-bars');
            this.$previewPanel.addClass('show-progress show-critical-bars');
        },

        resetConfig : function () {
            this.config = this.widget.config;
            this.positioner.resetConfig();
            this.barThemeCombo.value(BAR_THEMES.classic);
        },

        initToolbar : function () {
            var that = this;
            this.$toolbarContainer = this.$layout.find('.toolbar-container');

            var $themePicker = this.$toolbarContainer.find('#theme-selector');
            this.barThemeCombo = new oui.ComboBox({
                viewFns : {
                    display : function (theme) {
                        return themeValue(theme);
                    },
                    text : function (theme) {
                        return that.options.locale.barThemes[theme.label];
                    }
                },
                forceDisplay : true,
                keyboardInput : false,
                menuSize : 'input-width'
            }, $themePicker, getEnumOptions(BAR_THEMES));
            this.barThemeCombo.init();
            this.detectAppliedTheme();
            this.barThemeCombo.on('valueSelected', function (theme) {
                that.widget.applyTheme(theme);
            });

            this.$toolbarContainer.find('#gantt-config-add-bar').on('click', function () {
                that.widget.addBar();
            });

            this.$toolbarContainer.find('#gantt-config-add-label').on('click', function () {
                that.widget.addLabel();
            });

            this.$toolbarContainer.find('#gantt-config-configure-bars').on('click', function () {
                that.configureBars();
            });
            this.$toolbarContainer.find('#gantt-config-preview-bars').on('click', function () {
                that.previewBars();
            });

            this.$previewPanel.on('click', '.gantt-bar', function (event) {
                that.$toolbarContainer.find('#gantt-config-configure-bars').click();
                that.widget.selectBar($(this).data('bar-id'));
            });

            this.$previewPanel.on('click', '.bar-label', function (event) {
                event.preventDefault();
                event.stopPropagation();
                that.$toolbarContainer.find('#gantt-config-configure-bars').click();
                that.widget.selectLabel($(this).data('label-id'));
            });

            this.$toolbarContainer.find('.reset-button').on('click', function () {
                that.widget.applyDefaultConfig();
            });
        },

        detectAppliedTheme : function () {
            var that = this;
            var matches = [];
            _.forEach(BAR_THEMES.order, function (themeId) {
                var theme = BAR_THEMES[themeId];
                var themeMatches = 0;
                _.forEach(that.config.bars.order, function (barId) {
                    if (_.includes(theme.pickerColors, that.config.bars.definitions[barId].render.color)) {
                        themeMatches++;
                    }
                });
                matches.push({ matchCount : themeMatches, theme : theme });
            });

            var appliedTheme = _.reduce(matches, function (max, current) {
                return max.matchCount >= current.matchCount ? max : current;
            }).theme;
            this.barThemeCombo.value(appliedTheme);
            this.widget.appliedTheme = appliedTheme;
        },

        // TODO: change to have it be summary - completed - 50% not started, then same set of critical
        previewBars : function (force) {
            if (!this.$layout.hasClass('preview-gantt-bars') || force) {
                this.positioner.updateConfig();
                this.$layout.addClass('preview-gantt-bars');
                this.updatePreview();
            }
        },

        updatePreview : function () {
            this.$previewPanel.empty();
            for (var i = 0; i < 10; i++) {
                this.addActivityRow({
                    left : 50 * i,
                    number : i
                });
            }
        },
        // todo, vary percent, summary, lower number
        addActivityRow : function (options) {
            options = _.defaults({}, options, { width : 200, left : 10 });
            var barsHtml = '';

            for (var i = 0; i < this.config.bars.order.length; i++) {
                var barId = this.config.bars.order[i];
                var barConfig = this.config.bars.definitions[barId];
                var left = options.left || 10;
                left = left + Math.random() * 25;
                var labelHtml = this.positioner.makeLabels({
                    barId : barId,
                    number : options.number + 1
                });
                barsHtml += previewBarTemplate(
                    {
                        size : BAR_HEIGHT[barConfig.render.size].style,
                        color : BAR_COLOR[barConfig.render.color].style,
                        progressColor : barConfig.render.progressColor != null ? BAR_COLOR[barConfig.render.progressColor].progressStyle : '',
                        showProgress : barConfig.render.showProgress,
                        id : barId,
                        top : barConfig.render.top,
                        width : parseInt(options.width + Math.random() * 50, 10),
                        left : parseInt(left, 10),
                        labelHtml : labelHtml
                    });
            }
            this.$previewPanel.append(previewRowTemplate({
                height : this.config.layout.rowHeight,
                barsHtml : barsHtml
            }));
        },

        configureBars : function () {
            this.$layout.removeClass('preview-gantt-bars');
        },

        // todo:separate create and listeners
        initBarOptions : function () {
            var that = this;
            this.$barType = this.$barOptions.find('#bar-type');
            this.barTypeCombo = new oui.ComboBox({
                viewFns : {
                    display : function (barType) {
                        var selected = [];
                        for (var i = 0; i < that.config.bars.order.length; i++) {
                            var barId = that.config.bars.order[i];
                            if (barId !== that.widget.selection.id) {
                                selected.push(that.config.bars.definitions[barId].type);
                            }
                        }
                        if (_.includes(selected, barType.key)) {
                            return '<span class="bar-type-selected">' + barType.label + '</span>';
                        }
                        return barType.label;
                    }
                },
                keyboardInput : false
            }, this.$barType, this.options.dataFns.getBarTypes());
            this.barTypeCombo.init();

            this.barTypeCombo.on('valueSelected', function (selected) {
                that.widget.updateBarType({ type : selected });
            });

            this.$barHeight = this.$barOptions.find('#bar-height');

            this.barHeightComobo = new oui.ComboBox({
                viewFns : {
                    display : function (val) {
                        return that.options.locale.barHeight[val.key];
                    }
                },
                keyboardInput : false
            }, this.$barHeight, getEnumOptions(BAR_HEIGHT));
            this.barHeightComobo.init();
            this.barHeightComobo.selected(BAR_HEIGHT.M);
            this.barHeightComobo.on('valueSelected', function (selected) {
                that.widget.updateBarHeight(selected);
            });

            this.$barColorPicker = this.$barOptions.find('#bar-color-picker');
            this.barColorPicker = new oui.ColorPicker({
                appendTo : this.$modal,
                fluid : true,
                // map: [ array of hex colors, without preceding # ]
                thumbnail : true // display a swatch of the color
            });
            this.barColorPicker.element().addClass('input-small').appendTo(this.$barColorPicker);

            this.barColorPicker.init();

            this.barColorPicker.element().closest('.controls').siblings('label').attr('for', this.barColorPicker.id());

            this.barColorPicker.on('change', function (selected) {
                that.widget.updateBarColor({ color : selected.substring(1) });
            });


            this.$barProgressColorPicker = this.$barOptions.find('#bar-progress-color-picker');
            this.barProgressColorPicker = new oui.ColorPicker({
                appendTo : this.$modal,
                fluid : true,
                // map: [ array of hex colors, without preceding # ]
                thumbnail : true // display a swatch of the color
            });
            this.barProgressColorPicker.element().addClass('input-small').appendTo(this.$barProgressColorPicker);

            this.barProgressColorPicker.init();

            this.barProgressColorPicker.element().closest('.controls').siblings('label').attr('for', this.barProgressColorPicker.id());

            this.barProgressColorPicker.on('change', function (selected) {
                that.widget.updateBarProgressColor({ color : selected.substring(1) });
            });

            this.$colorContainer = this.$barOptions.find('.color-container');

            this.$barOptions.find('.btn-swap-color').on('click', function () {
                that.widget.swapProgressColor({});
            });

            this.$barStart = this.$barOptions.find('#bar-start');
            this.$barEnd = this.$barOptions.find('#bar-end');
            this.$barName = this.$barOptions.find('#bar-name');
            this.$barProgress = this.$barOptions.find('#bar-progress');
            this.$barProgressCheckbox = this.$barOptions.find('#bar-show-progress');

            this.$barProgressCheckbox.on('change', function () {
                var showProgress = that.$barProgressCheckbox.is(':checked');
                that.widget.updateBarShowProgress(showProgress);
            });

            this.$barName.on('change', function () {
                that.widget.updateBarName(that.$barName.val());
            });
            this.barStartDropDown = this.initFieldDropdown({
                $pickerElement : this.$barStart,
                data : this.options.dataFns.getStartFields()
            });

            this.barStartDropDown.on('valueSelected', function (selected) {
                that.widget.updateBarStart(selected);
            });

            this.barEndDropDown = this.initFieldDropdown({
                $pickerElement : this.$barEnd,
                data : this.options.dataFns.getFinishFields()
            });

            this.barEndDropDown.on('valueSelected', function (selected) {
                that.widget.updateBarEnd(selected);
            });

            this.barProgressDropDown = this.initFieldDropdown({
                $pickerElement : this.$barProgress,
                data : this.options.dataFns.getProgressFields()
            });

            this.barProgressDropDown.on('valueSelected', function (selected) {
                that.widget.updateBarProgress(selected);
            });

            this.$showSummaryBarCheckbox = this.$barOptions.find('#bar-show-when-expanded');
            this.$showSummaryBarCheckbox.on('change', function () {
                that.widget.updateShowExpandedSummaryBar(that.$showSummaryBarCheckbox.is(':checked'));
            });
            this.$showSummaryBarCheckbox.prop('checked', this.config.chart.showExpandedSummaryBars);

            this.$showCriticalCheckbox = this.$barOptions.find('#bar-show-critical');
            this.$showCriticalCheckbox.parents('.control-group').hide();
            this.$showCriticalCheckbox.on('change', function () {
                that.widget.updateBarShowCritical(that.$showCriticalCheckbox.is(':checked'));
            });

            this.$barProgressStyle = this.$barOptions.find('#bar-progress-style');
            this.barProgressStyleCombo = new oui.ComboBox({
                viewFns : {
                    display : function (val) {
                        return that.options.locale.progressStyles[val.label];
                    }
                },
                keyboardInput : false,
                fluid : true
            }, this.$barProgressStyle, getEnumOptions(BAR_PROGRESS_STYLE));
            this.barProgressStyleCombo.on('valueSelected', function (selected) {
                that.widget.updateBarProgressStyle({ style : selected });
            });
        },

        initFieldDropdown : function (args) {
            var options = {
                appendTo : this.$modal,
                keyboardInput : true,
                allowEmpty : true,
                modal : true,
                required : false,
                viewFns : {
                    display : function (node) {
                        return node.name;
                    }
                }

            };
            var comboBox = new oui.ComboBox(options, args.$pickerElement, args.data);
            comboBox.init();
            return comboBox;
        },

        addBar : function (barId) {
            this.positioner.addBar(barId);
        },

        removeBar : function (barId) {
            this.positioner.removeBar(barId);
        },

        addLabel : function (labelId) {
            this.positioner.addLabel(labelId);
        },

        removeLabel : function (labelId) {
            this.positioner.removeLabel(labelId);
        },


        updateBarColor : function (barId) {
            this.positioner.updateBarColor(barId);
        },

        updateBarProgressColor : function (barId) {
            this.positioner.updateBarProgressColor(barId);
        },
        // Todo: change showing and hiding to be class based
        updateBarShowProgress : function (barId) {
            var barConfig = this.config.bars.definitions[barId];
            this.$colorContainer.toggleClass('show-progress-color-picker', barConfig.render.showProgress === true);
            this.$barProgressStyle.parents('.control-group-progress-style').toggle(barConfig.render.showProgress === true);
            if (barConfig.render.progressColor != null) {
                this.barProgressColorPicker.value('#' + barConfig.render.progressColor);
            }
            var barType = _.findWhere(this.options.dataFns.getBarTypes(), { key : barConfig.type });
            if (barType.configRender.showProgressField) {
                if (barConfig.render.showProgress) {
                    this.$barProgress.parents('.control-group').show();
                } else {
                    this.$barProgress.parents('.control-group').hide();
                }
            }
            if (barConfig.render.showProgress === true) {
                this.$barOptions.find('#bar-color-label').text(this.options.locale.progressColor).attr('title', this.options.locale.progressColor);
            } else {
                this.$barOptions.find('#bar-color-label').text(this.options.locale.barColor).attr('title', this.options.locale.barColor);
            }
            this.positioner.updateBarShowProgress(barId);
        },

        updateBarProgressStyle : function (barId) {
            this.positioner.updateBarProgressStyle(barId);
        },

        updateLabelColor : function (labelId) {
            this.positioner.updateLabelColor(labelId);
        },

        updateLabelField : function (options) {
            this.positioner.updateLabelField(options.labelId, options.labelField);
        },
        // todo: change hiding / showing to be class based
        applyType : function (type) {
            this.barTypeCombo.values(this.options.dataFns.getBarTypes());
            this.barTypeCombo.value(type);
            if (type.configRender.showNameField) {
                this.$barName.parents('.control-group').show();
            } else {
                this.$barName.parents('.control-group').hide();
            }
            if (type.configRender.showStartField) {
                this.$barStart.parents('.control-group').show();
            } else {
                this.$barStart.parents('.control-group').hide();
            }

            if (type.configRender.showEndField) {
                this.$barEnd.parents('.control-group').show();
            } else {
                this.$barEnd.parents('.control-group').hide();
            }

            if (type.configRender.showProgressField) {
                this.$barProgress.parents('.control-group').show();
            } else {
                this.$barProgress.parents('.control-group').hide();
            }

            if (type.configRender.showProgressCheckbox || type.configRender.showCriticalCheckbox) {
                this.$barProgressCheckbox.parents('.control-group').show();
            } else {
                this.$barProgressCheckbox.parents('.control-group').hide();
            }

            this.$barProgressCheckbox.closest('label').toggleClass('hidden', !type.configRender.showProgressCheckbox);
            this.$showCriticalCheckbox.closest('label').toggleClass('hidden', !type.configRender.showCriticalCheckbox);

            if (type.configRender.moveHeight) {
                this.$colorContainer.before(this.$barHeight.parents('.control-group'));
            } else {
                this.$barType.parents('.control-group').after(this.$barHeight.parents('.control-group'));
            }
        },

        initLabelOptions : function () {
            var that = this;
            this.$labelField = this.$labelOptions.find('#label-name');
            this.labelFieldDropDown = this.initFieldDropdown({
                $pickerElement : this.$labelField,
                data : this.options.dataFns.getLabelFields()
            });

            this.labelFieldDropDown.on('valueSelected', function (value) {
                that.widget.updateLabelField({ labelField : value });
            });

            this.$labelHeight = this.$labelOptions.find('#label-height');

            this.labelHeightCombo = new oui.ComboBox({
                viewFns : {
                    display : function (val) {
                        return that.options.locale.labelHeight[val.key];
                    }
                },
                keyboardInput : false
            }, this.$labelHeight, getEnumOptions(LABEL_HEIGHT));
            this.labelHeightCombo.init();
            this.labelHeightCombo.selected(LABEL_HEIGHT.M);

            this.labelHeightCombo.on('valueSelected', function (selected) {
                that.widget.updateLabelHeight(selected);
            });

            this.$labelColorPicker = this.$labelOptions.find('#label-color-picker');
            this.labelColorPicker = new oui.ColorPicker({
                appendTo : this.$modal.find('.modal-body'),
                fluid : true,
                // map: [ array of hex colors, without preceding # ]
                thumbnail : true // display a swatch of the color
            });
            this.labelColorPicker.element().addClass('input-small').appendTo(this.$labelColorPicker);

            this.labelColorPicker.init();

            this.labelColorPicker.on('change', function (selected) {
                that.widget.updateLabelColor({ color : selected.substring(1) });
            });
        },

        selectBar : function (barId) {
            this.applyBarSettings(barId);
            this.$labelOptions.addClass('hidden');
            this.$barOptions.removeClass('hidden');
            this.positioner.selectBar(barId);
        },

        selectLabel : function (labelId) {
            this.applyLabelSettings(labelId);
            this.$labelOptions.removeClass('hidden');
            this.$barOptions.addClass('hidden');
            this.positioner.selectLabel(labelId);
        },
        // todo: change the way hide / show works
        applyBarSettings : function (barId) {
            var barConfig = this.config.bars.definitions[barId];
            var barType = _.findWhere(this.options.dataFns.getBarTypes(), { key : barConfig.type });
            this.applyType(barType);
            this.barHeightComobo.value(BAR_HEIGHT[barConfig.render.size]);
            this.barColorPicker.value('#' + barConfig.render.color);
            this.barProgressColorPicker.value('#' + barConfig.render.progressColor);
            if (barType.configRender.showProgressCheckbox) {
                this.$barProgressCheckbox.prop('checked', barConfig.render.showProgress === true);
            }
            if (barType.configRender.showNameField) {
                this.$barName.val(barConfig.name);
            }
            if (barType.configRender.showStartField && barConfig.keys) {
                this.barStartDropDown.value({ dataIndex : barConfig.keys.start });
            } else {
                this.barStartDropDown.value();
            }
            if (barType.configRender.showEndField && barConfig.keys) {
                this.barEndDropDown.value({ dataIndex : barConfig.keys.end });
            } else {
                this.barStartDropDown.value();
            }
            if (barType.configRender.showProgressField && barConfig.keys) {
                this.barProgressDropDown.value({ dataIndex : barConfig.keys.percent });
            } else {
                this.barStartDropDown.value();
            }

            if (barConfig.render.showProgress && barType.configRender.showProgressField) {
                this.$barProgress.parents('.control-group').show();
            } else {
                this.$barProgress.parents('.control-group').hide();
            }

            if (barId === 'summary') {
                this.$showSummaryBarCheckbox.parents('.control-group').show();
//                    this.$colorContainer.before(this.$barHeight.parents('.control-group'));
            } else {
                this.$showSummaryBarCheckbox.parents('.control-group').hide();
            }

            this.$colorContainer.toggleClass('show-progress-color-picker', barConfig.render.showProgress === true);
            this.$barProgressStyle.parents('.control-group-progress-style').toggle(barConfig.render.showProgress === true);
            if (barConfig.render.showProgress === true) {
                this.$barOptions.find('#bar-color-label').text(this.options.locale.progressColor).attr('title', this.options.locale.progressColor);
            } else {
                this.$barOptions.find('#bar-color-label').text(this.options.locale.barColor).attr('title', this.options.locale.barColor);
            }

            this.$showCriticalCheckbox.prop('checked', barConfig.render.showCritical);
            if (barConfig.render.progressStyle != null) {
                this.barProgressStyleCombo.value({ key : barConfig.render.progressStyle });
            }
        },

        applyLabelSettings : function (labelId) {
            var labelConfig = this.config.labels[labelId];
            this.labelHeightCombo.value(LABEL_HEIGHT[labelConfig.render.size]);
            this.labelColorPicker.value('#' + labelConfig.render.color);
            this.labelFieldDropDown.value({ dataIndex : labelConfig.key });
        }
    });

    return configuratorView;
});
