define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.logger',
    'ganttapi.gantt/enums/barHeight',
    'ganttapi.gantt/enums/barColor',
    'ganttapi.gantt/enums/labelHeight',
    'ganttapi.gantt/enums/labelPosition',
    'ganttapi.gantt/enums/barProgressStyle',
    'markup!ganttapi.gantt/config/templates/bar-config.htm:.htm-bar-holder',
    'markup!ganttapi.gantt/config/templates/bar-config.htm:.htm-summary-bar-holder',
    'markup!ganttapi.gantt/config/templates/bar-config.htm:.htm-bar-placer',
    'markup!ganttapi.gantt/config/templates/bar-config.htm:.htm-bar-label',
    'markup!ganttapi.gantt/config/templates/bar-config.htm:.htm-label-placer'
], function ($, _, logger, BAR_HEIGHT, BAR_COLOR, LABEL_HEIGHT, LABEL_POSITION, PROGRESS_STYLE, barHolderTemplate, summaryBarHolderTemplate, barPlacerTemplate, labelTemplate, labelPlacerTemplate) {
    'use strict';

    var barPositioner = function (view) {
        this.view = view;
        this.widget = this.view.widget;
        this.config = this.view.widget.config;
    };

    _.assign(barPositioner.prototype, {
        init : function () {
            this.$layoutCanvas = this.view.$layout.find('.gantt-layout-canvas');
            this.resetConfig();
            this.initListeners();
        },

        resetConfig : function () {
            this.config = this.view.widget.config;
            this.barModels = {};
            this.$layoutCanvas.empty();
            this.renderBars();
            this.toggleDelete();
            this.$selectedItem = $();
        },

        initListeners : function () {
            var that = this;
            var dragTimeout;
            this.$layoutCanvas.on('mousedown', '.gantt-bar-holder', function (event) {
                var $barHolder = $(this);
                if (!$barHolder.hasClass('selected')) {
                    that.widget.selectBar($(this).data('bar-id'));
                }
                if (!$barHolder.hasClass('gantt-summary-bar-holder')) {
                    dragTimeout = setTimeout(function () {
                        that.$movingBar = $barHolder;
                        that.handleDragBar({ grabEvent : event, offset : event.offsetY });
                    }, 150);
                }
            });
            this.$layoutCanvas.on('mousedown', '.delete-icon', function (event) {
                event.stopPropagation();
                event.preventDefault();
            });
            this.$layoutCanvas.on('mousedown', '.bar-label', function (event) {
                event.stopPropagation();
                event.preventDefault();
                var $label = $(this);
                if (!$label.hasClass('selected')) {
                    that.widget.selectLabel($label.data('label-id'));
                }

                dragTimeout = setTimeout(function () {
                    that.$movingLabel = $label;
                    that.handleDragLabel({ grabEvent : event, offset : event.offsetY });
                }, 150);
            });

            this.$layoutCanvas.on('mouseup.clear-drag-timeout', function () {
                clearTimeout(dragTimeout);
            });

            that.$layoutCanvas.on('mouseenter', '.delete-icon', function (event) {
                var label = $();
                if ($(this).is('.delete-label')) {
                    label = $(this).parent().parent();
                }
                var selectionBox = $(this).parent();
                selectionBox.addClass('show-remove-highlight');
                label.addClass('show-remove-highlight');
                selectionBox.on('mouseleave', function () {
                    selectionBox.removeClass('show-remove-highlight');
                    label.removeClass('show-remove-highlight');
                });
                clearTimeout(dragTimeout);
            });

            that.$layoutCanvas.on('click', '.delete-bar', function () {
                that.widget.removeBar({
                    barId : $(this).parents('.gantt-bar-holder').data('bar-id')
                });
            });

            that.$layoutCanvas.on('click', '.delete-label', function () {
                that.widget.removeLabel({
                    labelId : $(this).parents('.bar-label').data('label-id')
                });
            });
        },
        handleDragLabel : function () {
            var that = this;
            that.$movingLabel.addClass('label-dragging');
            this.$layoutCanvas.addClass('bar-dragging');
            this.$layoutCanvas.find('.gantt-bar').addClass('disable-hover');
            var movingLabelId = that.$movingLabel.data('label-id');
            var label = this.config.labels[movingLabelId];

            var $placeHolder = this.$placeHolder = $(labelPlacerTemplate(
                {
                    id : label.id,
                    label : _.findWhere(this.widget.flatLabelFields, { dataIndex : label.key }).name,
                    locale : that.widget.options.locale,
                    size : LABEL_HEIGHT[label.render.size].style,
                    position : LABEL_POSITION[label.render.position].style,
                    color : label.render.color
                }));

            this.barModels[label.barId].$bar.find('.gantt-bar').append($placeHolder);
            var barOffset = this.barModels[label.barId].$bar.offset();
            var barHeight = this.barModels[label.barId].height;
            var labelWidth = this.$movingLabel.width();
            var previousMouseX = 0;
            var previousMouseY = 0;
            var labelLeft = this.$placeHolder.offset().left;
            var labelRight = labelLeft + labelWidth;
            this.$layoutCanvas.on('mousemove.label-drag', function (event) {
                // Handle Moving Left and Right
                var mouseX = event.pageX - barOffset.left - labelWidth / 2;
                var positionOrderIndex;
                if (previousMouseX > event.pageX && event.pageX < labelLeft) {
                    positionOrderIndex = LABEL_POSITION.order.indexOf(label.render.position);
                    if (positionOrderIndex > 0) {
                        that.moveLabel({
                            label : label,
                            newPosition : LABEL_POSITION.order[positionOrderIndex - 1]
                        });
                        labelLeft = that.$placeHolder.offset().left;
                        labelRight = labelLeft + labelWidth;
                    }
                } else if (event.pageX > previousMouseX && event.pageX > labelRight) {
                    positionOrderIndex = LABEL_POSITION.order.indexOf(label.render.position);
                    if (positionOrderIndex < LABEL_POSITION.order.length - 1) {
                        that.moveLabel({
                            label : label,
                            newPosition : LABEL_POSITION.order[positionOrderIndex + 1]
                        });
                        labelLeft = that.$placeHolder.offset().left;
                        labelRight = labelLeft + labelWidth;
                    }
                }
                that.$movingLabel.css('left', mouseX);
                // Handle Moving Up and Down
                if (event.pageY > previousMouseY && event.pageY > (barOffset.top + barHeight)) { // movingdown
                    positionOrderIndex = that.sortedBars.indexOf(label.barId);
                    if ((positionOrderIndex < that.sortedBars.length - 1) && label.barId !== 'summary') {
                        that.moveLabel({
                            label : label,
                            resetVerticalPosition : true,
                            newBarId : that.sortedBars[positionOrderIndex + 1]
                        });
                        barOffset = that.barModels[label.barId].$bar.offset();
                        barHeight = that.barModels[label.barId].height;
                    } else if (label.render.verticalPosition !== 'B') {
                        that.moveLabel({
                            label : label,
                            verticalPosition : 'B'
                        });
                    }
                } else if (event.pageY < previousMouseY && event.pageY < barOffset.top) {
                    positionOrderIndex = that.sortedBars.indexOf(label.barId);
                    if (positionOrderIndex > 0) {
                        that.moveLabel({
                            label : label,
                            resetVerticalPosition : true,
                            newBarId : that.sortedBars[positionOrderIndex - 1]
                        });
                        barOffset = that.barModels[label.barId].$bar.offset();
                        barHeight = that.barModels[label.barId].height;
                    } else if (label.render.verticalPosition !== 'A') {
                        that.moveLabel({
                            label : label,
                            verticalPosition : 'A'
                        });
                    }
                } else if (label.render.verticalPosition != null && (event.pageY > barOffset.top && event.pageY < barOffset.top + barHeight)) {
                    // Move it back into the bar
                    that.moveLabel({
                        label : label,
                        resetVerticalPosition : true
                    });
                }

                previousMouseY = event.pageY;
                previousMouseX = event.pageX;
            });

            this.$layoutCanvas.on('mouseup.label-drag mouseleave.label-drag', function (event) {
                $placeHolder.remove();
                that.selectLabel(movingLabelId);
                that.$movingLabel.removeClass('label-dragging').css('left', '');
                that.$layoutCanvas.off('mousemove.label-drag mouseup.label-drag mouseleave.label-drag');
                that.widget._trigger('config-updated');
            });
        },

        moveLabel : function (options) {
            var classToRemove = '';
            var classToAdd = '';
            if (options.newPosition != null) {
                classToRemove = LABEL_POSITION[options.label.render.position].style;
                classToAdd = LABEL_POSITION[options.newPosition].style;
                this.$placeHolder.removeClass(classToRemove);
                this.$movingLabel.removeClass(classToRemove);

                this.$placeHolder.addClass(classToAdd);
                this.$movingLabel.addClass(classToAdd);
                options.label.render.position = options.newPosition;
            }
            if (options.resetVerticalPosition) {
                classToRemove = options.label.render.verticalPosition != null ? LABEL_POSITION[options.label.render.verticalPosition].style : '';
                this.$placeHolder.removeClass(classToRemove);
                this.$movingLabel.removeClass(classToRemove);
            }
            if (options.verticalPosition != null) {
                classToAdd = LABEL_POSITION[options.verticalPosition].style;
                this.$placeHolder.addClass(classToAdd);
                this.$movingLabel.addClass(classToAdd);
                options.label.render.verticalPosition = options.verticalPosition;
            } else {
                delete options.label.render.verticalPosition;
            }
            if (options.newBarId != null) {
                var newBar = this.barModels[options.newBarId];
                options.label.barId = options.newBarId;
                newBar.$bar.find('.gantt-bar').append(this.$movingLabel).append(this.$placeHolder);
            }
        },

        handleDragBar : function () {
            var that = this;
            that.$movingBar.addClass('bar-dragging');
            this.$layoutCanvas.addClass('bar-dragging');

            var movingBarId = that.$movingBar.data('bar-id');
            var movingBarConfig = this.config.bars.definitions[movingBarId];
            var movingBar = this.movingBar = this.barModels[movingBarId];
            var $placeHolder = this.$placeHolder = $(barPlacerTemplate(
                {
                    size : BAR_HEIGHT[movingBarConfig.render.size].style,
                    color : BAR_COLOR[movingBarConfig.render.color].style,
                    progressColor : movingBarConfig.render.progressColor != null ? BAR_COLOR[movingBarConfig.render.progressColor].progressStyle : '',
                    showProgress : movingBarConfig.render.showProgress,
                    id : movingBarId + 'placer',
                    top : this.barModels[movingBarId].top,
                    locale : this.widget.options.locale
                }));
            $placeHolder.css('left', movingBar.$bar.position().left);
            this.$layoutCanvas.append($placeHolder);

            var layoutTop = this.$layoutCanvas.offset().top;
            var barHeight = that.$movingBar.height();
            var previousMouseY = 0;
            this.$layoutCanvas.on('mousemove.bar-drag', function (event) {
                var mouseY = event.pageY - layoutTop;
                var newTop = mouseY - barHeight / 2;
                if (newTop < 0) {
                    newTop = 0;
                }

                that.$movingBar.css('top', newTop);
                if (mouseY === previousMouseY) {
                    return;
                }

                var barAbove = null;
                for (var i = 0; i < that.sortedBars.length; i++) {
                    var bar = that.barModels[that.sortedBars[i]];

                    if (i > 0) {
                        barAbove = that.barModels[that.sortedBars[i - 1]];
                    }
                    if (i === 0 && bar.id === movingBarId && mouseY < bar.top && mouseY < previousMouseY) { // Trying to separate at top
                        if (!bar.separated) {
                            that.updateMovingBar({
                                top : bar.top - 14,
                                separated : true
                            });
                        }
                        break;
                    }

                    // Inside the bar, swap for now, need to add inside logic
                    if (mouseY > bar.top && mouseY < (bar.top + bar.height)) {
                        if (mouseY > previousMouseY && bar.overlapped) {
                            break;
                        }
                        if (mouseY < movingBar.top || mouseY > movingBar.top + movingBar.height) {
                            if (movingBar.top > bar.top && mouseY < previousMouseY) { // Moving up
                                if (movingBar.positionTo == null) {
                                    if (bar.height > 9 && movingBar.height > 9) { // Place in the middle for M and up bars
                                        that.updateMovingBar({
                                            top : bar.top + bar.height / 2,
                                            positionTo : bar.id,
                                            overlapping : true
                                        });
                                        bar.overlapped = true;
                                        break;
                                    }
                                }
                                that.updateMovingBar({
                                    swap : bar
                                });
                            } else if (movingBarId !== bar.id && movingBar.top < bar.top && mouseY > previousMouseY) {
                                if (barAbove.id === movingBarId) {
                                    that.updateMovingBar({
                                        swap : bar
                                    });
                                } else {
                                    that.updateMovingBar({
                                        top : bar.top + bar.height
                                    });
                                }
                            }
                        }
                        break;
                        // In between 2 separated bars, either connect to the top or bottom
                    }
                    else if (barAbove != null && mouseY < bar.top && mouseY > (barAbove.top + barAbove.height)) { // In between 2 bars
                        if (barAbove.id === movingBarId) {
                            if (mouseY > previousMouseY) {
                                that.updateMovingBar({
                                    top : bar.top - barAbove.height
                                });
                            }
                        } else if (mouseY < previousMouseY) {
                            that.updateMovingBar({
                                top : barAbove.top + barAbove.height
                            });
                        }
                        break;
                    }
                    else if (i === that.sortedBars.length - 1 && bar.id === movingBarId && mouseY > (bar.top + bar.height)) {
                        if (bar.overlapping) {
                            that.updateMovingBar({
                                top : barAbove.top + barAbove.height
                            });
                        }
                        else if (!bar.separated && mouseY > previousMouseY) {
                            that.updateMovingBar({
                                top : bar.top + 14,
                                separated : true
                            });
                        }
                        break;
                    }
                }

                previousMouseY = mouseY;
            });

            this.resetBarTops();

            this.$layoutCanvas.on('mouseup.bar-drag mouseleave.bar-drag', function (event) {
                that.$layoutCanvas.off('mousemove.bar-drag');
                that.$layoutCanvas.removeClass('bar-dragging');
                that.$movingBar.removeClass('bar-dragging');
                that.$layoutCanvas.off('mouseup.bar-drag mouseleave.bar-drag mousemove.bar-drag');

                $placeHolder.remove();
                that.orderBars();
                that.overlapBars();
                that.resetBarTops();
                that.updateConfig();
                that.selectBar(movingBarId);
                that.widget._trigger('config-updated');
            });
        },

        overlapBars : function () {
            var previousBar = null;
            for (var i = 0; i < this.sortedBars.length; i++) {
                var bar = this.barModels[this.sortedBars[i]];
                bar.overlapping = false;
                bar.overlapped = false;
                bar.connectedTop = false;
                bar.connectedBottom = false;
                bar.level = 0;
                if (previousBar != null) {
                    var previousBottom = previousBar.top + previousBar.height;
                    if (bar.top < previousBottom) {
                        bar.overlapping = true;
                        bar.positionTo = previousBar.id;
                        previousBar.overlapped = true;
                        bar.level = previousBar.level + 1;
                        bar.$bar.css('left', 120 + (bar.level * 10));
                    } else {
                        if (bar.top === previousBottom) {
                            bar.connectedTop = true;
                            bar.level = previousBar.level;
                        }
                        previousBar.connectedBottom = bar.top === previousBottom;
                    }
                }
                bar.$bar.css('left', 120 + (bar.level * 10));
                previousBar = bar;
            }
        },

        updateMovingBar : function (options) {
            if (options.swap != null) {
                if (this.movingBar.top < options.swap.top) {
                    options.top = this.movingBar.top + options.swap.height;
                    this.updateBarTop(options.swap, this.movingBar.top);
                } else {
                    options.top = options.swap.top;
                    this.updateBarTop(options.swap, options.swap.top + this.movingBar.height);
                }
            }

            this.movingBar.top = options.top;
            this.movingBar.separated = options.separated;
            this.movingBar.overlapping = options.overlapping;
            this.movingBar.positionTo = options.positionTo;
            this.movingBar.position = options.position;
            this.$placeHolder.css('top', this.movingBar.top);

            this.sortBars();
            this.classifyBarPositions();
        },

        makeLabels : function (options) {
            var that = this;
            var labels = '';
            _.forEach(_.filter(_.values(that.config.labels), function (label) {
                return label.barId === options.barId;
            }), function (label) {
                var templateOptions = {
                    id : label.id,
                    label : _.findWhere(that.widget.flatLabelFields, { dataIndex : label.key }).name,
                    locale : that.widget.options.locale,
                    size : LABEL_HEIGHT[label.render.size].style,
                    position : LABEL_POSITION[label.render.position].style + ' ' +
                    (label.render.verticalPosition != null ? LABEL_POSITION[label.render.verticalPosition].style : ''),
                    showBackground : label.render.color === 'ffffff' ? 'show-background ' : '',
                    color : label.render.color
                };
                labels += labelTemplate(templateOptions);
            });
            return labels;
        },

        updateBarTop : function (barModel, top) {
            barModel.top = top;
            if (barModel.$bar.hasClass('slide-down')) {
                top = top + 7;
            } else if (barModel.$bar.hasClass('slide-up')) {
                top = top - 7;
            }

            barModel.$bar.css('top', top);
        },

        selectBar : function (barId) {
            var that = this;

            this.$selectedItem.removeClass('selected');
            this.$selectedItem.find('.gantt-bar').removeClass('disable-hover');
            this.$selectedItem = $(this.$layoutCanvas.find('.gantt-bar-holder[data-bar-id="' + barId + '"]'));
            this.$selectedItem.addClass('selected');
            this.$selectedItem.find('.gantt-bar').addClass('disable-hover');
            var prevAll = this.$selectedItem.prevAll('.gantt-bar-holder');
            var nextAll = this.$selectedItem.nextAll('.gantt-bar-holder');

            this.resetBarTops();
            var selectedBar = this.barModels[barId];
            this.$selectedItem.css('top', selectedBar.top);
            if (prevAll.length === 0 && nextAll.length) { // If this is the first and there are following slide it up
                if (!this.$selectedItem.hasClass('gantt-summary-bar-holder')) {
                    this.$selectedItem.css('top', selectedBar.top - 7).addClass('slide-up');
                }
            } else if (prevAll.length) {
                if (nextAll.length === 0) {
                    if (!this.$selectedItem.hasClass('gantt-summary-bar-holder')) {
                        this.$selectedItem.css('top', selectedBar.top + 7).addClass('slide-down');
                    }
                } else {
                    prevAll.each(function () {
                        if (!$(this).hasClass('gantt-summary-bar-holder')) {
                            $(this).css('top', that.barModels[$(this).data('bar-id')].top - 7).addClass('slide-up');
                        }
                    });
                    nextAll.each(function () {
                        if (!$(this).hasClass('gantt-summary-bar-holder')) {
                            $(this).css('top', that.barModels[$(this).data('bar-id')].top + 7).addClass('slide-down');
                        }
                    });
                }
            }
        },

        selectLabel : function (labelId) {
            this.$selectedItem.removeClass('selected');
            this.$selectedItem.find('.gantt-bar').removeClass('disable-hover');
            this.$selectedItem = $(this.$layoutCanvas.find('.bar-label[data-label-id="' + labelId + '"]'));
            this.$selectedItem.addClass('selected');
            this.resetBarTops();
        },

        resetBarTops : function () {
            _.forEach(_.values(this.barModels), function (bar) {
                bar.$bar.css('top', bar.top).removeClass('slide-up slide-down');
            });
        },

        renderBars : function () {
            this.renderSummaryBar();

            for (var i = 0; i < this.config.bars.order.length; i++) {
                var barId = this.config.bars.order[i];
                var barConfig = this.config.bars.definitions[barId];
                var top = 70 + (barConfig.render.top - 3) * 1.5;
                var height = BAR_HEIGHT[barConfig.render.size].height * 1.5;
                var $barContainer = $(barHolderTemplate(
                    {
                        size : BAR_HEIGHT[barConfig.render.size].style,
                        color : _.get(BAR_COLOR, barConfig.render.color, BAR_COLOR['66c4be']).style,
                        progressColor :  _.get(BAR_COLOR, barConfig.render.progressColor, BAR_COLOR['66c4be']).progressStyle,
                        showProgress : barConfig.render.showProgress,
                        id : barId,
                        top : top,
                        locale : this.widget.options.locale
                    }));
                $barContainer.find('.gantt-bar').append(this.makeLabels({
                    barId : barId
                }));
                this.$layoutCanvas.append($barContainer);
                this.barModels[barId] = {
                    id : barId,
                    $bar : $barContainer,
                    top : top,
                    height : height,
                    bottom : top + height
                };
            }
            this.orderBars();
            this.classifyBarPositions();
        },

        renderSummaryBar : function () {
            if (this.config.summaryBar != null) {
                var barId = 'summary';
                var barConfig = this.config.summaryBar;
                var top = 20;
                var height = BAR_HEIGHT[barConfig.render.size].height * 1.5;
                var $barContainer = $(summaryBarHolderTemplate(
                    {
                        size : BAR_HEIGHT[barConfig.render.size].style,
                        color : BAR_COLOR[barConfig.render.color].style,
                        progressColor : barConfig.render.progressColor != null ? BAR_COLOR[barConfig.render.progressColor].progressStyle : '',
                        showProgress : barConfig.render.showProgress,
                        id : barId,
                        top : top,
                        locale : this.widget.options.locale
                    }));
                $barContainer.find('.gantt-bar').append(this.makeLabels({
                    barId : barId
                }));
                this.$layoutCanvas.append($barContainer);
                this.barModels[barId] = {
                    id : barId,
                    $bar : $barContainer,
                    top : top,
                    height : height,
                    bottom : top + height
                };
            }
        },

        addBar : function (barId) {
            var barConfig = this.config.bars.definitions[barId];
            var lastBar = this.barModels[_.last(this.sortedBars)];

            var top = lastBar.top + 3 * 1.5 + lastBar.height;
            var height = BAR_HEIGHT[barConfig.render.size].height * 1.5;
            var $barContainer = $(barHolderTemplate(
                {
                    size : BAR_HEIGHT[barConfig.render.size].style,
                    color : BAR_COLOR[barConfig.render.color].style,
                    progressColor : barConfig.render.progressColor ? BAR_COLOR[barConfig.render.progressColor].progressStyle : '',
                    showProgress : barConfig.render.showProgress,
                    id : barId,
                    top : top,
                    locale : this.widget.options.locale
                }));
            this.$layoutCanvas.append($barContainer);
            this.barModels[barId] = { id : barId, $bar : $barContainer, top : top, height : height, bottom : top + height };
            this.orderBars();
            this.classifyBarPositions();
            this.toggleDelete();
        },

        addLabel : function (labelId) {
            var labelConfig = this.config.labels[labelId];
            var bar = this.barModels[labelConfig.barId];
            var label = _.findWhere(this.widget.flatLabelFields, { dataIndex : labelConfig.key });
            var labelText = '';
            if (label != null) {
                labelText = label.name;
            }

            bar.$bar.find('.gantt-bar').append(
                labelTemplate({
                    id : labelConfig.id,
                    label : labelText,
                    locale : this.widget.options.locale,
                    size : LABEL_HEIGHT[labelConfig.render.size].style,
                    position : LABEL_POSITION[labelConfig.render.position].style,
                    color : labelConfig.render.color,
                    showBackground : labelConfig.render.color === 'ffffff' ? 'show-background ' : ''
                }));
        },

        classifyBarPositions : function () {
            var previousBar = null;
            for (var i = 0; i < this.sortedBars.length; i++) {
                var bar = this.barModels[this.sortedBars[i]];
                bar.overlapping = false;
                bar.overlapped = false;
                bar.connectedTop = false;
                bar.connectedBottom = false;
                bar.level = 0;
                if (previousBar != null) {
                    var previousBottom = previousBar.top + previousBar.height;
                    if (bar.top < previousBottom) {
                        bar.overlapping = true;
                        bar.positionTo = previousBar.id;
                        previousBar.overlapped = true;
                        bar.level = previousBar.level + 1;
                        bar.$bar.css('left', 120 + (bar.level * 10));
                    } else {
                        if (bar.top === previousBottom) {
                            bar.connectedTop = true;
                            bar.level = previousBar.level;
                        }
                        previousBar.connectedBottom = bar.top === previousBottom;
                    }
                }
                bar.$bar.css('left', 120 + (bar.level * 10));
                if (bar === this.movingBar) {
                    this.$placeHolder.css('left', 120 + (bar.level * 10));
                }
                previousBar = bar;
            }
        },

        updateConfig : function () {
            var previousBar = this.barModels[this.sortedBars[0]];
            var previousBarConfig = this.config.bars.definitions[previousBar.id];
            previousBarConfig.render.top = 0;
            var bottom = previousBarConfig.render.top + BAR_HEIGHT[previousBarConfig.render.size].height;
            var totalHeight = bottom;
            var bar, barConfig, i;
            for (i = 1; i < this.sortedBars.length; i++) {
                bar = this.barModels[this.sortedBars[i]];
                barConfig = this.config.bars.definitions[bar.id];
                if (bar.connectedTop) {
                    barConfig.render.top = previousBarConfig.render.top + BAR_HEIGHT[previousBarConfig.render.size].height;
                } else if (bar.overlapping) {
                    barConfig.render.top = previousBarConfig.render.top + BAR_HEIGHT[previousBarConfig.render.size].height / 2;
                } else {
                    barConfig.render.top = previousBarConfig.render.top + BAR_HEIGHT[previousBarConfig.render.size].height + 3;
                }
                previousBar = bar;
                previousBarConfig = barConfig;
                bottom = previousBarConfig.render.top + BAR_HEIGHT[previousBarConfig.render.size].height;
                if (bottom > totalHeight) {
                    totalHeight = bottom;
                }
            }
            var rowPadding = 3;
            var topOffset = rowPadding;
            var labelAbove = _.findWhere(_.values(this.config.labels), { render : { verticalPosition : 'A' } });
            var labelBelow = _.findWhere(_.values(this.config.labels), { render : { verticalPosition : 'B' } });
            if (labelAbove != null) {
                rowPadding = parseInt(LABEL_HEIGHT[labelAbove.render.size].height / 2, 10) + 1;
                topOffset = rowPadding * 2;
            } else if (labelBelow != null) {
                rowPadding = parseInt(LABEL_HEIGHT[labelBelow.render.size].height / 2, 10) + 1;
                topOffset = 0;
            }

            if (totalHeight + rowPadding * 2 < 26) {
                topOffset = (26 - totalHeight) / 2;
            }
            topOffset = parseInt(topOffset, 10);
            for (i = 0; i < this.sortedBars.length; i++) {
                bar = this.barModels[this.sortedBars[i]];
                barConfig = this.config.bars.definitions[bar.id];
                barConfig.render.top = barConfig.render.top + topOffset;
                bar.top = 70 + (barConfig.render.top - 3) * 1.5;
                bar.$bar.css({ top : bar.top });
            }
            if (totalHeight < 26 - rowPadding * 2) {
                totalHeight = 26;
            } else {
                totalHeight = totalHeight + rowPadding * 2;
            }
            this.config.layout.rowHeight = totalHeight;
        },

        sortBars : function () {
            this.sortedBars = _.without(_.pluck(_.sortBy(_.values(this.barModels), function (barValues) {
                return barValues.top;
            }), 'id'), 'summary');
        },

        orderBars : function () {
            var that = this;
            this.sortBars();

            var previousBar;
            _.forEach(this.sortedBars, function (barId) {
                var bar = that.barModels[barId];
                if (previousBar != null) {
                    previousBar.$bar.after(bar.$bar);
                }
                previousBar = bar;
            });
            this.config.bars.order = _.clone(this.sortedBars);
        },

        updateLabelColor : function (labelId) {
            var labelConfig = this.config.labels[labelId];
            var $label = this.$layoutCanvas.find('.bar-label[data-label-id="' + labelId + '"]');
            if (labelConfig.render.color === 'ffffff') {
                $label.addClass('show-background');
            } else {
                $label.removeClass('show-background');
            }
            $label.css('color', '#' + labelConfig.render.color);
        },

        updateLabelField : function (labelId, labelField) {
            var $label = this.$layoutCanvas.find('.bar-label[data-label-id="' + labelId + '"] span');
            $label.text(labelField.name);
        },

        updateBarColor : function (barId) {
            var barConfig = this.config.bars.definitions[barId];
            this.$layoutCanvas.find('.gantt-bar[data-bar-id="' + barId + '"]').removeClass(function (index, css) {
                return (css.match(/(^|\s)bar-color-\S+/g) || []).join(' ');
            }).addClass(BAR_COLOR[barConfig.render.color].style).removeClass('');
        },

        updateBarProgressColor : function (barId) {
            var barConfig = this.config.bars.definitions[barId];
            this.$layoutCanvas.find('.gantt-bar[data-bar-id="' + barId + '"]').removeClass(function (index, css) {
                return (css.match(/(^|\s)bar-progress-color-\S+/g) || []).join(' ');
            }).addClass(barConfig.render.progressColor != null ? BAR_COLOR[barConfig.render.progressColor].progressStyle : '').removeClass('');
        },

        updateBarShowProgress : function (barId) {
            this.updateBarProgressColor(barId);
            var barConfig = this.config.bars.definitions[barId];
            this.$layoutCanvas.find('.gantt-bar[data-bar-id="' + barId + '"]').toggleClass('bar-in-progress', barConfig.render.showProgress === true);
        },

        updateBarProgressStyle : function (barId) {
            var barConfig = this.config.bars.definitions[barId];
            this.$layoutCanvas.find('.gantt-bar[data-bar-id="' + barId + '"]').removeClass('bar-progress-modern bar-progress-classic bar-progress-classic-web')
                .addClass(barConfig.render.progressStyle != null ? PROGRESS_STYLE[barConfig.render.progressStyle].style : '').removeClass('');
        },

        updateBarHeight : function (barId) {
            var barConfig = this.config.bars.definitions[barId];
            var changingBar = this.barModels[barId];
            var originalHeight = this.barModels[barId].height;
            var newHeight = 1.5 * BAR_HEIGHT[barConfig.render.size].height;
            var heightChange = newHeight - originalHeight;
            var barsBelow = this.sortedBars.slice(this.sortedBars.indexOf(barId) + 1);

            for (var i = 0; i < barsBelow.length; i++) {
                var bar = this.barModels[barsBelow[i]];
                if (i === 0) {
                    if (bar.overlapping) {
                        if (newHeight > 9) {
                            heightChange = newHeight / 2 + changingBar.top - bar.top;
                        } else {
                            heightChange = changingBar.top + newHeight - bar.top;
                        }
                    }
                }
                this.updateBarTop(bar, bar.top + heightChange);
            }
            changingBar.height = newHeight;

            changingBar.$bar.find('.gantt-bar').removeClass(function (index, css) {
                return (css.match(/(^|\s)gantt-bar-\S+/g) || []).join(' ');
            }).addClass(BAR_HEIGHT[barConfig.render.size].style);
        },

        updateLabelHeight : function (labelId) {
            var barConfig = this.config.labels[labelId];
            var $label = $(this.$layoutCanvas.find('.bar-label[data-label-id="' + labelId + '"]'));

            $label.removeClass('bar-label-small bar-label-medium bar-label-large').addClass(LABEL_HEIGHT[barConfig.render.size].style);
        },

        removeBar : function (barId) {
            var barToDelete = this.barModels[barId];
            barToDelete.$bar.remove();

            var heightChange = -barToDelete.height;
            var newHeight = 0;
            var barsBelow = this.sortedBars.slice(this.sortedBars.indexOf(barId) + 1);

            for (var i = 0; i < barsBelow.length; i++) {
                var bar = this.barModels[barsBelow[i]];
                if (i === 0) {
                    if (bar.overlapping) {
                        heightChange = barToDelete.top + newHeight - bar.top;
                    }
                }
                this.updateBarTop(bar, bar.top + heightChange);
            }
            delete this.barModels[barId];
            this.sortBars();
            this.widget.selectBar(barsBelow[0] || _.last(this.sortedBars));
            this.toggleDelete();
        },

        toggleDelete : function () {
            this.$layoutCanvas.toggleClass('disable-delete-bars', this.sortedBars.length <= 1);
        },

        removeLabel : function (labelId) {
            var $label = $(this.$layoutCanvas.find('.bar-label[data-label-id="' + labelId + '"]'));
            var barParent = $label.parents('.gantt-bar-holder').data('bar-id');
            this.widget.selectBar(barParent);
            $label.remove();
        }

    });

    return barPositioner;
});
