define([
    'gridapi.jquery',
    'gridapi.lodash',
    'ganttapi.gantt/locale',
    'gridapi.logger',
    'gridapi.jqueryui',
    'gridapi.oui',
    'markup!ganttapi.gantt/config/templates/options-config.htm:.htm-layout'
], function ($, _, locale, logger, jqui, oui, layoutMarkup) {
    'use strict';

    var configuratorView = function (widget) {
        this.options = widget.options;
        this.widget = widget;
        this.config = this.widget.config;
    };

    _.assign(configuratorView.prototype, {
        init : function () {
            this.$element = this.widget.element;
            this.$modal = this.$element.parents('.modal');
            this.$layout = $(layoutMarkup({ locale : this.options.locale }));

            this.$element.append(this.$layout);
            this.config = this.widget.config;
            this.timeouts = {};

            this.$showRelationships = this.$layout.find('#show-relationships');
            this.$relationshipOptions = this.$layout.find('.relationship-options');
            this.$showNonDriving = this.$layout.find('#show-non-driving');
            this.$showCriticalPath = this.$layout.find('#show-critical-path');
            this.$showSightLines = this.$layout.find('#show-sight-lines');
            this.$showUnavailableTime = this.$layout.find('#show-unavailable');
            this.$showDateLine = this.$layout.find('#show-date-line');
            this.$timescaleType = this.$layout.find('#timescale-type');
            this.removeDisabled();
            this.initListeners();
            this.applyConfig();
        },

        removeDisabled : function () {
            var that = this;
            _.forEach(this.options.disabled, function (option) {
                switch (option) {
                case 'showRelationships':
                    that.$showRelationships.parents('.relationship-options').hide();
                    break;
                case 'showCriticalPath':
                    that.$showCriticalPath.parent().hide();
                    break;
                case 'showNonDriving':
                    that.$showNonDriving.parent().hide();
                    break;
                case 'showDateLine':
                    that.$showDateLine.parent().hide();
                    break;
                case 'showSightLines':
                    that.$showSightLines.parent().hide();
                    break;
                case 'showUnavailableTime':
                    that.$showUnavailableTime.parent().hide();
                    break;
                }
            });
        },


        initListeners : function () {
            var that = this;
            this.$showRelationships.on('change', function (event) {
                var showRelationships = that.$showRelationships.is(':checked');
                that.config.relationships.showRelationships = showRelationships;
                that.$relationshipOptions.toggleClass('show-relationships', showRelationships);
                that.widget._trigger('config-updated');
            });
            this.$showNonDriving.on('change', function (event) {
                that.config.relationships.showNonDriving = that.$showNonDriving.is(':checked');
                that.widget._trigger('config-updated');
            });
            this.$showCriticalPath.on('change', function (event) {
                that.config.relationships.showCriticalPath = that.$showCriticalPath.is(':checked');
                that.widget._trigger('config-updated');
            });
            this.$showSightLines.on('change', function (event) {
                that.config.chart.showSightLines = that.$showSightLines.is(':checked');
                that.widget._trigger('config-updated');
            });
            this.$showUnavailableTime.on('change', function (event) {
                that.config.chart.showUnavailableTime = that.$showUnavailableTime.is(':checked');
                that.widget._trigger('config-updated');
            });
            this.$showDateLine.on('change', function (event) {
                that.config.chart.showDateLine = that.$showDateLine.is(':checked');
                that.widget._trigger('config-updated');
            });

            this.timescaleTypeDropDown = new oui.ComboBox({
                appendTo : that.$layout.find('.timescale-options'),
                keyboardInput : false,
                allowEmpty : false,
                modal : true,
                required : false,
                viewFns : {
                    display : function (node) {
                        return node.name;
                    }
                }
            }, this.$timescaleType, this.widget.getTimescaleTypes());
            this.timescaleTypeDropDown.init();

            this.timescaleTypeDropDown.on('valueSelected', function (timescale) {
                that.config.timescale.type = timescale.key;
                that.widget._trigger('config-updated');
            });
        },

        applyConfig : function () {
            this.$relationshipOptions.toggleClass('show-relationships', this.config.relationships.showRelationships);
            this.$showRelationships.prop('checked', this.config.relationships.showRelationships);
            this.$showNonDriving.prop('checked', this.config.relationships.showNonDriving);
            this.$showCriticalPath.prop('checked', this.config.relationships.showCriticalPath);
            this.$showSightLines.prop('checked', this.config.chart.showSightLines);
            this.$showDateLine.prop('checked', this.config.chart.showDateLine);
            this.$showUnavailableTime.prop('checked', this.config.chart.showUnavailableTime);
            this.timescaleTypeDropDown.value({ key : this.config.timescale.type });
        }
    });

    return configuratorView;
});
