define([
    'gridapi.jquery',
    'gridapi.lodash',
    'ganttapi.gantt/locale',
    'gridapi.logger',
    'gridapi.jqueryui',
    'gridapi.oui',
    'ganttapi.gantt/enums/timescaleTypes',
    'ganttapi.gantt/config/optionsConfigView'
], function ($, _, locale, logger, jqui, oui, TIMESCALE_TYPE,
             OptionsConfigView) {
    'use strict';

    $.widget('pgbu.ganttOptionsConfigurator', {
        // Defaults
        options : {
            locale : locale.optionsConfigurator,
            config : {},
            disabled : []
        },


        _create : function () {
            this.widgetEventPrefix = 'gantt-config-';

            this.timeouts = {};

            var defaults = {
                relationships : {
                    showRelationships : true,
                    showNonDriving : true,
                    showCriticalPath : true
                },
                chart : {
                    showDateLine : true,
                    showProgress : true,
                    showCriticalBars : true,
                    showSightLines : false,
                    showUnavailableTime : false
                },
                timescale : {
                    type : 'QUARTER_MONTH'
                }


            };
            this.config = _.defaults({}, this.options.config, defaults);
            this.view = new OptionsConfigView(this);
            this.view.init();
        },

        getTimescaleTypes : function () {
            var that = this;
            var timescales = [];
            _.forEach(TIMESCALE_TYPE.order, function (timescale) {
                var ts = _.clone(TIMESCALE_TYPE[timescale], true);
                ts.name = that.options.locale.timescales[ts.label];
                timescales.push(ts);
            });
            return timescales;
        },

        getUpdatedConfig : function () {
            var values = _.pick(this.config, 'relationships', 'chart', 'timescale');
            delete values.chart.showExpandedSummaryBars;
            return values;
        },


        destroy : function () {
            // remove any editors and menus
            _.forEach(this.timeouts, function (timeout) {
                clearTimeout(timeout);
            });
            this.element.children().each(function () {
                $(this).remove();
            });

            this.element.empty();
            this._tearDownTriggers();
            // Does nice things like unbind all namespaced events on the original element
            $.Widget.prototype.destroy.call(this);
        },

        _tearDownTriggers : function () {
            this.element.off(this.widgetEventPrefix);
        }

    });
});
