define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.moment',
    'ganttapi.gantt/timescaleView',
    'ganttapi.gantt/barRenderer',
    'ganttapi.gantt/relationshipRenderer',
    'ganttapi.gantt/relationshipHandlers',
    'ganttapi.gantt/barHandlers',
    'ganttapi.gantt/scrollHandlers',
    'ganttapi.gantt/enums/recordTypes',
    'gridapi.grid/constants',
    'gridapi.grid/utils',
    'markup!ganttapi.gantt/templates/gantt.htm:.htm-gantt-layout',
    'markup!ganttapi.gantt/templates/gantt.htm:.htm-pseudoscroller',
    'markup!ganttapi.gantt/templates/bars.htm:.htm-gantt-row'
], function ($, _, moment, TimescaleView, BarRenderer, RelationshipRenderer, relationshipMixins,
             barMixins, scrollMixins, RECORD_TYPES, constants, utils, ganttMarkup, pseudoScrollerTemplate,
             rowTemplate) {
    // stop exiting keyup functions
    'use strict';

    var $ganttMarkup = $(ganttMarkup());
    var calculateScrollbarWidth = utils.calculateScrollbarWidth;
    var ganttView = function (widget) {
        this.visible = false;
        this.options = widget.options;
        this.widget = widget;
        this.$gantt = $ganttMarkup.clone();

        this.pseudoScrollers = {};
    };

    _.merge(ganttView.prototype, relationshipMixins);
    _.merge(ganttView.prototype, barMixins);
    _.merge(ganttView.prototype, scrollMixins);
    _.merge(ganttView.prototype, {
        init : function () {
            var that = this;
            this.timeouts = {};
            this.$tooltip = $();
            this.ganttRowsetId = this.widget.ganttRowsetId;
            that.widget.element.append(that.$gantt);
            this.$parent = that.$gantt.parent();
            this.$ganttResizer = this.$gantt.find('.gantt-resizer');
            this.$gridContainer = this.$gantt.find('.grid-container');
            this.$ganttContainer = this.$gantt.find('.gantt-container');
            this.$timescaleViewport = this.$gantt.find('.timescale-viewport');
            this.$timescaleContainer = this.$timescaleViewport.find('.timescale-container');
            this.$chartViewport = this.$gantt.find('.chart-viewport');
            this.$chartContainer = this.$chartViewport.find('.chart-container');
            this.$barContainer = this.$chartContainer.find('.bar-container');
            this.$relationshipContainer = this.$chartContainer.find('.relationship-container');
            this.$dataDateLine = this.$chartContainer.find('.data-date-line');
            this.$sightlineContainer = this.$chartContainer.find('.sightline-container');
            this.$unavailableContainer = this.$chartContainer.find('.unavailable-container');

            this.quickAddHeight = 0;

            this.timescale = new TimescaleView({
                widget : this.widget,
                ganttView : this,
                options : this.options.timescale
            });
            this.timescale.createTimeScale();
            this.remakeRelationshipRenderers();
            this.remakeBarRenderers();
            this.visibleRelationships = {};
            this.updateRelationshipDebounce();

            this.$chartContainer.css('width', this.timescale.settings.width + 'px');
            this.$ganttContainer.append(this.createPseudoScroller('vertical', 'bar'));
            this.$ganttContainer.append(this.createPseudoScroller('horizontal', 'bar'));
            this.$ganttContainer.on('keydown', that.keyDown());
            this.$ganttContainer.on('keyup', that.keyUp());

            this.$scrollShims = {
                $bottom : this.$ganttContainer.find('.bottom-scroll-shim'),
                $right : this.$ganttContainer.find('.right-scroll-shim')
            };

            if (!('WebkitAppearance' in document.documentElement.style)) { // Sad sad panda, we have css that changes the width of the scrollbars for chrome and safari.
                var scrollbarSize = calculateScrollbarWidth();
                this.options.scrollbar.extraHeight = scrollbarSize + 1;// Add one for the borderbox and bottom border
                this.options.scrollbar.extraWidth = scrollbarSize;
            }
            this.$scrollShims.$bottom.css('height', this.options.scrollbar.extraHeight);
            this.$scrollShims.$right.css('width', this.options.scrollbar.extraWidth);
            this.resetItemCache();
            this.initListeners();
            this.applyChartOptions();
            this.applyRelationshipOptions();
            this.setDataDate();
        },

        applyLayoutConfig : function () {
            var needsResize = false;
            if (this.options.config.layout.showGrid) {
                this.applyGanttSize(this.options.config.layout.ganttPercent);
                this.$ganttResizer.css('left', 100 - this.options.config.layout.ganttPercent + '%');
            } else {
                this.showGrid(false);
                needsResize = true;
            }
            return needsResize;
        },

        updateRelationshipDebounce : function () {
            this.updateAllRelationships = _.debounce(function () {
                var that = this;
                if (this.options.config.relationships.showRelationships) {
                    _.forEach(_.values(that.visibleRelationships), function (relationship) {
                        that.makeRelationship({ relationship : relationship });
                    });
                }
            }, this.options.performance.updateRelationshipsDebounce);
        },

        showGrid : function (showGrid) {
            if (showGrid) {
                this.$gantt.removeClass('gantt-hide-grid');
            } else {
                this.$gantt.addClass('gantt-hide-grid');
                this.$chartViewport.width(this.$gantt.width());
                this.$timescaleViewport.width(this.$gantt.width());
            }
            var that = this;
            _.delay(function () {
                that.resize();
            }, 1000);
        },

        showGantt : function (showGantt) {
            var repopulate = showGantt !== this.visible;
            if (showGantt) {
                this.$gantt.removeClass('gantt-hide-gantt');
                this.$ganttContainer.css('max-width', '');
                this.$ganttContainer.attr('tabindex', '0');
                this.widget.grid._setOption('layout', { rowHeight : this.options.layout.rowHeight });
            } else {
                this.$gantt.addClass('gantt-hide-gantt');
                this.$ganttContainer.css('max-width', (this.options.scrollbar.extraWidth + 2) + 'px');
                this.$ganttContainer.attr('tabindex', '-1');
                this.widget.grid._setOption('layout', { rowHeight : 26 });
            }
            // repopulate will, itself, resize
            if (repopulate) {
                this.widget.repopulate();
            }
            this.visible = showGantt;
        },

        remakeRelationshipRenderers : function () {
            var that = this;
            this.relationshipRender = new RelationshipRenderer({
                ganttView : that,
                widget : that.widget,
                timescale : that.timescale
            });
        },
        remakeBarRenderers : function () {
            var that = this;
            this.barRenderers = {};
            this.relationshipBarConfig = null;
            this.relationshipBarId = null;
            if (this.options.config.summaryBar != null) {
                this.barRenderers.summary = new BarRenderer({
                    config : this.options.config.summaryBar,
                    id : 'summary',
                    definition : that.options.barTypes[this.options.config.summaryBar.type],
                    ganttView : that,
                    widget : that.widget,
                    timescale : that.timescale
                });
            }
            _.forEach(this.options.config.bars, function (config, index) {
                var id = config.type + '-' + index;
                if (index === 0) {
                    that.defaultBarId = id;
                }
                that.barRenderers[id] = new BarRenderer({
                    config : config,
                    id : id,
                    definition : that.options.barTypes[config.type],
                    ganttView : that,
                    widget : that.widget,
                    timescale : that.timescale
                });
                if (config.render.showRelationships) {
                    if (that.relationshipBarConfig == null) {
                        that.relationshipBarConfig = config;
                        that.relationshipBarId = id;
                    }
                }
            });

            if (that.relationshipBarId == null) {
                that.relationshipBarId = that.defaultBarId;
                that.relationshipBarConfig = that.barRenderers[that.relationshipBarId].config;
            }
        },

        applyChartOptions : function () {
            this.$chartContainer.toggleClass('show-progress', this.options.config.chart.showProgress);
            this.$chartContainer.toggleClass('show-critical-bars', this.options.config.chart.showCritical);
            this.$chartContainer.toggleClass('show-date-line', this.options.config.chart.showDateLine);
            this.$chartContainer.toggleClass('show-sight-lines', this.options.config.chart.showSightLines);
            this.$chartContainer.toggleClass('show-expanded-summary-bars', this.options.config.chart.showExpandedSummaryBars);
            this.$chartContainer.toggleClass('show-unavailable-time', this.options.config.chart.showUnavailableTime);
            this.$timescaleContainer.toggleClass('show-unavailable-time', this.options.config.chart.showUnavailableTime);
        },

        applyRelationshipOptions : function () {
            this.$relationshipContainer.toggleClass('show-relationships', this.options.config.relationships.showRelationships);
            this.$relationshipContainer.toggleClass('show-non-driving', this.options.config.relationships.showNonDriving);
            this.$relationshipContainer.toggleClass('show-critical-path', this.options.config.relationships.showCriticalPath);
        },

        resetItemCache : function () {
            this.ganttItems = {};
        },

        getTimescaleHeight : function () {
            return this.$timescaleContainer.height();
        },

        selectRow : function (args) {
            if (!this.options.config.layout.showGantt) {
                return;
            }
            this.getBarRow(args.recordId).addClass('bar-selected');
        },

        clearSelection : function () {
            if (!this.options.config.layout.showGantt) {
                return;
            }
            this.$barContainer.find('.bar-selected').removeClass('bar-selected');
        },

        zoomToFit : function () {
            this.timescale.zoomToFit();
        },

        initListeners : function () {
            var that = this;
            this.attachScrollListeners();
            var currentPosition = -(that.pseudoScrollers.horizontal.$scroller.scrollLeft());
            var throttledHorizontalHandler = _.throttle(function () {
                var newPosition = -(that.pseudoScrollers.horizontal.$scroller.scrollLeft());
                if (currentPosition !== newPosition) {
                    that.$chartContainer.css('left', newPosition);
                    that.$timescaleContainer.css('left', newPosition);
                    if (!that.$gantt.is('.resizing-timescale')) {
                        that.timescale.updateBucketLabelPositions();
                    }
                    currentPosition = newPosition;
                    that.pseudoScrollers.horizontal.position = currentPosition;
                }
            }, that.options.performance.horizontalScrollThrottle);

            that.pseudoScrollers.horizontal.$scroller.on('scroll', throttledHorizontalHandler);
            this.$chartContainer.on('click', '.bar-row', that.onRowClick());

            this.$chartContainer.on('dblclick', '.summary-bar', that.onSummaryBarDoubleClick());
            this.$chartContainer.on('dblclick', '.bar-row', that.onRowDoubleClick());
            this.$chartContainer.on('mouseenter', '.gantt-bar', that.onBarEnter());
            this.$chartContainer.on('mouseleave', '.gantt-bar', that.onBarExit());
            $(document.body).on('mouseenter', '.bar-tooltip', that.onBarTooltipEnter());
            $(document.body).on('mouseleave', '.bar-tooltip', that.onBarExit());

            this.$ganttResizer.on('mousedown', function (event) {
                var ganttOffset = that.$gantt.offset().left;
                $(this).addClass('dragging');
                that.$gantt.on('mousemove.gantt-resize', _.throttle(function (event) {
                    if (that.$ganttResizer.is('.dragging')) {
                        that.$ganttResizer.css('left', event.clientX - ganttOffset);
                        var newPosition = ((that.$ganttResizer.offset().left - ganttOffset) / that.$gantt.outerWidth()) * 100;
                        that.applyGanttSize(100 - newPosition);
                    }
                }, 75));
                $(document.body).on('mouseup.gantt-resize mouseleave.gantt-resize', function (event) {
                    that.$gantt.off('mousemove.gantt-resize ');
                    $(document.body).off('mouseup.gantt-resize mouseleave.gantt-resize');
                    that.$ganttResizer.removeClass('dragging');
                    var newPosition = ((that.$ganttResizer.offset().left - ganttOffset) / that.$gantt.outerWidth()) * 100;
                    that.applyGanttSize(100 - newPosition);
                });
                that.$ganttResizer.css('left', event.mouseX - ganttOffset);
            });

            this.$chartContainer.add(this.$timescaleContainer).on('mousedown', function (event) {
                var dragTimeout = setTimeout(function () {
                    var startLeft = that.pseudoScrollers.horizontal.$scroller.scrollLeft();
                    var startTop = that.pseudoScrollers.vertical.$scroller.scrollTop();
                    var startX = event.pageX;
                    var startY = event.pageY;
                    var xMovement, yMovement;

                    that.$gantt.addClass('dragging-viewport');
                    $(document.body).on('mousemove.drag-viewport', function (event) {
                        xMovement = startX - event.pageX;
                        yMovement = startY - event.pageY;
                        that.pseudoScrollers.horizontal.$scroller.scrollLeft(startLeft + (xMovement) * 2);
                        that.pseudoScrollers.vertical.$scroller.scrollTop(startTop + yMovement);
                    });
                    $(document.body).on('mouseup.drag-viewport mouseleave.drag-viewport', function () {
                        $(document.body).off('mouseup.drag-viewport mouseleave.drag-viewport');
                        _.delay(function () {
                            that.$gantt.removeClass('dragging-viewport');
                        }, 50);
                        $(document.body).off('mousemove.drag-viewport');
                    });
                }, 220);

                $(document.body).on('mouseup.gantt-drag-viewport', function () {
                    clearTimeout(dragTimeout);
                    $(document.body).off('mouseup.gantt-drag-viewport');
                });
            });

            this.pseudoScrollers.horizontal.$scroller.on('mousedown', function () {
                that.$gantt.addClass('dragging-viewport');
                $(document.body).on('mouseup.drag-viewport', function () {
                    $(document.body).off('mouseup.drag-viewport');
                    that.$gantt.removeClass('dragging-viewport');
                });
            });

            this.$timescaleContainer.on('transitionend', function () {
                that.timescale.updateBucketLabelPositions();
            });

            this.$relationshipContainer.on('mouseenter', '.relationship', that.onRelationshipEnter());
            this.$relationshipContainer.on('mouseleave', '.relationship', that.onRelationshipExit());
            $(document.body).on('mouseenter', '.relationship-tooltip', that.onRelationshipTooltipEnter());
            $(document.body).on('mouseleave', '.relationship-tooltip', that.onRelationshipExit());
        },

        createPseudoScroller : function (type, id) {
            var that = this;
            var $scroller = $(pseudoScrollerTemplate({ type : type, id : 'gantt-' + this.widget.uuid + '-' + id }));
            that.pseudoScrollers[type] = {
                $content : $scroller.find('.empty-content'),
                $scroller : $scroller,
                position : 0
            };
            return $scroller;
        },

        resizeChart : function (ganttHeight, ganttWidth) {
            var chartHeight = ganttHeight - (this.options.layout.timescaleHeight + 1);
            var statusBarHeight = 0;
            if (this.widget.grid && this.widget.grid.options.statusBar.enabled) {
                statusBarHeight = this.widget.grid.options.statusBar.height;
                chartHeight -= statusBarHeight;
            }
            var requiresHorizontalScroll = true;
            if (requiresHorizontalScroll) {
                chartHeight -= this.options.scrollbar.extraHeight;
                this.$scrollShims.$bottom.css('display', 'block');
            } else {
                this.$scrollShims.$bottom.css('display', 'none');
            }
            this.$chartViewport.css('height', chartHeight + 'px');
            var requiresVerticalScroll = true;
            this.chartWidth = ganttWidth;
            if (requiresVerticalScroll) {
                this.chartWidth -= this.options.scrollbar.extraWidth;
                this.$scrollShims.$right.css('display', 'block');
            } else {
                this.$scrollShims.$right.css('display', 'none');
            }
            if (!this.options.config.layout.showGantt) {
                this.chartWidth = this.options.scrollbar.extraWidth;
            }

            this.pseudoScrollers.horizontal.$scroller.css('width', this.chartWidth);
            this.pseudoScrollers.horizontal.$scroller.css('left', '0');
            this.pseudoScrollers.horizontal.$content.css('width', this.timescale.settings.width);
            this.pseudoScrollers.horizontal.$scroller.css('top', ganttHeight - this.options.scrollbar.extraHeight - 2 - statusBarHeight);
            this.pseudoScrollers.vertical.$scroller.css('top', this.options.layout.timescaleHeight + this.quickAddHeight + 2 + 'px');
            this.pseudoScrollers.vertical.$scroller.css('height', chartHeight - this.quickAddHeight);

            this.$chartViewport.css('width', this.chartWidth + 'px');
            this.$timescaleViewport.css('width', this.chartWidth + 'px');

            this.$chartContainer.css('width', this.timescale.settings.width + 'px');

            this.timescale.updateBucketLabelPositions();
        },

        setDataDate : function () {
            if (this.options.dataDate != null && moment.isMoment(this.options.dataDate)) {
                this.$dataDateLine.css('left', this.timescale.getOffset(this.options.dataDate) + 'px'); // moment here should be data date
            }
        },

        scrollFocusIntoView : function (args) {
            if (!this.options.config.layout.showGantt) {
                return;
            }
            var hScroller = this.pseudoScrollers.horizontal;
            if (args.barId == null) {
                args.barId = this.defaultBarId;
            }
            if (hScroller != null) {
                //>>excludeStart("perf", pragmas.production)
                var start;
                if (this.options.debug) {
                    start = window.performance.now();
                }
                //>>excludeEnd("perf")
                if (args.recordId == null || args.recordId === constants.DUMMY_RECORD_ID) {
                    args.recordId = this.widget.grid._getFirstRecordId();
                }
                var barCache = this.ganttItems[args.recordId];
                if (barCache != null) {
                    var bar = barCache[args.barId];
                    if (bar == null) {
                        bar = _.first(_.values(barCache));
                    }
                    if (bar.start == null || bar.end == null) {
                        return;
                    }
                    var width = bar.render.width,
                        leftOffset = -hScroller.position;
                    if (args.center) {
                        if (width < this.chartWidth) { // Center the bar
                            if ((bar.render.left < leftOffset || // If its not showing the start of the bar
                                (bar.render.left + width) > (leftOffset + this.chartWidth)) || args.force) { // if its not showing the end of the bar
                                var negativeSpace = this.chartWidth - width;
                                leftOffset = bar.render.left - negativeSpace / 2;
                            }
                        } else {
                            leftOffset = bar.render.left - 20;
                        }
                    } else {
                        leftOffset = bar.render.left - 50;
                    }

                    // Make the ui a bit snappier if you delay this
                    setTimeout(function () {
                        hScroller.$scroller.scrollLeft(leftOffset); // add some padding so its not right against the edge
                    }, 1);
                    //>>excludeStart("perf", pragmas.production)
                    if (this.options.debug) {
                        this.widget._trigger('performance-debug', null,
                            {
                                start : start,
                                end : window.performance.now(),
                                name : 'scroll-Focus-Into-View',
                                type : 'view'
                            });
                    }
                    //>>excludeEnd("perf")
                }
            }
        },

        getBarRow : function (recordId) {
            return this.$barContainer.find('.bar-row[data-id="' + recordId + '"]');
        },

        updateRow : function (args) {
            if (!this.options.config.layout.showGantt) {
                return;
            }
            var oldBar = this.getBarRow(args.recordId);
            if (oldBar.length) {
                this.updateGanttRow(oldBar);
            }
        },

        removeRow : function (args) {
            if (!this.options.config.layout.showGantt) {
                return;
            }
            this.getBarRow(args.recordId).remove();
            var that = this;
            var relationships = this.$relationshipContainer.find('.relationship[data-predecessor="' + args.recordId + '"],.relationship[data-successor="' +
                args.recordId + '"]');
            relationships.each(function () {
                delete that.visibleRelationships[$(this).attr('data-id')];
            }).remove();
            this.updateAllRelationships();
        },

        removeRelationship : function (relationshipId) {
            if (this.visibleRelationships[relationshipId] != null) {
                this.$relationshipContainer.find('.relationship[data-id="' + relationshipId + '"]').remove();
                delete this.visibleRelationships[relationshipId];
            }
        },

        makeRelationship : function (args) {
            if (args.relationship == null) {
                return null;
            }
            if (this.visibleRelationships[args.relationship.id] == null) {
                var relationshipToAdd = this.relationshipRender.makeRelationship(
                    {
                        relationship : args.relationship,
                        barId : this.relationshipBarId,
                        barConfig : this.relationshipBarConfig,
                        lineSize : 1,
                        linePadding : 2,
                        offset : 10
                    });
                if (relationshipToAdd.length) {
                    this.visibleRelationships[args.relationship.id] = args.relationship;
                    return relationshipToAdd;
                }
            } else {
                this.relationshipRender.remakeRelationship(
                    {
                        relationship : args.relationship,
                        barId : this.relationshipBarId,
                        barConfig : this.relationshipBarConfig,
                        lineSize : 1,
                        linePadding : 2,
                        offset : 10
                    });
            }
        },

        adjustForQuickAddRow : function (enabled) {
            var height = this.options.layout.rowHeight;
            if (!enabled) {
                height = 0;
            }
            this.quickAddHeight = height;
            this.$chartContainer.css('top', height + 2);
        },

        makeRow : function (recordId) {
            if (recordId === 'DUMMY-NEW-RECORD') {
                this.adjustForQuickAddRow(true);
                return '<div id="0gantt-row-DUMMY-NEW-RECORD" data-id="DUMMY-NEW-RECORD" class="bar-row" style="height:26px;"/>';
            }
            return this.makeGanttRow(recordId);
        },

        makeGanttRow : function (recordId) {
            var barMarkup = '';
            var parentIndicator = '';
            var node = this.widget.grid.view.virtualRows[this.ganttRowsetId].nodes[recordId];
            var ganttAttributes = this.widget.getGanttAttributes({
                recordId : recordId
            });
            if (ganttAttributes.type === RECORD_TYPES.SUMMARY && this.barRenderers.summary != null) {
                barMarkup = this.barRenderers.summary.makeBar({
                    recordId : recordId,
                    record : this.widget._getRecord(recordId),
                    barAttributes : this.widget.getBarAttributes({
                        recordId : recordId,
                        barConfig : this.options.config.summaryBar
                    }),
                    ganttAttributes : ganttAttributes,
                    barConfig : this.options.config.summaryBar,
                    node : node
                });
            } else {
                for (var i = 0; i < this.options.config.bars.length; i++) {
                    var barConfig = this.options.config.bars[i];
                    var barRenderer = this.barRenderers[this.getBarConfigId(barConfig, i)];
                    var renderOptions = {
                        recordId : recordId,
                        record : this.widget._getRecord(recordId),
                        barAttributes : this.widget.getBarAttributes({
                            recordId : recordId,
                            barConfig : barConfig
                        }),
                        ganttAttributes : ganttAttributes,
                        barConfig : barConfig,
                        node : node
                    };

                    barMarkup += barRenderer.makeBar(renderOptions);
                }
            }

            return rowTemplate({
                id : this.widget.uuid + 'gantt-row-' + recordId,
                recordId : recordId,
                bars : barMarkup,
                parentIndicator : parentIndicator,

                height : this.options.layout.rowHeight
            });
        },

        getBarConfigId : function (barConfig, index) {
            return barConfig.type + '-' + index;
        },

        getElementDataId : function (element) {
            return this.widget.grid.view.fixRecordIdType(element.attr('data-id'));
        },

        updateGanttRow : function ($barRow) {
            var recordId = this.getElementDataId($barRow);
            var typeChanged = false;
            var node = this.widget.grid.view.virtualRows[this.ganttRowsetId].nodes[recordId];
            var ganttAttributes = this.widget.getGanttAttributes({
                recordId : recordId
            });
            var $bar;
            delete this.ganttItems[recordId];
            if (ganttAttributes.type === RECORD_TYPES.SUMMARY && this.barRenderers.summary != null) {
                $bar = $barRow.find('.gantt-bar[data-bar="summary"]');
                if ($bar.length) {
                    this.barRenderers.summary.updateBar({
                        recordId : recordId,
                        record : this.widget._getRecord(recordId),
                        barAttributes : this.widget.getBarAttributes({
                            recordId : recordId,
                            barConfig : this.options.config.summaryBar
                        }),
                        ganttAttributes : ganttAttributes,
                        barConfig : this.options.config.summaryBar,
                        node : node,
                        $bar : $barRow.find('.gantt-bar[data-bar="summary"]')
                    });
                } else {
                    typeChanged = true;
                }
            } else {
                for (var i = 0; i < this.options.config.bars.length; i++) {
                    var barConfig = this.options.config.bars[i];
                    $bar = $barRow.find('.gantt-bar[data-bar="' + barConfig.type + '-' + i + '"]');
                    if ($bar.length) {
                        var barRenderer = this.barRenderers[this.getBarConfigId(barConfig, i)];
                        var barAttributes = this.widget.getBarAttributes({
                            recordId : recordId,
                            barConfig : barConfig
                        });
                        var renderOptions = {
                            ganttAttributes : ganttAttributes,
                            recordId : recordId,
                            record : this.widget._getRecord(recordId),
                            barAttributes : barAttributes,
                            node : node,
                            $bar : $bar

                        };

                        barRenderer.updateBar(renderOptions);
                    }
                    else {
                        typeChanged = true;
                    }
                }
            }
            if (typeChanged) {
                $barRow.replaceWith(this.makeGanttRow(recordId));
            }
            this.updateRowRelationships({ recordId : recordId });
        },

        updateRowRelationships : function (args) {
            if (!this.options.config.layout.showGantt) {
                return;
            }
            var that = this;
            if (this.options.config.relationships.showRelationships) {
                var relationships = this.widget.getRelationships({ recordId : args.recordId });
                var relationshipMarkup = '';
                _.forEach(relationships, function (relationship) {
                    var newRelationship = that.makeRelationship({ relationship : relationship });
                    if (newRelationship != null) {
                        relationshipMarkup += newRelationship;
                    }
                });
                if (relationshipMarkup.length) {
                    that.$relationshipContainer.append(relationshipMarkup);
                }
            }
        },

        updateRelationship : function (args) {
            if (!this.options.config.layout.showGantt) {
                return;
            }
            var relationship = this.widget.getRelationship({ relationshipId : args.relationshipId });
            var relationshipMarkup = '';
            var newRelationship = this.makeRelationship({ relationship : relationship });
            if (newRelationship != null) {
                relationshipMarkup += newRelationship;
            }
            if (relationshipMarkup.length) {
                this.$relationshipContainer.append(relationshipMarkup);
            }
        },

        resizeScale : function (args) {
            var that = this;
            if (args.change == null) {
                return;
            }
            var newScale = this.timescale.appliedTimescale.options.bucketSize + args.change;
            if (newScale < 5) {
                newScale = 5;
            } else if (newScale > 500) {
                newScale = 200;
            }
            if (newScale !== this.timescale.settings.bucketSize) {
                this.timescale.resizeTimescale(newScale, args.quick);

                if (args.quick) {
                    this.$barContainer.find('.gantt-bar').each(function () {
                        var $bar = $(this);
                        var id = that.getElementDataId($bar.parent());
                        var barId = $bar.attr('data-bar');
                        var item = that.ganttItems[id][barId];
                        item.render.width = item.units.width * newScale;
                        item.render.left = item.units.offset * newScale;
                        item.render.right = item.render.left + item.render.width;
                        if (!$bar.is('.milestone-bar')) {
                            $bar.width(item.render.width);
                        }
                        $bar.css('left', item.render.left + 'px');
                    });
                }
                else {
                    this.ganttItems = {};
                    this.$barContainer.find('.bar-row').each(function () {
                        that.updateGanttRow($(this));
                    });
                    this.setDataDate();
                }
                if (!args.quick) {
                    this.scrollFocusIntoView({ recordId : this.getElementDataId(this.$barContainer.find('.bar-selected')) });
                    this.resize();
                    this.timescale.updateBucketLabelPositions();
                }
            }
        },

        updateTimescaleRange : function () {
            this.timescale.setTimescaleRange();
            this.changeTimescale(this.timescale.appliedTimescaleId, true);
        },

        changeTimescale : function (timescale, force, options) {
            var that = this;
            if (timescale !== this.timescale.appliedTimescaleId || force) {
                this.ganttItems = {};
                this.options.config.timescale.type = timescale;
                this.$gantt.addClass('transitioning-timescale');
                _.delay(function () {
                    that.$gantt.removeClass('transitioning-timescale');
                }, 1000);
                this.timescale.applyTimescale(timescale);
                this.$barContainer.find('.bar-row').each(function () {
                    that.updateGanttRow($(this));
                });
                this.setDataDate();
                this.resizeChart(this.ganttHeight, this.ganttContainerWidth);
                that.scrollFocusIntoView({ recordId : that.getElementDataId(that.$barContainer.find('.bar-selected')) });
                that.widget._trigger('view-updated', null, {
                    action : 'timescale-change',
                    userAction : options && options.userAction,
                    scale : that.timescale.appliedTimescaleId,
                    min : that.timescale.appliedTimescale.options.range.start,
                    max : that.timescale.appliedTimescale.options.range.end,
                    granularity : that.timescale.appliedTimescale.options.granularity
                });
            }
        },

        makeRows : function (rowIds) {
            var that = this;
            var createdRows = _.map(rowIds, function (rowId) {
                return that.makeRow(rowId);
            });

            if (this.options.config.relationships.showRelationships) {
                var relationshipMarkup = '';
                _.forEach(rowIds, function (recordId) {
                    var relationships = that.widget.getRelationships({ recordId : recordId });

                    _.forEach(relationships, function (relationship) {
                        var newRelationship = that.makeRelationship({ relationship : relationship });
                        if (newRelationship != null) {
                            relationshipMarkup += newRelationship;
                        }
                    });
                });
                that.$relationshipContainer.append(relationshipMarkup);
            }

            return createdRows;
        },

        attachRows : function (args) {
            if (!this.options.config.layout.showGantt) {
                return;
            }
            var that = this;
            var rowsToRemove = $();
            switch (args.action) {
            case 'replace' :
                that.$barContainer.empty();
                break;
            case 'append' :
                rowsToRemove = that.$barContainer.find('.bar-row:lt(' + args.actionIndex + ')');
                break;
            case 'prepend' :
                rowsToRemove = that.$barContainer.find('.bar-row:gt(' + args.actionIndex + ')');
                break;
            case 'insert' :
                // Remove from the end the number you are trying to add, must work or further pruning breaks;
                rowsToRemove = that.$barContainer.find('.bar-row:last');
                break;
            case 'insert-skip-delete':
                // We have to do this after
                break;
            }
            that.updateAllRelationships();
            rowsToRemove.remove();
            var rowMarkup = this.makeRows(args.rows);
            switch (args.action) {
            case 'replace' :
            case 'append' :
                that.$barContainer.append(rowMarkup);
                break;
            case 'prepend' :
                that.$barContainer.prepend(rowMarkup);
                break;
            case 'insert-skip-delete':
            case 'insert' :
                that.$barContainer.children().eq(args.actionIndex).after(rowMarkup);
                break;
            }
        },

        applyGanttSize : function (ganttSize) {
            this.$ganttContainer.css('width', ganttSize + '%');
            this.$gridContainer.css('width', 100 - ganttSize + '%');
            this.widget.resize();
            this.options.config.layout.ganttPercent = ganttSize;
        },

        resize : function () {
            var width, height;
            var widthIsPercent = this.options.layout.width.indexOf('%') !== -1;
            var heightIsPercent = this.options.layout.height.indexOf('%') !== -1;
            if (widthIsPercent || heightIsPercent) {
                this.$gantt.hide();// hide if percentage
            }

            if (widthIsPercent) {
                width = this.$parent.width() * (parseFloat(this.options.layout.width) / 100);
                if (this.options.layout.widthShim != null) {
                    width -= parseInt(this.options.layout.widthShim, 10);
                }
            } else {
                width = parseFloat(this.options.layout.width);
            }

            if (heightIsPercent) {
                height = this.$parent.height() * (parseFloat(this.options.layout.height) / 100);

                if (this.options.layout.heightShim != null) {
                    height -= parseInt(this.options.layout.heightShim, 10);
                }
            } else {
                height = parseFloat(this.options.layout.height);
            }

            if (widthIsPercent || heightIsPercent) {
                this.$gantt.show();
            }

            this.ganttWidth = parseInt(width, 10);
            this.ganttHeight = parseInt(height, 10);

            // Set the size of the grid.
            this.$gantt.css({
                width : this.ganttWidth,
                height : this.ganttHeight
            });
            this.ganttContainerWidth = this.$ganttContainer.width();
            this.resizeChart(this.ganttHeight, this.ganttContainerWidth);
        },

        attachScrollListeners : function () {
            var that = this;
            var scroller = this.pseudoScrollers.vertical;
            var eventName = 'wheel'; // Standardized event
            if (document.onmousewheel !== undefined && document.onwheel === undefined) {
                eventName = 'mousewheel'; // Only needed for Safari
            }
            this.$chartViewport.on(eventName, function (event) {
                function getWheelDelta(e) {
                    if (!(e.wheelDeltaY || e.deltaY || e.wheelDelta)) {
                        e = e.originalEvent;
                    }

                    if (e.deltaY) {
                        return -e.deltaY;
                    } else if (e.wheelDeltaY) {
                        return e.wheelDeltaY;
                    } else if (e.wheelDelta) {
                        return e.wheelDelta;
                    }
                }

                var deltaY = getWheelDelta(event);
                var scrollAmount = 0;
                if (deltaY > 0) {
                    scrollAmount = -that.options.layout.rowHeight;
                } else if (deltaY < 0) {
                    scrollAmount = that.options.layout.rowHeight;
                }
                if (scrollAmount !== 0) {
                    scroller.$scroller.scrollTop(scroller.$scroller.scrollTop() + scrollAmount);
                    return false;
                }
            });
        },

        setEscapeMode : function (escape) {
            if (escape !== this.escapeMode) {
                this.escapeMode = escape;
                if (this.escapeMode) {
                    this.$ganttContainer.addClass('escape-mode');
                } else {
                    this.$ganttContainer.removeClass('escape-mode');
                }
            }
        },

        _syncGridListeners : function () {
            var that = this;

            this.$gridContainer.on('grid-view-sync-long-render', function (event, args) {
                that.toggleLongRender(args);
            });

            this.$gridContainer.on('grid-view-sync-clear-selection', function (event, args) {
                that.clearSelection(args);
            });

            this.$gridContainer.on('grid-view-sync-select-row', function (event, args) {
                that.selectRow(args);
            });

            this.$gridContainer.on('grid-view-sync-remove-row', function (event, args) {
                that.removeRow(args);
            });

            this.$gridContainer.on('grid-view-sync-update-row', function (event, args) {
                that.updateRow(args);
            });

            this.$gridContainer.on('grid-view-sync-scroll-focus-into-view', function (event, args) {
                if (that.options.config.chart.syncGridViewFocus) {
                    that.scrollFocusIntoView(args);
                }
            });

            this.$gridContainer.on('grid-view-sync-attach-rows', function (event, args) {
                that.attachRows(args);
            });

            this.$gridContainer.on('grid-view-sync-get-header-height', function (event, args) {
                return that.getTimescaleHeight();
            });

            this.$gridContainer.on('grid-view-sync-content-height-sync', function (event, args) {
                that.$relationshipContainer.css('height', args.newHeight);
            });

            this.$gridContainer.on('grid-view-sync-scroll-top', function (event, args) {
                that.$relationshipContainer.css('top', '-' + args.top + 'px');
            });

            this.$gridContainer.on('grid-view-sync-escape-mode', function (event, args) {
                that.setEscapeMode(args.escapeMode);
            });
        },

        toggleLongRender : function (args) {
            if (!this.options.config.layout.showGantt) {
                return;
            }
            this.$chartViewport.toggleClass('long-render', args.longRender);
            this.pseudoScrollers.vertical.$scroller.toggleClass('long-render', args.longRender);
        },
        hideTooltip : function () {
            this.$tooltip.remove();
            this.$tooltip = $();
        }
    });
    return ganttView;
});
