define([
    'gridapi.jquery',
    'gridapi.lodash'
], function ($, _) {
    'use strict';

    return {
        onRelationshipEnter : function () {
            var that = this;
            return function (event) {
                that.timeouts.relationshipEnter = setTimeout(function () {
                    if (!that.$gantt.hasClass('dragging-viewport')) {
                        that.showRelationshipTooltip(event);
                    }
                }, that.options.tooltips.delay);
            };
        },

        onRelationshipExit : function () {
            var that = this;
            return function (event) {
                clearTimeout(that.timeouts.relationshipEnter);
                that.timeouts.relationshipExit = setTimeout(function () {
                    that.hideTooltip();
                }, 50);
            };
        },

        onRelationshipTooltipEnter : function () {
            var that = this;
            return function (event) {
                clearTimeout(that.timeouts.relationshipExit);
            };
        },

        showRelationshipTooltip : function (event) {
            var that = this;
            this.hideTooltip();
            var relationshipId = $(event.currentTarget).attr('data-id');
            this.$tooltip = $(this.relationshipRender.makeTooltip({ event : event, relationshipId : relationshipId }));
            $('body').append(this.$tooltip);
            var width = this.$tooltip.outerWidth();
            var height = this.$tooltip.outerHeight();
            var left = event.clientX - width / 2;
            var top = event.clientY + 8;

            var $target = $(event.target);
            if ($target.is('.relationship-vertical')) {
                if ($target.is('.relationship-right') && $target.offset().left + 10 + width < $(window).width()) {
                    left = $target.offset().left + 10;
                    top = event.clientY - height / 2;
                    that.$tooltip.addClass('tooltip-right');
                } else {
                    left = $target.offset().left - 5 - width;
                    top = event.clientY - height / 2;
                    that.$tooltip.addClass('tooltip-left');
                }
            } else {
                if ($target.is('.relationship-top') && $target.offset().top - height > 0) {
                    top = $target.offset().top - height - 5;
                    that.$tooltip.addClass('tooltip-above');
                } else {
                    top = $target.offset().top + 10;
                    that.$tooltip.addClass('tooltip-below');
                }

                if (left + width > $(window).width()) {
                    left = $(window).width() - width - 10;
                }
            }


            _.delay(function () {
                that.$tooltip.css('left', left);
                that.$tooltip.css('top', top);
                that.$tooltip.addClass('show-tooltip');
            }, 20);
        }
    };
});
