define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/view/menuUtils',
    'markup!gridapi.columnGroupMenu/menuRoot.htm',
    'markup!gridapi.columnGroupMenu/menuItem.htm',
    'markup!gridapi.columnGroupMenu/menuHeader.htm'
], function ($, _, menus, menuRootTemplate, menuItemTemplate, menuHeaderTemplate) {
    'use strict';

    var DIVIDER = '<li class="divider" role="separator"></li>';
    var GROUP_PARENT = '<ul class="columns-list" role="presentation" tabindex="-1"></ul>';
    function ColumnGroupMenuView(columnGroupMenu, group) {
        this.columnGroupMenu = columnGroupMenu;
        this.grid = this.columnGroupMenu.grid;
        this.group = group;
        this.$root = null;
        this.$menu = null;
    }

    ColumnGroupMenuView.prototype = {
        render : function (args) {
            var $opener = this.$opener = args.$opener;
            var position = $opener.offset();
            var openerWidth = $opener.width();
            $opener.addClass('opened');
            this.$root = $(menuRootTemplate({
                id : args.id,
                left : position.left + 'px',
                top : (position.top + args.$opener.outerHeight()) + 'px'
            }));
            this.$menu = this.$root.find('[role=menu]');
            var locale = this.grid.options.locale;
            this.$menu.append($(menuHeaderTemplate({
                text : locale.selectedColumns
            })));
            this._sections = {};
            this._sections.selected = $(GROUP_PARENT).attr('data-section', 'selected').appendTo(this.$menu);
            this.$menu.append(DIVIDER);
            this.$menu.append($(menuHeaderTemplate({
                text : locale.unselectedColumns
            })));
            this._sections.unselected = $(GROUP_PARENT).attr('data-section', 'unselected').appendTo(this.$menu);
            this._refreshSections();
            this._attachToBody();
            this._initListeners();
            this.$menu.css('left', position.left - this.$menu.width() + (openerWidth * 2) - 2);
            this._selectInitialMenuItem(args);
        },

        _refreshSections : function () {
            var that = this;
            this.$menu.find('.columns-list').empty();
            var columnSections = _.groupBy(this.group.columns, function (col) {
                return col.visible ? 'selected' : 'unselected';
            });
            _.forEach(columnSections, function (columns, section) {
                var $section = that._sections[section];
                _.forEach(columns, function (column) {
                    if (column.dataIndex !== 'empty-column') {
                        $section.append(menuItemTemplate({
                            name : column.title,
                            dataIndex : column.dataIndex,
                            classes : 'column-choice'
                        }));
                    }
                });
            });
        },

        _attachToBody : function () {
            $(document.body).append(this.$root);
        },

        _initListeners : function () {
            this.$menu.on('click', '.columns-list a', this._clickColumn.bind(this));
            this.events = menus.initDestroyEvents({
                grid : this.grid,
                $el : this.$menu,
                destroyFn : this.columnGroupMenu.destroy,
                rootSelector : '.columngroup-dropdown'
            });

            var that = this;
            var $sections = $(this._sections.selected).add(this._sections.unselected);
            $sections.sortable({
                connectWith : '.columns-list',
                handle : '.pgbu-icon-drag-drop-dots',
                update : function (ev, ui) {
                    var columnId = ui.item.attr('data-index');
                    var $newGroup = ui.item.parents('[data-section]');
                    var isNowVisible = !!$newGroup.is('[data-section=selected]');
                    var column = _.findWhere(that.group.columns, { dataIndex : columnId });
                    var indexWithinGroup = $newGroup.find('li').index(ui.item);
                    that._updateColumnPlacement(column, isNowVisible, indexWithinGroup);
                }
            });
        },
        _canHideColumn : function (column) {
            return this.grid.view._canHideColumn(this.group, column);
        },
        _updateColumnPlacement : function (column, isNowVisible, indexWithinGroup) {
            column.visible = isNowVisible;
            var columns = this.group.columns;
            columns.splice(columns.indexOf(column), 1);

            var matchingIndices = [];
            for (var i = 0, len = columns.length; i < len; ++i) {
                if (columns[i].visible === column.visible) {
                    matchingIndices.push(i);
                }
            }

            var insertPos;
            if (matchingIndices.length > indexWithinGroup) {
                insertPos = matchingIndices[indexWithinGroup];
            } else {
                var lastMatchingIndex = _.last(matchingIndices);
                if (lastMatchingIndex === (columns.length - 1)) {
                    columns.push(column);
                } else {
                    insertPos = lastMatchingIndex + 1;
                }
            }
            if (insertPos != null) {
                columns.splice(insertPos, 0, column);
            }
            this.grid.view._refreshColumnVisibility();
        },
        _clickColumn : function (ev) {
            ev.preventDefault();
            var dataIndex = $(ev.target).closest('.column-choice').attr('data-index');
            var column = _.findWhere(this.group.columns, { dataIndex : dataIndex });
            this._toggleColumn(column);
        },
        _toggleColumn : function (column) {
            if (column.visible && !this._canHideColumn(column)) {
                return;
            }

            this.grid.view.switchColumnVisibilty(column);
            this.columnGroupMenu.destroy();
            this.grid.view.refocusHeader(this.group);
        },

        _selectInitialMenuItem : function (args) {
            this.columnGroupMenu.allowRefocus();
            var $firstThing = this.$menu.find('li a').first();
            if (args.fromKeyboard && $firstThing.length) {
                $firstThing.focus();
            } else {
                this.$menu.focus();
            }
        },

        destroy : function (args) {
            if (this.$root != null) {
                if (this.$opener) {
                    this.$opener.removeClass('opened');
                }
                this.$root.remove();
                this.$root = null;
                this.$menu = null;
                this.events.destroy();
                this.events = null;
                this.grid.view._disownMenu(this.grid.view.componentIds.columnGroupMenu);
            }
        }
    };

    return ColumnGroupMenuView;
});
