define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.columnMenu/columnSubMenus'
], function ($, _, subMenu) {
    'use strict';
    var MENU_SELECTOR = '.pgbu-grid-menu';

    return {
        'sort-asc' : function ($li, options) {
            if (!$li.is('.disabled')) {
                var $menu = $li.closest(MENU_SELECTOR),
                    menuApi = $menu.data('menu-api');
                options.grid.widget.sortRows(options.column.dataIndex, this.ctrlKey, 'asc');
                menuApi.destroy();
            }
        },
        'sort-dsc' : function ($li, options) {
            if (!$li.is('.disabled')) {
                var $menu = $li.closest(MENU_SELECTOR),
                    menuApi = $menu.data('menu-api');
                options.grid.widget.sortRows(options.column.dataIndex, this.ctrlKey, 'dsc');
                menuApi.destroy();
            }
        },
        'check-column' : function ($li, options) {
            // destroy deepest sub menu
            $li.parent().find('.sub-menu').remove();

            if ($li.is('.sub-menu-item')) {
                // make the selection appear disappeared from sub menu item
                $li.addClass('item-focused-transparent');

                // expand column sub menu
                var index = $li.data('dataLayoutIndex'),
                    columnLayout = $li.data('columnLayout');
                // get the column layout for the next level if not root level
                if (_.isNumber(index)) {
                    columnLayout = columnLayout.subMenus[index];
                }
                subMenu.init(options.grid, $li, 'columnToggle', {
                    toggleLayout : columnLayout,
                    column : options.column
                });
            }
        },
        'column' : function ($li, options) {
            var $menu = $li.closest(MENU_SELECTOR),
                menuApi = $menu.data('menu-api');

            if (!$li.is('.disabled')) {
                var checkbox = $li.find('input[type="checkbox"]');
                // if event came from checkbox then no need to reset its current state
                if (!$(this.target).is('input[type="checkbox"]')) {
                    if (checkbox.is(':checked')) {
                        checkbox.removeAttr('checked');
                    } else {
                        checkbox.attr('checked', 'checked');
                    }
                }
                var col = options.grid.columnDefinitions[$li.attr('data-index')];
                options.grid.switchColumnVisibilty(col);

                // enable sorting capabilities for column
                if (options.column.dataIndex === col.dataIndex) {
                    var $rootMenu = $li.closest('.root-menu');
                    var $liSorts = $rootMenu.find('li[data-action^="sort-"]');
                    if (col.visible) {
                        $liSorts.removeClass('disabled');
                    } else {
                        $liSorts.addClass('disabled');
                    }
                }

                // disable last available chec column
                menuApi.disableLastColumnToggle();
            }
        }
    };
});
