define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/utils/strings',
    'gridapi.grid/view/menuUtils',
    'markup!gridapi.columnMenu/menuRoot.htm',
    'markup!gridapi.columnMenu/menuItem.htm',
    'markup!gridapi.columnMenu/menuHeader.htm'
], function ($, _, strings, menus, menuRootTemplate, menuItemTemplate, menuHeaderTemplate) {
    'use strict';

    var DIVIDER = '<li class="divider" role="separator"></li>';
    function ColumnMenuView(columnMenu, column, group) {
        this.columnMenu = columnMenu;
        this.grid = this.columnMenu.grid;
        this.column = column;
        this.groupId = group;
        this.group = this.grid.view._getGroup(this.groupId);
        this.$root = null;
        this.$menu = null;
    }

    ColumnMenuView.prototype = {
        render : function (args) {
            var actions = this._initActions();
            if (actions.length) {
                var that = this;
                var menuText = menuRootTemplate({
                    left : args.left,
                    top : args.top
                });
                this.$root = $(menuText);
                this.$menu = this.$root.find('[role=menu]');
                this.$menu.data('menu-api', this.columnMenu);
                _.forEach(actions, function (action) {
                    if (strings.isString(action)) {
                        that.$menu.append(action);
                        return;
                    }

                    var itemText = action.template(action.templateArgs);
                    var $item = $(itemText);
                    that.$menu.append($item);
                    if (action.action) {
                        $item.on('click', that[action.action].bind(that));
                    }
                });
                this._attachToBody();
                this._selectInitialMenuItem(args);
            }
        },

        _initActions : function () {
            var actions = [];
            var locale = this.grid.options.locale;

            if (this.column.sortable) {
                var sortOrder = this.columnMenu._getColumnSortOrder(this.column);
                actions.push({
                    template : menuHeaderTemplate,
                    templateArgs : {
                        text : locale.sort
                    }
                });
                actions.push({
                    template : menuItemTemplate,
                    templateArgs : {
                        iconText : locale.sort,
                        icon : 'ascending',
                        menuLabel : locale.sortAscending,
                        classes : sortOrder === 'asc' ? 'active' : ''
                    },
                    action : '_sortAscending'
                });
                actions.push({
                    template : menuItemTemplate,
                    templateArgs : {
                        iconText : locale.sort,
                        icon : 'descending',
                        menuLabel : locale.sortDescending,
                        classes : sortOrder === 'dsc' ? 'active' : ''
                    },
                    action : '_sortDescending'
                });
            }
            var resizable = this.column.resizable;
            var canHide = this._canHideColumn();
            if ((resizable || canHide) && actions.length) {
                actions.push(DIVIDER);
            }
            if (resizable) {
                actions.push({
                    template : menuItemTemplate,
                    templateArgs : {
                        // consider this
                        // icon : 'gateway-integration',
                        icon : '',
                        iconText : '',
                        menuLabel : locale.columnAdjustToContent,
                        classes : ''
                    },
                    action : '_autoResizeColumn'
                });
            }
            if (canHide) {
                actions.push({
                    template : menuItemTemplate,
                    templateArgs : {
                        icon : '',
                        iconText : '',
                        menuLabel : locale.columnHideColumn,
                        classes : ''
                    },
                    action : '_hideColumn'
                });
            }

            return actions;
        },

        _sortAscending : function (ev) {
            this.columnMenu.destroy();
            this.grid.sortRows(this.column.dataIndex, ev.ctrlKey, 'asc');
            ev.preventDefault();
        },
        _sortDescending : function (ev) {
            this.columnMenu.destroy();
            this.grid.sortRows(this.column.dataIndex, ev.ctrlKey, 'dsc');
            ev.preventDefault();
        },
        _autoResizeColumn : function (ev) {
            this.columnMenu.destroy();
            this.grid.view.autoResizeColumn(this.column, this.groupId);
            ev.preventDefault();
        },
        _canHideColumn : function () {
            if (!this.grid.options.columnMenu.canHideColumns) {
                return false;
            }
            return this.grid.view._canHideColumn(this.group, this.column);
        },
        _hideColumn : function (ev) {
            this.columnMenu.destroy();

            var refocusColumn;
            var columns = this.grid.view._visibleColumns(this.group);
            this.grid.view.switchColumnVisibilty(this.column);
            var columnIndex = columns.indexOf(this.column);
            if (columnIndex > 0) {
                refocusColumn = columns[columnIndex - 1];
            } else {
                refocusColumn = columns[columnIndex + 1];
            }
            this.grid.view.refocusHeader(this.group, refocusColumn);
            ev.preventDefault();
        },

        _attachToBody : function () {
            $(document.body).append(this.$root);
            this.events = menus.initDestroyEvents({
                grid : this.grid,
                $el : this.$menu,
                destroyFn : this.columnMenu.destroy,
                rootSelector : '.column-dropdown'
            });
        },

        _selectInitialMenuItem : function (args) {
            this.columnMenu.allowRefocus();
            var $firstThing = this.$menu.find('li a').first();
            if (args.fromKeyboard && $firstThing.length) {
                $firstThing.focus();
            } else {
                this.$menu.focus();
            }
        },

        destroy : function (args) {
            if (this.$root != null) {
                this.$root.remove();
                this.$root = null;
                this.$menu = null;
                this.events.destroy();
                this.events = null;
                this.grid.view._disownMenu(this.grid.view.componentIds.columnMenu);
            }
        }
    };

    return ColumnMenuView;
});
