define([
    'gridapi.jquery',
    'gridapi.lodash'
], function ($, _) {
    'use strict';

    return function (grid) {
        var $columnResizeLine = $('<div class="col-resize-line hidden"></div>');
        var MINIMUM_WIDTH_BUMPER = grid.options.sizing.minColumnWidth,
            $th = $(),
            gridOffset,
            viewportOffset,
            thCurrentWidth,
            thPositionleft,
            thOffset,
            maxDrag = {},
            dataIndex,
            group;

        // add column resize bar after table markup
        grid.$grid.append($columnResizeLine);

        var mousemoveFn = function () {
            return function (event) {
                // left click button
                if (event.which === 1) {
                    calcColumnResizeBarFn(event.clientX + $(window).scrollLeft());
                    var distanceFromViewEdge = $columnResizeLine.offset().left - viewportOffset,
                        visibleAmount = $th.closest('table').position().left + thPositionleft + thCurrentWidth,
                        width = distanceFromViewEdge + thCurrentWidth - visibleAmount,
                        totalWidth;
                    if (width < MINIMUM_WIDTH_BUMPER) {
                        width = MINIMUM_WIDTH_BUMPER;
                    }

                    grid.columnDefinitions[dataIndex].width = width;

                    if (grid.columnDefinitions[dataIndex].percent != null) {
                        totalWidth = _.find(grid.config.groups, function (group) {
                            return group.id === grid.columnDefinitions[dataIndex].groupId;
                        }).width;
                        grid.columnDefinitions[dataIndex].percent = (width / totalWidth * 100);
                    }
                    if (group === 'timephased') {
                        $columnResizeLine.addClass('hidden');
                        grid.viewports['$group-' + group].find('th').attr('width', width);
                    } else {
                        grid.viewports['$group-' + group].find('th[data-index="' + dataIndex + '"]').attr('width', width);
                    }
                    grid.resize();
                }
            };
        };

        var mouseupFn = function () {
            return function (event) {
                if (event.which === 1) {
                    // switch of mouse events
                    grid.$grid.off('mousemove.resize');
                    $(document.body).off('mouseup.gridresize');

                    // remove resizing styles
                    $th.parent().removeClass('resize');
                    $columnResizeLine.removeClass('hidden');
                    var distanceFromViewEdge = $columnResizeLine.offset().left + ($columnResizeLine.width() / 2) - viewportOffset,
                        visibleAmount = $th.closest('table').position().left + thPositionleft + thCurrentWidth,
                        width = distanceFromViewEdge + thCurrentWidth - visibleAmount;

                    if (width < MINIMUM_WIDTH_BUMPER) {
                        width = MINIMUM_WIDTH_BUMPER;
                    }

                    grid.viewports['$group-' + $th.closest('div.viewport').data('group')]
                        .find('th[data-index="' + dataIndex + '"]').attr('width', width);
                    var columnDefinition = grid.columnDefinitions[dataIndex];

                    columnDefinition.width = width;

                    if (columnDefinition.percent == null) {
                        columnDefinition.percent = null;
                    }

                    if (columnDefinition.displayType === 'timephased') {
                        grid.viewports['$group-' + group]
                            .find('th').attr('width', width);
                        grid.timephasedColumnGroup.resizeTimescale(width);
                        _.forEach(grid.columnDefinitions, function (definition) {
                            if (definition.displayType === 'timephased') {
                                definition.width = width;
                            }
                        });
                    }

                    $columnResizeLine.addClass('hidden');
                    grid.widget._trigger('view-updated', null, { action : 'column-resize', column : $th.attr('data-index'), size : width, userAction : true });
                    grid.getHeaderHeight();
                    grid.resize();
                    grid.reselectRecords();
                    grid.$grid.removeClass('resizing-column');
                    setTimeout(function () {
                        $th.removeClass('column-resizing');
                    }, 10);
                }
                return false;
            };
        };

        /**
         * This function needs to be FAST as it occurs in mousemove
         * @param clientX - x coordinate of the mouse
         * @private
         */
        var calcColumnResizeBarFn = function (clientX) {
            // Check minimum width, check if its within the viewport
            if ((thOffset + MINIMUM_WIDTH_BUMPER) < clientX && clientX >= maxDrag.left && clientX < maxDrag.right) {
                $columnResizeLine.css('left', clientX - gridOffset);
            }

            // using css here instead of show and hide as will be quicker
            $columnResizeLine.removeClass('hidden');
        };

        return {
            /**
             * Trigger column resize
             * @param $columnHeader - column being resized
             * @param clientX - start position of column resize drag
             * @private
             */
            initColumnResize : function ($columnHeader, clientX) {
                $th = $columnHeader;
                dataIndex = $th.attr('data-index');
                group = $th.closest('div.viewport').data('group');
                $th.addClass('column-resizing');
                gridOffset = grid.$grid.offset().left;
                var $viewport = $columnHeader.closest('div.viewport');
                viewportOffset = $viewport.offset().left;
                maxDrag.left = viewportOffset + 5;
                maxDrag.right = viewportOffset + $viewport.width() + 150;
                thCurrentWidth = parseInt($th.attr('width'), 10);
                thPositionleft = $columnHeader.position().left;
                thOffset = $columnHeader.offset().left;
                // add class for resizing column styles

                // switch mouse move and down event to on
                grid.$grid.addClass('resizing-column');
                grid.$grid.on('mousemove.resize', mousemoveFn());
                $(document.body).on('mouseup.gridresize', mouseupFn());
                $columnResizeLine.css('left', clientX - gridOffset);
                // move column resize bar into position
                calcColumnResizeBarFn(clientX + $(window).scrollLeft());
            }
        };
    };
});
