define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.columns/core/renderer',
    'gridapi.columns/core/editor',
    'gridapi.columns/core/headerRenderer'
], function ($, _, renderer, editor, headerRenderer) {
    'use strict';

    return {
        defaultOptions : {
            disableColumnFilter: true
        },
        editor : editor.extend({

        }),

        headerRenderer : headerRenderer.extend({

        }),

        renderer : renderer.extend({

            getValueMarkup : function (renderOptions) {
                var isChecked = this.isValueChecked(renderOptions);
                var checkedAttr = isChecked ? 'checked="checked"' : '';
                var disableForPrint = renderOptions.printable ? 'onclick="return false;" ' : '';
                var input = '<input aria-checked="' + isChecked + '" tabindex="-1" type="checkbox" ' + checkedAttr +
                    disableForPrint + (renderOptions.readOnly || !renderOptions.editable ? 'disabled' : '') + '/>';
                return input;
            },

            getAttributes : function (renderOptions) {
                var isChecked = this.isValueChecked(renderOptions);
                var tooltip = renderOptions.locale.unselected;
                if (isChecked) {
                    tooltip = renderOptions.locale.selected;
                }
                return ' title="' + tooltip + '" ';
            },

            getDescriptionLabel : function (renderOptions) {
                if (renderOptions.editable && !renderOptions.readOnly && !renderOptions.locked) {
                    return renderOptions.descriptionIds.cellState.checkboxCell;
                }
            },

            isValueChecked : function (renderOptions) {
                var data = renderOptions.data;
                return data === true || data === 'true' || data === 'Y' || data === 'yes';
            },

            getClasses : function (/* renderOptions */) {
                return 'cell-renderer-checkbox';
            }
        })
    };
});
