define([
    'gridapi.lodash',
    'gridapi.grid/columns/core/BaseColumnBuilder',
    'gridapi.columns/core/columnFilterRenderer',
    'ojs/ojcore',
    'gridapi.logger'
], function (_, BaseColumnBuilder, columnFilterRenderer, oj, logger) {
    'use strict';

    /**
     * @class ColumnBuilder
     *
     * @param {BaseColumnBuilder#args} args
     *
     */
    function ColumnBuilder(args) {
        BaseColumnBuilder.apply(this, arguments);
    }

    ColumnBuilder.prototype = _.assign(Object.create(BaseColumnBuilder.prototype), {
        getDefaultColumn: function () {
            var defaultColumn = BaseColumnBuilder.prototype.getDefaultColumn.apply(this, arguments);
            return _.assign(defaultColumn, {
                columnFilterRenderer : new columnFilterRenderer()
            });
        },
        extend : function () {
            this.deprecate();
        },
        deprecate : function () {
            if (this.definedColumn.displayType === 'spinner') {   // for backward compatibility with LUX 0.7
                logger.warn("displayType === 'spinner' is not supported post LUX 1.0 release, please use displayType: 'number' and set 'spinner' = true");
            }
        },
        //override for columns/core/BaseColumnBuilder
        _setColumnTypeDefaults : function () {
            var columnType = this.types[this.definedColumn.displayType || this.defaultColumn.displayType];
            if (columnType != null) {
                if (columnType.editor != null) {
                    this.newColumn.editor = new columnType.editor();
                }
                if (columnType.headerRenderer != null) {
                    this.newColumn.headerRenderer = new columnType.headerRenderer();
                }

                if (columnType.renderer != null) {
                    this.newColumn.renderer = new columnType.renderer();
                }

                if (columnType.formatter != null || columnType.getDefaultFormatter != undefined) {
                    this.newColumn.formatter = (columnType.getDefaultFormatter != undefined) ? columnType.getDefaultFormatter(this.definedColumn) : columnType.formatter;
                }
                if (columnType.defaultOptions != null) {
                    _.assign(this.newColumn, columnType.defaultOptions);
                }
            }
        }
    });

    return ColumnBuilder;
});
