/*
 * If you are planning to upgrade to LUX 1.0 grid, please make following changes to columns configuration options.
 *
 * Default cost and percent 'converters' have been set in columnDefFunctions.js for all of the following column types.
 * Please refer to OJET number converters and validators for more options on formatting and validation.
 * All of the following columns are supported through one columnFile. (i.e. numberColumn.js ).
 * displayType: 'number'
 * displayType: 'cost'
 * displayType: 'percent'
 * displayType: 'number', spinner: true
 * displayType: 'cost', spinner: true
 * displayType: 'percent', spinner: true
 *
 * Use of spinner column is deprecated, instead please use following
 * displayType: 'number' and set 'spinner' = true,
 * spinner column is now removed from columnDefinitionFns.js, (keeping the old file for reference spinnerColumn-obsolete-see-numberColumn.js)
 */
define(
    ['gridapi.jquery',
        'gridapi.lodash',
        'gridapi.columns/core/editor',
        'gridapi.columns/core/renderer',
        'gridapi.columns/core/headerRenderer',
        'ojs/ojcore',
        'knockout',
        'ojs/ojinputtext',
        'ojs/ojinputnumber'
    ],
    function ($, _, editor, renderer, headerRenderer) {
        'use strict';

        return {
            defaultOptions : {
                allowAllKeys : false,
                disableColumnFilter: false
            },

            getDefaultFormatter : function (definedColumn) {
                // this is column level formatter, no row based info here.
                var formatter;

                if (definedColumn.displayType === 'number') {
                    // Note: we can not use defaultFormatter with OJET components
                    formatter = oj.Validation.converterFactory('number').createConverter({});
                }
                if (definedColumn.displayType === 'cost') {
                    formatter = oj.Validation.converterFactory('number')
                        .createConverter({
                            style: 'currency',
                            currency: 'USD',
                            currencyDisplay: 'code',
                            pattern: '¤ ##,##0.00' });
                }
                if (definedColumn.displayType === 'percent') {
                    formatter = oj.Validation.converterFactory('number')
                        .createConverter({
                            style:'percent',
                            maximumFractionDigits: 2,
                            useGrouping: false });
                }

                return formatter;
            },

            renderer: renderer.extend({
                format: function (renderOptions) {
                    return (renderOptions.column.getFormatter == undefined) ? renderOptions.column.formatter.format(renderOptions.data) : renderOptions.column.getFormatter(renderOptions.record).format(renderOptions.data);
                }
            }),

            headerRenderer: headerRenderer.extend({

            }),

            editor: editor.extend({

                make: function () {
                    var that = this;

                    if (_.isUndefined(this.column.spinner) || this.column.spinner === false || this.column.spinner === 'false') { // no spinner
                        that.numORspin = 'ojInputText';
                        that['cell-editor-class'] = 'cell-editor-number';
                    } else { // it's a spinner
                        that.numORspin = 'ojInputNumber';
                        that['cell-editor-class'] = 'cell-editor-spinner';
                    }

                    var $editor = $('<div class="cell-editor cell-editor-spinner"></div>');
                    that.fromKeyboard = false;

                    this.numField = $('<input>', {
                        class: 'cell-editor-numField'
                    });

                    $editor.append(this.numField);

                    this.numField[that.numORspin]({
                        converter: (that.column.getFormatter == undefined) ? that.column.formatter : that.column.getFormatter(that.record),
                        validators: that.column.validators
                    });

                    return $editor;
                },

                getNewData: function () {
                    var valid = this.validateUserInput();
                    var data = '';
                    /* note: if data is valid, we take the plain value without $ sign as per converter on ojComponet,
                     * if data is invalid we save the whatever string user has entered without any conveter on it
                     */
                    if (valid) {
                        data = this.numField[this.numORspin]('option', 'value');
                    } else {
                        data = this.getEditorInput().val();
                    }
                    // console.log(' data: ' + data + ' valid: ' + valid);
                    return {
                        value: data,
                        valid: valid
                    };
                },

                validateUserInput: function (/* data */) {
                    return this.numField[this.numORspin]('validate');
                },

                render: function (/* startString */) {
                    var that = this;
                    /*
                     * Note: if data entered by user in previous attempt is invalid (mediator know is invalid, but actual data is intact)
                     * then you can not use 'option' method to set the data.
                     * console.log('this.data-> ' + this.data);     console.log('previoul mediator validity-> ' + !this.invalid);
                     */
                    if (!this.invalid) {
                        this.numField[that.numORspin]('option', 'value', this.data);
                    } else {
                        this.getEditorInput().val(this.data);
                    }
                },

                destroy: function () {
                    var that = this;
                    var $editor = this.getEditor();
                    if ($editor.length) {
                        setTimeout(function () {
                            that.numField[that.numORspin]('destroy');
                            $editor.remove();
                        }, 10);
                    }
                },

                onKeyDown: function (event) {
                    if (this.column.allowAllKeys || this.checkKeyCode(event)) {
                        editor.prototype.onKeyDown.call(this, event); // Default behavior for all allowed keys
                    } else {
                        event.preventDefault();
                    }
                },

                checkKeyCode: function (event) {
                    if (this.column.spinnerType === 'number') {
                        var keyCode = event.keyCode;
                        return (keyCode >= 48 && keyCode <= 57) || // 0-9
                            (keyCode >= 96 && keyCode <= 105) || // numpad 0-9
                            (keyCode === $.ui.keyCode.ENTER) ||
                            (keyCode === $.ui.keyCode.TAB) ||
                            (keyCode === $.ui.keyCode.ESCAPE) ||
                            (keyCode === $.ui.keyCode.COMMA) ||
                            (keyCode === $.ui.keyCode.LEFT) ||
                            (keyCode === $.ui.keyCode.RIGHT) ||
                            (keyCode === $.ui.keyCode.UP) ||
                            (keyCode === $.ui.keyCode.DOWN) ||
                            (keyCode === $.ui.keyCode.END) ||
                            (keyCode === $.ui.keyCode.HOME) ||
                            (keyCode === $.ui.keyCode.BACKSPACE) ||
                            (keyCode === $.ui.keyCode.DELETE) ||
                            (keyCode === $.ui.keyCode.NUMPAD_DECIMAL) ||
                            (keyCode === $.ui.keyCode.NUMPAD_SUBTRACT) ||
                            (keyCode === $.ui.keyCode.NUMPAD_ADD) ||
                            (keyCode === 189) || // - _ negative
                            (keyCode === 187) || // = + positive
                            (keyCode === 190) || // period
                            (keyCode === 65 && event.ctrlKey) || // Allow for control a
                            (keyCode === 67 && event.ctrlKey) || // Allow for control c
                            (keyCode === 86 && event.ctrlKey); // Allow for control v
                    } else {
                        return true;
                    }
                }
            })
        };
    });
