define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.columns/checkColumn',
    'gridapi.columns/core/editor',
    'gridapi.columns/core/headerRenderer'
], function ($, _, checkColumn, editor, headerRenderer) {
    'use strict';

    return {
        editor : editor.extend({
        }),
        headerRenderer : headerRenderer.extend({
            getAriaMarkup : function (column, options) {
                var descriptionIds = this.getDescribedBy(column, options);
                var tooltipText = column.tooltip ? column.tooltip : column.title;
                var html = '';
                html += ' title="' + tooltipText + '"';
                html += ' aria-label="' + tooltipText + '"';
                if (descriptionIds.length) {
                    html += ' aria-describedby="' + descriptionIds.join(' ') + '" ';
                }
                return html;
            },
            getDescribedBy : function (column, options) {
                var describedby = headerRenderer.prototype.apply(this, arguments);
                describedby.push(options.descriptionIds.cellState.unselected);
                describedby.push(options.descriptionIds.cellState.checkboxCellAll);
                return describedby;
            }
        }
        ),
        renderer : checkColumn.renderer.extend({
            getValueMarkup : function (renderOptions) {
                if (renderOptions.recordId === 'DUMMY-NEW-RECORD') {
                    return '';
                }
                var isChecked = this.isValueChecked(renderOptions);
                var input = '<input ';
                input += 'aria-checked="' + isChecked + '" ';
                input += ' tabindex="-1" type="checkbox" ';
                input += isChecked ? 'checked="checked"' : '';
                input += renderOptions.printable ? 'onclick="return false;" ' : '';
                input += '/>';
                return input;
            },

            getAttributes : function (renderOptions) {
                // Issue: LUX-230
                var titleValue = this.isValueChecked(renderOptions) ? renderOptions.locale.selected : renderOptions.locale.unselected;
                return ' title="' + titleValue + '" ';
            },

            getClasses : function () {
                return 'cell-renderer-selector-checkbox';
            },

            getDescriptionLabel : function (renderOptions) {
                // Issue 230
                if (renderOptions.editable && !renderOptions.readOnly && !renderOptions.locked) {
                    return renderOptions.descriptionIds.cellState.checkboxCell;
                }
                // return renderOptions.descriptionIds.selectRowCell;
            }

        })

    };
});
