define(
    ['gridapi.jquery',
        'gridapi.lodash',
        'gridapi.columns/core/editor',
        'gridapi.columns/core/renderer',
        'gridapi.columns/core/headerRenderer',
        'ojs/ojcore',
        'knockout',
        'ojs/ojselectcombobox'
    ],
    function ($, _, editor, renderer, headerRenderer) {
        'use strict';

        return {
            defaultOptions : {
                width : '100px',
                columnClass : 'select-column',
                selectValues : {},
                selectMultiple : false,
                listDelim : ',',
                describedBy : '',
                disableColumnFilter: false
            },

            headerRenderer: headerRenderer.extend({

            }),

            formatter: {
                format: function (data, column) {
                    var text = '';

                    if (_.isArray(data)) {
                        // data should be an array of ids
                        _.forEach(data, function (id, index) {
                            text += column.selectValues[id].name;
                            if (index < data.length - 1) {
                                text += column.listDelim;
                            }
                        });
                    } else if (column.selectValues[data] != null) {
                        text = column.selectValues[data].name;
                    }

                    return text;
                }
            },

            renderer: renderer.extend({
                format: function (renderOptions) {
                    var selectedValueObj = "";
                    if (_.isFunction(renderOptions.column.selectValues)) {
                        selectedValueObj = renderOptions.column.selectValues(renderOptions.record, renderOptions.column, renderOptions.data)[renderOptions.data];
                        return (!_.isUndefined(selectedValueObj)) ? selectedValueObj.name : "";
                    } else {
                        return renderOptions.column.formatter.format(renderOptions.data, renderOptions.column);
                    }
                }
            }),

            editor: editor.extend({
                make: function () {
                    var that = this;
                    var $editor = $('<div class="cell-editor cell-editor-select"></div>');
                    that.fromKeyboard = false;
                    this.combo = $('<select>', {
                        class: 'cell-editor-combo'
                    });

                    _.forEach(this.getPossibleSelectValues(), function (val, key) {
                        $('<option value=' + key + '>' + val.name + '</option>')
                            .appendTo(that.combo);
                    });

                    $editor.append(this.combo);

                    this.combo.ojSelect({
                        minimumResultsForSearch: 1E5,
                        // value: '_orange_',
                        rootAttributes: {
                            style: 'max-width:20em',
                            class: 'lux-oj-select' // lux-505
                        },
                        // lux-505
                        beforeExpand: function (ev /*, ui */) {
                            // setTimeout is used because there is no afterExpand event on ojSelect
                            setTimeout(function () {
                                var val = $editor.find('.lux-oj-select').attr('id');
                                $('[data-oj-containerid=\'' + val + '\']').addClass('lux-data-grid-select-column-menu');
                            }, 100);
                        }
                    });

                    setTimeout(function () {
                        $('.oj-select-choice').focus();
                        // $('.oj-select-choice').trigger('mousedown');  // LUX-openDropdown .. open dropdown when editor is opend
                        $('.oj-select-choice').on('dblclick keypress keyup keydown', function (event) {
                            // console.log('event channel-> on-> ' + $editor.attr('class') + ' type-> '+ event.type);
                            switch (event.type) {
                            case 'keypress':
                                that.onKeyPress(event);
                                break;
                            case 'keyup':
                                that.onKeyUp(event);
                                break;
                            case 'keydown':
                                that.onKeyDown(event);
                                break;
                            }
                        });
                    }, 100);

                    return $editor;
                },

                render: function (/* startString */) {
                    var that = this;
                    if (!this.invalid) {
                        this.ignoreSelectText = true; // console.log(this.data);
                        this.combo.ojSelect('option', 'value', [this.data]);
                        this.combo.one('ojoptionchange', function (ev /*, data */) {
                            if (!that.column.selectMultiple && !that.fromKeyboard) { // if it is from a keyboard event let the keyup of the editor commit it.
                                that.timeout = setTimeout(function () {
                                    that.commit();
                                }, 250);
                            }
                        });
                    } else {
                        this.getEditorInput().val(this.data);
                    }
                },

                getNewData: function () {
                    return {
                        value: this.combo.ojSelect('option', 'value')[0], // Note: although ojSelect likes an Array, we don't save data in array
                        valid: true
                    };
                },

                /*
                 * Issue 252  When a select column cell is being edited, it should have the same type of aria-labelledby value as other column types,
                 * to identify the row and column header for the cell. Investigation: The problem is that, the aria-labelledby and aria-describedby
                 * are added (by the editor.js) to whatever is retured by getEditorInput function. So implimenting this function (overriding it in getEditorInput)
                 */
                getEditorInput: function () {
                    return this.getEditor().find('.oj-select-choice');
                },

                /* eslint-disable no-unused-vars */
                /*
                 * part of issue 252
                 */
                drawFocusToInput: function (select) {
                    if (this.disabled) {
                        this.getEditor().get(0).focus();
                    } else {
                        var $editorInput = this.getEditorInput();
                        // draw input cursor into editor
                        if ($editorInput.length) {
                            select = !this.ignoreSelectText && (select || this.detectSelectText());
                            $editorInput.get(0).focus();
                        }
                    }
                },
                /* eslint-enable no-unused-vars */

                getPossibleSelectValues: function () {
                    if (_.isFunction(this.column.selectValues)) {
                        return this.column.selectValues(this.record, this.column);
                    } else {
                        return this.column.selectValues;
                    }
                },

                onKeyDown: function (event) {
                    switch (event.keyCode) {
                    case $.ui.keyCode.TAB:
                        event.preventDefault();
                            /* falls through */
                    case $.ui.keyCode.ESCAPE:
                    case $.ui.keyCode.ENTER:
                        this.fromKeyboard = true;
                        editor.prototype.onKeyDown.apply(this, arguments);
                        break;
                    }
                },

                destroy: function () {
                    var $editor = this.getEditor();
                    if ($editor.length) {
                        $editor.remove();
                    }
                    clearTimeout(this.timeout);
                }
            })
        };
    });
