define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.columns/core/renderer',
    'gridapi.columns/core/editor',
    'gridapi.columns/toggleHeaderRenderer'
], function ($, _, renderer, editor, headerRenderer) {
    'use strict';

    return {
        defaultOptions : {
            width : '75px',
            columnClass : 'check-column',
            headerToggle : false,
            expandable : false,
            groupingFns : {
                formatValue : function (data, recordGroup, column, locale) {
                    if (recordGroup.groupValue) {
                        return locale.selected;
                    } else {
                        return locale.unselected;
                    }
                },
                getGroupValue : function (data) {
                    return (data === true || data === 'true' || data === 'Y' || data === 'yes');
                }
            }
        },
        editor : editor.extend({}),

        headerRenderer : headerRenderer.extend({}),

        renderer : renderer.extend({

            getValueMarkup : function (renderOptions) {
                if (this.showCheckBox(renderOptions)) {
                    var isChecked = this.isValueChecked(renderOptions);
                    var checkedAttr = isChecked ? 'checked="checked"' : '';
                    var disableForPrint = renderOptions.printable ? 'onclick="return false;" ' : '';
                    var input = '<input class="cell-descendant" id="' + renderOptions.id + '-input"' +
                        this.getAriaLabeledBy(renderOptions) + this.getAriaDescribedBy(renderOptions) + '" aria-checked="' + isChecked + '" tabindex="-1"' +
                        ' type="checkbox" ' + checkedAttr +
                        disableForPrint + (renderOptions.readOnly || !renderOptions.editable ? 'disabled' : '') + '/>';
                    return input;
                } else {
                    return '';
                }
            },

            showCheckBox : function (/* renderOptions */) {
                return true;
            },

            getAttributes : function (renderOptions) {
                var isChecked = this.isValueChecked(renderOptions);
                var tooltip = renderOptions.locale.unselected;
                if (isChecked) {
                    tooltip = renderOptions.locale.selected;
                }
                return ' title="' + tooltip + '" ';
            },

            getDescriptionLabel : function (renderOptions) {
                if (renderOptions.editable && !renderOptions.readOnly && !renderOptions.locked) {
                    return renderOptions.descriptionIds.cellState.checkboxCell;
                }
            },

            isValueChecked : function (renderOptions) {
                var data = renderOptions.data;
                return data === true || data === 'true' || data === 'Y' || data === 'yes';
            },

            format : function (renderOptions) {
                return renderOptions.data == null ? '' : renderOptions.data;
            },

            getClasses : function (/* renderOptions */) {
                return 'cell-renderer-checkbox';
            }
        })
    };
});
