define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/gridMixins',
    'gridapi.grid/eventHandlers',
    'gridapi.grid/utils/keyCode'
], function ($, _, mixins, eventHandlers, keyCode) {
    'use strict';

    var AdvancedColumnFilterEditor = function (column, options) {
        var that = this;
        _.forEach(options.advancedFilterFns, function (fnc, fncName) {
            that[fncName] = fnc || that[fncName];
        });
        this.column = column;
        this.init(options);
        return this;
    };

    AdvancedColumnFilterEditor.extend = mixins.extendFn;

    _.assign(AdvancedColumnFilterEditor.prototype, {
        init : function (options) {
            this.advancedFilters = options.advancedFilters;
            this.$element = this.makeElement();
            this.attachListeners();
            this.setFilterData(options.initialData);
            this.debouncedUpdate = _.debounce(this.updateFilter.bind(this), 250);
        },
        makeElement : function () {
            var $input = $();
            if (_.get(this.column, 'advancedFilters.enabled', false)) {
                $input = $('<input type="text"/>'); //todo template
            }
            return $input;
        },
        getElement : function () {
            return this.$element;
        },
        updateFilter : function () {
            this.advancedFilters.advancedFilterUpdated(this);
        },
        getFilterData : function () {
            return this.$element.val();
        },
        setFilterData : function (value) {
            this.$element.val(value);
        },
        attachListeners: function () {
            var that = this;
            this.$element.on('focus', function () {
                that.editorFocused();
                that.advancedFilters.view.selectColumnFilterCell(that.column.dataIndex);
            });
            this.$element.on('blur', function (event) {
                event.preventDefault();
                event.stopPropagation();
            });
            this.$element.on('click', function () {
                that.focusEditor();
            });
            // stop double click events
            this.$element.on('dblclick', function (event) {
                event.stopPropagation();
                event.preventDefault();
            });
            this.$element.on('keypress', function (event) {
                that.onKeyPress(event);
            });
            this.$element.on('keydown', function (event) {
                that.onKeyDown(event);
            });
            this.$element.on('keyup', function (event) {
                that.onKeyUp(event);
            });
        },
        destroy : function () {
            this.$element.off();
        },
        focusEditor : function () {
            this.$element.focus();
        },
        onKeyPress : function (event) {
            switch (event.keyCode) {
            case keyCode.ESCAPE :
            case keyCode.ENTER :
                event.stopPropagation();
                event.preventDefault();
                break;
            }
        },
        onKeyDown : function (event) {
            switch (event.keyCode) {
            case keyCode.TAB :
                event.stopPropagation();
                event.preventDefault();
                this.onTab(event);
                break;
            case keyCode.LEFT :
            case keyCode.RIGHT :
                event.stopPropagation();
                break;
            default :
                break;
            }
            this.debouncedUpdate();
        },
        onTab : function (event) {
            event.preventDefault();
            this.advancedFilters.focusNextFilter(this.column.dataIndex, !event.shiftKey);
        },
        editorFocused : _.noop,
        onKeyUp : _.noop
    });

    return AdvancedColumnFilterEditor;
});
