define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/utils/keyCode',
    'gridapi.moment',
    'gridapi.columns/core/editor',
    'gridapi.columns/core/renderer',
    'gridapi.columns/core/headerRenderer',
    'gridapi.oui'
], function ($, _, keyCode, moment, editor, renderer, headerRenderer, oui) {
    'use strict';

    return {

        defaultOptions : {
            width : '100px',
            columnClass : 'date-column',
            dateType : 'date'
        },
        headerRenderer : headerRenderer.extend({}),

        renderer : renderer.extend({
            format : function (renderOptions) {
                return renderOptions.column.formatter.format(renderOptions.data);
            },
            getExcelCell : function (renderOptions) {
                return renderer.prototype.getExcelCell.apply(this, [{
                    formattedValue : this.format(renderOptions)
                }]);
            }
        }),
        editor : editor.extend({
            show : function (startString) {
                this.render(startString);

                var $editor = this.getEditor();
                $editor.show();
            },

            onKeyDown : function (event) {
                var that = this;
                switch (event.keyCode) {
                case keyCode.TAB :
                    that.onTab(event);
                    break;
                case keyCode.ENTER:
                        // Happens before change event and ends up committing same value
                    this.getEditor().find('input').trigger('change');
                    /* falls through */
                case keyCode.ESCAPE:
                    editor.prototype.onKeyDown.apply(this, arguments);
                    break;
                }
            },

            make : function () {
                var that = this;
                var $editor = $('<div class="cell-editor cell-editor-date"></div>');
                this.isValid = true;
                this.dateField = new oui.DateField({
                    openOnFocus : true,
                    type : this.column.dateType,
                    unselectable : this.column.unselectable,
                    formatter : this.column.formatter,
                    fluid : true,
                    number : {
                        formatter : this.column.timeFormatter
                    },
                    min : this.column.minValue,
                    max : this.column.maxValue,
                    appendTo : $editor
                }, this.data);

                if (this.column.dateType === 'date') {
                    this.dateField.on('change', function () {
                        if (!that.commitFinished) {
                            that.commit({});
                        }
                    });
                }

                $editor.prepend(this.dateField.element());

                return $editor;
            },

            getNewData : function () {
                var data = this.dateField.date();
                var valid = this.validateUserInput(data);
                return {
                    value : data,
                    valid : valid
                };
            },

            getEditorInput : function () {
                return this.getEditor().find('> .orcl-widget input');
            },

            validateUserInput : function (data) {
                return data == null || moment.isMoment(data);
            },

            render : function (startString) {
                this.dateField.init();
                if (startString != null) {
                    // set typed character into input
                    this.getEditorInput().val(startString);
                    this.ignoreSelectText = true;
                } else {
                    // if the data is not valid then just display it
                    this.getEditorInput().val(this.formatValueForInput());
                }
            }
        })
    };
});
