define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.columns/core/editor',
    'gridapi.columns/core/renderer',
    'gridapi.columns/core/headerRenderer'
], function ($, _, editor, renderer, headerRenderer) {
    'use strict';

    return {
        defaultOptions : {
            enableEditor : false,
            width : '150px',
            columnClass : 'link-column'
        },
        renderer : renderer.extend({
            getValueMarkup : function (options) {
                if (!this.isLink(options)) {
                    return renderer.prototype.getValueMarkup.call(this, options);
                } else {
                    var markup = '<a  id="' + options.id + '-link" tabindex="-1" data-trigger="link-clicked" class="cell-descendant cell-link" ';
                    markup += this.getAriaLabeledBy(options);
                    markup += this.getAriaDescribedBy(options);
                    markup += this.getLinkAttributes(options) + ' >';
                    markup += options.formattedValue + '</a>';
                    return markup;
                }
            },
            getLinkAttributes : function (/* options */) {
                return '';
            },
            getDescriptionLabel : function (renderOptions) {
                var labels = this.isLink(renderOptions) ? renderOptions.descriptionIds.cellState.linkCell : '';
                if (renderOptions.editable && !renderOptions.readOnly && !renderOptions.locked && renderOptions.column.enableEditor) {
                    labels += ' ' + renderOptions.descriptionIds.cellState.editableCell;
                }
                return labels;
            },
            isLink: function (/* options */) {
                return true;
            },
            getExcelCell : function (renderOptions) {
                var link, url;

                if (this.isLink(renderOptions)) {
                    link = $(this.getValueMarkup(renderOptions));
                    url = link.attr('href') || link.find('a').attr('href');
                }

                if (url && url.substring(0, 4) === 'http') {
                    url = url.replace(/&/g, _.escape('&'));
                    var linkText = _.escape(_.unescape(this.getExcelCellData(renderOptions)));
                    return {
                        link: true,
                        url: url,
                        value: linkText
                    };
                } else {
                    return renderer.prototype.getExcelCell.apply(this, arguments);
                }
            }
        }),
        headerRenderer : headerRenderer.extend({

        }),
        editor : editor.extend({
            getNewData : function () {
                var string = this.getEditorInput().val();
                if (string != null) {
                    string = string.trim();
                }
                return {
                    value : string,
                    valid : this.validateUserInput(string)
                };
            }
        })
    };
});
