define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.oui',
    'gridapi.grid/utils/keyCode',
    'gridapi.columns/core/editor',
    'gridapi.columns/core/renderer',
    'gridapi.columns/core/headerRenderer',
    'markup!gridapi.grid/templates/grid.htm:.htm-rich-text-modal'
], function ($, _, oui, keyCode, editor, renderer, headerRenderer, modalTemplate) {
    'use strict';

    function findFocusable($parent) {
        return $parent
        .find('a[href], button:not([disabled]), input:not([disabled]), textarea:not([disabled]), select:not([disabled]), [tabindex]', 'iframe')
        .filter(':visible:not([tabindex=-1]), input[type="file"]').add($parent.find('iframe'));
    }

    return {
        defaultOptions : {
            hasDisabledMode : true,
            width : '150px',
            columnClass : 'rich-text-column'
        },
        renderer : renderer.extend({}),

        formatter : {
            format : function (value) {
                // strip out markup just for the text input area.  not for the rich text editor
                if (_.isUndefined(value)) {
                    return '';
                } else if (value.length) {
                    return value.replace(/<(?:.|\n)*?>/gm, '');
                } else {
                    return '';
                }
            },
            validate : function (/* value */) {
                return true;
            }
        },

        headerRenderer : headerRenderer.extend({}),

        editor : editor.extend({

            make : function () {
                var that = this;
                // create a text area
                this.$modal = $(modalTemplate({ locale : this.grid.options.locale, title : this.column.title }));
                this.editor = new oui.Editor({
                    fill : true,
                    fluid : true,
                    appendAll : this.$modal.find('.modal-body')
                }, this.data);

                this.$modal.find('.modal-body').append(this.editor.element());

                this.$modal.find('iframe').one('load', function (event) {
                    $(event.target.contentDocument).on('keydown', function (event) {
                        if (event.target.ownerDocument !== document &&
                            event.keyCode === keyCode.TAB) {
                            var focusable = findFocusable(that.$modal);
                            var index = focusable.index(that.$modal.find('iframe'));

                            if (event.shiftKey) {
                                _.defer(function () {
                                    if (document.activeElement !== focusable[index - 1]) {
                                        $(focusable[index - 1]).focus();
                                    }
                                });
                            } else {
                                _.defer(function () {
                                    if (document.activeElement !== focusable[index + 1]) {
                                        $(focusable[index + 1]).focus();
                                    }
                                });
                            }
                        }
                    });
                });

                return this.$modal;
            },

            getNewData : function () {
                return {
                    value : this.editor.document(),
                    valid : true
                };
            },

            drawFocusToInput : function () {

            },

            initListeners : function () {
                var that = this;
                this.$modal.find('.rich-text-modal-cancel').on('click', function () {
                    that.$modal.off('hidden');
                    that.close();
                });
                this.$modal.find('.rich-text-modal-accept').on('click', function () {
                    that.$modal.off('hidden');
                    that.commit();
                });
                this.$modal.find('.rich-text-modal-close').on('click', function () {
                    that.$modal.off('hidden');
                    that.close();
                });

                this.$modal.on('hidden', function (event) {
                    if ($(event.target).is('.grid-rich-text-modal')) {
                        that.close();
                    }
                });
            },

            destroy : function () {
                this.$modal.modal('hide');
                var $editor = this.getEditor();
                if ($editor.length) {
                    $editor.remove();
                }
            },

            validateUserInput : function (/* input */) {
                return true;
            },

            show : function (/* startString */) {
                var readOnly = this.column.readOnly;
                this.$modal.modal('show');
                this.editor.readonly(readOnly);
                this.$modal.find('.rich-text-modal-close-btn').toggleClass('hidden', !readOnly);
                this.$modal.find('.rich-text-modal-accept, .rich-text-modal-cancel').toggleClass('hidden', readOnly);
            },

            render : function (/* startString */) {
                this.editor.data(this.data);
            },
            setPosition : function ($editor) {
            },

            setSize : function ($editor) {
            },


            getEditorInput : function () {
                return this.getEditor();
            },

            onKeyDown : function (event) {
                if (!(event.keyCode === keyCode.ENTER && !event.shiftKey && !this.column.readOnly)) {
                    event.stopPropagation();
                }
            }
        })
    };
});
