define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/utils/keyCode',
    'gridapi.columns/core/editor',
    'gridapi.columns/core/renderer',
    'gridapi.columns/core/headerRenderer',
    'gridapi.oui'
], function ($, _, keyCode, editor, renderer, headerRenderer, oui) {
    'use strict';

    return {
        defaultOptions : {
            width : '100px',
            columnClass : 'select-column',
            allowClearCells : false,
            selectValues : {},
            selectMultiple : false,
            listDelim : ','
        },
        headerRenderer : headerRenderer.extend({}),

        formatter : {
            format : function (data, column) {
                var text = '';

                if (_.isArray(data)) {
                    // data should be an array of ids
                    _.forEach(data, function (id, index) {
                        text += column.selectValues[id].name;
                        if (index < data.length - 1) {
                            text += column.listDelim;
                        }
                    });
                } else if (column.selectValues[data] != null) {
                    text = column.selectValues[data].name;
                }

                return text;
            }
        },

        renderer : renderer.extend({
            format : function (renderOptions) {
                return renderOptions.column.formatter.format(renderOptions.data, renderOptions.column);
            }
        }),

        editor : editor.extend({
            getNewData : function () {
                var selected = this.combo.selected();
                return {
                    value : _.get(selected, 'id', null),
                    valid : true
                };
            },

            getPossibleSelectValues : function () {
                if (_.isFunction(this.column.selectValues)) {
                    return this.column.selectValues(this.record, this.column);
                } else {
                    return this.column.selectValues;
                }
            },

            render : function (startString) {
                this.combo.init();
                if (startString != null) {
                    // set typed character into input
                    this.getEditorInput().val(startString);
                    this.getEditorInput().trigger('input');// This allows the typeahead drop down stuff to work properly
                    this.ignoreSelectText = true;
                } else {
                    var data;

                    if (_.isString(this.data) || _.isNumber(this.data)) {
                        data = { id : this.data };
                    } else {
                        data = this.data;
                    }

                    this.combo.value(data);
                }
            },

            onKeyDown : function (event) {
                switch (event.keyCode) {
                case keyCode.TAB :
                    event.preventDefault();
                    /* falls through */
                case keyCode.ESCAPE:
                case keyCode.ENTER:
                    this.fromKeyboard = true;
                    editor.prototype.onKeyDown.apply(this, arguments);
                    break;
                }
            },

            drawFocusToInput : function (select) {
                if (this.disabled) {
                    this.getEditor().get(0).focus();
                } else {
                    var $editorInput = this.getEditorInput();
                    // draw input cursor into editor
                    if ($editorInput.length) {
                        select = !this.ignoreSelectText && (select || this.detectSelectText());
                        $editorInput.on('focus', function () {
                            this.selectionStart = this.selectionEnd = this.value.length; // fix placement of cursor for  firefox
                            if (select) {
                                this.select();
                            }
                        });
                        // $editorInput.get(0).blur();
                        $editorInput.get(0).focus();
                    } else {
                        this.getEditor().find('button').get(0).focus();
                    }
                }
            },

            make : function () {
                var that = this;
                var $editor = $('<div class="cell-editor cell-editor-select"></div>');
                that.fromKeyboard = false;
                this.combo = new oui.ComboBox(_.assign({}, this.column.ouiOptions, {
                    fluid : true,
                    appendTo : $editor,
                    openOnFocus : true,
                    keyboardInput : this.column.keyboardInput || false
                })).on('selection-change identical', function (event) {
                    if (!that.column.selectMultiple && !that.fromKeyboard) { // if it is from a keyboard event let the keyup of the editor commit it.
                        that.timeout = setTimeout(function () {
                            that.commit({});
                        }, 250);
                    }
                });

                this.combo.values(this.getPossibleSelectValues());

                $editor.prepend(this.combo.element());

                this.combo.element().find('input, button').first().on('keydown', function () {
                    that.fromKeyboard = true;
                });

                return $editor;
            },
            destroy : function () {
                var $editor = this.getEditor();
                if ($editor.length) {
                    $editor.remove();
                }
                clearTimeout(this.timeout);
            }
        })
    };
});
