define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/utils/keyCode',
    'gridapi.columns/core/editor',
    'gridapi.columns/core/renderer',
    'gridapi.columns/core/headerRenderer'
], function ($, _, keyCode, editor, renderer, headerRenderer) {
    'use strict';
    return {
        defaultOptions : {
            hasDisabledMode : true,
            minWidth : 200,
            width : '150px',
            columnClass : 'text-area-column'
        },
        renderer : renderer.extend({}),

        headerRenderer : headerRenderer.extend({}),

        editor : editor.extend({

            make : function () {
                var that = this;

                // create a text area
                var editorMarkup = '<div tabindex="0" class="cell-editor cell-editor-textarea">' +
                    '<textarea rows="3" cols="100" ' + (this.disabled ? ' disabled ' : '') +
                    '></textarea></div>';
                var $editor = $(editorMarkup);

                // keep scroll event to ourselvemakes - not doing will cause no scroll in the text area
                var $editorInput = that.getEditorInput($editor);
                $editorInput.on('mousewheel', function (event) {
                    event.stopPropagation();
                });

                return $editor;
            },

            getNewData : function () {
                var string = this.getEditorInput().val();
                if (this.column.escaping.autoEscapeEditor) {
                    string = this.column.escaping.unescape(string);
                }
                if (string != null) {
                    string = string.trim();
                }
                return {
                    value : string,
                    valid : this.validateUserInput(string)
                };
            },

            setSize : function ($editor) {
                var cellWidth = parseInt(this.$cell.css('width'), 10);
                // ensure editor is not larger than the grid
                var suggestedWidth = cellWidth;

                if (suggestedWidth < this.column.minWidth) {
                    suggestedWidth = this.column.minWidth;
                }
                var gridBoundary = this.grid.$grid.offset().left + this.grid.$grid.width();
                var positionLeft = parseInt($editor.css('left'), 10);

                if (positionLeft + suggestedWidth > gridBoundary) {
                    $editor.css('left', gridBoundary - suggestedWidth);
                }
                $editor.find('textarea').css('width', suggestedWidth);
            },

            getEditorInput : function () {
                return this.getEditor().find('textarea');
            },

            onKeyPress : function () {
            },

            onKeyDown : function (event) {
                if (!(event.keyCode === keyCode.ENTER && !event.shiftKey && !this.column.readOnly)) {
                    editor.prototype.onKeyDown.apply(this, arguments);
                }
            }
        })
    };
});
