define([
    'gridapi.jquery',
    'gridapi.lodash',
    'moment',
    'gridapi.oui',
    'markup!gridapi.grid/config/templates/print-config.htm:.htm-layout',
    'markup!gridapi.grid/config/templates/print-config.htm:.htm-gantt-settings'
], function ($, _, moment, oui, layout, ganttSettings) {
    'use strict';

    var configuratorView = function (widget) {
        this.widget = widget;
        this.options = widget.options;
        this.config = widget.options.config;
    };

    _.assign(configuratorView.prototype, {
        init : function () {
            this.$element = this.widget.element;

            this._buildLayout();
            this._initForm();

            this.$element.append(this.$layout);
        },

        _buildLayout : function () {
            this.$layout = $(layout({ locale : this.options.locale }));
            if (this.options.gantt) {
                this.$layout.find('.print-settings-container').append(ganttSettings({
                    locale: this.options.locale
                }));
            }
        },

        _initForm : function () {
            var numberSpinnerFormatter = {
                format: function (num) {
                    if (num == null) {
                        return '';
                    }
                    return num.toString();
                },
                unformat: function (str) {
                    return parseInt(str, 10);
                },
                validate: function (num) {
                    return !_.isNaN(parseInt(num, 10));
                }
            };

            var percentSpinnerFormatter = _.assign({}, numberSpinnerFormatter, {
                format: function (num) {
                    return numberSpinnerFormatter.format(num) + '%';
                }
            });

            var radioGroupFns = {
                getValue: function (elem) {
                    return elem.filter(':checked').val();
                },
                reset: function (elem) {
                    return elem.prop('checked', false);
                },
                setValue: function (elem, value) {
                    elem.filter('[value=' + value + ']').prop('checked', true);
                }
            };

            var gridFieldDefinitions = {
                orientation : radioGroupFns,
                pages : {
                    options : {
                        spinner: {
                            controls: true
                        },
                        min: 1,
                        formatter: numberSpinnerFormatter
                    }
                }
            };

            var ganttFieldDefinitions = {
                repeatColumn : radioGroupFns,
                scale : {
                    options : {
                        spinner: {
                            controls: true
                        },
                        min: 10,
                        max: 100,
                        formatter: percentSpinnerFormatter
                    }
                },
                columns : {
                    options : {
                        spinner: {
                            controls: true
                        },
                        min: 1,
                        formatter: numberSpinnerFormatter
                    }
                }
            };

            var fieldDefinitions = this.options.gantt ?
                _.assign({}, gridFieldDefinitions, ganttFieldDefinitions) :
                gridFieldDefinitions;

            this.form = new oui.Form({
                fieldDefinitions : fieldDefinitions
            }, this.$layout);

            _.defaults(this.config, {
                orientation: 'portrait',
                pages: 1,
                scale: 0.75,
                columns: 10,
                repeat: 'first',
                startDate: moment(),
                endDate: moment()
            });

            this.form.init({
                orientation: {
                    portrait: this.config.orientation === 'portrait',
                    landscape: this.config.orientation === 'landscape'
                },
                pages: this.config.pages,
                scale: this.config.scale * 100,
                columns: this.config.columns,
                repeat: {
                    first: this.config.repeat === 'first',
                    all: this.config.repeat === 'all',
                    none: this.config.repeat === 'none'
                },
                startDate: this.config.startDate,
                endDate: this.config.endDate
            });
        },

        getFormData : function () {
            var data = this.form.data();

            data.scale /= 100;
            data.orientation = _.findKey(data.orientation);
            data.repeat = _.findKey(data.repeat);

            return data;
        }
    });

    return configuratorView;
});
