define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.logger',
    'gridapi.jqueryui',
    'gridapi.oui',
    'gridapi.grid/enums/groupingStyle',
    'gridapi.grid/enums/groupingTheme',
    'gridapi.grid/config/gridPreviewDataGenerator',
    'markup!gridapi.grid/config/templates/theme-config.htm:.htm-layout',
    'markup!gridapi.grid/config/templates/theme-config.htm:.htm-theme-color-value'
], function ($, _, logger, jqui, oui, GROUPING_STYLES, GROUPING_THEME, dataGenerator, layoutMarkup, themeValueMarkup) {
    'use strict';
    var configuratorView = function (widget) {
        this.options = widget.options;
        this.widget = widget;
        this.config = this.widget.config;
    };

    _.assign(configuratorView.prototype, {
        init : function () {
            this.$element = this.widget.element;
            this.$modal = this.$element.parents('.modal');
            this.$layout = $(layoutMarkup({ locale : this.options.locale }));
            this.$gridContainer = this.$layout.find('.grid-preview-fieldset');
            this.$groupingStyleInput = this.$layout.find('#grouping-style');
            this.$groupingColorInput = this.$layout.find('#grouping-colors');
            this.$element.append(this.$layout);
            this.config = this.widget.config;
            this.setupGrid();
            this.initListeners();
        },
        resize : function () {
            this.$gridContainer.grid('resize');
        },
        initListeners : function () {
            var that = this;
            this.barProgressStyleCombo = new oui.ComboBox({
                viewFns : {
                    display : function (val) {
                        return that.options.locale.groupingStyle[val.label];
                    }
                },
                keyboardInput : false,
                fluid : false
            }, this.$groupingStyleInput, [
                GROUPING_STYLES.MODERN_NOTCHES, GROUPING_STYLES.MODERN_LINE, GROUPING_STYLES.MODERN_BANDS, GROUPING_STYLES.CLASSIC_BANDS, GROUPING_STYLES.NONE]
            );
            this.barProgressStyleCombo.on('valueSelected', function (selected) {
                that.config.theme.style = selected.key;
                that.$gridContainer.grid('option', 'theme', {
                    style : selected.key
                });
            });
            this.barProgressStyleCombo.value({ key : this.config.theme.style });

            this.groupingColorCombo = new oui.ComboBox({
                viewFns : {
                    display : function (value) {
                        return themeValueMarkup({ style : value.style,
                            name : that.options.locale.groupingThemes[value.label] });
                    },
                    text : function (value) {
                        return that.options.locale.groupingThemes[value.label];
                    }
                },
                forceDisplay : true,
                keyboardInput : false,
                fluid : false
            }, this.$groupingColorInput, [
                GROUPING_THEME.PASTEL, GROUPING_THEME.HIGHLIGHTER, GROUPING_THEME.DARK, GROUPING_THEME.GREY, GROUPING_THEME.CLASSIC_PASTEL, GROUPING_THEME.CLASSIC_VIBRANT]);

            this.groupingColorCombo.value({ key : this.config.theme.colors });
            this.groupingColorCombo.on('valueSelected', function (selected) {
                that.config.theme.colors = selected.key;
                that.$gridContainer.grid('option', 'theme', {
                    colors : selected.key
                });
            });
        },

        setupGrid : function () {
            var records = dataGenerator.get({ locale : this.options.locale });
            this.$gridContainer.grid({
                enableQuickAddRow : true,
                enableCellChangeTrigger : true,
                mediatorFns : {
                    getRecordAttributes : function (record, defaults) {
                        var attributes = {};
                        _.defaults(attributes, defaults);
                        attributes.invalid = this.invalidFields[record[this.recordKeys.id]];

                        if (this.modified[record[this.recordKeys.id]] != null) {
                            attributes.modified = this.modified[record[this.recordKeys.id]];
                        }
                        if (this.readOnly[record[this.recordKeys.id]] != null) {
                            attributes.readOnly = this.readOnly[record[this.recordKeys.id]];
                        } else {
                            var readOnly = this.checkRecordPermissions(record);
                            attributes.readOnly = readOnly;
                            this.readOnly[record[this.recordKeys.id]] = readOnly;
                        }
                        if (record.children != null && record.children.length) {
                            attributes.isNode = true;
                            attributes.hasChildren = true;
                            attributes.expanded = true;
                            if (this.options.fakelazy) {
                                attributes.childrenLoaded = this.fakeAjaxLoad[record[this.recordKeys.id]];
                            } else {
                                attributes.childrenLoaded = true;
                            }
                            attributes.groupable = false;
                        }

                        attributes.type = record[this.recordKeys.type];

                        if (record[this.recordKeys.id] < 0) {
                            attributes.newRow = true;
                        }

                        if (this.dataIcons[record[this.recordKeys.id]] != null) {
                            attributes.dataIcons = this.dataIcons[record[this.recordKeys.id]];
                        } else {
                            var recordIcons = this.lookupRecordIcons(record);
                            attributes.dataIcons = recordIcons;
                            this.dataIcons[record[this.recordKeys.id]] = recordIcons;
                        }
                        return attributes;
                    }

                },
                tree : true,
                treeMaxLevel : 10,
                sortRootNodes : true,
                classes : {
                    grid : ['grid-grouping-modern-bands'],
                    cellSelected : 'cell-selected-none ',
                    cellSelectedTop : 'cell-selected-top-none ',
                    cellSelectedBottom : 'cell-selected-bottom-none ',
                    cellFocused : 'cell-focused-none ',
                    rowSelected : 'row-selected-none ',
                    rowSelectedTop : 'row-selected-top-none ',
                    rowSelectedBottom : 'row-selected-bottom-none ',
                    rowFocused : 'row-focused-none ',
                    rowHovered : 'row-hovered-none '
                },
                editable : false,
                layout : {
                    width : '100%',
                    height : '250px',
                    removeHeaderRow : true,
                    rowsets : [
                        {
                            id : 'a',
                            type : 'flex'
                        }
                    ],
                    columnGroups : [
                        {
                            type : 'flex',
                            id : 'y',
                            columns : ['title', 'description']
                        }
                    ]
                },
                columns : {
                    title : {
                        title : 'Name',
                        dataIndex : 'title',
                        expandable : true,
                        groupTitle : true,
                        percent : 70
                    },
                    description : {
                        title : 'Description',
                        dataIndex : 'description',
                        width : '200px',
                        percent : 30
                    }

                }
            });
            this.$gridContainer.grid('setStore', records);
            this.$gridContainer.grid('option', 'theme', this.config.theme);
        }
    });

    return configuratorView;
});
