define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.logger',
    'gridapi.grid/constants',
    'gridapi.grid/gridWidgetOptions',
    'gridapi.grid/view/rowRenderer',
    'gridapi.grid/selectionModel',
    'gridapi.grid/columns/core/columnResizeFns',
    'gridapi.grid/columns/core/columnMover',
    'gridapi.grid/sizeFns',
    'gridapi.grid/rows/rowMoveFns',
    'gridapi.columns/core/generateColumn',
    'gridapi.grid/eventHandlers',
    'gridapi.grid/contextMenu',
    'gridapi.columnMenu/columnMenu',
    'gridapi.columnGroupMenu/ColumnGroupMenu',
    'gridapi.columns/textColumn',
    'gridapi.columns/toggleHeaderRenderer',
    'gridapi.grid/enums/groupingStyle',
    'gridapi.grid/enums/groupingTheme',
    'gridapi.grid/timephasedColumnGroup',
    'gridapi.grid/rowExpansionManager',
    'gridapi.grid/advancedFilters/AdvancedFilters',
    'gridapi.grid/exporter',
    'gridapi.grid/data/attrFns',
    'gridapi.grid/touchInteractions/touchInteractions',
    'gridapi.grid/utils/browser',
    'markup!gridapi.grid/templates/grid.htm:.htm-grid-layout',
    'markup!gridapi.grid/templates/grid.htm:.htm-viewport',
    'markup!gridapi.grid/templates/grid.htm:.htm-row',
    'markup!gridapi.grid/templates/grid.htm:.htm-header-shim',
    'markup!gridapi.grid/templates/grid.htm:.htm-pseudoscroller',
    'markup!gridapi.grid/templates/grid.htm:.htm-rowset',
    'markup!gridapi.grid/templates/grid.htm:.htm-scroll-shims',
    'markup!gridapi.grid/templates/grid.htm:.htm-group-resizer',
    'markup!gridapi.grid/templates/grid.htm:.htm-rowset-resizer',
    'markup!gridapi.grid/templates/grid.htm:.htm-cell-error-popup',
    'markup!gridapi.grid/templates/grid.htm:.htm-cell-error-template',
    'markup!gridapi.grid/templates/grid.htm:.htm-loading-spinner',
    'markup!gridapi.grid/templates/grid.htm:.htm-rowset-message',
    'markup!gridapi.grid/templates/grid.htm:.htm-summary-description',
    'markup!gridapi.grid/templates/grid.htm:.htm-cell-selection',
    'markup!gridapi.grid/templates/grid.htm:.htm-status-bar',
    'markup!gridapi.grid/templates/grid.htm:.htm-status-bar-toggle'
], function ($, _, logger, constants, defaultOptions,
             RowRenderer, SelectionModel, ColumnResizer, ColumnMover,
             SizingUtil, rowMoveFns, generateColumn, eventHandlers,
             ContextMenu, ColumnMenu, ColumnGroupMenu, textColumn, toggleHeaderRenderer,
             GROUPING_STYLE, GROUPING_THEME, TimephasedColumnGroup, RowExpansionManager, AdvancedFilters,
             exporter, attrFns, TouchInteractions, browserUtils, gridMarkup, viewportTemplate, trTemplate,
             thShimTemplate, pseudoScrollerTemplate, rowsetTemplate, scrollShims,
             groupResizer, rowsetResizer, errorBox, errorTemplate,
             loadingTemplate, emptyRowsetTemplate, descriptionTemplate, cellSelectionTemplate,
             statusBarTemplate, statusBarToggleTemplate) {
        // stop exiting keyup functions
    'use strict';

    var FROM_BLUR_ARG = { fromBlur : true };
    var $gridMarkup = $(gridMarkup()),
        $scrollShims = $(scrollShims()),
        $errorBox = $(errorBox());
    var calculateScrollbarWidth = SizingUtil.calculateScrollbarWidth;
    var selectorGroup = {
        id : constants.ROW_SELECTOR_GROUP,
        type : constants.ROWSET_TYPE.FIXED,
        columns : [constants.ROW_SELECTOR],
        internalDisplayOnly : true
    };

    var menuButtonGroup = {
        id : constants.MENU_BUTTON_GROUP,
        type : constants.ROWSET_TYPE.FIXED,
        columns : [constants.MENU_BUTTON_COLUMN],
        ignoreBorders : true,
        internalDisplayOnly : true
    };

    var digitCheck = /^-?\d/;

    var selectorColumn = {
        title : null,
        dataIndex : constants.ROW_SELECTOR,
        displayType : 'rowSelector',
        align : 'center',
        width : '30px',
        draggable : false,
        headerAlign : 'center',
        enableEditor : false,
        internalDisplayOnly : true,
        filldown : false
    };

    var emptyColumn = {
        dataIndex : constants.EMPTY_COLUMN,
        visible : true,
        width : '0px',
        title : '',
        enableEditor : false,
        resizable : false,
        displayColumn : true,
        internalDisplayOnly : true,
        filldown : false,
        headerRendererFns : {
            getAriaRole : function () {
                return 'presentation';
            }
        },
        advancedFilters : {
            enabled : false
        }
    };

    var menuButtonColumn = {
        dataIndex : constants.MENU_BUTTON_COLUMN,
        visible : true,
        width : '22px',
        title : '',
        readOnly : true,
        resizable : false,
        displayColumn : true,
        internalDisplayOnly : true,
        filldown : false,
        rendererFns : {

            getValueMarkup : function (renderOptions) {
                var markup = '<div class="row-menu-button" title="' + renderOptions.locale.cellState.menuLabel + '"> <i class="pgbu-icon pgbu-icon-cog" title=""/></div>';
                if (renderOptions.showQuickAdd) {
                    markup = '<div class="quick-add-button" title="' + renderOptions.quickAddHint + '"> <i class="pgbu-icon pgbu-icon-add-button" title=""/></div>';
                }
                return markup;
            },
            getClasses : function (renderOptions) {
                if (renderOptions.attributes.disableRowMenu) {
                    return ' menu-button-cell row-menu-disabled';
                } else {
                    return ' menu-button-cell';
                }
            }
        },
        headerRendererFns : {
            getAriaRole : function () {
                return 'presentation';
            }
        }
    };

    var QUICK_ADD_ROWSET = {
        type : constants.ROWSET_TYPE.FIXED,
        id : constants.QUICK_ADD_ROWSET_ID,
        excludeRecordCount : true
    };


    var gridView = function (widget) {
        var that = this;
        this.widget = widget;
        this.options = widget.options;
        this._attrs = widget._attrs;
        this.setRowHeight();

        if (!('WebkitAppearance' in document.documentElement.style)) { // Sad sad panda, we have css that changes the width of the scrollbars for chrome and safari.
            var scrollbarSize = calculateScrollbarWidth();
            this.options.scrollbar.extraHeight = scrollbarSize + 1;// Add one for the borderbox and bottom border
            this.options.scrollbar.extraWidth = scrollbarSize;
        }


        this.$el = this.options.el;

        // add row selector column
        if (this.options.selectionMode === 'check') {
            var exists = _.find(this.options.layout.columnGroups, function (group) {
                return group.id === constants.ROW_SELECTOR_GROUP;
            });

            var indexOfRowSelector = 0;

            if (this.options.layout.columnGroups[0].id === constants.MENU_BUTTON_COLUMN) {
                indexOfRowSelector = 1;
            }

            if (!exists) {
                this.options.layout.columnGroups.splice(0, indexOfRowSelector, _.clone(selectorGroup));
            }

            this.options.columns[constants.ROW_SELECTOR] = _.clone(selectorColumn);
            this.options.columns[constants.ROW_SELECTOR].tooltip = this.options.locale.selectDeselectAll;
            this.options.columns[constants.ROW_SELECTOR].headerRenderer = toggleHeaderRenderer.extend({
                attachListeners : function ($columnTh) {
                    $columnTh.find('input[type="checkbox"]').on('click', function (event) {
                        var $checkbox = $(this);
                        var recordsToCheck = [];
                        for (var w = 0; w < that.config.rowsets.length; w++) {
                            recordsToCheck = recordsToCheck.concat(that.virtualRows[that.config.rowsets[w].id].records);
                        }
                        that.widget.checkRows(recordsToCheck, $checkbox.is(':checked'), true);
                        event.stopImmediatePropagation();
                    });
                }
            });
        }

        if (this.options.contextMenu.enabled && this.options.contextMenu.enableRowMenuButton) {
            if (this.options.layout.columnGroups[0].id !== constants.MENU_BUTTON_GROUP) {
                this.options.layout.columnGroups.unshift(_.clone(menuButtonGroup));
            }
            this.options.columns[constants.MENU_BUTTON_COLUMN] = _.clone(menuButtonColumn);
        } else if (this.options.layout.columnGroups[0].id === constants.MENU_BUTTON_GROUP) {
            this.options.layout.columnGroups.shift();
        }

        this.firstRowsetId = _.find(this.options.layout.rowsets, function (rowset) {
            return rowset.id !== constants.HEADERS_ROWSET_ID;
        }).id;
        this.config = widget.config;

        if (this.options.search.columnFilter && !this.options.advancedFilters.enabled && !_.findWhere(this.options.layout.rowsets, { id : constants.COLUMNFILTER_ROWSET_ID })) { // LUX-506
            this.options.layout.rowsets.unshift({
                id : constants.COLUMNFILTER_ROWSET_ID,
                type : constants.ROWSET_TYPE.FIXED,
                height : this.options.layout.headerHeight + 'px'
            });
        } else if (this.options.advancedFilters.enabled) {
            this.advancedFilters = new AdvancedFilters({
                widget : this.widget,
                view : this
            });
            if (!_.findWhere(this.options.layout.rowsets, { id : constants.ADVANCED_FILTER_ROWSET_ID })) {
                this.options.layout.rowsets.unshift({
                    id : constants.ADVANCED_FILTER_ROWSET_ID,
                    type : constants.ROWSET_TYPE.FIXED,
                    height : this.options.layout.headerHeight + 'px'
                });
            }
        }

        if (!this.options.layout.removeHeaderRow && this.options.layout.rowsets[0].id !== constants.HEADERS_ROWSET_ID) {
                // add rowset for headers
            this.options.layout.rowsets.unshift({
                id : constants.HEADERS_ROWSET_ID,
                type : constants.ROWSET_TYPE.FIXED,
                excludeRecordCount : true,
                height : this.options.layout.headerHeight + 'px'
            });
        }
        var quickAddIndex = _.findIndex(this.options.layout.rowsets, function (rowset) {
            return rowset.id === constants.QUICK_ADD_ROWSET_ID;
        });
        if (this.options.quickAdd.enabled) {
            if (quickAddIndex < 0) {
                var insertIndex = _.findIndex(this.options.layout.rowsets, function (rowset) {
                    return rowset.id === that.options.quickAdd.addAfterRowset;
                }) + 1;
                this.options.layout.rowsets.splice(insertIndex, 0, _.clone(QUICK_ADD_ROWSET));
            }
        } else if (quickAddIndex > 0) {
            this.options.layout.rowsets.splice(quickAddIndex, 1);
        }


        var lastGroup = _.last(this.options.layout.columnGroups);
        if (lastGroup.columns[lastGroup.columns.length - 1] !== emptyColumn.dataIndex) {
            lastGroup.columns.push(emptyColumn.dataIndex);
        }
        this.options.columns[emptyColumn.dataIndex] = _.clone(emptyColumn);

        this.recordKeys = this.options.recordKeys;
        // Holds references to the fake scrollers that are overlaid ontop of the various groups and rowsets
        this.pseudoScrollers = {
            vertical : {}, // Handles scrolling for flexible rowsets. Keyed off of rowset.id
            horizontal : {} // Handle scroll for flexible column groups. Keyed off of group.id
        };

        // Jquery saved selectors to keep from having to do a find. Keyed off of the rowset.id
        this.$rowsets = {};
        // Contains references to viewports accessed via the [rowsetId][groupId]
        this.viewports = {};
        // Used to wait for a long press to initiate row dragging
        this.rowDragTimeout = 0;
        // the vertical and horizontal draggers that let a user customize the size of groups and rowsets
        this.resizers = {
            group : {},
            rowset : {}
        };

        this.rowDragTimeout = 0;
        this.$grid = $gridMarkup.clone();
        this.updateGroupBandClasses();
        this.applyGridClasses();
        this.$grid.attr('id', this.widget.id);
        this.componentIds = {
            editor : this.widget.uuid + 'grid-cell-editor',
            menu : this.widget.uuid + 'grid-menu',
            errorPopup : this.widget.uuid + 'grid-errors',
            columnMenu : this.widget.uuid + 'grid-columnmenu',
            columnGroupMenu : this.widget.uuid + 'grid-columngroupmenu'
        };

        this.$errorBox = $errorBox.clone();
        this.$errorBox.attr('id', this.componentIds.errorPopup);
        if (this.options.inModal) {
            this.$errorBox.addClass('in-modal');
        }
        this.columnDefinitions = {};
        this.cellDefinitions = {};

            // initiate table models
        this.rowRenderer = new RowRenderer({
            options : this.options,
            view : this,
            widget : this.widget
        });
        this.selectionModel = new SelectionModel(this);
        this.columnResizer = new ColumnResizer(this);
        this.rowMover = rowMoveFns(this);

        this.columnMover = new ColumnMover(this);
        this.columnMenu = new ColumnMenu(this);
        this.columnGroupMenu = new ColumnGroupMenu(this);
        if (this.options.timephased.enabled || this.options.timephased.supportsTimephased) {
            this.timephasedColumnGroup = new TimephasedColumnGroup({
                widget : this.widget,
                gridView : this
            });
        }

        this.rowExpansionManager = new RowExpansionManager({
            widget : this.widget,
            gridView : this,
            options : this.options.rowExpansion
        });


        this.virtualRows = {};

        this.currentEditor = null;
        this.rowHeaders = [];
        this.descriptionIds = {};
        this.loadingRecords = {};
        this.touchInteractions = new TouchInteractions({
            eventHandlers: eventHandlers,
            widget : this.widget,
            gridView : this,
            options : this.options.touchInteractions
        });
    };

    _.assign(gridView.prototype, {
        remove : function () {
            this.$grid.empty();
            this.$el.empty();
            // wont remove on $el do the action of the above 2 statements?
            this.$el.remove();
        },

        setRowHeight : function () {
            this.ROW_HEIGHT = this.options.layout.rowHeight;
            this.CELL_HEIGHT = this.ROW_HEIGHT - this.options.borders.row;
        },


        init : function () {
            var that = this;
            that.generateColumnDefinitions();
            if (this.options.timephased.enabled) {
                this.timephasedColumnGroup.createTimescale();
            }
            that.generateCellDefinitions();
            // Link rowsets
            var prevRowset;
            _.assign(that.config.rowsets, that.options.layout.rowsets);
            _.forEach(that.config.rowsets, function (rowset) {
                rowset.sortable = rowset.sortable != null ? rowset.sortable : true;
                rowset.prev = prevRowset;
                if (prevRowset != null && rowset.type !== constants.ROWSET_TYPE.MANUAL) {
                    prevRowset.next = rowset;
                }
                if (rowset.type !== constants.ROWSET_TYPE.MANUAL) {
                    prevRowset = rowset;
                }
                that.virtualRows[rowset.id] = {
                    displayTop : 0,
                    displayBottom : 10,
                    visibleRows : 0,
                    renderedTop : -1,
                    renderedBottom : -1,
                    records : [],
                    nodes : {}
                };
            });
            that.firstRowset = _.first(that.config.rowsets);
            that.widget.element.append(that.$grid);
            that.$loading = loadingTemplate({ loading : that.options.locale.loading });
            that.$grid.append(loadingTemplate({ loading : that.options.locale.loading }));
            if (that.options.contextMenu.enabled) {
                that.contextMenu = new ContextMenu(that.widget);
            }

            that.addGridDescriptions();
            // Fill the layout
            that.createRowsets();
            that.createStatusBar();

            // construct table shell
            that.generateViewports();
            // Create and fix columns
            that.createColumns();

            // create rowset headers
            that.createRowsetHeaders();
            if (that.options.timephased.enabled) {
                that.timephasedColumnGroup.applyToView();
            }
            that.initListeners();
            that.applyAriaSettings();
            that.setEscapeMode(true);
            that.getHeaderHeight();
            that.updateHeaderOverlay();
        },

        addGridDescriptions : function () {
            var that = this;
            var $descriptionContainer = this.$grid.find('.grid-descriptions');
            var describedBy = [];

            if (this.options.locale.gridSummary) {
                this.descriptionIds.gridSummary = this.widget.uuid + 'grid-description-summary';
                describedBy.push(this.descriptionIds.gridSummary);
                $descriptionContainer.append(descriptionTemplate({
                    id : this.descriptionIds.gridSummary,
                    message : this.options.locale.gridSummary
                }));
            }

            if (this.options.locale.gridUsage) {
                this.descriptionIds.gridUsage = this.widget.uuid + 'grid-description-usage';
                describedBy.push(this.descriptionIds.gridUsage);
                $descriptionContainer.append(descriptionTemplate({
                    id : this.descriptionIds.gridUsage,
                    message : this.options.locale.gridUsage
                }));
            }

            // LUX-277:  There is no indication by JAWS that the selector (check box) column is available.
            if (this.options.locale.rowSelectorCol && this.options.selectionMode === 'check') {
                this.descriptionIds.rowSelectorCol = this.widget.uuid + 'grid-description-rowSelectorCol';
                describedBy.push(this.descriptionIds.rowSelectorCol);
                $descriptionContainer.append(descriptionTemplate({
                    id : this.descriptionIds.rowSelectorCol,
                    message : this.options.locale.rowSelectorCol
                }));
            }

            if (describedBy.length) {
                this.$grid.attr('aria-describedby', describedBy.join(' '));
            }


            this.descriptionIds.cellState = {};
            _.forEach(this.options.locale.cellState, function (message, key) {
                that.descriptionIds.cellState[key] = that.widget.uuid + 'grid-description-cell-' + key;
                $descriptionContainer.append(descriptionTemplate({
                    id : that.descriptionIds.cellState[key],
                    message : message
                }));
            });


            var labels = [];

            if (this.options.locale.gridLabel) {
                this.descriptionIds.gridLabel = this.widget.uuid + 'grid-description-grid-label';
                $descriptionContainer.append(descriptionTemplate({
                    id : this.descriptionIds.gridLabel,
                    message : this.options.locale.gridLabel
                }));
                labels.push(this.descriptionIds.gridLabel);
            }

            if (this.options.locale.escapeModeText) {
                this.descriptionIds.escapeModeFilm = this.widget.uuid + 'grid-description-escapemode';
                this.descriptionIds.escapeModeText = this.descriptionIds.escapeModeFilm + '-message';
                this.$grid.find('.escape-mode-film').attr('id', this.descriptionIds.escapeModeFilm);
                this.$grid.find('.escape-mode-message').append(this.options.locale.escapeModeText).attr('id', this.descriptionIds.escapeModeText);
                labels.push(this.descriptionIds.escapeModeFilm);
            }

            if (labels.length) {
                this.$grid.attr('aria-labelledby', labels.join(' '));
            }

            this.descriptionIds.columnEditors = {};
            _.forEach(this.options.locale.columnEditors, function (message, key) {
                that.descriptionIds.columnEditors[key] = that.widget.uuid + 'grid-description-editor-' + key;
                $descriptionContainer.append(descriptionTemplate({
                    id : that.descriptionIds.columnEditors[key],
                    message : message
                }));
            });
        },

        applyAriaSettings : function () {
            if (!this.options.editable) {
                this.$grid.attr('aria-readonly', true);
                this.$grid.addClass('uneditable-grid');
            }


            this.resetGridRole();
        },

        applyGridClasses : function () {
            if (this.options.contextMenu.enabled && this.options.contextMenu.enableRowMenuButton) {
                this.$grid.addClass('has-context-menu');
            }
            this.$grid.addClass(this.options.classes.grid.join(' '));
            this.applyVerticalLines();
            this.updateReadOnlyStyling();
        },

        applyVerticalLines : function () {
            if (this.options.showVerticalLines) {
                this.$grid.addClass('vertical-lined-grid');
            } else {
                this.$grid.removeClass('vertical-lined-grid');
            }
        },

        resetGridRole : function () {
            if (this.options.tree || this.widget.groupingApplied) {
                this.$grid.attr('role', 'treegrid');
            } else {
                this.$grid.attr('role', 'grid');
            }
        },

        setLoading : function (loading) {
            this.$grid.toggleClass(this.options.classes.loading, loading);
            this.isLoading = loading;
        },

        setDisabled : function () {
            this.$grid.toggleClass(this.options.classes.disabled, this.options.disabled);
        },

        createRowsetHeaders : function () {
            var that = this;
            _.forEach(that.config.rowsets, function (rowset) {
                if (rowset.id !== constants.HEADERS_ROWSET_ID && rowset.type !== constants.ROWSET_TYPE.MANUAL) {
                    var header = rowset.header;
                    if (header != null) {
                        that.createHeaderRow(rowset.id, header.title);
                        that.virtualRows[rowset.id].headerRow = true;
                    }
                }
            });
        },
        // Move into table functions
        createHeaderRow : function (rowsetId, title) {
            var that = this;
            var groups = that.config.groups;
            var titleSet = false;
            _.forEach(groups, function (group) {
                var $tr = $(trTemplate({ id : 'header', groupId : group.id, ariaAttributes : ' role="row" ' }));
                var $th = $('<th class="pgbu-cell"><div class="cell-renderer"></div></th>');
                $th.attr('colspan', group.columns.length);
                // Issue LUX 83: (ROWSETS: ESTABLISH RELATIONSHIP BETWEEN ROWSET HEADING AND CELLS IN THAT ROWSET)
                $th.find('.cell-renderer').attr('id', that.widget.uuid + '-' + 'rowsetId-' + rowsetId);
                // should be derived from the row
                if (!titleSet && !group.internalDisplayOnly) {
                    $th.find('.cell-renderer').text(title);
                    titleSet = true;
                }
                $tr.append($th);
                that.viewports[rowsetId][group.id].$thead.append($tr);
            });
        },
        // can stay slightly edit

        initListeners : function () {
            var that = this;
            that.touchInteractions.initListeners();
            that.$grid.on('keypress', $.proxy(eventHandlers.keypress, that)());
            that.$grid.on('keyup', $.proxy(eventHandlers.keyup, that)());
            that.$grid.on('keydown', $.proxy(eventHandlers.keydown, that)());
            that.$grid.on('mousedown', $.proxy(eventHandlers.mousedown, that)());
            that.$grid.on('mousedown', '.cell-selection-handle', eventHandlers.mouseDownCellDrag.bind(that)());
            that.$grid.on('focusin', '.pseudo-scroller', function (event) {
                event.preventDefault();
                that.widget.requestBrowserFocus(true, event);
            });
            that.$grid.on('blur', function (event) {
                var $newFocus = $(event.relatedTarget);
                if (!$newFocus.closest('.pgbu-grid-menu').length) {
                    that._clearColumnGroupActive();
                    that.closeMenus(FROM_BLUR_ARG);
                }
            });
            that.$grid.on('change', $.proxy(eventHandlers.change, that)());
            that.$grid.on('dblclick', $.proxy(eventHandlers.doubleClick, that)());
            that.$grid.on('click', '.menu-button-cell:not(.row-menu-disabled)', $.proxy(eventHandlers.clickMenu, that)());
            that.$grid.on('click', '.quick-add-button', function () {
                if (that.widget._trigger('before-quick-add-row', null, {
                    columnId : null,
                    dataIndex : null
                }) === false) {
                    return;
                }
                that.widget._commitNewRecord({
                    ignoreEmpty : true,
                    focusAfterAdd : true
                });
            });
            that.$grid.on('click', '[data-grid-trigger], [data-trigger]', $.proxy(eventHandlers.clickTrigger, that)());
            if (that.options.contextMenu.enabled) {
                that.$grid.on('contextmenu', $.proxy(eventHandlers.contextMenu, that)());
            }
            that.$grid.on('click', '.column-group-menu', that._openColumnGroupMenuEvent.bind(that));
            that.$grid.on('touchstart', '.grid-rowset', function (event) {
                var id = $(this).attr('data-id');
                var scroller = that.$grid.find('#' + that.$grid.attr('id') + '-scroller-vertical-' + id);
                var horizontalGroup = $(event.target).closest('[data-group]').attr('data-group');
                var horizontalScroller = that.$grid.find('#' + that.$grid.attr('id') + '-scroller-horizontal-' + horizontalGroup);
                if (event.originalEvent.touches.length === 1) {
                    var start = {
                        x : event.originalEvent.touches[0].pageX,
                        y : event.originalEvent.touches[0].pageY
                    };
                    var originalScrollTop = scroller.scrollTop();
                    var originalScrollLeft = horizontalScroller.scrollLeft();

                    that.$grid.on('touchmove.grid', function (event) {
                        horizontalScroller.triggerHandler($.Event(event, { touches : event.originalEvent.touches }), {
                            scrollTop : originalScrollTop,
                            start : start,
                            scrollLeft : originalScrollLeft
                        });
                        scroller.triggerHandler($.Event(event, { touches : event.originalEvent.touches }), {
                            scrollTop : originalScrollTop,
                            start : start,
                            scrollLeft : originalScrollLeft
                        });
                    });
                }

                that.$grid.on('touchend.grid touchcancel.grid', function () {
                    that.$grid.off('.grid');
                });
            });
            that.$grid.on('scroll', $.proxy(eventHandlers.scroll, that)());
            that.$grid.on('mousemove', $.proxy(eventHandlers.mousemove, that)());
            that.$grid.on('mouseleave', $.proxy(eventHandlers.mouseleave, that)());
            _.forEach(that.resizers.group, function (resizer) {
                resizer.$selector.on('mousedown', function (event) {
                    event.preventDefault();
                    that.resizeGroup(resizer);
                });
            });

            _.forEach(that.resizers.rowset, function (resizer) {
                resizer.$selector.on('mousedown', function (event) {
                    event.preventDefault();
                    that.resizeRowset(resizer);
                });
            });

            var verticalScrollHeight = this.ROW_HEIGHT;
            _.forEach(that.pseudoScrollers.vertical, function (scroller, rowsetId) {
                var $rowset = that.$rowsets[rowsetId];
                // Setup vertical scroll listeners on the actual pseudoScrollers
                var verticalScrollTimeout;// Used for when full page renders are being done. This will keep it snappy
                var verticalScrollDelay = that.options.performance.verticalScrollTimeout;
                var throttledScrollHandler = _.throttle(function () {
                    var scrollTop = scroller.$scroller.scrollTop();
                    var round = Math.round(scrollTop / that.ROW_HEIGHT);
                    var ceil = Math.ceil(scrollTop / that.ROW_HEIGHT);

                    if (ceil === that.virtualRows[rowsetId].records.length - that.virtualRows[rowsetId].visibleRows) {
                        that.virtualRows[rowsetId].displayTop = ceil;
                    } else {
                        that.virtualRows[rowsetId].displayTop = round;
                    }
                    if (verticalScrollDelay > 0) { // If there is a delay already, check if a full rowset render is being done, if not remove the delay
                        if (that.virtualRows[rowsetId].renderedBottom > that.virtualRows[rowsetId].displayTop &&
                                that.virtualRows[rowsetId].renderedTop < (that.virtualRows[rowsetId].displayTop + (that.virtualRows[rowsetId].visibleRows * 0.8))) {
                            verticalScrollDelay = 0;
                        } else {
                            $rowset.addClass('long-render');
                            scroller.$scroller.addClass('long-render');
                            if (scroller.originalScroller != null) {
                                scroller.originalScroller.$scroller.addClass('long-render');
                            }
                            that.widget._trigger('view-sync-long-render', null, { longRender : true });
                        }
                    }
                    clearTimeout(verticalScrollTimeout); // Clear last render attempt
                    verticalScrollTimeout = setTimeout(function () {
                        that.renderRowset(rowsetId);
                        $rowset.removeClass('long-render');
                        scroller.$scroller.removeClass('long-render');
                        if (scroller.originalScroller != null) {
                            scroller.originalScroller.$scroller.removeClass('long-render');
                        }
                        that.widget._trigger('view-sync-long-render', null, { longRender : false });
                    }, verticalScrollDelay); // Set a timeout to render the rowset
                    verticalScrollDelay = that.options.performance.verticalScrollTimeout; // Reset the value (this value is changed in the mousewheel method)
                }, that.options.performance.verticalScrollThrottle);

                scroller.$scroller.on('scroll', throttledScrollHandler);

                scroller.$scroller.on('touchmove.mobile', function (event, original) {
                        // Using clone event from above
                    if (event.touches.length === 1) {
                        event.preventDefault();
                        event.stopPropagation();
                        var current = {
                            x : event.touches[0].pageX,
                            y : event.touches[0].pageY
                        };
                        scroller.$scroller.scrollTop(original.scrollTop + (original.start.y - current.y));
                        throttledScrollHandler();
                    }
                });

                var eventName = 'wheel'; // Standardized event
                if (document.onmousewheel !== undefined && document.onwheel === undefined) {
                    eventName = 'mousewheel'; // Only needed for Safari
                }
                $rowset.on(eventName, function (event) {
                    function getWheelDelta(e) {
                        if (!(e.wheelDeltaY || e.deltaY || e.wheelDelta)) {
                            e = e.originalEvent;
                        }

                        if (e.deltaY != null) {
                            return -e.deltaY;
                        } else if (e.wheelDeltaY != null) {
                            return e.wheelDeltaY;
                        } else if (e.wheelDelta != null) {
                            return e.wheelDelta;
                        }
                        return 0;
                    }

                    var deltaY = getWheelDelta(event);
                    var scrollAmount = 0;
                    if (deltaY > 0) {
                        scrollAmount = -verticalScrollHeight;
                    } else if (deltaY < 0) {
                        scrollAmount = verticalScrollHeight;
                    }
                    if (scrollAmount !== 0) {
                        scroller.$scroller.scrollTop(scroller.$scroller.scrollTop() + scrollAmount);
                        verticalScrollDelay = 0;
                        return false;
                    }
                });
            });

            _.forEach(that.pseudoScrollers.horizontal, function (scroller, group) {
                var $tablesToAdjust = that.viewports['$group-' + group].find('table,.viewport-overlay,.timephased-container');
                var currentPosition = -(scroller.$scroller.scrollLeft());
                var groupConfig = _.find(that.config.groups, function (groupC) {
                    return groupC.id === group;
                });

                var throttledHorizontalHandler = _.throttle(function () {
                    var newPosition = -(scroller.$scroller.scrollLeft());
                    if (that.widget.syncedGrid != null) {
                        that.widget.syncedGrid.view.pseudoScrollers.horizontal[group].$scroller.scrollLeft(-newPosition);
                    }
                    if (currentPosition !== newPosition && currentPosition !== newPosition - 2) {
                        groupConfig.positionLeft = newPosition;
                        if (newPosition !== 0) {
                            newPosition -= 2;
                        }
                        $tablesToAdjust.css('left', newPosition);
                        if (groupConfig.timephased) {
                            that.timephasedColumnGroup.updateBucketLabelPositions();
                        }
                        that.closeEditor();
                        that.closeMenus();
                        currentPosition = newPosition;
                    }
                }, that.options.performance.horizontalScrollThrottle);

                scroller.$scroller.on('scroll', throttledHorizontalHandler);
                // swipe support ???? unsure
                // Need to handle a similar situation for horizontal scrollwheel.
                var eventName = 'wheel'; // Standardized event
                if (document.onmousewheel !== undefined && document.onwheel === undefined) {
                    eventName = 'mousewheel'; // Only needed for Safari
                }
                $tablesToAdjust.parent().on(eventName, function (event) {
                    function getWheelDelta(e) {
                        if (!(e.wheelDeltaX || e.deltaX || e.wheelDelta)) {
                            e = e.originalEvent;
                        }

                        if (e.deltaX != null) {
                            return -e.deltaX;
                        } else if (e.wheelDeltaX != null) {
                            return e.wheelDeltaX;
                        }
                        return 0;
                    }

                    var deltaX = -1 * getWheelDelta(event);
                    if (deltaX !== 0) {
                        scroller.$scroller.scrollLeft(scroller.$scroller.scrollLeft() + deltaX);
                    }
                });
                scroller.$scroller.on('touchmove.mobile', function (event, original) {
                    // Using cloned event from above
                    if (original && event.touches.length === 1) {
                        event.preventDefault();
                        event.stopPropagation();
                        var current = {
                            x : event.touches[0].pageX,
                            y : event.touches[0].pageY
                        };

                        scroller.$scroller.scrollLeft(original.scrollLeft + (original.start.x - current.x));
                        throttledHorizontalHandler();
                    }
                });
            });
        },

        removeListeners : function () {
            var that = this;
            that.$grid.off('keypress');
            that.$grid.off('keyup');
            that.$grid.off('keydown');
            that.$grid.off('mousedown');
            that.$grid.off('change');
            that.$grid.off('dblclick');
            that.$grid.off('click');
            that.$grid.off('contextmenu');
            that.$grid.off('scroll');
            that.$grid.off('mousemove');
            that.$grid.off('mouseleave');
            _.forEach(that.resizers.group, function (resizer) {
                resizer.$selector.off('mousedown');
            });

            _.forEach(that.resizers.rowset, function (resizer) {
                resizer.$selector.off('mousedown');
            });

            _.forEach(that.pseudoScrollers.vertical, function (scroller /*, rowsetId */) {
                scroller.$scroller.off('scroll');
            });

            _.forEach(that.pseudoScrollers.horizontal, function (scroller /*, group */) {
                scroller.$scroller.off('scroll');
            });
        },

        /*
         * Create the rowset structure and add neccessary pseudoScrollers, and scrolling
         * shims for spacing.
         */
        createRowsets : function () {
            var that = this;
            var $scrollers = $();
            _.forEach(that.config.rowsets, function (rowset) {
                var $rowset;
                if (rowset.type === constants.ROWSET_TYPE.FIXED || rowset.type === constants.ROWSET_TYPE.MANUAL) {
                    $rowset = $(rowsetTemplate(rowset));
                } else {
                    $rowset = $(rowsetTemplate(rowset));
                    // Create scrollers and save references to them
                    $scrollers = $scrollers.add(that.createPseudoScroller('vertical', rowset));
                }
                // store all rowset options
                $rowset.data('rowset', rowset);
                if (rowset.type === constants.ROWSET_TYPE.MANUAL) {
                    rowset.insertDom($rowset, rowset.id);
                }

                that.$grid.append($rowset);
                that.$rowsets[rowset.id] = $rowset;

                if (rowset.type === constants.ROWSET_TYPE.FLEX) {
                    that.$grid.append(that.createRowsetResizer(rowset));
                }
            });

            // Remove the resize bar for the last rowset
            var lastFlexRowset = _.last(_.keys(that.resizers.rowset));
            if (lastFlexRowset != null) {
                var resizer = that.resizers.rowset[lastFlexRowset];
                if (resizer != null) {
                    resizer.$selector.remove();
                    delete that.resizers.rowset[lastFlexRowset];
                }
            }

            var $shims = $scrollShims.clone();
            that.$grid.find('.grid-rowset').last().after($shims);
            that.$grid.append($scrollers);

            that.$scrollShims = {
                $bottom : $shims.filter('.bottom-scroll-shim'),
                $right : $shims.filter('.right-scroll-shim')
            };
            that.$scrollShims.$bottom.css('height', that.options.scrollbar.extraHeight);
            that.$scrollShims.$right.css('width', that.options.scrollbar.extraWidth);
        },

        updateHeaderOverlay : function () {
            if (this.$rowsets.headers != null) {
                var $rowsetOverlay = this.$rowsets.headers.find('.rowset-overlay');
                $rowsetOverlay.empty();
                if (this.options.headerOverlay != null && this.options.headerOverlay.length) {
                    $rowsetOverlay.append(this.options.headerOverlay);
                }
            }
        },

        createPseudoScroller : function (type, area) {
            var that = this;

            var id = this.widget.uuid + 'grid-scroller-' + type + '-' + area.id;
            var $scroller = $(pseudoScrollerTemplate({ type : type, id : id }));
            that.pseudoScrollers[type][area.id] = {
                $content : $scroller.find('.empty-content'),
                $scroller : $scroller
            };
            if (area.ganttRowset) {
                area.replacementScroller.originalScroller = that.pseudoScrollers[type][area.id];
                area.replacementScroller.$scroller.off('scroll.gridsync');
                area.replacementScroller.$scroller.on('scroll.gridsync', function () {
                    area.replacementScroller.originalScroller.$scroller.scrollTop(area.replacementScroller.$scroller.scrollTop());
                });
                that.pseudoScrollers[type][area.id] = area.replacementScroller;
            }
            return $scroller;
        },

        generateViewports : function () {
            var that = this;

            _.forEach(that.config.rowsets, function (rowset) {
                that.viewports['$rowset-' + rowset.id] = $();
                that.viewports[rowset.id] = {};
            });

            _.forEach(that.config.groups, function (group) {
                that.viewports['$group-' + group.id] = $();
                _.forEach(that.config.rowsets, function (rowset) {
                    var viewport = {};
                    viewport.type = group.type;
                    viewport.group = group.id;
                    viewport.rowset = rowset.id;
                    viewport.viewportId = that.widget.uuid + 'grid-viewport-rowset-' + rowset.id + '-group-' + group.id;
                    viewport.menuText = '';
                    if (rowset.id === constants.HEADERS_ROWSET_ID && that._hasColumnGroupMenu(group)) {
                        viewport.menuText = that.options.locale.openColumnGroupMenu;
                    }
                    viewport.$viewport = $(viewportTemplate(viewport));
                    viewport.$viewport.css('background-size', that.ROW_HEIGHT + 'px ' + that.ROW_HEIGHT + 'px');
                    viewport.$table = viewport.$viewport.find('table');
                    viewport.$tbody = viewport.$table.find('tbody');
                    viewport.tbody = '';
                    viewport.$thead = viewport.$table.find('thead');
                    viewport.thead = [];
                    viewport.$tfoot = viewport.$table.find('tfoot');
                    viewport.tfoot = [];
                    viewport.groupConfig = group; // this will probably change to group etc for now so you can access it
                    viewport.rowsetConfig = rowset;
                    viewport.$overlay = viewport.$viewport.find('.viewport-overlay');

                    that.viewports[rowset.id][group.id] = viewport;

                    // Save selectors
                    that.viewports['$rowset-' + rowset.id] = that.viewports['$rowset-' + rowset.id].add(viewport.$viewport);
                    that.viewports['$group-' + group.id] = that.viewports['$group-' + group.id].add(viewport.$viewport);

                    // Add viewport as data reference on the dom
                    viewport.$viewport.data('viewport', viewport);

                    // Put it on
                    if (rowset.type !== constants.ROWSET_TYPE.MANUAL) {
                        that.$rowsets[rowset.id].append(viewport.$viewport);
                    }
                    if (rowset.id === constants.HEADERS_ROWSET_ID && group.title != null) {
                        that.hasNamedColumnGroups = true;
                        that.$grid.addClass('grid-with-named-groups');
                        var $titleElem = $('<div class="column-group-header"><span class="group-header-border"/><span class="group-header-name"> <i class="pgbu-icon-small pgbu-icon-folder"></i>' + group.title + '</span><span class="group-header-border"/></div>');
                        $titleElem.insertAfter(viewport.$viewport.find('.viewport-borders'));
                    }
                    if (group.title != null) {
                        viewport.$viewport.addClass('viewport-named-group');
                    }
                });
                // Create horizontal scrollers for the proper divs
                if (group.type === constants.GROUP_TYPE.FLEX) {
                    if (_.some(group.columns, function (column) {
                        return column.visible;
                    })) {
                        that.$grid.append(that.createPseudoScroller('horizontal', group));
                        that.$grid.append(that.createGroupResizer(group));
                    }
                }
            });
            // Remove the resize bar for the last group
            var lastFlexGroup = _.last(_.keys(that.resizers.group));
            if (lastFlexGroup != null) {
                var resizer = that.resizers.group[lastFlexGroup];
                if (resizer != null) {
                    resizer.$selector.remove();
                    delete that.resizers.group[lastFlexGroup];
                }
            }
        },

        createStatusBar : function () {
            var updateRecordCountFn = $.noop;
            if (this.options.statusBar.enabled) {
                var $statusBar = $(statusBarTemplate({
                    locale : this.options.locale,
                    id : this.widget.uuid + 'grid-status-bar',
                    countString : this.options.statusBar.countLabel || this.options.locale.statusBar.countLabel
                }));
                this.$statusBar = $statusBar;
                this.$recordCounter = $statusBar.find('.record-count');
                this.$recordCountGroup = $statusBar.find('.record-count-group');
                this.$statusBarLegend = $statusBar.find('.grid-legend');
                this.$statusBarToolbar = $statusBar.find('.toggle-toolbar');
                this.$statusBarLegend.prepend(this.options.statusBar.markup);
                this.$exportToExcelLink = $statusBar.find('.grid-export-container');

                if (this.options.statusBar.markup != null && this.options.statusBar.markup.length) {
                    if (this.options.statusBar.toggleItems.length) {
                        this.$statusBarLegend.addClass('with-divider');
                    }
                }

                if (this.options.statusBar.recordCountEnabled && this.options.export.enabled) {
                    this.$exportToExcelLink.css('margin-left', '30px');
                }

                if (this.options.statusBar.readOnlyStylingEnabled) {
                    this.$readOnlyToggle = $statusBar.find('.readonly-toggle');
                    var $toggle = $(statusBarToggleTemplate({
                        locale : this.options.locale,
                        label : this.options.locale.statusBar.readOnlyCellStyle,
                        labelPosition : 'before',
                        classes : 'read-only-toggle',
                        id : 'grid' + this.widget.uuid + '-statusbar-toggle-read-only'
                    }));
                    this.$readOnlyToggle = $toggle.filter('.btn-slider');
                    this.$statusBarToolbar.append($toggle);
                }


                for (var i = 0; i < this.options.statusBar.toggleItems.length; i++) {
                    this.createStatusBarToggle(this.options.statusBar.toggleItems[i]);
                }
                this.$recordCountGroup.toggleClass('hidden', !this.options.statusBar.recordCountEnabled);
                this.$exportToExcelLink.toggleClass('hidden', !this.options.export.enabled);

                this.$grid.append($statusBar);
                this.updateStatusBarPosition();
                this.addStatusBarListeners();
                this.updateReadOnlyStyling();

                updateRecordCountFn = _.debounce(function () {
                    if (this.options.statusBar.recordCountEnabled) {
                        var recordCount = 0;
                        if (this.options.tree && this.options.statusBar.getRecordCount == null && _.get(this.widget, 'store.getRecordCount') == null) {
                            logger.warn('Please implement getRecordCount for your tree grid.');
                        }
                        if (_.has(this.options.statusBar, 'getRecordCount')) {
                            recordCount = this.options.statusBar.getRecordCount();
                        } else if (_.get(this.widget, 'store.getRecordCount') != null) {
                            recordCount = this.widget.store.getRecordCount();
                        } else {
                            for (var i = 0; i < this.config.rowsets.length; i++) {
                                var rowset = this.config.rowsets[i];
                                if (!rowset.excludeRecordCount) {
                                    recordCount += this.getRowsetRecordCount(rowset.id);
                                }
                            }
                        }
                        this.$recordCounter.text(recordCount);
                    }
                }, 250);
            }
            this.updateRecordCount = updateRecordCountFn;
        },

        getRowsetRecordCount : function (rowsetId) {
            var recordCount;
            if (this.widget.groupingApplied || this.widget.options.tree) {
                recordCount = _.size(_.keys(this.virtualRows[rowsetId].nodes));
                recordCount -= _.get(_.countBy(this.widget.recordGrouper.recordGroups, function (group, groupKey) {
                    return _.includes(groupKey, 'group-' + rowsetId) ? 'inRowset' : 'outOfRowset';
                }), 'inRowset', 0);
            } else {
                recordCount = this.virtualRows[rowsetId].records.length;
            }

            return recordCount;
        },

        createStatusBarToggle : function (toggleAction) {
            var that = this;
            var $toggle = $(statusBarToggleTemplate({
                locale : this.options.locale,
                label : toggleAction.label,
                labelPosition : toggleAction.labelPosition || 'before',
                classes : toggleAction.classes || '',
                id : 'grid' + this.widget.uuid + '-statusbar-toggle-' + toggleAction.action
            }));
            var $toggleBtn = $toggle.filter('.btn-slider');
            $toggleBtn.toggleClass('active', toggleAction.enabled());
            $toggleBtn.on('click', function (event) {
                if (toggleAction.onChange != null) {
                    toggleAction.onChange(!$toggleBtn.hasClass('active'));
                } else {
                    that.widget._trigger('status-bar-toggle-change', event, {
                        state : !$toggleBtn.hasClass('active'),
                        action : toggleAction.action,
                        toggleItem : toggleAction
                    });
                }
            });
            this.$statusBarToolbar.append($toggle);
        },

        addStatusBarListeners : function () {
            var that = this;
            if (this.$readOnlyToggle != null) {
                this.$readOnlyToggle.on('click', function (event) {
                    that.options.theme.readOnlyStyling = !that.$readOnlyToggle.hasClass('active');
                    that.updateReadOnlyStyling(true);
                });
            }
            this.$exportToExcelLink.on('click', function (event) {
                that.exportToExcel();
            });
        },

        updateReadOnlyStyling : function (skipToggle) {
            this.$grid.toggleClass('cell-uneditable-highlight', this.options.theme.readOnlyStyling);
            if (this.$readOnlyToggle != null && !skipToggle) {
                this.$readOnlyToggle.toggleClass('active', this.options.theme.readOnlyStyling);
            }
        },

        updateStatusBarPosition : function () {
            if (this.options.contextMenu.enabled && this.options.contextMenu.enableHoverRowMenuButton) {
                this.$statusBar.addClass('adjust-for-context-menu');
            } else {
                this.$statusBar.removeClass('adjust-for-context-menu');
            }
            if (this.$readOnlyToggle != null) {
                this.$readOnlyToggle.toggleClass('active', this.$grid.hasClass('cell-uneditable-highlight'));
            }
        },

        // layout functions
        createGroupResizer : function (group) {
            var that = this;
            var resizer = {};
            resizer.$selector = $(groupResizer(group));
            resizer.group = group;
            that.resizers.group[group.id] = resizer;
            return resizer.$selector;
        },

        createRowsetResizer : function (rowset) {
            var that = this;
            var resizer = {};
            resizer.$selector = $(rowsetResizer(rowset));
            resizer.rowset = rowset;
            that.resizers.rowset[rowset.id] = resizer;
            return resizer.$selector;
        },

        generateCellDefinitions : function () {
            var that = this;
            var cells = this.options.cells || {};
            var defs = cells.definitions;

            _.forEach(defs, function (cellOptions, dataIndex) {
                if (cellOptions.cellIdentifier == null) {
                    cellOptions.cellIdenfitier = dataIndex;
                }
                that.cellDefinitions[dataIndex] = generateColumn({
                    definedColumn : _.clone(cellOptions),
                    modifiers : { cellAttrsOnly : true },
                    options : that.options.defaultColumnOptions
                });
            });
            that.widget.cellDefinitions = that.cellDefinitions;
        },

        generateColumnDefinitions : function () {
            var prevGroup = null;
            var that = this;
            var clonedGroups;
            if (this.options.timephased.enabled || this.options.timephased.supportsTimephased) {
                var timephasedColumnGroup = _.find(that.options.layout.columnGroups, function (columnGroup) {
                    return columnGroup.timephased;
                });
                this.timephasedGroupId = timephasedColumnGroup.id;
                timephasedColumnGroup.columns = [];
                var timephasedDefinitions = this.timephasedColumnGroup.getColumnDefinitions();
                _.forEach(timephasedDefinitions, function (definition) {
                    that.columnDefinitions[definition.dataIndex] = generateColumn({
                        definedColumn : _.clone(definition),
                        options : that.options.defaultColumnOptions
                    });
                    timephasedColumnGroup.columns.push(definition.dataIndex);
                });
            }

            _.forEach(that.options.columns, function (columnOptions, dataIndex) {
                if (columnOptions.dataIndex == null) {
                    columnOptions.dataIndex = dataIndex;
                }
                that.columnDefinitions[dataIndex] = generateColumn({
                    definedColumn : _.clone(columnOptions),
                    options : that.options.defaultColumnOptions
                });
            });
            that.widget.columnDefinitions = that.columnDefinitions;

            clonedGroups = _.cloneDeep(that.options.layout.columnGroups);

            this.addIconColumns(clonedGroups);

            var maxStackedCells = 0;
            _.forEach(clonedGroups, function (group) {
                if (!group.disabled) {
                    group.placementModifier = 1;
                    if (group.stackedCells != null && group.stackedCells.length) {
                        group.showStackedCells = true;
                        group.placementModifier = group.stackedCells.length;
                        maxStackedCells = Math.max(maxStackedCells, group.stackedCells.length);
                    }
                    group.positionLeft = 0;
                    group.prev = prevGroup;
                    if (prevGroup != null) {
                        prevGroup.next = group;
                    }
                    prevGroup = group;
                    // Build column definitions
                    for (var c = 0; c < group.columns.length; ++c) {
                        var column = that.columnDefinitions[group.columns[c]];
                        //>>excludeStart("validation", pragmas.production)
                        if (!column) {
                            logger.fatal('Column with name `' + group.columns[c] + '` does not exist - cannot generate column definitions');
                        }
                        //>>excludeEnd("validation")
                        group.columns[c] = column;
                        group.columns[c].groupId = group.id;
                        if (group.stackedCells != null && group.stackedCells.length) {
                            group.columns[c].stackedCells = true;
                        }
                    }

                    that.config.groups.push(group);
                }
            });
            if (maxStackedCells >= 1) {
                if (maxStackedCells === 1) {
                    that.options.layout.rowHeight = defaultOptions.layout.rowHeight;
                } else {
                    that.options.layout.rowHeight = maxStackedCells * that.options.stackedCells.rowHeight + this.options.borders.row + 1;
                }
                this.setRowHeight();
            }
        },

        addIconColumns : function (clonedGroups) {
            var that = this;
            _.forEach(this.options.iconColumns, function (iconColumn, idx) {
                var iconColumnIndex = iconColumn.dataIndex + '-' + iconColumn.position + '-' + idx;
                that.columnDefinitions[iconColumnIndex] = generateColumn({
                    definedColumn : {
                        dataIndex : iconColumnIndex,
                        displayType : 'joinedIcon',
                        iconArgs : iconColumn,
                        internalDisplayOnly : true,
                        width : iconColumn.width,
                        joinedColumns : [iconColumn.dataIndex],
                        rendererFns : iconColumn.rendererFns,
                        tooltip : iconColumn.headerTooltip
                    },
                    options : that.options.defaultColumnOptions,
                    sorter : iconColumn.sorter,
                    dataType : iconColumn.dataType || 'boolean'
                });
                var joinedColumns = [iconColumnIndex];
                that.columnDefinitions[iconColumn.dataIndex].joinedColumns = joinedColumns.concat(that.columnDefinitions[iconColumn.dataIndex].joinedColumns);
                _.forEach(clonedGroups, function (clonedGroup) {
                    var joinedIndex = clonedGroup.columns.indexOf(iconColumn.dataIndex);
                    if (joinedIndex > -1) {
                        var insertIndex = iconColumn.position === 'after' ? joinedIndex + 1 : joinedIndex;
                        clonedGroup.columns.splice(insertIndex, 0, iconColumnIndex);
                    }
                });
            });
        },

        createColumns : function () {
            var that = this;
            var first = true;
            var groupTitleColumn, groupTitleGroupId;

            // loop through each group so we can get the assigned columns
            _.forEach(that.config.groups, function createByGroup(group) {
                _.forEach(group.columns, function initGroupColumns(column) {
                    if (column.visible) {
                        if (that.expandableDataIndex == null && !column.internalDisplayOnly && column.expandable !== false) {
                            groupTitleColumn = column;
                            groupTitleColumn.colSpan = 1;
                            groupTitleGroupId = group.id;
                            that.expandableDataIndex = column.dataIndex;
                            if (that.quickAddHintColumn == null) {
                                that.quickAddHintColumn = column.dataIndex;
                            }
                        } else if (groupTitleColumn != null && column.rollupRole == null && (!column.internalDisplayOnly || column.displayType === 'joinedIcon') && !column.timephased && groupTitleGroupId === group.id) {
                            groupTitleColumn.colSpan++;
                            column.hideGroupCell = true;
                        } else {
                            groupTitleColumn = null;
                        }
                        if (first && !column.internalDisplayOnly) {
                            that.firstVisibleColumn = column.dataIndex;
                            first = false;
                        }
                        if (!column.internalDisplayOnly) {
                            that.lastVisibleColumn = column.dataIndex;
                        }
                        if (column.rowHeader) {
                            that.rowHeaders.push(column.dataIndex);
                        }
                    }
                });

                if (that.quickAddHintColumn == null) {
                    that.quickAddHintColumn = that.firstVisibleColumn;
                }
                if (that.options.quickAdd.hintColumn != null) {
                    that.quickAddHintColumn = that.options.quickAdd.hintColumn;
                }

                // loop through columns for the group otherwise add header shims into other rowsets
                _.forEach(that.config.rowsets, function createRowsetAndColumns(rowset) {
                    var viewport = that.viewports[rowset.id][group.id];
                    var role = 'presentation';
                    if (rowset.id === constants.HEADERS_ROWSET_ID) {
                        role = 'row';
                    }
                    if (rowset.id === constants.COLUMNFILTER_ROWSET_ID) {  // LUX-506
                        role = 'row';
                    }
                    viewport.$headerRow = $('<tr role="' + role + '" ></tr>').appendTo(viewport.$thead);
                    _.forEach(group.columns, function createViewportColumn(column) {
                        if (column.visible) {
                            viewport.$headerRow.append(that.createColumn(column, rowset.id));
                        }
                    });
                });
            });
            if (!that.rowHeaders.length) {
                that.rowHeaders.push(that.firstVisibleColumn);
            }
            that.rowHeaders = _.uniq(that.rowHeaders);
        },

        createColumn : function (column, rowsetId) {
            var that = this;
            var $column = $();
            // add columns only in the first rowset
            if (rowsetId === constants.HEADERS_ROWSET_ID) {
                // create row selector column markup
                var columnText = column.headerRenderer.create(column, {
                    id : this.getHeaderId(column.dataIndex),
                    descriptionIds : that.descriptionIds,
                    popoverTooltip : this.widget.options.locale.popoverTooltip
                });
                $column = $(columnText);
                this.attachColumnListeners(column, $column);
                column.headerRenderer.attachListeners($column);
            } else {
                $column = $(thShimTemplate(column));
            }

            if (rowsetId === constants.ADVANCED_FILTER_ROWSET_ID) {
                $column = this.advancedFilters.make(column);
            }
            if (rowsetId === constants.COLUMNFILTER_ROWSET_ID) {
                $column = $(column.columnFilterRenderer.create(column,
                    {
                        id : that.widget.uuid + 'grid-column-filter-' + column.dataIndex,
                        headerId : that.getHeaderId(column.dataIndex),
                        descriptionIds : that.descriptionIds,
                        placeHolderText : that.options.locale.columnFilterPlaceholderText,
                        searchFields: that.widget.columnFilterInstance.searchFields
                    }));
                column.columnFilterRenderer.attachListeners(column, $column, that.widget);
            }

            return $column;
        },


        attachColumnListeners : function (column, $column) {
            var that = this;
            var columnTimeout = 0;
            var clearColumnResizingClass = function () {
                $column.removeClass('column-resizing');
            };
            $column.find('.col-resize-anchor').on('mousedown',
                    function (event) {
                        event.preventDefault();
                        event.stopPropagation();
                        var startX = event.clientX;
                        var $th = $(this).parents('th');
                        columnTimeout = setTimeout(function () {
                            that.columnResizer.initColumnResize($th, startX);
                            that.closeEditor();
                            $column.addClass('column-resizing');
                        }, 200);
                    });

            // column listeners

            $column.on('mousedown', function (event) {
                event.preventDefault();
                var $md = $(this);
                columnTimeout = setTimeout(function () {
                    var $th = $md.closest('th');
                    var column = that.columnDefinitions[$th.attr('data-index')];
                    if (column.draggable) {
                        that.columnMover.initColumnDrag($th,
                                $md.closest('div.viewport').data('viewport'),
                                event.clientX);
                    }
                }, 200);
                that.closeEditor();
                that.setEscapeMode(false);
            });


            $column.on('mouseup', function (event) {
                clearTimeout(columnTimeout);
                that.widget.timeouts.columnResize = setTimeout(clearColumnResizingClass, 100);
            });


            $column.on('dblclick', '.col-resize-anchor', function (event) {
                var $th = $(this).closest('th');
                var group = $th.closest('div.viewport').data('group');
                var column = that.columnDefinitions[$th.attr('data-index')];
                that.autoResizeColumn(column, group);
            });

            // event for sorting
            $column.on('click', function (event) {
                var $target = $(event.target);
                if ($target.is('.col-resize-anchor') || $column.is('.column-resizing') || $target.is('input') || $target.closest('.orcl-popover').length) {
                    clearTimeout(that.widget.timeouts.columnResize);
                    clearColumnResizingClass();
                    return;
                }
                var $th = $(this);
                // close any editor or menus
                that.closeEditor();
                that.closeMenus();

                var column = that.columnDefinitions[$th.attr('data-index')];
                if (column.dataIndex !== constants.ROW_SELECTOR && column.sortable) {
                    that.widget.sortRows(column.dataIndex, event.ctrlKey || event.metaKey);
                    that.recalculateHeaderHeight = true;
                    that.widget.resize();
                    return false;
                }
            });

            $column.on('contextmenu', function (event) {
                var $target = $(event.target);
                that.openColumnMenu($target.closest('th'), event, false);
                event.stopPropagation();
                event.preventDefault();
            });

            $column.find('input[type="checkbox"]').on('change', function (event) {
                var $checkbox = $(this);
                var recordsToCheck = [];
                for (var w = 0; w < that.config.rowsets.length; w++) {
                    recordsToCheck = recordsToCheck.concat(that.virtualRows[that.config.rowsets[w].id].records);
                }
                that.widget._trigger('header-toggle-checked', event, {
                    dataIndex : column.dataIndex,
                    checked : $checkbox.is(':checked'),
                    allRecords : recordsToCheck
                });
                return false;
            });
            that.touchInteractions.attachColumnListeners($column);
        },

        autoResizeColumn : function (column, group) {
            var values = [];
            var that = this;

            this.widget.store.iterateStore(function (record) {
                values.push(that.widget.store.getValueFromRecord(record, column.dataIndex));
            });

            if (column.formatter) {
                values = _.map(values, function (value) {
                    return column.formatter.format(value, column);
                });
            }

            var maxLength = _.max(values, 'length');
            var span = document.createElement('span');
            span.textContent = maxLength;
            span.classList.add('cell-renderer');
            span.style.fontWeight = 'bold';
            this.$grid.append(span);
            var width = span.getBoundingClientRect().width + 5;

            if (column.dataIndex === this.expandableDataIndex) {
                width += 20;
            }

            // TODO: currently just guessing because we don't have max level for all nodes
            if (this.widget.options.tree && this.expandableDataIndex === column.dataIndex) {
                width += 50;
            }

            width = _.max([this.widget.options.sizing.minColumnWidth, width]);

            $(span).remove();

            if (column.displayType === 'timephased') {
                this.viewports['$group-' + group]
                        .find('th').attr('width', width);
                this.timephasedColumnGroup.resizeTimescale(width);
                _.forEach(this.columnDefinitions, function (definition) {
                    if (definition.displayType === 'timephased') {
                        definition.width = width;
                    }
                });
            }

            this.widget.columnDefinitions[column.dataIndex].width = width;
            this.widget.columnDefinitions[column.dataIndex].percent = null;
            this.$grid.find('th[data-index="' + column.dataIndex + '"]').attr('width', width);
            this.widget._trigger('view-updated', null, {
                action : 'column-resize',
                userAction : true,
                column : column.dataIndex,
                size : column.width
            });
            this.getHeaderHeight();
            this.widget.resize();
            this.reselectRecords();
        },

        editSelectedCell : function () {
            var that = this;
            if (that.widget.focus.recordId != null) {
                if (!that.options.multiCellEdit) {
                    that.widget.selectCell();
                }
                // Due to the scrollTop function we need to give a break in execution as it closes the editor immediately otherwise
                setTimeout(function () {
                    that.widget.editRow({
                        recordId : that.widget.focus.recordId,
                        columnId : that.widget.focus.columnId,
                        subRow : that.widget.focus.subRow
                    });
                }, 10);
            }
        },

        getEditor : function (args) {
            var columnDef = this.getCombinedDefinition(args);
            return columnDef.editor;
        },

        getAttribute : function (rowIdentifier, columnIdentifier, column, attr) {
            if (_.includes(constants.COLUMN_ONLY_ATTRS, attr)) {
                return column[attr];
            }

            if (this.options.cells && this.options.cells.getType) {
                var def = this.cellDefinitions[this.options.cells.getType(rowIdentifier, columnIdentifier)];

                if (def != null && def[attr]) {
                    return def[attr];
                }
            }
            return column[attr];
        },

        getCombinedDefinition : function (args) {
            var columnDefintion = args.column;
            if (this.options.cells && this.options.cells.getType) {
                var def = this.cellDefinitions[this.options.cells.getType({
                    recordId : args.recordId,
                    dataIndex : args.column.dataIndex,
                    subRow : args.subRow
                })];

                if (def != null) {
                    return _.assign({}, args.column, _.omit(def, constants.COLUMN_ONLY_ATTRS));
                }
            }
            if (args.group.showStackedCells && !args.column.stackedCellHeader && args.column.useStackedContext) {
                columnDefintion = this.columnDefinitions[args.subRow];
            }
            return columnDefintion;
        },


        /*
         * Open the cell editor, this should only be called from the widget. If you need
         * to open the editor call widget.editRow
         *
         */
        openEditor : function (options) {
            if (this.currentEditor != null) {
                if (this.currentEditor.record === options.record && this.currentEditor.column.dataIndex === options.dataIndex) {
                    this.currentEditor.drawFocusToInput();
                    return;
                }
            }
            this.closeEditor();
            this.closeMenus();
            var that = this;
            var $td = $();

            this.widget.selectCell(options.recordId, options.dataIndex, constants.SHOW_CELL_LOCATION.MIDDLE, this.selectionModel.focus.recordId === options.recordId, options.subRow);

            $td = this.$grid.find('.' + that.options.classes.cellFocused);

            var column = this.getCombinedDefinition({
                recordId : options.recordId,
                dataIndex : options.dataIndex,
                column : that.columnDefinitions[options.dataIndex],
                subRow : options.subRow,
                group : options.group
            });
            var editor = column.editor;

            if (column.enableEditor || options.disabled) {
                // move cell focus
                $td.addClass('editing');

                // hide render cell contents whilst editing
                var $cellRenderer = $td.find('.cell-renderer');
                $cellRenderer.hide();
                var selection = that.widget.selection.recordIds;// get the selection at the time editing begins for multi select
                if (options.recordId === constants.DUMMY_RECORD_ID) {
                    options.record = that.widget._getProxyRecord();
                    options.recordId = options.record[that.widget.options.recordKeys.id];
                    options.fromQuickAddRow = true;
                }
                // create editor cell
                editor.initialize({
                    ariaLabels : that.getCellLabels(options.recordId, options.dataIndex, true),
                    invalid : options.invalid,
                    grid : that,
                    $cell : $td,
                    data : options.data,
                    column : column,
                    record : options.record,
                    disabled : options.disabled,
                    fromQuickAddRow : options.fromQuickAddRow,
                    recordAttributes : options.recordAttributes,
                    onPersistFn : function (persistOptions) {
                        // set the new data
                        persistOptions.hideError = options.hideError;
                        persistOptions.skipUndo = options.hideError;
                        persistOptions.recordAttributes = options.recordAttributes;
                        if (options.group.timephased) {
                            persistOptions.timephased = true;
                            persistOptions.timephasedColumn = that.columnDefinitions[options.dataIndex];
                            persistOptions.masterColumn = options.dataIndex;
                        } else if (options.group.showStackedCells) {
                            persistOptions.stackedCell = true;
                            persistOptions.subRow = options.subRow;
                            persistOptions.masterColumn = options.dataIndex;
                        }

                        if (options.fromQuickAddRow) {
                            if (persistOptions.valid) {
                                that.widget._newRecordCellUpdated({
                                    value : persistOptions.value,
                                    dataIndex : persistOptions.column.dataIndex,
                                    cellUpdatedOptions : persistOptions
                                });
                            }
                            if (persistOptions.quickAdd) {
                                that.widget._commitNewRecord(persistOptions);
                            }
                        } else if (that.options.multiCellEdit && selection.length > 1) {
                            if (selection.length >= that.widget.options.performance.bulkThreshold) {
                                that.widget.processBulkOperation(that.widget._multiCellUpdated.bind(that.widget, selection, persistOptions), []);
                            } else {
                                that.widget._multiCellUpdated(selection, persistOptions);
                            }
                        } else {
                            that.widget._cellUpdated(persistOptions);
                        }
                        this.commitStarted = false;
                        this.commitFinished = true;
                    },
                    onCloseFn : function () {
                        that.$grid.removeClass('cell-multi-edit cell-single-edit');
                        // remove editing class
                        $td.removeClass('editing');

                        // re-show rendered contents
                        $cellRenderer.show();

                        // null out the current editor
                        that.currentEditor = null;
                        that.resetRowHovered();
                        that.widget._trigger('editor-closed', null, {
                            record : options.record,
                            dataIndex : options.dataIndex
                        });
                        // draw focus back to the grid for continued navigation, settimeout for firefox.
                        that.widget.timeouts.editorCloseTimeout = setTimeout(function () {
                            that.widget.requestBrowserFocus(true);
                        }, 1);
                    }
                });

                // save the current editor so we can close it later
                that.currentEditor = editor;
                if (that.widget._getSelectionAny().length > 1) {
                    that.$grid.addClass('cell-multi-edit');
                } else if (that.widget._getSelectionAny().length === 1) {
                    that.$grid.addClass('cell-single-edit');
                }
                var startString = options.currentlyTyping ? that.currentlyTyping : null;
                this.currentlyTyping = '';
                editor.show(startString);
                editor.drawFocusToInput(options.selectText, startString);
            }
        },


        /*
         * Close the editor with a commit, or pass true to escape changes
         */
        closeEditor : function (forceClose, args) {
            if (!_.isObject(args)) {
                args = {};
            }
            if (forceClose && this.currentEditor != null) {
                this.currentEditor.close();
            }

            if (this.currentEditor != null) {
                // passing true tells the editor to still close even if validation is not correct
                // value is not persisted and no error message is displayed
                if (!this.currentEditor.getCommitStarted()) {
                    return this.currentEditor.commit({ quickAdd : args.quickAdd });
                }
            }

            return true;
        },

        closeMenus : function (args) {
            var menuClosed = false;
            // column menu
            if (this.columnMenu.isOpen()) {
                this.columnMenu.destroy(args); // disowns own menu
                menuClosed = true;
            }

            if (this.columnGroupMenu.isOpen()) {
                this.columnGroupMenu.destroy(args); // disowns own menu
                menuClosed = true;
            }

            if (this.contextMenu != null && this.contextMenu.isOpen) {
                this.contextMenu.close(args);
                this._disownMenu(this.componentIds.menu);
                menuClosed = true;
            }

            if (this.$errorBox.parent().length) {
                this.$errorBox.remove();
                this._disownMenu(this.componentIds.errorPopup);
                menuClosed = true;
            }
            return menuClosed;
        },

        clearClipboardStyles : function () {
            var stylesCleared = !(this.clearCutCopyCells() === 0 && this.clearCutCopyRows() === 0);
            this.widget.clipboard.hideStyling = true;
            return stylesCleared;
        },

        _hasColumnGroupMenu : function (group) {
            if (!this.options.columnGroupMenu.enabled) {
                return false;
            }
            var hasAnyOptions = this.options.menus.canHideColumns; // || ...other options, like renaming group?...
            if (!hasAnyOptions) {
                return false;
            }
            return !!group.hasColumnGroupMenu;
        },

        _establishOwnedMenu : function (menuId) {
            this.$grid.attr('aria-owns', menuId);
            this.$grid.attr('aria-activedescendant', menuId);
        },
        _disownMenu : function (id) {
            var currentOwns = (this.$grid.attr('aria-owns') || '').split(' ');
            var newOwns = _.difference(currentOwns, [id]).join(' ').trim();

            if (newOwns === '') {
                this.$grid.removeAttr('aria-owns');
            } else {
                this.$grid.attr('aria-owns', newOwns);
            }

            // assuming there'll only be one, relatively uncommon anyway
            if (this.$grid.attr('aria-activedescendant') === id) {
                this.$grid.removeAttr('aria-activedescendant');
            }
        },

        _openColumnGroupMenuEvent : function (ev) {
            var $menuOpener = $(ev.target).closest('.column-group-menu');
            var $viewport = $menuOpener.closest('.viewport.viewport-headers');
            var group = this._getGroup($viewport.attr('data-group'));
            return this.openColumnGroupMenu(group);
        },
        openColumnGroupMenu : function (group, fromKeyboard) {
            this.closeMenus();

            if (!this._hasColumnGroupMenu(group)) {
                return false;
            }
            var menuId = this.componentIds.columnGroupMenu;
            var $menuOpener = this.viewports['$rowset-headers'].filter('[data-group="' + group.id + '"]').find('.column-group-menu');
            var $menu = this.columnGroupMenu.render({
                fromKeyboard : fromKeyboard,
                $opener : $menuOpener,
                id : menuId,
                group : group
            });
            if ($menu) {
                this._establishOwnedMenu(menuId);
                return true;
            }
            return false;
        },

        openColumnMenu : function (focus, event, fromKeyboard) {
            if (!this.options.columnMenu.enabled) {
                return false;
            }

            this.closeMenus();
            var columnMenuId = this.componentIds.columnMenu;

            // LUX-276, focus was not on DOM when the 'space' key was pressed second time to reopen the menu, so find it again, focus is used for positioning the menu on DOM
            focus = $('#' + focus.attr('id'));

            var $menu = this.columnMenu.render(focus, event, fromKeyboard);
            if ($menu === 'fromKeyboardTwice') {
                return true;
            }
            if ($menu) {
                focus.addClass('pgbu-grid-column-menu-open');
                this._establishOwnedMenu(columnMenuId);
                $menu.attr('id', columnMenuId);
                return true;
            }
            return false;
        },

        openContextMenu : attrFns.decorateCache('_attrs', 'gridView#openContextMenu', function openContextMenu(args) {
            if (!this.options.contextMenu.enabled) {
                return;
            }
            if (args.recordId == null && args.record == null) {
                return;
            }

            var that = this;
            that.closeMenus();
            this.resetRowHovered();
            if (args.recordId == null && args.record !== null) {
                args.recordId = this.widget._getRecordId(args.record);
            }
            if (args.record == null) {
                args.record = this.widget._getRecord(args.recordId);
            }
            var disableRowMenu = this.widget._attrs.all(args.record).disableRowMenu;

            if (!this.options.contextMenu.allowDisjointSelection && !_.includes(this.widget.getSelection(), args.record)) {
                this.widget.selectRows([args.recordId]);
            }

            var $virtualRow = this.buildVirtualRow(args.recordId);
            if (!args.openedFromCell) {
                $virtualRow.addClass('row-menu-open');
                this.contextMenu.onClose(function () {
                    $virtualRow.removeClass('row-menu-open');
                    that.focusCellSelection();
                });
            }

            var internalDisplayOnly = that.getAttribute(args.recordId, that.widget.focus.columnId, that.columnDefinitions[that.widget.focus.columnId], 'internalDisplayOnly');
            var isJoinedIconColumn = args.$relativeElement && args.$relativeElement.hasClass('joined-icon-column');
            this.contextMenu.setOpenedFromCell((args.openedFromCell && (!internalDisplayOnly || isJoinedIconColumn)) || false);
            this.contextMenu.setContext(args.record);
            if (args.$relativeElement == null || !args.$relativeElement.is('td')) {
                if (this.contextMenu.openedFromCell) {
                    args.$relativeElement = $virtualRow.find('td[data-index="' + args.dataIndex + '"]');
                } else {
                    args.$relativeElement = $virtualRow.find('.row-menu-button');
                }
            }

            if (!disableRowMenu) {
                this.contextMenu.show(args.$relativeElement, args.offsetX);
                if (args.fromKeyboard) {
                    this.contextMenu.focusFirstItem();
                }
            }
            this.$grid.attr('aria-activedescendant', this.componentIds.menu);
            this.resetRowHovered();
        }),

        resetRowHovered : function () {
            this.$currentHoveredVirtualRow.removeClass(this.options.classes.rowHovered);
            this.$grid.removeClass('grid-state-row-hovered');
            this.$currentHoveredVirtualRow = $();
            this.currentHoverId = null;
        },

        /*
         * Build a virtual row selection to work with subrows split across all groups
         */
        buildVirtualRow : function (recordId, rowsetId) {
            if (rowsetId == null) {
                rowsetId = this.widget._getRecordRowset(recordId);
            }
            return this.viewports['$rowset-' + rowsetId].find('tbody tr[data-id="' + recordId + '"]');
        },

        clearCutCopyCells : function () {
            var cells = this.$grid.find('.cell-copied, .cell-cut');
            this.$grid.find('.cell-copied').removeClass('cell-copied cell-copied-top cell-copied-bottom');
            this.$grid.find('.cell-cut').removeClass('cell-cut cell-cut-top cell-cut-bottom');

            return cells.length;
        },

        clearCutCopyRows : function () {
            var rows = this.$grid.find('.row-cut, .row-copied');
            this.$grid.find('.row-cut').removeClass('row-cut row-cut-top row-cut-bottom');
            this.$grid.find('.row-copied').removeClass('row-copied row-copied-top row-copied-bottom');

            return rows.length;
        },

        /*
         * used by cut / copy record
         */
        applyCutCopyClassesToRecords : function (recordIds) {
            this.clearCutCopyCells();
            this.clearCutCopyRows();
            var that = this;
            var scope = that.widget.clipboard.scope;
            var dataIndex = that.widget.clipboard.cellIndex;
            _.forEach(recordIds, function (recordId) {
                var rowsetId = that.widget._getRecordRowset(recordId);
                var virtualRows = that.virtualRows[rowsetId];
                var recordIndex = that.getRecordIndex(recordId, rowsetId);
                if (recordIndex >= virtualRows.displayTop && recordIndex <= virtualRows.displayBottom) {
                    var classes = that.widget._attrs.all(that.widget._getRecord(recordId)).cutCopyClasses;
                    var virtualRow = that.buildVirtualRow(recordId, rowsetId);
                    if (scope === 'row') {
                        virtualRow.addClass(classes);
                    } else {
                        virtualRow.find('td[data-index="' + dataIndex + '"]').addClass(classes);
                    }
                }
            });
        },

        clearSelection : function () {
            var that = this;
            //>>excludeStart("perf", pragmas.compiled)
            var start;
            if (this.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            var $selectedRows = this.$grid.find('.' + this.options.classes.rowSelected).removeClass(this.options.classes.rowSelected + this.options.classes.rowSelectedTop + this.options.classes.rowSelectedBottom);
            $selectedRows.find('.' + this.options.classes.cellSelected).removeClass(this.options.classes.cellSelected + this.options.classes.cellSelectedTop + this.options.classes.cellSelectedBottom);
            this.$grid.find('.' + this.options.classes.rowFocused).removeClass(this.options.classes.rowFocused);
            this.$grid.find('.cell-selection-overlay').remove();
            this._clearColumnGroupActive();
            this.widget._trigger('view-sync-clear-selection', null, {});
            //>>excludeStart("perf", pragmas.compiled)
            if (this.options.debug) {
                that.widget._trigger('performance-debug', null, {
                    start : start,
                    end : window.performance.now(),
                    name : 'clear-selection',
                    type : 'view'
                });
            }
            //>>excludeEnd("perf")
        },


        clearChecked : function () {
            this.setHeaderCheckbox(constants.ROW_SELECTOR, false);
            this.$grid.find('td[data-index="row-selector"] input').prop('checked', false);
            this.$grid.find('.row-checked').removeClass('row-checked');
        },

        focusRow : function (recordId, rowsetId, columnId) {
            this.$grid.find('.' + this.options.classes.rowFocused).removeClass(this.options.classes.rowFocused);
            var virtualRow = this.buildVirtualRow(recordId, rowsetId);
            virtualRow.addClass(this.options.classes.rowFocused);
            this.selectionModel.cell.focusCell(virtualRow.find('td[data-index="' + columnId + ['"]']));
            this.reselectRecords({ skipFocus : true });
        },

        selectRow : function (args) {
            this.widget._trigger('view-sync-select-row', null, { recordId : args.recordId });
            if (!args.virtualRow.length) {
                return;
            }
            args.virtualRow.addClass(args.attributes.rowSelectedClasses);
            var cellFind = 'td[data-index="' + args.dataIndex + '"]';
            if (args.subRow != null) {
                cellFind += '[data-sub-row="' + args.subRow + '"]';
            }
            var $td = args.virtualRow.find(cellFind);
            $td.addClass(args.attributes.cellSelectedClasses);
            if (args.focus && $td.length) {
                this.$grid.find('.' + this.options.classes.rowFocused).removeClass(this.options.classes.rowFocused);
                args.virtualRow.addClass(this.options.classes.rowFocused);
                this.selectionModel.cell.focusCell($td);
            }
        },

        selectCell : function (args) {
            var viewport = this.viewports[args.rowsetId][args.groupId];
            var left = args.columnDefinition.positionLeft;
            var width = args.columnDefinition.renderedWidth;
            var iSGroupRecord = this.widget._isGroupRecordId(args.recordId);
            if (args.columnDefinition.colSpan > 1 && iSGroupRecord) {
                width = this.selectionModel.focus.$td.width();
            }

            if (this.expandableDataIndex === args.columnDefinition.dataIndex && (this.options.tree || this.widget.groupingApplied)) {
                var node = args.virtualRows.nodes[args.recordId];
                var reduceBy = 0;
                if (node != null) {
                    switch (this.options.theme.style) {
                    case GROUPING_STYLE.MODERN_BANDS.key :
                        reduceBy = (node.level) * 13 + 15;
                        if (!args.attributes.isNode) {
                            reduceBy -= 13;
                        }
                        break;
                    case GROUPING_STYLE.MODERN_NOTCHES.key:
                    case GROUPING_STYLE.CLASSIC_BANDS.key :
                        reduceBy = (node.level + 1) * 7;
                        if (!args.attributes.isNode) {
                            reduceBy -= 7;
                        }
                        break;
                    }
                }
                width -= reduceBy;
                left += reduceBy;
            }
            var classes = [args.attributes.cellSelectedClasses];
            if (this.selectionModel.focus.pivotId != null ? (this.selectionModel.focus.pivotId === args.recordId) : args.focus) {
                classes.push(this.options.classes.cellFullSelection);
            }
            if (this.widget.canEditCell(args.recordId, args.dataIndex, args.attributes, args.subRow) !== true) {
                classes.push('cell-selection-no-action');
            }
            if (args.attributes.invalid != null && args.attributes.invalid[args.dataIndex] != null && _.some(_.values(args.attributes.invalid[args.dataIndex]), function (error) {
                return !error.hidden;
            })) {
                var allWarnings = _.every(args.attributes.invalid[args.dataIndex], function (error) {
                    return error.warning;
                });

                if (allWarnings) {
                    classes.push('cell-selection-warning');
                } else {
                    classes.push('cell-selection-error');
                }
            }
            var height = this.options.layout.rowHeight + 1;
            var top = args.recordIndex * this.options.layout.rowHeight + (args.virtualRows.headerRow ? this.options.layout.rowHeight + 1 : 0);
            var filldownEnabled = args.virtualRows.records.length > 1 && !iSGroupRecord && this.widget._canFillDown({
                recordId : args.recordId,
                record : this.widget._getRecord(args.recordId),
                columnDefinition: args.columnDefinition
            });
            if (args.subRow != null && viewport.groupConfig.stackedCells) {
                height = Math.floor(this.ROW_HEIGHT / viewport.groupConfig.stackedCells.length);
                top = top + ((height) * this.selectionModel.focus.subRowPosition) + (this.options.borders.row / 2);
                filldownEnabled = false;
            }
            var $cellSelection = cellSelectionTemplate({
                filldown : filldownEnabled,
                focused : args.focus,
                recordId : args.recordId,
                dataIndex : args.dataIndex,
                width : width,
                top : top,
                left : left,
                height : height,
                classes : classes.join(' ')
            });
            viewport.$overlay.append($cellSelection);
        },

        reselectRecords : function (args) {
            var that = this, recordId, rowsetId, virtualRows, recordIndex, attributes, focus, i;
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (this.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            args = args || {};
            var $selectedRows = this.$grid.find('.' + this.options.classes.rowSelected);
            $selectedRows.removeClass(this.options.classes.rowSelected + this.options.classes.rowSelectedTop + this.options.classes.rowSelectedBottom);
            $selectedRows.find('.' + this.options.classes.cellSelected).removeClass(this.options.classes.cellSelected + this.options.classes.cellSelectedTop + this.options.classes.cellSelectedBottom);
            this.$grid.find('.cell-selection-overlay').remove();
            var selectedRecordIds = _.clone(this.widget.selection.recordIds);
            if (this.widget.focus.recordId && !_.includes(selectedRecordIds, this.widget.focus.recordId)) {
                selectedRecordIds.push(this.widget.focus.recordId);
            }

            for (i = 0; i < this.config.rowsets.length; i++) {
                rowsetId = this.config.rowsets[i].id;
                virtualRows = this.virtualRows[rowsetId];
                for (var j = virtualRows.displayTop; j <= virtualRows.displayBottom; j++) {
                    recordId = virtualRows.records[j];
                    recordIndex = j;
                    //>>excludeStart("perf", pragmas.production)
                    var start2;
                    if (this.options.debug) {
                        start2 = window.performance.now();
                    }
                    //>>excludeEnd("perf")
                    var contains = _.includes(selectedRecordIds, recordId);
                    //>>excludeStart("perf", pragmas.production)
                    if (this.options.debug) {
                        that.widget._trigger('performance-debug', null,
                            {
                                start : start2,
                                end : window.performance.now(),
                                name : 'reselect-contains',
                                type : 'view'
                            });
                    }
                    //>>excludeEnd("perf")
                    if (contains) {
                        focus = this.widget.focus.recordId === recordId;
                        attributes = this.widget._attrs.all(this.widget._getRecord(recordId));
                        this.selectRow(
                            {
                                virtualRow : this.buildVirtualRow(recordId, rowsetId),
                                dataIndex : this.widget.focus.columnId,
                                focus : args.skipFocus ? false : focus,
                                recordId : recordId,
                                subRow : this.widget.focus.subRow,
                                attributes : attributes
                            });

                        this.selectCell({
                            virtualRows : virtualRows,
                            rowsetId : rowsetId,
                            focus : focus,
                            recordId : recordId,
                            dataIndex : this.widget.focus.columnId,
                            recordIndex : recordIndex,
                            columnDefinition : this.columnDefinitions[this.widget.focus.columnId],
                            groupId : this.columnDefinitions[this.widget.focus.columnId].groupId,
                            attributes : attributes,
                            subRow : this.widget.focus.subRow
                        });
                        if (!args.skipFocus && this.widget.focus.record && this.widget._getRecordId(this.widget.focus.record) === recordId) {
                            this.selectionModel.cell._updateFocusModelFromIds(recordId, this.widget.focus.columnId, this.widget.focus.subRow);
                        }
                    }
                }
            }

            //>>excludeStart("perf", pragmas.production)
            if (that.options.debug) {
                that.widget._trigger('performance-debug', null,
                    {
                        start : start,
                        end : window.performance.now(),
                        name : 'reselect-records',
                        type : 'view'
                    });
            }
            //>>excludeEnd("perf")
        },


        checkRow : function (virtualRow, check) {
            if (check) {
                virtualRow.addClass('row-checked');
            } else if (virtualRow.is('.row-checked')) {
                virtualRow.removeClass('row-checked');
            }
            virtualRow.find('td[data-index="row-selector"] input').prop('checked', check);
            // Issue: LUX 230 , SELECTOR COLUMN:  CHECK BOX STATE NOT ACCESSIBLE, screen reader does not read the state change.
            virtualRow.find('td[data-index="row-selector"]').attr({ 'title' : check ? this.options.locale.selected : this.options.locale.unselected });
        },

        checkRecords : function (recordIds, check) {
            var that = this;
            _.forEach(recordIds, function (recordId) {
                var rowsetId = that.widget._getRecordRowset(recordId);
                var virtualRows = that.virtualRows[rowsetId];
                var recordIndex = that.getRecordIndex(recordId, rowsetId);
                if (recordIndex >= virtualRows.displayTop && recordIndex <= virtualRows.displayBottom) {
                    that.checkRow(that.buildVirtualRow(recordId), check);
                }
            });
        },

        refocusHeader : function (group, column) {
            this.requestFocus();
            if (column == null) {
                column = _.first(this._visibleColumns(group));
            }
            this.widget.selectHeaderCell(column.dataIndex);
        },
        _visibleColumns : function (group) {
            return _.filter(group.columns, { visible : true });
        },
        selectHeaderCell : function (columnId) {
            var $headers = this.viewports['$rowset-headers'];
            var column = this.columnDefinitions[columnId];
            var $viewport = $headers.filter('[data-group="' + column.groupId + '"]');
            $viewport.addClass('group-active');
            this.selectionModel.cell.selectHeader($headers.find('thead tr'), columnId);
            this.scrollFocusIntoView();
        },
        _clearColumnGroupActive : function () {
            var $rowHeader = this.viewports['$rowset-headers'];
            if ($rowHeader) {
                $rowHeader.removeClass('group-active');
            }
        },

        selectColumnFilterCell : function (columnId) {       // LUX-506
            var filterRowsetId = this.options.advancedFilters.enabled ? constants.ADVANCED_FILTER_ROWSET_ID : constants.COLUMNFILTER_ROWSET_ID;
            this.selectionModel.cell.selectColumnFilter(this.viewports['$rowset-' + filterRowsetId].find('thead tr'), columnId);
            this.scrollFocusIntoView();
        },

        moveRecord : function (recordId, location, oldRowset, recordAttributes) {
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (this.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            var that = this;
            // delete the records from the view and flat list
            that.deleteRecords([recordId], true, oldRowset);
            // Update the node settings for the record if this is a tree grid
            if (this.options.tree) {
                that.updateNode(recordId, location, recordAttributes, false);
            }

            // Readd the record to the and virtualrow list
            that.addRecord({ recordId : recordId, location : location, select : false });
            //>>excludeStart("perf", pragmas.production)
            if (that.options.debug) {
                that.widget._trigger('performance-debug', null, {
                    start : start,
                    end : window.performance.now(),
                    name : 'move-Records',
                    type : 'view',
                    info : recordId
                });
            }
            //>>excludeEnd("perf")
        },

        deleteRecords : function (recordIds, move, oldRowset, fromCollapse) {
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (this.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            var that = this;
            var modifiedRowsets = {};
            var reselect = false;

            if (that.options.tree && !fromCollapse) {
                var childIds = [];
                // If this is a tree grid and this is a legitemate delete we need to remove all children as well
                _.forEach(recordIds, function (recordId) {
                    var rowsetId = oldRowset != null ? oldRowset : that.widget._getRecordRowset(recordId);
                    var virtualRows = that.virtualRows[rowsetId];
                    var record = virtualRows.nodes[recordId];
                    // If parent and child are both in the clipboard and the parent is moved before the child the parent
                    // will collapse removing the children from virtual rows. Check to make sure it exists
                    if (record && record.expanded) {
                        childIds = childIds.concat(that.getChildIds(recordId, rowsetId));
                    }
                    that.deleteChildNodes(recordId, virtualRows);
                    // Clean up our nodes to they stay synced properly
                    delete virtualRows.nodes[recordId];
                });
                recordIds = recordIds.concat(childIds);
                recordIds = _.uniq(recordIds);
            }
            if (fromCollapse) {
                var recordId = recordIds[0];
                var rowsetId = oldRowset != null ? oldRowset : that.widget._getRecordRowset(recordId);
                var virtualRows = that.virtualRows[rowsetId];
                var top = virtualRows.displayTop;
                var bottom = virtualRows.displayBottom;
                for (var i = top; i <= bottom; i++) {
                    if (_.includes(recordIds, virtualRows.records[i])) {
                        that.deleteRow(virtualRows.records[i], rowsetId);
                        modifiedRowsets[rowsetId] = true;
                        virtualRows.renderedBottom--;
                        virtualRows.displayBottom--;
                    }
                }
                var recordIndex = that.getRecordIndex(recordId, rowsetId);
                var lastRecordId = _.last(recordIds);
                var lastChildRecordId = this.lastChild(lastRecordId, rowsetId);
                if (lastChildRecordId != null) {
                    lastRecordId = lastChildRecordId;
                }
                virtualRows.records.splice(recordIndex, _.indexOf(virtualRows.records, lastRecordId) - recordIndex + 1);
            } else {
                // Iterate through and delete the rows from the view. Adjust render values appropriately
                _.forEach(recordIds, function (recordId) {
                    var rowsetId = oldRowset != null ? oldRowset : that.widget._getRecordRowset(recordId);
                    var virtualRows = that.virtualRows[rowsetId];
                    var recordIndex = that.getRecordIndex(recordId, rowsetId);
                    if (recordIndex !== -1) {
                        if (!modifiedRowsets[rowsetId]) {
                            modifiedRowsets[rowsetId] = false;
                        }
                        if (recordIndex < virtualRows.displayTop) {
                            virtualRows.displayTop--;
                            virtualRows.renderedTop--;
                        } else if (recordIndex >= virtualRows.displayTop && recordIndex <= virtualRows.displayBottom) {
                            that.deleteRow(recordId, rowsetId);
                            virtualRows.renderedBottom--;
                            virtualRows.displayBottom--;
                            modifiedRowsets[rowsetId] = true;
                        }
                        // Remove the record from the flat list of visible items
                        virtualRows.records.splice(recordIndex, 1);
                        // If this record is selected we need to select the record below
                        if (that.selectionModel.focus.recordId === recordId) {
                            reselect = true;
                        }
                    }
                });
            }

            // Rerender the rowsets and reselect if required
            _.forEach(modifiedRowsets, function (isModified, rowsetId) {
                var virtualRows = that.virtualRows[rowsetId];
                if (isModified) {
                    if (virtualRows.displayTop < 0) {
                        virtualRows.displayTop = 0;
                    }
                    // Try and force it to fill the viewport
                    if (virtualRows.displayTop + virtualRows.visibleRows > virtualRows.records.length) {
                        virtualRows.displayTop = virtualRows.records.length - virtualRows.visibleRows;
                    }
                    that.renderRowset(rowsetId, { fromDelete : true });
                }
                // Select the record below if this is not part of a move action.
                if (that.selectionModel.focus.rowsetId === rowsetId) {
                    if (reselect && !move) {
                        var indexToSelect = that.selectionModel.focus.recordIndex;
                        if (indexToSelect >= virtualRows.records.length) {
                            indexToSelect = virtualRows.records.length - 1;
                        }
                        if (indexToSelect >= 0) {
                            var newSelectionId = virtualRows.records[indexToSelect];
                            if (newSelectionId === constants.DUMMY_RECORD_ID && indexToSelect > 0) {
                                newSelectionId = virtualRows.records[indexToSelect - 1];
                            }

                            that.widget.selectCell(newSelectionId);
                        } else {
                            that.widget._clearRowSelection();
                        }
                    }
                }
            });

            this.updateRecordCount();

            //>>excludeStart("perf", pragmas.production)
            if (this.options.debug) {
                this.widget._trigger('performance-debug', null,
                    {
                        start : start,
                        end : window.performance.now(),
                        name : 'delete-Records',
                        type : 'view',
                        info : recordIds
                    });
            }
            //>>excludeEnd("perf")
        },

        deleteRow : function (recordId, rowsetId) {
            var $virtualRow = this.buildVirtualRow(recordId, rowsetId);
            $virtualRow.remove();
            this.widget._trigger('view-sync-remove-row', null, { recordId : recordId });
        },


        attachRows : function (rowsetId, tableSection, rows, action, actionIndex, rowsRendered) {
            var that = this;

            _.forEach(that.config.groups, function (group) {
                var $tableSection = that.viewports[rowsetId][group.id][tableSection];

                switch (action) {
                case 'replace' :
                    $tableSection.empty();
                    break;
                case 'append' :
                    $tableSection.find('tr:lt(' + actionIndex * group.placementModifier + ')').remove();
                    break;
                case 'prepend' :
                    $tableSection.find('tr:gt(' + (actionIndex * group.placementModifier + group.placementModifier - 1) + ')').remove();
                    break;
                case 'insert' :
                    // Remove from the end the number you are trying to add, must work or further pruning breaks;
                    $tableSection.find('tr:gt(' + ((rowsRendered * group.placementModifier) - 1) + ')').remove();
                    break;
                case 'insert-skip-delete':
                    // We have to do this after
                    break;
                }
            });


            _.forEach(that.config.groups, function (group) {
                var rowMarkup = _.pluck(rows, group.id).join(' ');
                switch (action) {
                case 'replace' :
                case 'append' :
                    that.viewports[rowsetId][group.id][tableSection].append(rowMarkup);
                    break;
                case 'prepend' :
                    that.viewports[rowsetId][group.id][tableSection].prepend(rowMarkup);
                    break;
                case 'insert-skip-delete':
                case 'insert' :
                    that.viewports[rowsetId][group.id][tableSection].children().eq(actionIndex * group.placementModifier + group.placementModifier - 1).after(rowMarkup);
                    break;
                }
            });

            this.widget._trigger('view-sync-attach-rows', null, {
                action : action,
                actionIndex : actionIndex,
                rows : _.pluck(rows, 'RECORD_ID')

            });
        },

        getPrintContent : function (options) {
            if (options == null) {
                options = {};
            }
            options = _.defaults({}, options, {
                ROW_LIMIT : 2000,
                rowCount : 0,
                rowsets : _.pluck(this.config.rowsets, 'id')
            });
            var that = this;
            var $printContent = $('<div></div>');
            var multiLine = this.options.printMultiline ? 'multi-line' : '';
            $printContent.append('<div class="scrolly-grid printable ' + multiLine + '" tabIndex="0"></grid>');
            var $grid = $printContent.find('.scrolly-grid');
            if (this.options.showVerticalLines) {
                $grid.addClass('vertical-lined-grid');
            }

            var print = {
                $rowsets : {},
                viewports : {}
            };
            var excelRowData = {
                rows : []
            };

            var $scrollers = $();
            _.forEach(that.config.rowsets, function (rowset) {
                if (!options.forExport && (rowset.id === constants.HEADERS_ROWSET_ID || rowset.id === constants.QUICK_ADD_ROWSET_ID)) {
                    return;
                }
                print.viewports['$rowset-' + rowset.id] = $();
                print.viewports[rowset.id] = {};

                var $rowset;
                if (rowset.type === constants.ROWSET_TYPE.FIXED || rowset.type === constants.ROWSET_TYPE.MANUAL) {
                    $rowset = $(rowsetTemplate(rowset));
                } else {
                    $rowset = $(rowsetTemplate(rowset));
                    // Create scrollers and save references to them
                    $scrollers = $scrollers.add($(pseudoScrollerTemplate({ type : 'vertical', id : rowset.id })));
                }
                // store all rowset options
                $rowset.data('rowset', rowset);
                $rowset.find('.rowset-overlay').remove();
                if (rowset.type === constants.ROWSET_TYPE.MANUAL) {
                    rowset.insertDom($rowset, rowset.id);
                }
                print.$rowsets[rowset.id] = $rowset;
                $grid.append($rowset);
                var viewport = {};
                viewport.type = constants.GROUP_TYPE.FIXED;
                viewport.group = 'print';
                viewport.rowset = rowset.id;
                viewport.viewportId = that.widget.uuid + 'grid-viewport-rowset-' + rowset.id + '-group-print';
                viewport.menuText = '';
                viewport.$viewport = $(viewportTemplate(viewport));
                viewport.$table = viewport.$viewport.find('table');
                viewport.$tbody = viewport.$table.find('tbody');
                viewport.tbody = '';
                viewport.$thead = viewport.$table.find('thead');
                viewport.thead = [];
                viewport.$tfoot = viewport.$table.find('tfoot');
                viewport.tfoot = [];
                print.viewports[rowset.id].print = viewport;

                if (rowset.type !== constants.ROWSET_TYPE.MANUAL) {
                    $rowset.append(viewport.$viewport);
                }
            });

            var first = true;
            var firstRowset = true;
            // loop through each group so we can get the assigned columns
            _.forEach(that.config.rowsets, function (rowset) {
                if (!options.forExport && (rowset.id === constants.HEADERS_ROWSET_ID || rowset.id === constants.QUICK_ADD_ROWSET_ID)) {
                    return;
                }
                var viewport = print.viewports[rowset.id].print;
                viewport.$headerRow = $('<tr></tr>').appendTo(viewport.$thead);
                _.forEach(that.config.groups, function (group) {
                    // loop through columns for the group otherwise add header shims into other rowsets
                    _.forEach(group.columns, function (column) {
                        if (column.visible && (!column.internalDisplayOnly || (!options.forExport && column.displayType === 'joinedIcon'))) {
                            if (first && column.dataIndex !== constants.ROW_SELECTOR) {
                                that.firstVisibleColumn = column.dataIndex;
                                first = false;
                            }

                            var $col = that.createColumn(column, (options.forExport || !firstRowset) ? rowset.id : constants.HEADERS_ROWSET_ID);
                            if ($col.hasClass('timephased-column')) {
                                $col.find('.cell-renderer')
                                        .attr('style', function (i, style) {
                                            return (style || '') + 'white-space: normal !important;';
                                        })
                                        .text($col.attr('title'));
                            }
                            viewport.$headerRow.append($col);

                            if (column.dataIndex !== constants.EMPTY_COLUMN) {
                                that.lastVisibleColumn = column.dataIndex;
                            }
                        }
                    });
                });
                firstRowset = false;
            });

            if (options.forExport && options.rowCount === 0) {
                // rowCount = 2: 1 for headers, 1 for 1-index rather than 0-index
                options.rowCount = 2;
            }

            _.forEach(options.rowsets, function (rowsetId) {
                if (rowsetId === constants.QUICK_ADD_ROWSET_ID || (!options.forExport && rowsetId === constants.HEADERS_ROWSET_ID)) {
                    return;
                }

                options.rowsetId = rowsetId;
                options.rowsetStart = 0;
                if (options.forExport) {
                    var rows = that.getExcelRowData(options);
                    Array.prototype.push.apply(excelRowData.rows, rows);
                } else {
                    var rowsetRows = that.getPrintRows(options).join(' ');
                    print.viewports[rowsetId].print.$tbody.append(rowsetRows);
                }
            });

            if (options.forExport) {
                var headers = {
                    attrs : 'x14ac:dyDescent="0.25"',
                    cells : []
                };
                var columnNum = 1;
                print.viewports.headers.print.$thead.find('th').each(function () {
                    var $this = $(this);
                    if ($this.attr('data-index') !== constants.MENU_BUTTON_COLUMN) {
                        var cell = {
                            name : exporter.getExcelCellName(1, columnNum),
                            inline : true,
                            wrapText : true
                        };

                        if ($this.hasClass('timephased-column')) {
                            cell.value = $this.attr('title');
                        } else {
                            cell.value = _.escape($this[0].innerText);
                        }

                        headers.cells.push(cell);
                        columnNum++;
                    }
                });
                excelRowData.headers = headers;
                return excelRowData;
            } else {
                $printContent.prepend($(window.document.head).html());

                return $printContent;
            }
        },

        getExcelRowData : function (options) {
            var that = this;
            var rows = [];

            for (var i = options.rowsetStart; i < that.virtualRows[options.rowsetId].records.length; i++) {
                if (options.rowCount < options.ROW_LIMIT) {
                    var recordId = that.virtualRows[options.rowsetId].records[i];
                    var excelRows = that.makeExcelRow(recordId, options.rowsetId, options.rowCount);
                    Array.prototype.push.apply(rows, excelRows);
                    options.rowCount += excelRows.length;
                }
            }

            return rows;
        },

        getPrintRows : function (options) {
            var that = this;
            var printRows = [];

            for (var i = options.rowsetStart; i < that.virtualRows[options.rowsetId].records.length; i++) {
                if (options.rowCount < options.ROW_LIMIT) {
                    var recordId = that.virtualRows[options.rowsetId].records[i];
                    var rows = that.makePrintRow(recordId, options.rowsetId);
                    Array.prototype.push.apply(printRows, rows);
                    options.rowCount++;
                }
            }

            return printRows;
        },

        openPrintView : function (skipPrint) {
            var printPopupWindow = window.open('', '', 'scrollbars=1,left=20,top=20,width=1024,height=768');
            printPopupWindow.document.write('<!DOCTYPE html>' + this.getPrintContent().html());
            printPopupWindow.addEventListener('load', function () {
                if (!skipPrint) {
                    printPopupWindow.print();
                }
            }, false);
            printPopupWindow.document.close();
            printPopupWindow.focus();
        },

        beginExport : function (options) {
            if (options == null) {
                options = {};
            }
            this.export = {
                rowsets : _.pluck(this.config.rowsets, 'id'),
                currentRowsetIndex : 0,
                currentRecordIndex : 0,
                chunkSize : options.chunkSize || 250,
                exportSize : _.reduce(_.values(this.virtualRows), function (memo, virtualRowset) {
                    return virtualRowset.records.length + memo;
                }, 0)
            };
            this.export.data = {
                totalChunks : Math.ceil(this.export.exportSize / this.export.chunkSize),
                loadedChunks : 0
            };
        },

        endExport : function () {
            this.export = null;
        },

        getExportChunk : function () {
            var chunkCount = 0;
            var rows = [];
            for (var i = this.export.currentRowsetIndex; i < this.export.rowsets.length; i++) {
                var chunkRowData = this.getExcelRowData({
                    rowsetId : this.export.rowsets[i],
                    rowsetStart : this.export.currentRecordIndex,
                    rowCount : chunkCount,
                    ROW_LIMIT : this.export.chunkSize
                });
                chunkCount += chunkRowData.length;
                Array.prototype.push.apply(rows, chunkRowData);
                if (chunkCount < this.export.chunkSize) {
                    this.export.currentRecordIndex = 0;
                    this.export.currentRowsetIndex = i + 1;
                } else {
                    this.export.currentRecordIndex = this.export.currentRecordIndex + chunkRowData.length;
                }
            }
            this.export.data.loadedChunks++;
            this.export.data.percentLoaded = this.export.data.loadedChunks / this.export.data.totalChunks;
            delete this.export.data.currentChunk;
            this.export.data.currentChunk = rows;
            return this.export.data;
        },

        resetExport : function () {
            this.export = null;
        },

        exportToExcel : function () {
            var excelData = this.getPrintContent({ forExport : true, ROW_LIMIT : 1000000 });
            exporter.toExcel(excelData, this);
        },

        exportToCSV : function () {
            var exportData = this.getPrintContent({ forExport : true, ROW_LIMIT : 1000000 });
            exporter.toCSV(exportData, this.options.export.fileName);
        },

        updateRow : function (recordId, rowsetId) {
            var virtualRows = this.virtualRows[rowsetId];
            var recordIndex = this.getRecordIndex(recordId, rowsetId);
            if (recordIndex < virtualRows.displayTop || recordIndex > virtualRows.displayBottom) {
                return;
            }

            var oldRow = this.buildVirtualRow(recordId, rowsetId);
            if (!oldRow.length) {
                return;
            }
            var newRow = this.makeRow(recordId, rowsetId);
            var that = this;
            _.forEach(newRow, function (rowString, groupId) {
                if (groupId === 'RECORD_ID') { // Ignore gantt syncing call
                    return;
                }
                var row = oldRow.filter('tr[data-group-id="' + groupId + '"]');
                var replacement = $(rowString);
                // If the row was selected need to maintain this selection
                if (row.is('.' + that.options.classes.rowSelected)) {
                    row.first().before(replacement);
                    row.remove();

                    var cellSelected = row.find('td.' + that.options.classes.cellFocused);
                    if (cellSelected.length) {
                        var lookupString = 'td[data-index="' + cellSelected.attr('data-index') + '"]';
                        if (cellSelected.attr('data-sub-row') != null) {
                            lookupString += '[data-sub-row="' + cellSelected.attr('data-sub-row') + '"]';
                        }
                        that.selectionModel.cell.focusCell(replacement.find(lookupString));
                    }
                } else {
                    row.first().before(replacement);
                    row.remove();
                }
            });
            this.widget._trigger('view-sync-update-row', null, { recordId : recordId });
        },

        populateRecords : function (recordId, location) {
            var virtualRows = this.virtualRows[location.rowsetId];
            virtualRows.records.push(recordId);
            var node = virtualRows.nodes[recordId];
            if (node != null) {
                return node.expanded;
            }
            this.updateRecordCount();

            return false;
        },

        containsRecord : function (recordId, rowsetId) {
            if (recordId == null) {
                return false;
            }
            var recordIndex = this.getRecordIndex(recordId, rowsetId);
            return recordIndex !== -1;
        },

        recordAbove : function (recordId) {
            if (recordId == null) {
                return null;
            }
            var rowsetId = this.widget._getRecordRowset(recordId);
            var virtualRows = this.virtualRows[rowsetId];
            var recordIndex = this.getRecordIndex(recordId, rowsetId);
            if (recordIndex > 0) {
                return virtualRows.records[recordIndex - 1];
            }
        },

        recordBelow : function (recordId) {
            if (recordId == null) {
                return null;
            }
            var rowsetId = this.widget._getRecordRowset(recordId);
            var virtualRows = this.virtualRows[rowsetId];
            var recordIndex = this.getRecordIndex(recordId, rowsetId);
            if (recordIndex <= virtualRows.records.length - 2) {
                return virtualRows.records[recordIndex + 1];
            }
        },

        previousSibling : function (recordId) {
            if (recordId == null) {
                return null;
            }
            var rowsetId = this.widget._getRecordRowset(recordId);
            var virtualRows = this.virtualRows[rowsetId];
            var recordIndex = this.getRecordIndex(recordId, rowsetId);
            if (this.options.tree || this.widget.groupingApplied) {
                var node = virtualRows.nodes[recordId];
                for (var i = recordIndex - 1; i >= 0; --i) {
                    var newNodeLevel = virtualRows.nodes[virtualRows.records[i]].level;
                    if (node.level === newNodeLevel) {
                        return virtualRows.records[i];
                    }
                    if (newNodeLevel < node.level) {
                        return null;
                    }
                }
            } else {
                return virtualRows.records[recordIndex - 1];
            }
            return null;
        },

        nextSibling : function (recordId) {
            if (recordId == null) {
                return null;
            }
            var rowsetId = this.widget._getRecordRowset(recordId);
            var virtualRows = this.virtualRows[rowsetId];
            var recordIndex = this.getRecordIndex(recordId, rowsetId);

            if (this.options.tree || this.widget.groupingApplied) {
                var node = virtualRows.nodes[recordId];
                // If parent and child are cut when the parent gets moved it is collapsed which means the virtual rows
                // no longer has any way to get siblings for the child when that is attempted.
                if (node) {
                    for (var i = recordIndex + 1; i < virtualRows.records.length; ++i) {
                        var newNodeLevel = virtualRows.nodes[virtualRows.records[i]].level;
                        if (node.level === newNodeLevel) {
                            return virtualRows.records[i];
                        }
                        if (newNodeLevel < node.level) {
                            return null;
                        }
                    }
                }
            } else {
                return virtualRows.records[recordIndex + 1];
            }
            return null;
        },

        siblingIdsOfRecordAtLevel : function (recordId) {
            if (recordId == null) {
                return null;
            }

            var rowsetId = this.widget._getRecordRowset(recordId);
            var virtualRows = this.virtualRows[rowsetId];
            if (this.options.tree || this.widget.groupingApplied) {
                var result = [];
                var node = virtualRows.nodes[recordId];
                // TODO comment copied from nextSibling
                // If parent and child are cut when the parent gets moved it is collapsed which means the virtual rows
                // no longer has any way to get siblings for the child when that is attempted.
                if (node) {
                    var nodes = virtualRows.nodes;
                    var records = virtualRows.records;
                    for (var i = 0, len = records.length; i < len; ++i) {
                        var newNode = nodes[records[i]];
                        if (node.level === newNode.level) {
                            result.push(records[i]);
                        }
                    }
                }
                return result;
            } else {
                return virtualRows.records;
            }
            return null;
        },


        lastChild : function (recordId, rowsetId, precedence) {
            if (recordId == null) {
                return null;
            }
            if (rowsetId == null) {
                rowsetId = this.widget._getRecordRowset(recordId);
            }
            var virtualRows = this.virtualRows[rowsetId];
            var recordIndex = this.getRecordIndex(recordId, rowsetId);
            if (this.options.tree || this.widget.groupingApplied) {
                var node = virtualRows.nodes[recordId];
                var lastChild = null;
                for (var i = recordIndex + 1; i < virtualRows.records.length; ++i) {
                    var newNode = virtualRows.nodes[virtualRows.records[i]];
                    if (node.level + 1 === newNode.level && (precedence == null || newNode.precedence <= precedence)) {
                        lastChild = virtualRows.records[i];
                    }
                    if (node.level >= newNode.level || (precedence != null && newNode.precedence > precedence)) {
                        return lastChild;
                    }
                }
                return lastChild;
            }

            return null;
        },

        getMoveLeftSibling : function (recordId, parentId, rowsetId) {
            if (rowsetId == null) {
                rowsetId = this.widget._getRecordRowset(recordId);
            }
            var recordNode = this.virtualRows[rowsetId].nodes[recordId];
            var parentNode = this.virtualRows[rowsetId].nodes[parentId];
            if (recordNode.precedence === parentNode.precedence) {
                return parentId;
            } else if (parentNode.parent != null) {
                return this.lastChild(parentNode.parent.id, rowsetId, recordNode.precedence);
            } else {
                return this.getRootSibling(rowsetId, recordNode.precedence);
            }
        },

        getRootSibling : function (rowsetId, precedence) {
            if (this.options.tree || this.widget.groupingApplied) {
                var virtualRows = this.virtualRows[rowsetId];
                var lastChild = null;
                for (var i = 0; i < virtualRows.records.length; ++i) {
                    var newNode = virtualRows.nodes[virtualRows.records[i]];
                    if (0 === newNode.level && (precedence == null || newNode.precedence <= precedence)) {
                        lastChild = virtualRows.records[i];
                    }
                    if (precedence != null && newNode.precedence > precedence) {
                        return lastChild;
                    }
                }
                return lastChild;
            }
        },

        parentRecordId : function (recordId) {
            if (this.options.tree && recordId != null) {
                var rowsetId = this.widget._getRecordRowset(recordId);
                var virtualRows = this.virtualRows[rowsetId];
                var recordIndex = this.getRecordIndex(recordId, rowsetId);

                var node = virtualRows.nodes[recordId];
                for (var i = recordIndex - 1; i >= 0; --i) {
                    if (node.level === virtualRows.nodes[virtualRows.records[i]].level + 1) {
                        return virtualRows.records[i];
                    }
                }
            }
            return null;
        },


        findTreeInsertIndex : function (afterIndex, afterId, virtualRows) {
            var afterNode = virtualRows.nodes[afterId];
            // If the node is expanded find the end of the children/subchildren to add the record after
            if (afterIndex == null && afterNode != null && afterNode.expanded) { // don't override the afterIndex if it is already specified
                for (var i = afterIndex + 1; i <= virtualRows.records.length; ++i) {
                    if (virtualRows.nodes[virtualRows.records[i]] == null || virtualRows.nodes[virtualRows.records[i]].level <= afterNode.level) {
                        afterIndex = i - 1;
                        break;
                    }
                }
            }
            return afterIndex;
        },

        addRecord : function (options) {
            var location = options.location;
            var recordId = options.recordId;
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (this.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            var virtualRows = this.virtualRows[location.rowsetId];
            var afterIndex = -1;
            if (options.insertIndex != null) {
                afterIndex = options.insertIndex;
            } else {
                // If sibling id is set use that as the insert point
                if (location.siblingId != null) {
                    afterIndex = this.getRecordIndex(location.siblingId, location.rowsetId);
                    // In a tree grid make sure the sibling does not have expanded children
                    if (this.options.tree || this.widget.groupingApplied) {
                        // Expand the parent node of the record you are adding if it is not already
                        if (location.parentId != null) {
                            if (afterIndex < 0) { //If the parent node is not expanded, expand it this will render the added record
                                var wasToggled = this.toggleChildren({
                                    recordId : location.parentId,
                                    rowsetId : location.rowsetId,
                                    toggle : true
                                });
                                if (!options.fromExpand && wasToggled) {
                                    return;
                                } else {
                                    afterIndex = this.getRecordIndex(this.lastChild(location.parentId), location.rowsetId);
                                }
                            }
                        }
                        // Readjust add point to below all the children
                        if (afterIndex >= 0) {
                            afterIndex = this.findTreeInsertIndex(afterIndex, location.siblingId, virtualRows);
                        }
                    }
                }
                // If no sibling is set or the sibling can't be found place it under the parent
                if (location.parentId != null && (location.siblingId == null || afterIndex < 0)) {
                    afterIndex = this.getRecordIndex(location.parentId, location.rowsetId);
                }
                // If there is sorting applied ignore and add to the bottom of the parent
                if (options.orderApplied) {
                    var lastChild = this.lastChild(location.parentId, location.rowsetId);
                    if (lastChild != null) {
                        afterIndex = this.getRecordIndex(lastChild, location.rowsetId);
                    }
                }
            }
            virtualRows.records.splice(afterIndex + 1, 0, recordId);

            // If rows do not need to be rendered in
            if ((afterIndex < 0 && virtualRows.displayTop > 0) || // inserted at the very top but not viewing
                    (afterIndex >= 0 && afterIndex < virtualRows.displayTop) || // Standard case of top not in view
                    (afterIndex >= virtualRows.displayBottom && afterIndex >= virtualRows.visibleRows)) {
 // standard case of not in view below

                // If a record is inserted above what is displaying change display top to account
                if (afterIndex < virtualRows.displayTop) {
                    virtualRows.displayTop++;
                    virtualRows.renderedTop++;
                    virtualRows.displayBottom++;
                    virtualRows.renderedBottom++;
                }
                // If you need to select this record change display top to its index
                if (options.select) {
                    virtualRows.displayTop = afterIndex + 1;
                }
                // If rows do need to be rendered
            } else {
                var rows = [this.makeRow(recordId, location.rowsetId)];
                // Attempting to add to be start of a rowset when the view is scrolled to the top
                // Prepend row and trim end records, display and render values remain the same
                if (afterIndex === -1 && virtualRows.displayTop === 0) {
                    this.attachRows(location.rowsetId, '$tbody', rows, 'prepend', virtualRows.visibleRows - 1);
                    // If the view is not full need to fix renderedBottom count
                    if (virtualRows.records.length <= virtualRows.visibleRows) {
                        virtualRows.renderedBottom++;
                        virtualRows.displayBottom++;
                    }
                    // Attemping to add to the end of a rowset
                } else if (afterIndex === virtualRows.records.length - 2) {
                    // Adjust render and view references so renderRowset performs properly
                    virtualRows.renderedBottom++;
                    virtualRows.displayBottom++;

                    var removeCount = 1;
                    // If the rowset is not full do not remove any records else increase render counts for the top
                    if (virtualRows.visibleRows >= virtualRows.displayBottom) {
                        removeCount = 0;
                    } else {
                        virtualRows.displayTop++;
                        virtualRows.renderedTop++;
                    }
                    this.attachRows(location.rowsetId, '$tbody', rows, 'append', removeCount);
                } else {
                    var insertType = 'insert';
                    var insertAt = afterIndex - virtualRows.displayTop;

                    // if the view is not full
                    if (virtualRows.records.length <= virtualRows.visibleRows + 1 ||
                        // detect when view is not fully flushed and adjust the bottom
                            (virtualRows.renderedBottom - virtualRows.renderedTop) < virtualRows.visibleRows) {
                        virtualRows.renderedBottom++;
                        virtualRows.displayBottom++;
                        insertType = 'insert-skip-delete';
                    }

                    this.attachRows(location.rowsetId, '$tbody', rows, insertType, insertAt, virtualRows.visibleRows);
                }
            }
            this.updateRecordCount();
            //>>excludeStart("perf", pragmas.production)
            if (this.options.debug) {
                this.widget._trigger('performance-debug', null,
                    {
                        start : start,
                        end : window.performance.now(),
                        name : 'add-Record',
                        type : 'view',
                        info : recordId
                    });
            }
            //>>excludeEnd("perf")
        },

        // Redraw the elements of the rowset, only creates the rows it needs
        renderRowset : function (rowsetId, args) {
            var forceFullRender = (args === true); // args can be object, not as flag here, but for gridWidget's bulkModification processing
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (this.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            var that = this;
            var virtualRows = this.virtualRows[rowsetId];
            // Ensure display
            if (virtualRows.displayTop < 0) {
                virtualRows.displayTop = 0;
            }

            var displayBottom = virtualRows.displayTop + virtualRows.visibleRows;
            // Check we dont try and draw non existant records
            if (displayBottom >= virtualRows.records.length) {
                displayBottom = virtualRows.records.length - 1;
                // Reverse flush the viewport if we have modified the displayBottom
                virtualRows.displayTop = displayBottom - virtualRows.visibleRows + 1;
                if (virtualRows.displayTop < 0) {
                    virtualRows.displayTop = 0;
                }
            }

            virtualRows.displayBottom = displayBottom;
            // If the data is already rendered do nothing
            if (virtualRows.renderedTop === virtualRows.displayTop && !forceFullRender && virtualRows.renderedBottom === virtualRows.displayBottom) {
                return;
            }
            // if you have too much and need to strip it out remove them
            if (virtualRows.renderedTop === virtualRows.displayTop && virtualRows.renderedBottom >= virtualRows.displayBottom && !forceFullRender) {
                for (var j = virtualRows.displayBottom + 1; j <= virtualRows.renderedBottom; j++) {
                    this.deleteRow(virtualRows.records[j], rowsetId);
                }
                // Rendered bottom is different so before we return update the rendered to the display
                this.virtualRows[rowsetId].renderedBottom = virtualRows.displayBottom;
                return;
            }

            var rowStart = virtualRows.displayTop,
                rowEnd = displayBottom,
                actionIndex = 0,
                action = '';

            // If the sets of rows are disjoint add them all and clear everything, or if we need to add to the top AND BOTTOM
            if (forceFullRender || virtualRows.displayTop > virtualRows.renderedBottom || // Top > Bottom means everything TO BE rendered is below current viewed data
                    virtualRows.displayBottom <= virtualRows.renderedTop || // Bottom <= Top means everything TO BE  rendered is above the current
                    (virtualRows.displayTop < virtualRows.renderedTop && virtualRows.displayBottom > virtualRows.renderedBottom) // If it needs items on the top and bottom replace
                ) {
                action = 'replace';
                // If the beginning is partially rendered we add items to the bottom of the grid
            } else if ((virtualRows.displayTop > virtualRows.renderedTop && virtualRows.displayTop < virtualRows.renderedBottom) || // If DT > RT top is there so we are cutting top and adding bottom
                    virtualRows.displayBottom > virtualRows.renderedBottom || // If DB > RB adding more to the bottom, might not strip top
                    virtualRows.displayTop === virtualRows.displayBottom) { // If DT == DB its one row
                rowStart = virtualRows.renderedBottom + 1;
                // index of rows to remove before
                actionIndex = virtualRows.displayTop - virtualRows.renderedTop;
                action = 'append';

                // If the end is partially rendered, we prepend items to the top of the grid
            } else if (virtualRows.displayBottom <= virtualRows.renderedBottom && // If DB <= RB means we are cutting from the bottom
                    virtualRows.displayTop < virtualRows.renderedTop) {
 // If DT < RT it means we are adding to the top

                rowEnd = virtualRows.renderedTop - 1;
                actionIndex = virtualRows.displayBottom - virtualRows.renderedTop; // Index of how rows to cut after
                action = 'prepend';
            }

            var rows = [];
            for (var i = rowStart; i <= rowEnd; i++) {
                rows.push(that.makeRow(virtualRows.records[i], rowsetId));
            }

            that.attachRows(rowsetId, '$tbody', rows, action, actionIndex);
            this.virtualRows[rowsetId].renderedTop = virtualRows.displayTop;
            this.virtualRows[rowsetId].renderedBottom = virtualRows.displayBottom;
            virtualRows.recordsModified = false;
            //>>excludeStart("perf", pragmas.production)
            if (that.options.debug) {
                that.widget._trigger('performance-debug', null,
                        { start : start, end : window.performance.now(), name : 'render-Rowset', type : 'view' });
            }
            //>>excludeEnd("perf")
            this.viewports['$rowset-' + rowsetId].find('.viewport-overlay').css({
                top : -that.ROW_HEIGHT * virtualRows.displayTop
            });
            this.$rowsets[rowsetId].find('.rowset-overlay').css({
                top : -that.ROW_HEIGHT * virtualRows.displayTop
            });

            that.widget._trigger('view-sync-scroll-top', null, {
                top : that.ROW_HEIGHT * virtualRows.displayTop, displayTop : virtualRows.displayTop,
                totalRows : virtualRows.records.length, displayBottom : virtualRows.displayBottom
            });
            that.closeEditor();
            that.closeMenus();
        },

        makeRow : function (recordId, rowsetId) {
            return this.rowRenderer.makeRow(recordId, rowsetId);
        },

        makePrintRow : function (recordId, rowsetId) {
            var printRows = [this.rowRenderer.makePrintRow(recordId, rowsetId, 0)];
            var lastRow = printRows[0];

            while (lastRow.stacked) {
                lastRow = this.rowRenderer.makePrintRow(recordId, rowsetId, lastRow.stacked);
                printRows.push(lastRow);
            }

            return _.map(printRows, 'row');
        },

        makeExcelRow : function (recordId, rowsetId, rowNum) {
            var excelRows = [this.rowRenderer.makeExcelRow(recordId, rowsetId, rowNum, 0)];
            var lastRow = excelRows[0];
            var nextRowNum = rowNum + 1;

            while (lastRow.stacked) {
                lastRow = this.rowRenderer.makeExcelRow(recordId, rowsetId, nextRowNum, lastRow.stacked);
                excelRows.push(lastRow);
                nextRowNum++;
            }

            return excelRows;
        },

        canDragRows : function () {
            return !this.widget.groupingApplied && this.options.rows.draggable && !this.config.sortBy.length;
        },

        showCellErrors : function (recordId, dataIndex, $target) {
            var recordAttributes = this.widget._attrs.all(this.widget._getRecord(recordId));
            var recordErrors = _.get(recordAttributes, 'timephased.invalid.' + dataIndex, _.get(recordAttributes.invalid, dataIndex, null));
            var cellErrors = recordErrors != null ?
                    _.filter(
                        _.values(recordErrors), function (value) {
                            return value != null;
                        }
                    )
                    : [];

            if (cellErrors != null && cellErrors.length) {
                var allWarnings = _.every(cellErrors, function (error) {
                    return error.warning;
                });
                if (allWarnings) {
                    this.$errorBox.addClass('warning-box');
                } else {
                    this.$errorBox.removeClass('warning-box');
                }
            }

            var errorBoxId = this.componentIds.errorPopup;
            var labeledBy = [];
            if (cellErrors != null && cellErrors.length > 0) {
                this.$errorBox.empty();
                for (var i = 0; i < cellErrors.length; i++) {
                    var errorId = errorBoxId + ':' + i;
                    this.$errorBox.append(errorTemplate({ id : errorId, error : cellErrors[i].message }));
                    labeledBy.push(errorId);
                }
                this.$errorBox.attr('aria-labelledby', labeledBy.join(' '));
                var subRow;
                var realDataIndex = dataIndex;
                if (dataIndex.indexOf('$$') !== -1) {
                    realDataIndex = dataIndex.substring(0, dataIndex.indexOf('$$'));
                    subRow = dataIndex.substring(dataIndex.indexOf('$$') + 2);
                }
                var lookup = 'tr[data-id="' + recordId + '"] td[data-index="' + realDataIndex + '"]';
                if (subRow != null) {
                    lookup += '[data-sub-row="' + subRow + '"]';
                }
                $target = this.$grid.find(lookup);
                var $body = $(window);

                $(document.body).append(this.$errorBox);

                var currentOwns = (this.$grid.attr('aria-owns') || '').split(' ');
                this.$grid.attr('aria-owns', _.union(currentOwns, [errorBoxId]).join(' ').trim());

                var top = $target.offset().top + $target.height() - $body.scrollTop();
                var left = $target.offset().left - $body.scrollLeft() + ($target.outerWidth() / 2 - this.$errorBox.outerWidth() / 2);

                if (left + this.$errorBox.outerWidth() > window.pageXOffset + $(window).width()) {
                    // Bottom right of the error box is close to the triangle
                    left = $target.offset().left + $target.outerWidth() - this.$errorBox.outerWidth();
                    top -= this.$errorBox.outerHeight() + 10;
                }
                if (left < 0) {
                    left = 0;
                }

                this.$errorBox.css('top', top);
                this.$errorBox.css('left', left);
                this.$grid.attr('aria-activedescendant', this.componentIds.errorPopup);
                this.setEscapeMode(false);
            }
        },

        getHeaderId : function (dataIndex) {
            return this.widget.uuid + 'grid-header-' + dataIndex;
        },

        getCellId : function (recordId, dataIndex, subRow) {
            return this.widget.uuid + 'grid-row-' + recordId + '-col-' + dataIndex + (subRow != null ? ('-' + subRow) : '');
        },

        getCellLabels : function (recordId, dataIndex, skip) {
            var headers = [];
            if (this.descriptionIds.cellState.rowHeaderLabel && !skip) {
                headers.push(this.descriptionIds.cellState.rowHeaderLabel);
            }
            for (var i = 0; i < this.rowHeaders.length; i++) {
                var rowHeader = this.rowHeaders[i];

                headers.push(this.getCellId(recordId, rowHeader));
            }
            if (dataIndex !== constants.MENU_BUTTON_COLUMN) {
                if (this.descriptionIds.cellState.columnHeaderLabel && !skip) {
                    headers.push(this.descriptionIds.cellState.columnHeaderLabel);
                }
                headers.push(this.getHeaderId(dataIndex));


                if (this.descriptionIds.cellState.cellLabel && !skip) {
                    headers.push(this.descriptionIds.cellState.cellLabel);
                }
            }
            return headers.join(' ');
        },

        attachDOMTables : function (attach, selector, parent) {
            // detach table body off the dom whilst appending all the rows
            for (var w = 0; w < this.config.rowsets.length; w++) {
                for (var g = 0; g < this.config.groups.length; g++) {
                    var viewport = this.viewports[this.config.rowsets[w].id][this.config.groups[g].id];
                    if (attach) {
                        viewport[parent].after(viewport[selector]);
                    } else {
                        viewport[selector].detach();
                    }
                }
            }
        },

        clearTables : function () {
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (this.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            for (var w = 0; w < this.config.rowsets.length; w++) {
                for (var g = 0; g < this.config.groups.length; g++) {
                    var viewport = this.viewports[this.config.rowsets[w].id][this.config.groups[g].id];
                    viewport.$tbody.empty();
                    viewport.$tfoot.empty();
                }
                this.virtualRows[this.config.rowsets[w].id].records = [];
                this.virtualRows[this.config.rowsets[w].id].originalOrder = null;
            }
            //>>excludeStart("perf", pragmas.production)
            if (this.options.debug) {
                this.widget._trigger('performance-debug', null,
                        { start : start, end : window.performance.now(), name : 'clear-Tables', type : 'view' });
            }
            //>>excludeEnd("perf")
        },

        resetHeaders : function () {
            for (var w = 0; w < this.config.rowsets.length; w++) {
                for (var g = 0; g < this.config.groups.length; g++) {
                    var viewport = this.viewports[this.config.rowsets[w].id][this.config.groups[g].id];
                    viewport.$thead.empty();
                }
            }
            this.createColumns();
        },

        toggleColumns : function (columns, toggle) {
            var columnDelta = 0;
            var expandableColumn = _.get(this.columnDefinitions, this.expandableDataIndex);
            for (var i = 0; i < columns.length; i++) {
                var column = this.columnDefinitions[columns[i]];
                if (column.enabled !== false && column.visible !== toggle) {
                    column.visible = toggle;
                    columnDelta += toggle ? 1 : -1;
                }
            }
            if (columnDelta !== 0 && expandableColumn) {
                _.set(expandableColumn, 'colSpan', expandableColumn.colSpan + columnDelta);
            }
            this._refreshColumnVisibility();
            // TODO trigger view-updated event? note - this is called by gridWidget#toggleColumns, which is invoked by Prime's GridSettingsDialog column controller
        },

        switchColumnVisibilty : function (column) {
            column.visible = !column.visible;
            this._refreshColumnVisibility();
            this.widget._trigger('view-updated', null, {
                action : (column.visible) ? 'column-added' : 'column-removed',
                column : column.dataIndex,
                visibility : column.visible
            });
        },

        _refreshColumnVisibility : function () {
            var that = this;
            that.recalculateHeaderHeight = true;
            that.resetHeaders();
            _.forEach(that.config.rowsets, function (rowset) {
                if (rowset.id !== constants.HEADERS_ROWSET_ID) {
                    that.renderRowset(rowset.id, true);
                }
            });
            that.resize();
        },

        getColumnVisibleCount : function () {
            var visibleCount = 0;

            _.forEach(this.columnDefinitions, function (column) {
                if (column.dataIndex !== constants.ROW_SELECTOR) {
                    if (column.visible === true) {
                        visibleCount++;
                    }
                }
            });

            return visibleCount;
        },

        getRecordOrder : function (rowsetId) {
            return _.clone(this.virtualRows[rowsetId].records);
        },

        applySortOrder : function (rowsetId, recordOrder) {
            var virtualRows = this.virtualRows[rowsetId];
            if (virtualRows.originalOrder == null) {
                virtualRows.originalOrder = virtualRows.records;
            }
            virtualRows.records = recordOrder;
        },

        // When repopulate you need to clean the original sort order so invalid values do not get applied.
        clearOriginalSortOrder : function () {
            var that = this;
            _.forEach(this.config.rowsets, function (rowset) {
                var virtualRows = that.virtualRows[rowset.id];
                if (virtualRows.originalOrder != null) {
                    virtualRows.originalOrder = null;
                }
            });
        },

        clearSort : function () {
            var that = this;
            _.forEach(this.config.rowsets, function (rowset) {
                var virtualRows = that.virtualRows[rowset.id];
                if (virtualRows.originalOrder != null) {
                    virtualRows.records = that.virtualRows[rowset.id].originalOrder;
                    virtualRows.originalOrder = null;
                }
            });
        },

        getHeaderHeight : function () {
            var height = 0;
            if (this.options.gantt.enabled || this.options.timephased.enabled) {
                this.$rowsets.headers.addClass('rowset-headers-triple').removeClass('rowset-headers-double');
                this.recalculateHeaderHeight = false;
                this.config.rowsets[0].height = 45;
                return;
            }
            if (this.options.layout.removeHeaderRow) {
                this.recalculateHeaderHeight = false;
                return;
            }
            if (!this.$grid.is(':visible') || this.$grid.height() <= 0) {
                this.recalculateHeaderHeight = true;
                return;
            } else {
                this.recalculateHeaderHeight = false;
            }

            if (this.$rowsets.headers != null) {
                var headers = this.$rowsets.headers.find('.cell-renderer');
                _.forEach(headers, function (header) {
                    var headerHeight = $(header).innerHeight();
                    if (headerHeight > height) {
                        height = headerHeight;
                    }
                    if (height > 40) {
                        return;
                    }
                });

                if (height === 0) { // if height is still 0 due to some sort of visibility issue make it resize again
                    this.recalculateHeaderHeight = true;
                    return;
                }

                if (height > 27 && height < 40) {
                    this.config.rowsets[0].height = 36;
                    this.$rowsets.headers.addClass('rowset-headers-double').removeClass('rowset-headers-triple');
                } else if (height > 40) {
                    this.config.rowsets[0].height = 48;
                    this.$rowsets.headers.addClass('rowset-headers-triple').removeClass('rowset-headers-double');
                } else {
                    this.config.rowsets[0].height = this.options.layout.headerHeight;
                    this.$rowsets.headers.removeClass('rowset-headers-triple rowset-headers-double');
                }
                if (this.hasNamedColumnGroups) {
                    this.config.rowsets[0].height += 20;
                }
            }
            return height;
        },

        resize : attrFns.decorateCache('_attrs', 'gridView#resize', function resize(options) {
            //>>excludeStart("perf", pragmas.compiled)
            window.incrementCallCount('gridView#resize');
            //>>excludeEnd("perf")
            var singleColumnSize;
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (this.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            if (this.recalculateHeaderHeight) {
                this.getHeaderHeight();
            }

            var width, height;
            var $parent = this.$grid.parent();
            var widthIsPercent = this.options.layout.width.indexOf('%') !== -1;
            var heightIsPercent = this.options.layout.height.indexOf('%') !== -1;
            var refocus = this.$grid.is(':focus');
            var scrollLefts = [];
            if (widthIsPercent || heightIsPercent) {
                scrollLefts = _.map(this.pseudoScrollers.horizontal, function (scroller) {
                    return {
                        $scroller : scroller.$scroller,
                        scrollLeft : scroller.$scroller[0].scrollLeft
                    };
                });
                this.$grid.hide();// hide if percentage
            }


            if (widthIsPercent) {
                width = $parent.width() * (parseFloat(this.options.layout.width) / 100);
                if (this.options.layout.widthShim != null) {
                    width -= parseInt(this.options.layout.widthShim, 10);
                }
            } else {
                width = parseFloat(this.options.layout.width);
            }

            if (heightIsPercent) {
                height = $parent.height() * (parseFloat(this.options.layout.height) / 100);

                if (this.options.layout.heightShim != null) {
                    height -= parseInt(this.options.layout.heightShim, 10);
                }
            } else {
                height = parseFloat(this.options.layout.height);
            }

            if (widthIsPercent || heightIsPercent) {
                this.$grid.show();
                _.forEach(scrollLefts, function handleResizeScrollLefts(scrollObj) {
                    scrollObj.$scroller.scrollLeft(scrollObj.scrollLeft);
                });
            }
            // Bug for IE, when focused resize will lose focus on the hide.
            if (refocus) {
                this.widget.requestBrowserFocus(true);
            }

            var gridWidth = parseInt(width, 10),
                gridHeight = parseInt(height, 10);
            // Set the size of the grid.
            this.$grid.css('width', gridWidth);
            this.$grid.css('height', gridHeight);


            gridHeight -= this.options.borders.grid;
            if (this.options.statusBar.enabled) {
                gridHeight -= this.options.statusBar.height;
            }
            var requiresVerticalShim = SizingUtil.calculateRowsetHeights(this, gridHeight, false) && !this.options.gantt.enabled;
            var singleColumn = null, singleColumnDeductions = 0;
            if (this.options.layout.singleColumn) {
                if (this.options.contextMenu.enabled && this.options.contextMenu.enableRowMenuButton) {
                    singleColumn = this.config.groups[1].columns[0];
                } else {
                    singleColumn = this.config.groups[0].columns[0];
                }

                singleColumnDeductions = requiresVerticalShim ? this.options.scrollbar.extraWidth : 0;
                if (this.options.contextMenu.enabled && this.options.contextMenu.enableRowMenuButton) {
                    singleColumnDeductions += this.options.contextMenu.rowMenuButtonWidth;
                }
                singleColumnSize = (gridWidth - singleColumnDeductions) + 'px';
                singleColumn.width = singleColumnSize;
            }
            var requiresHorizontalShim = SizingUtil.calculateGroupWidths(this, gridWidth, requiresVerticalShim);
            // Need to recalculate
            if (requiresHorizontalShim) {
                // If no verticalshim was required before the addition of the horizontal shim might require it now,
                // redo the vertical shim calculation and it if has changed redo the horizontal calculations.
                if (SizingUtil.calculateRowsetHeights(this, gridHeight, requiresHorizontalShim) && !requiresVerticalShim && !this.options.gantt.enabled) {
                    if (this.options.layout.singleColumn) {
                        var deductions = requiresVerticalShim ? this.options.scrollbar.extraWidth : 0;
                        if (this.options.contextMenu.enabled && this.options.contextMenu.enableRowMenuButton) {
                            deductions += this.options.contextMenu.rowMenuButtonWidth;
                        }
                        singleColumnSize = (gridWidth - deductions) + 'px';
                        singleColumn.width = singleColumnSize;
                    }
                    SizingUtil.calculateGroupWidths(this, gridWidth, true);
                }

                this.$scrollShims.$bottom.css('display', 'block');
            } else {
                this.$scrollShims.$bottom.css('display', 'none');
            }
            if (requiresVerticalShim) {
                this.$scrollShims.$right.css('display', 'block');
            } else {
                this.$scrollShims.$right.css('display', 'none');
            }
            if (this.options.layout.singleColumn) {
                this.$grid.find('th[data-index="' + singleColumn.dataIndex + '"]').attr('width', singleColumnSize);
            }

            SizingUtil.resizeRowsetsFn(this, requiresVerticalShim);
            SizingUtil.resizeGroupsFn(this, gridHeight);


            // IE fix, when you resize a div it resets the horizontal scrolling of the group. However it does not trigger a scrollevent
            // this call will do nothing if the scroll values are the same.
            _.forEach(this.pseudoScrollers.horizontal, function (scroller) {
                scroller.$scroller.scroll();
            });
            if (this.recalculateHeaderHeight && (options == null || options.fromResize !== true)) {
                this.resize({ fromResize : true });
            }

            if (this.options.timephased.enabled) {
                this.timephasedColumnGroup.updateBucketLabelPositions();
            }

            if (this.options.gantt.enabled && this.options.statusBar.enabled) {
                this.$statusBar.css('padding-right', '10px');
            }

            //>>excludeStart("perf", pragmas.production)
            if (this.options.debug) {
                this.widget._trigger('performance-debug', null,
                        { start : start, end : window.performance.now(), name : 'resize', type : 'view' });
            }
            //>>excludeEnd("perf")
        }),

        resizeGroup : function (resizer, mouseX) {
            var gridOffset = this.$grid.offset().left;
            resizer.$selector.addClass('dragging');
            this.$grid.addClass('resizing-column');
            this.$grid.on('mousemove.group-resize', _.throttle(SizingUtil.onGroupDragFn(this, resizer, gridOffset), 20));
            $(document.body).on('mouseup.group-resize', SizingUtil.resizeDraggedGroupFn(this, resizer));
            SizingUtil.moveGroupResizeBarFn(this, resizer, mouseX - gridOffset);
        },

        resizeRowset : function (resizer, mouseY) {
            var gridOffset = this.$grid.offset().top;
            resizer.$selector.addClass('dragging');
            this.$grid.on('mousemove.rowset-resize', _.throttle(SizingUtil.onRowsetDragFn(resizer, gridOffset), 20));
            $(document.body).on('mouseup.rowset-resize', SizingUtil.resizeDraggedRowsetFn(this, resizer));
            SizingUtil.moveRowsetResizeBarFn(resizer, mouseY - gridOffset);
        },

        scrollFocusIntoView : function () {
            var focusedColumn = this.columnDefinitions[this.selectionModel.focus.columnId];
            var viewport = this.selectionModel.focus.viewport;
            var hScroller = viewport != null ? this.pseudoScrollers.horizontal[viewport.group] : null;
            if (hScroller != null) {
                //>>excludeStart("perf", pragmas.production)
                var start;
                if (this.options.debug) {
                    start = window.performance.now();
                }
                //>>excludeEnd("perf")
                var width = parseInt(focusedColumn.width, 10),
                    viewWidth = viewport.groupConfig.width,
                    adjustedWidth = (viewWidth < width ? viewWidth - 4 : width),
                    selectionPosition = focusedColumn.positionLeft,
                    tablePosition = viewport.groupConfig.positionLeft,
                    requiredToView = tablePosition + selectionPosition + adjustedWidth,
                    missingAmount = requiredToView - viewWidth;

                // Make the ui a bit snappier if you delay this
                setTimeout(function () {
                    if (missingAmount > 0) {
                        hScroller.$scroller.scrollLeft(hScroller.$scroller.scrollLeft() + missingAmount); // add some padding so its not right against the edge
                    } else if (requiredToView < adjustedWidth) {
                        hScroller.$scroller.scrollLeft(hScroller.$scroller.scrollLeft() + requiredToView - adjustedWidth - 5); // subtract a bit so its not against the edge
                    }
                }, 1);
                //>>excludeStart("perf", pragmas.production)
                if (this.options.debug) {
                    this.widget._trigger('performance-debug', null,
                        {
                            start : start,
                            end : window.performance.now(),
                            name : 'scroll-Focus-Into-View',
                            type : 'view'
                        });
                }
                //>>excludeEnd("perf")
            }
        },
        // Returns true if the scollbar needs to be updated still, IE flicker bug
        renderFocus : function (recordId, rowsetId, columnId, location) {
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (this.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            if (recordId != null) {
                var virtualRows = this.virtualRows[rowsetId];

                this.selectionModel.focus.recordId = recordId;
                this.selectionModel.focus.rowsetId = rowsetId;
                if (virtualRows[this.selectionModel.focus.recordIndex] !== recordId) {
                    this.selectionModel.focus.recordIndex = _.indexOf(virtualRows.records, recordId);
                }
                if (columnId != null) {
                    this.selectionModel.focus.columnId = columnId;
                }

                var updateScrollbar = this.renderRecord(this.selectionModel.focus.recordId,
                        this.selectionModel.focus.rowsetId,
                        location, this.selectionModel.focus.recordIndex);

                //>>excludeStart("perf", pragmas.production)
                if (this.options.debug) {
                    this.widget._trigger('performance-debug', null,
                            { start : start, end : window.performance.now(), name : 'render-focus', type : 'view' });
                }
                //>>excludeEnd("perf")

                return updateScrollbar;
            }
            return false;
        },

        getRecordIndex : function (recordId, rowsetId) {
            if (rowsetId == null) {
                rowsetId = this.widget._getRecordRowset(recordId);
            }
            var virtualRows = this.virtualRows[rowsetId];
            for (var i = virtualRows.displayTop; i < virtualRows.records.length; ++i) {
                if (virtualRows.records[i] === recordId) {
                    return i;
                }
            }
            return _.indexOf(virtualRows.records, recordId);
        },

        _canHideColumn : function (group, column) {
            if (!group.canHideColumns) {
                return false;
            }
            if (!this.options.menus.canHideColumns) {
                return false;
            }
            var canHideRequired = this.options.menus.canHideRequiredColumns;
            if (column.required && !canHideRequired) {
                return false;
            }
            var columns = group && _.filter(group.columns, function (col) {
                return col.visible && col.dataIndex !== 'empty-column';
            });
            return columns && columns.length > 1;
        },
        _getGroup : function (groupId) {
            return _.findWhere(this.config.groups, { id : groupId });
        },
        getColumnGroupId : function (columnId) {
            for (var i = 0; i < this.config.groups.length; i++) {
                var group = this.config.groups[i];
                for (var j = 0; j < group.columns.length; j++) {
                    var column = group.columns[j];
                    if (column.dataIndex === columnId) {
                        return group.id;
                    }
                }
            }
        },

        // Renders the requested record into the view, the location attribute is where you want
        // the record to be displayed in the rowset: 'top' 'middle' 'bottom'
        // returns true if renderRowset is required meaning we need to update the scrollbar (IE performance fix for flicker
        renderRecord : function (recordId, rowsetId, location, recordIndex) {
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (this.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            var virtualRows = this.virtualRows[rowsetId];

            if (recordIndex == null) {
                recordIndex = this.getRecordIndex(recordId, rowsetId);
            }

            // Detemine what the top index should be.
            // Already in view return
            if (recordIndex >= virtualRows.displayTop && recordIndex <= virtualRows.displayTop + virtualRows.visibleRows - 1) {
                return false;
            }
            var newTopIndex = recordIndex;
            if (location == null || location === 'top') {
                newTopIndex = recordIndex;
            } else if (location === 'middle') {
                newTopIndex = recordIndex - Math.floor(virtualRows.visibleRows / 2);
            } else if (location === 'bottom') {
                newTopIndex = recordIndex - virtualRows.visibleRows + 1; // +1 is for index offset
            }
            // Try and force it to fill the viewport
            if (newTopIndex + virtualRows.visibleRows > virtualRows.records.length) {
                newTopIndex = virtualRows.records.length - virtualRows.visibleRows;
            }
            // Not enough records to do what its trying, just render from 0
            if (newTopIndex < 0) {
                newTopIndex = 0;
            }

            // Set the top and rerender the rowset
            virtualRows.displayTop = newTopIndex;
            this.renderRowset(rowsetId);
            // This used to be where we updated scrollbar IE flicker means we return true and have it done after select
            //>>excludeStart("perf", pragmas.production)
            if (this.options.debug) {
                this.widget._trigger('performance-debug', null,
                        { start : start, end : window.performance.now(), name : 'render-record', type : 'view' });
            }
            //>>excludeEnd("perf")
            return true;
        },

        // Delayed scrollbar updating brought to you by IEs poor performance
        updateRowsetScrollbar : function (rowsetId) {
            if (this.pseudoScrollers.vertical[rowsetId] != null) {
                var virtualRows = this.virtualRows[rowsetId];
                this.pseudoScrollers.vertical[rowsetId].$scroller.scrollTop(virtualRows.displayTop * this.ROW_HEIGHT);
                this.viewports['$rowset-' + rowsetId].find('.viewport-overlay').css({
                    top : -this.ROW_HEIGHT * virtualRows.displayTop
                });
                this.$rowsets[rowsetId].find('.rowset-overlay').css({
                    top : -this.ROW_HEIGHT * virtualRows.displayTop
                });
            }
        },

        expandToRecord : function (recordId, rowsetId) {
            var originalRecordId = recordId;
            var node = this.virtualRows[rowsetId].nodes[recordId];
            var parentsToExpand = [];
            if (node == null) {
                var record, recordPlacement;
                while (recordId != null) {
                    record = this.widget._getRecord(recordId);
                    if (record != null) {
                        recordPlacement = this.widget._getRecordPlacement(record, true);
                        if (recordPlacement != null) {
                            recordId = recordPlacement.parentId;
                            if (recordId != null) {
                                record = this.widget._getRecord(recordId);
                            } else {
                                record = null;
                            }
                            if (record != null) {
                                parentsToExpand.push(recordId);
                            } else {
                                recordId = null;
                            }
                        } else {
                            recordId = null;
                        }
                    } else {
                        return false;// parent not loaded can not expand it
                    }
                }
            } else {
                var parentNode = node.parent;
                while (parentNode != null) {
                    if (!parentNode.expanded) {
                        parentsToExpand.push(parentNode.id);
                    }
                    parentNode = parentNode.parent;
                }
            }

            if (parentsToExpand.length) {
                parentsToExpand = parentsToExpand.reverse();
                for (var i = 0; i < parentsToExpand.length; i++) {
                    this.ensureExpanded({
                        nodeId : parentsToExpand[i],
                        rowsetId : rowsetId
                    });
                }
            }
            node = this.virtualRows[rowsetId].nodes[originalRecordId];
            return node != null;
        },

        ensureExpanded : function (args) {
            var nodeId = args.nodeId;
            var rowsetId = args.rowsetId;
            var expandChildren = args.expandChildren;
            //>>excludeStart("validation", pragmas.production)
            logger.assert(nodeId != null && rowsetId, 'nodeId and rowsetId must both be given');
            //>>excludeEnd("validation")
            var node = this.virtualRows[rowsetId].nodes[nodeId];
            if (node == null && !this.expandToRecord(nodeId, rowsetId)) {
                return false;
            } else {
                node = this.virtualRows[rowsetId].nodes[nodeId];
            }
            var parentNode = node.parent;
            if (parentNode != null) {
                this.ensureExpanded({
                    nodeId : parentNode.id,
                    rowsetId : rowsetId,
                    skipResize : args.skipResize
                });
                this.toggleChildren({
                    recordId : parentNode.id,
                    rowsetId : rowsetId,
                    toggle : true,
                    skipResize : args.skipResize
                });
            }
            if (!node.expanded || expandChildren) {
                return this.toggleChildren({
                    recordId : node.id,
                    rowsetId : rowsetId,
                    toggle : true,
                    expandChildren : expandChildren,
                    skipResize : args.skipResize
                });
            }

            return false;
        },

        toggleChildren : function (args) {
            var that = this;
            var recordId = args.recordId;
            var rowsetId = args.rowsetId;
            var toggle = args.toggle;
            var expandChildren = args.expandChildren;
            //>>excludeStart("validation", pragmas.production)
            logger.assert(recordId != null && rowsetId, 'toggleChildren must be given record/rowset to toggle');
            //>>excludeEnd("validation")
            var virtualRows = that.virtualRows[rowsetId];
            var actionTaken = false;
            var node = virtualRows.nodes[recordId];
            if (toggle == null) {
                toggle = !that.virtualRows[rowsetId].nodes[recordId].expanded;
            }
            if (toggle) {
                if (that.options.lazyload && !node.loaded) {
                    if (node.loading) {
                        return that.loadingRecords[node.id];
                    }
                    node.loading = true;
                    actionTaken = this.widget.store.loadChildren(recordId);
                    if (!_.isBoolean(actionTaken)) {
                        that.loadingRecords[node.id] = actionTaken;
                        actionTaken.done(function () {
                            delete that.loadingRecords[node.id];
                        });
                    }
                } else if (!node.expanded || expandChildren) {
                    that.expandNode(recordId, expandChildren);
                    actionTaken = true;
                } else {
                    actionTaken = true;
                }
            } else if (!node.loading) {
                that.collapseNode(recordId);
                actionTaken = true;
            }
            if (!args.skipResize) {
                that.resize();
            }
            that.updateRow(recordId, rowsetId);
            that.resetRowHovered();

            return actionTaken;
        },

        childrenLoaded : function (recordId) {
            var that = this;
            var virtualRows = that.virtualRows[that.widget._getRecordRowset(recordId)];
            var node = virtualRows.nodes[recordId];
            node.loading = false;
            that.expandNode(recordId);

            that.updateRow(recordId, this.widget._getRecordRowset(recordId));
            that.resize();
        },

        addRowExpansion : function (recordId) {
            this.rowExpansionManager.expandRow(recordId);
        },


        expandNode : function (nodeId, expandChildren, addChildNodes, nodesToInsert) {
            var that = this;
            var rowsetId = that.widget._getRecordRowset(nodeId);
            var virtualRows = that.virtualRows[rowsetId];
            var node = virtualRows.nodes[nodeId];
            if (!node.isNode || (node.expanded && !expandChildren && !addChildNodes)) {
                return;
            }
            var addAtEnd = false;
            var insertIndex;
            if (nodesToInsert == null) {
                insertIndex = this.getRecordIndex(nodeId, rowsetId);
                nodesToInsert = [];
                addAtEnd = true;
            }
            this.addRowExpansion(nodeId);
            var children = this.widget._getChildIds(nodeId);

            _.forEach(children, function (childId) {
                var childNode = virtualRows.nodes[childId];
                var forceAddChildren = false;
                var location, record;
                if (childNode == null) {
                    record = that.widget._getRecord(childId);
                    location = that.widget._getRecordPlacement(record);
                    childNode = that.updateNode(childId, location, that._attrs.treeAttrs(record));
                }
                if (!node.expanded || addChildNodes) { // Add Child Nodes for when something is expanded but its parent is being reexpanded
                    nodesToInsert.push(childId);
                    forceAddChildren = true; // Need to force children to be added as the node is expanded but children need to be readded
                }
                var alreadyExpanded = childNode.expanded;
                if (alreadyExpanded || expandChildren) {
                    that.expandNode(childId, expandChildren, forceAddChildren, nodesToInsert);
                    if (!alreadyExpanded) {
                        that.updateRow(childNode.id, rowsetId);
                    }
                }
            });
            node.expanded = true;
            if (addAtEnd) {
                virtualRows.records.splice.apply(virtualRows.records, [insertIndex + 1, 0].concat(nodesToInsert));
                that.renderRowset(rowsetId, true);
            }
        },


        collapseNode : function (nodeId, collapseChildren) {
            var that = this;
            var virtualRows = that.virtualRows[that.widget._getRecordRowset(nodeId)];
            var node = virtualRows.nodes[nodeId];
            if (node.expanded) {
                this.rowExpansionManager.collapseRow(nodeId);
                var recordsToDelete = this.getChildIds(nodeId, that.widget._getRecordRowset(nodeId), collapseChildren);
                if (recordsToDelete.length) {
                    that.deleteRecords(recordsToDelete, false, null, true);
                }
            }
            node.expanded = false;
        },

        getChildIds : function (nodeId, rowsetId, collapseChildren) {
            var that = this;
            var virtualRows = that.virtualRows[rowsetId],
                topNode = virtualRows.nodes[nodeId],
                topLevel = topNode.level,
                currentIndex = this.getRecordIndex(nodeId, rowsetId),
                nextNode, nextId,
                childIds = [];

            for (var i = currentIndex + 1; i < virtualRows.records.length; i++) {
                nextId = virtualRows.records[i];
                nextNode = virtualRows.nodes[nextId];

                // Might already be removed from view
                if (nextNode != null) {
                    if (nextNode.level > topLevel) {
                        childIds.push(nextId);

                        if (collapseChildren) {
                            nextNode.expanded = false;
                        }
                    } else if (nextNode.level <= topLevel) {
                        break;
                    }
                }
            }

            return childIds;
        },

        deleteChildNodes : function (recordId, virtualRows) {
            var that = this;
            var children = this.widget._getChildRecords(recordId);
            _.forEach(children, function (child) {
                var childId = that.widget._getRecordId(child);
                if (virtualRows.nodes[childId] != null) {
                    that.deleteChildNodes(childId, virtualRows);
                    delete virtualRows.nodes[childId];
                }
            });
        },

        collapseAllNodes : function () {
            var that = this;
            _.forEach(_.keys(this.virtualRows), function (key) {
                var virtualRows = that.virtualRows[key];
                _.forEach(_.clone(virtualRows.records), function (nodeId) {
                    var node = virtualRows.nodes[nodeId];
                    if (node.expanded) {
                        that.collapseNode(nodeId, true);
                    }
                });
                that.renderRowset(key, true);
            });

            that.resize();
        },

        ancestorsBanned : function (nodeId, bannedAncestors) {
            var nodeParent = nodeId;
            while (nodeParent != null) {
                if (_.includes(bannedAncestors, nodeParent)) {
                    return true;
                }
                nodeParent = this.parentRecordId(nodeParent);
            }
            return false;
        },

        getNode : function (nodeId, rowsetId) {
            return this.virtualRows[rowsetId].nodes[nodeId];
        },

        getNodeLevel : function (nodeId, rowsetId) {
            var virtualRows = this.virtualRows[rowsetId];
            var node = virtualRows.nodes[nodeId];
            return node != null ? node.level : undefined;
        },

        getRecordIds : function (records) {
            var that = this;
            var ids = [];
            _.forEach(records, function (record) {
                ids.push(that.widget._getRecordId(record));
            });
            return ids;
        },

        orderByViewIndex : function (records) {
            var that = this;
            return _.sortBy(records, function (record) {
                return that.getRecordIndex(that.widget._getRecordId(record));
            });
        },

        checkRecordsHaveChildren : function (records, rowsetId) {
            if (records.length <= 0) {
                return false;
            }
            var virtualRows = this.virtualRows[rowsetId];
            return _.every(this.getRecordIds(records), function (recordId) {
                var node = virtualRows.nodes[recordId];
                return node != null && node.isNode && (!node.loaded || node.hasChildren);
            });
        },

        // Only checks that there are records above and below. This is to save the storeWrapper some hassle
        canMoveRecords : function (records, rowsetId, direction) {
            // If rowset is null return false. Will get called again when focus is set
            if (records.length <= 0 || rowsetId == null || records.length > 1000) {
                return false;
            }
            var recordIds = this.getRecordIds(records);
            var virtualRows = this.virtualRows[rowsetId];
            var recordIndexes = [];
            var canMove = false;
            var nodes = [], previousNode, nodeLevel, nodeParent, i;

            for (i = 0; i < recordIds.length; i++) {
                var recordId = recordIds[i];
                if (this.options.tree) {
                    var node = virtualRows.nodes[recordIds[i]];
                    if (node != null) {
                        nodes.push(node);
                        if (i === 0) {
                            nodeLevel = node.level;
                            nodeParent = node.parent;
                        } else {
                            // If you have nodes from different parents, you can not do what you are trying to do
                            if (node.level !== nodeLevel || node.parent !== nodeParent) {
                                return false;
                            }
                        }
                    } else {
                        return false; // Return false for trying to move items from different rowsets.
                    }
                }

                recordIndexes.push({ recordId : recordId, index : this.getRecordIndex(recordId, rowsetId) });
            }

            recordIndexes = _.sortBy(recordIndexes, 'index');
            var contiguousSelection = true;
            for (i = 1; i < recordIndexes.length; i++) {
                if (recordIndexes[i].index - 1 !== recordIndexes[i - 1].index) { // If they are not contiguous return false
                    contiguousSelection = false;
                    break;
                }
            }
            switch (direction) {
            case 'left' :
                // If tree grid and its not at the root level
                if (this.options.tree) {
                    canMove = nodeLevel > 0;
                    if (this.options.data != null && canMove) {
                        canMove = this.widget._getTypeFromId(nodeParent.id) === this.widget._getTypeFromId(_.first(recordIds));
                    }
                }
                break;
            case 'right' :
                if (this.options.tree && contiguousSelection) { // If tree grid and all the rows are in order AND there is an eligible row above it can move under
                    previousNode = virtualRows.nodes[this.previousSibling(_.first(recordIndexes).recordId)];
                    var sameType = true;
                    if (this.options.data != null && previousNode != null) {
                        sameType = this.widget._getTypeFromId(previousNode.id) === this.widget._getTypeFromId(_.first(recordIds));
                    }
                    canMove = previousNode != null && sameType &&
                                (previousNode.isNode || previousNode.canHaveChildren);
                    if (canMove && this.options.treeMaxLevel != null) {
                        canMove = nodeLevel < this.options.treeMaxLevel;
                    }
                }
                break;
            case 'up' :
                if (this.options.tree) { // For trees
                    if (contiguousSelection) { // If they are all together, it can move up if there is a node above it.
                        if (_.first(recordIndexes).index > 0) {
                            // Could use previous sibling here but its more expensive and a lower level indicates there is a parent above it that it can go over
                            previousNode = virtualRows.nodes[virtualRows.records[_.first(recordIndexes).index - 1]];
                            canMove = (previousNode.level >= nodeLevel) && (previousNode.precedence >= _.first(nodes).precedence);
                        }
                    } else {
                        canMove = true;
                    }
                } else {
                    canMove = _.first(recordIndexes).index > 0 || !contiguousSelection;
                }
                break;
            case 'down':
                if (this.options.tree) {
                    if (contiguousSelection) {
                        if (_.last(recordIndexes).index < virtualRows.records.length - 1) {
                            for (i = _.last(recordIndexes).index + 1; i < virtualRows.records.length; ++i) {
                                var nextNode = virtualRows.nodes[virtualRows.records[i]];
                                if (nextNode.level === nodeLevel) {
                                    canMove = (nextNode.id !== constants.DUMMY_RECORD_ID) && (nextNode.precedence <= _.last(nodes).precedence);
                                    break;
                                } else if (nextNode.level < nodeLevel) {
                                    break;
                                }
                            }
                        }
                    } else {
                        canMove = true;
                    }
                } else {
                    var length = virtualRows.records.length - ((_.last(virtualRows.records) === constants.DUMMY_RECORD_ID) ? 2 : 1);

                    canMove = _.last(recordIndexes).index < length || !contiguousSelection;
                }
                break;

            }
            return canMove;
        },

        updateNode : function (nodeId, location, attributes, expanded) {
            if (nodeId == null) {
                return;
            }
            var rowsetId = location.rowsetId,
                nodes = this.virtualRows[rowsetId].nodes,
                parentId = location.parentId,
                siblingId = location.siblingId,
                parentNode = nodes[parentId],
                siblingNode = nodes[siblingId],
                node = nodes[nodeId];

            if (node == null) {
                node = {
                    id : nodeId,
                    expanded : attributes.childrenLoaded && attributes.expanded
                };
                nodes[nodeId] = node;
            }

            node.loaded = attributes.childrenLoaded;
            node.isNode = attributes.isNode;
            node.hasChildren = attributes.hasChildren;
            node.canHaveChildren = attributes.canHaveChildren;
            node.level = parentNode != null ? parentNode.level + 1 : (siblingNode != null ? siblingNode.level : 0);
            node.parent = parentNode;
            node.precedence = attributes.precedence;
            node.sibling = siblingNode;
            if (node.isNode && expanded != null) {
                node.expanded = expanded;
            }

            return node;
        },
        requestFocus : function () {
            if (browserUtils.isIE()) {
                this.$grid.get(0).setActive();
            } else {
                this.$grid.get(0).focus();
            }
        },
        updateGroupBandClasses : function () {
            var bandStyle = GROUPING_STYLE[this.options.theme.style];
            var bandColors = GROUPING_THEME[this.options.theme.colors];
            this.groupBandClasses = _.clone(bandColors.classes);

            this.$grid.removeClass(function (index, css) {
                return (css.match(/(^|\s)grid-grouping-\S+/g) || []).join(' ');
            }).addClass(bandStyle.style);

            this.$grid.removeClass(function (index, css) {
                return (css.match(/(^|\s)grid-band-theme-\S+/g) || []).join(' ');
            });
            if (GROUPING_STYLE.NONE !== bandStyle) {
                this.$grid.addClass(bandColors.style);
            }
        },

        setHeaderCheckbox : function (dataIndex, checked) {
            var $thCheckbox = this.$grid.find('th[data-index="' + dataIndex + '"] input');
            if ($thCheckbox.length) {
                $thCheckbox.prop('checked', checked || checked === 'mixed');

                if (checked === 'mixed') {
                    $thCheckbox.prop('indeterminate', true);
                } else {
                    $thCheckbox.prop('indeterminate', false);
                }
            }
        },

        focusCellSelection : function () {
            if (!this.escapeMode) {
                var $selectedCell = this.$grid.find('.cell-focused');
                if ($selectedCell.find('.cell-descendant').length) {
                    $selectedCell = $selectedCell.find('.cell-descendant');
                }
                this.$grid.attr('aria-activedescendant', $selectedCell.attr('id'));
            }
        },

        setColumnsSorted : function () {
            var $ths = this.viewports['$rowset-' + _.first(this.config.rowsets).id].find('thead th');
            var orderTitles = {
                asc : this.options.locale.sortedAscending,
                dsc : this.options.locale.sortedDescending,
                none : this.options.locale.notSorted
            };

            $ths.removeClass('col-sort-asc col-sort-dsc col-sorted').attr('aria-sort', 'none'); // LUX-clientSorting
            _.forEach(this.config.sortBy, function (sortConf, index, list) {
                var currentHeader = $ths.filter('th[data-index="' + sortConf.dataIndex + '"]');

                currentHeader
                    .addClass('col-sorted')
                    .addClass(sortConf.order === 'asc' ? 'col-sort-asc' : 'col-sort-dsc')
                    .attr('aria-sort', sortConf.order === 'asc' ? 'ascending' : 'descending');

                if (list.length > 1) {
                    currentHeader.find('.column-sort-icon')
                            .attr('title', orderTitles[sortConf.order])
                            .attr('data-sort-order', (index + 1));
                } else {
                    currentHeader.find('.column-sort-icon')
                            .attr('title', orderTitles[sortConf.order])
                            .removeAttr('data-sort-order');
                }
            });
        },
        setColumnsFiltered : function () {
            var $ths = this.viewports['$rowset-' + _.first(this.config.rowsets).id].find('thead th');
            $ths.removeClass('col-filtered');
            this.widget.columnMatcher.forEach(function (column) {
                var $currentHeader = $ths.filter('th[data-index="' + column.dataIndex + '"]');
                $currentHeader.addClass('col-filtered');
            });
        },

        setEscapeMode : function (escape, bypassSelect) {
            if (escape !== this.escapeMode) {
                this.escapeMode = escape;
                if (this.escapeMode) {
                    this.$grid.addClass('escape-mode');
                    this.$grid.removeAttr('aria-activedescendant');
                } else {
                    this.$grid.removeClass('escape-mode');
                    if (!bypassSelect) {
                        var recordId;
                        if (this.widget.focus.record != null) {
                            recordId = this.widget._getRecordId(this.widget.focus.record);
                        } else {
                            recordId = this.findFirstRow();
                        }
                        this.widget.selectCell(recordId);
                    }
                    this.focusCellSelection();
                }
                this.widget._trigger('view-sync-escape-mode', null, { escapeMode : this.escapeMode });
            }
        },

        setRowsetMessage : function (message, rowsetId) {
            if (rowsetId == null || this.$rowsets[rowsetId] == null) {
                rowsetId = this.widget.defaultRowset;
            }
            this.$rowsets[rowsetId].find('.rowset-message').remove();
            if (message != null) {
                // TODO: figure out better way to append to a specific viewport
                // In the meantime, find the first viewport that is not the menu button group and prepend
                // it there
                this.$rowsets[rowsetId].find('.viewport:not(.viewport-menu-button-group):not(.viewport-row-selector-group):eq(0)')
                        .prepend(emptyRowsetTemplate({ message : message }));
            }
        },

        fixRecordIdType : function (recordId) {
            if (recordId != null && this.options.recordKeys.numericalId && digitCheck.test(recordId)) { // g to detect grouping, D for DUMMY NEW RECORD
                return parseInt(recordId, 10);
            }
            return recordId;
        },

        findFirstRow : function (skipQuickAdd) {
            var that = this;

            var rowset = _.find(this.options.layout.rowsets, function (rowset) {
                return rowset.id !== constants.HEADERS_ROWSET_ID && that.virtualRows[rowset.id].records.length && (!skipQuickAdd || rowset.id !== constants.QUICK_ADD_ROWSET_ID);
            });

            if (rowset) {
                return that.virtualRows[rowset.id].records[0];
            }
        },

        findFirstNonGroupRow : function (skipQuickAdd) {
            var that = this;

            var rowset = _.find(this.options.layout.rowsets, function (rowset) {
                return rowset.id !== constants.HEADERS_ROWSET_ID && that.virtualRows[rowset.id].records.length && (!skipQuickAdd || rowset.id !== constants.QUICK_ADD_ROWSET_ID);
            });

            return _.find(that.virtualRows[rowset.id].records, function (recordId) {
                return !that.widget._isGroupRecordId(recordId);
            });
        },

        changeTimescale : function (timescale) {
            this.options.timephased.scale = timescale;
            this.timephasedColumnGroup.applyTimescale(timescale);
            this.timephasedColumnGroup.resizeTimescale(this.options.timephased.bucketWidth);
            this.viewports['$group-' + this.timephasedGroupId].find('th.timephased-column').attr('width', this.options.timephased.bucketWidth);
            this.widget.resize();
            this.toggleColumns([]);
        },

        setTimescaleView : function () {
            this.timephasedColumnGroup.setViewRange(this.options.timephased.view);
            this.changeTimescale(this.options.timephased.scale);
            this.widget.selectCell(null, this.timephasedColumnGroup.firstVisibleTimephasedColumn, null, null);
        },

        resetVirtualRows : function () {
            var that = this;
            this.virtualRows = {};
            _.forEach(that.config.rowsets, function (rowset) {
                that.virtualRows[rowset.id] = {
                    headerRow : rowset.header != null,
                    displayTop : 0,
                    displayBottom : 10,
                    visibleRows : 0,
                    renderedTop : -1,
                    renderedBottom : -1,
                    records : [],
                    nodes : {}
                };
            });
        },

        destroy : function () {
            this.closeEditor(true);    // force close
            this.closeMenus();
            this.removeListeners();
            this.rowExpansionManager.destroy();
            if (this.advancedFilters) {
                this.advancedFilters.destroy();
            }
        }
    });
    return gridView;
}
);
