define([
    'gridapi.lodash',
    'gridapi.logger',
    'gridapi.grid/rollups'
], function (_, logger, rollups) {
    'use strict';

    var api = {
        rollups : rollups,
        normalizeDataIndices : function normalizeDataIndices(dataIndices) {
            var result;
            if (dataIndices != null) {
                if (_.isArray(dataIndices)) {
                    result = _.compact(dataIndices);
                    if (result.length === 0) {
                        result = null;
                    }
                } else if (_.isString(dataIndices)) {
                    //>>excludeStart("validation", pragmas.production)
                    // TODO investigate removing in a major release
                    logger.warn('dataIndices to rollup should be array - file a bug against grid, w/ stacktrace, for perf boost');
                    //>>excludeEnd("validation")
                    result = [dataIndices];
                }
            }
            return result;
        },

        /*
         * Given a list of leaf nodes, this will return a processing order for rollups such that parent nodes only occur after all their children
         *  have been processed, and parent nodes only appear in the result once.
         *
         */
        rollupOrder : function rollupOrder(groupIds, getGroupFn) {
            /* eslint-disable no-continue */
            var i, len;
            var breadthQueued = [];
            var currentQueue = groupIds;
            var parentQueue = [];

            // step 1: kind of a negative breadth-first search, putting parents of leaf nodes on queue once leafs are in.  parents are duplicated, that's ok
            while (currentQueue.length) {
                for (i = 0, len = currentQueue.length; i < len; ++i) {
                    var groupId = currentQueue[i];
                    var group = getGroupFn(groupId);
                    if (group == null) {
                        continue;
                    }
                    breadthQueued.push(groupId);
                    var parentId = group.parentId;
                    if (parentId != null) {
                        parentQueue.push(parentId);
                    }
                }
                currentQueue = _.uniq(parentQueue);
                parentQueue = [];
            }

            // step 2: to avoid re-rolling parent nodes, we only take the last occurence of any particular node from step 1
            var seenNodes = {};
            var result = [];
            for (i = breadthQueued.length - 1; i >= 0; --i) {
                var queuedGroupId = breadthQueued[i];
                if (!seenNodes[queuedGroupId]) {
                    seenNodes[queuedGroupId] = true;
                    result.push(queuedGroupId);
                }
            }
            return result.reverse();
            /* eslint-enable no-continue */
        }
    };
    return api;
});
