define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/utils/keyCode',
    'gridapi.grid/gridMixins',
    'gridapi.logger',
    'markup!gridapi.grid/templates/expandedRow.htm:.htm-row-layout'
], function ($, _, keyCode, mixins, logger, expandedRowTemplate) {
    'use strict';

    var RowExpansionManager = function (args) {
        var that = this;
        that.options = args.options;
        that.widget = args.widget;
        that.view = args.gridView;

        _.forEach(that.options.rowExpansionFns, function (fnc, fncName) {
            that[fncName] = fnc || that[fncName];
        });

        that.visibleExpandedRows = {};

        that.expandedRows = [];
    };


    RowExpansionManager.prototype = {
        constructor : RowExpansionManager,

        expandRow : function (recordId) {
            if (this.options.enabled) {
                var that = this;
                var rowsetId = that.widget._getRecordRowset(recordId);
                var virtualRows = that.view.virtualRows[rowsetId];
                var node = virtualRows.nodes[recordId];
                if (!node.isNode || (node.expanded)) {
                    return;
                }
                var $rowExpansion = this.createExpansion({
                    recordId : recordId,
                    recordIndex : virtualRows.records.indexOf(recordId) + 1
                });
                this.visibleExpandedRows[recordId] = $rowExpansion;
                this.expandedRows.push(recordId);
                if (this.options.placement === 'rowset') {
                    this.view.$rowsets[rowsetId].find('.rowset-overlay').append($rowExpansion);
                } else if (this.options.placement === 'viewport') {
                    this.view.viewports[rowsetId][this.options.viewportPlacement].$overlay.append($rowExpansion);
                }
                this.repositionExpansions();
            }
        },

        collapseRow : function (recordId) {
            if (this.options.enabled) {
                var $rowExpansion = this.visibleExpandedRows[recordId] || $();
                $rowExpansion.remove();
                this.repositionExpansions();
            }
        },

        createExpansion : function (args) {
            var marginLeft = 7; // spacer size
            if (this.options.placement === 'rowset' && this.widget.options.contextMenu.enableRowMenuButton) {
                marginLeft += 22;// size of context menu
            }
            return $(expandedRowTemplate({
                id : 'grid' + this.widget.uuid + '-expandedRow-' + args.recordId,
                recordId : args.recordId,
                recordIndex : args.recordIndex,
                top : args.recordIndex * this.view.ROW_HEIGHT + 1,
                height : this.view.ROW_HEIGHT * this.options.heightInRows,
                marginLeft : marginLeft
            }));
        },
        // This will be refactored to work similar to relationship rendering to only draw visible items.
        // Not neccessary for the demo but it will be an important performance gain.
        // It also does not run when it would add new rows
        repositionExpansions : function () {
            var that = this;
            this.repositionTimeout = setTimeout(function () {
                that.repositionTimeout = null;
                _.forEach(that.expandedRows, function (recordId) {
                    var rowsetId = that.widget._getRecordRowset(recordId);
                    var virtualRows = that.view.virtualRows[rowsetId];

                    var top = (virtualRows.records.indexOf(recordId) + 1) * that.view.ROW_HEIGHT + 1;
                    that.visibleExpandedRows[recordId].css('top', top + 'px');
                });
            }, 1);
        },

        destroy : function () {
            if (this.repositionTimeout) {
                window.clearTimeout(this.repositionTimeout);
            }
        }
    };

    return RowExpansionManager;
});
