define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/utils'
], function ($, _, utils) {
    'use strict';

    var formatString = utils.formatString;
    return function (grid) {
        // MARKUP
        var $rowMoveShade = grid.$grid.find('.row-move-shade'),
            $rowMoveBar = grid.$grid.find('.row-move-bar'),
            $rowsetDrop = grid.$grid.find('.rowset-drop-shade');

        var $rows = $(), rows = [], offsets = {}, heights = {}, $scroller, rowsetConfig, whereToGo, rowsetId,
            positions = {}, direction = -1, scrollTimer, currentDropRowset = null, prevPointer;

        var mousemoveFn = function (autoScrollInterval) {
            return function (event) {
                onMoveFn(event.clientY + $(window).scrollTop(), autoScrollInterval);
            };
        };
        var onMoveFn = function (mouseY, autoScrollInterval) {
            direction = mouseY - offsets.grid - positions.pointer;
            positions.pointer = mouseY - offsets.grid;

            positionShadeFn(autoScrollInterval);
            if (currentDropRowset == null) {
                positionMoveBarFn();
            }
        };
        var mouseupFn = function (throttled) {
            return function (event) {
                if (event.which === 1 || event.type === 'touchend') { // for mouseEvent (1: Left button) OR touchEvent
                    // switch of mouse events
                    grid.$grid.off('mousemove.row-move');
                    $(document.body).off('mouseup.row-move');
                    clearTimeout(scrollTimer);
                    if (currentDropRowset == null) {
                        var virtualRows = grid.virtualRows[rowsetId];
                        var afterId = null;
                        if (whereToGo > (virtualRows.headerRow ? 1 : 0)) {
                            if (whereToGo > virtualRows.records.length) {
                                whereToGo = virtualRows.records.length - 1;
                            } else {
                                whereToGo -= (1 + (virtualRows.headerRow ? 1 : 0));
                            }
                            afterId = virtualRows.records[virtualRows.displayTop + whereToGo];
                        }
                        grid.widget._moveRows({ siblingId : afterId, rowsetId : rowsetId });
                    } else {
                        grid.widget._dropRows(rowsetId, currentDropRowset.rowset);
                    }

                    // hide the column move shade and bar
                    $rowMoveBar.hide();
                    $rowMoveShade.hide();
                    $rowsetDrop.hide();

                    $rows.removeClass('row-moving');
                    currentDropRowset = null;
                    whereToGo = null;
                }
                throttled.cancel(); // cancel throttled mousemove on mouseup
            };
        };

        var positionShadeFn = function (autoScrollInterval) {
            if (scrollTimer != null) {
                clearTimeout(scrollTimer);
                scrollTimer = null;
            }
            // Check standard case of moving within the same rowset
            if (positions.pointer > offsets.rowsetTop + heights.row && positions.pointer + heights.row < offsets.rowsetBottom) {
                $rowMoveShade.css('top', positions.pointer - heights.halfrow);
                $rowMoveShade.show();
                $rowsetDrop.hide();
                currentDropRowset = null;
                return;
            }
            // Check if the mouse is outside of the rowset
            if (positions.pointer < offsets.rowsetTop || positions.pointer > offsets.rowsetBottom) {
                positionRowsetDrop();
                return;
            } else {
                $rowsetDrop.hide();
                currentDropRowset = null;
            }

            // Check rowset boundaries for scrolling
            var topposition = positions.pointer - heights.row,
                bottomposition = positions.pointer + heights.row,
                scrollAmount = 0;
            if (topposition < offsets.rowsetTop) {
                topposition = offsets.rowsetTop;
                scrollAmount = 15;
            } else if (bottomposition > offsets.rowsetBottom) {
                topposition = offsets.rowsetBottom - heights.row;
                scrollAmount = -15;
            }
            $rowMoveShade.css('top', topposition);
            $rowMoveShade.show();

            // scroll if on edge
            if (scrollAmount !== 0) {
                scrollTimer = setInterval(function () {
                    scrollFn(scrollAmount);
                }, autoScrollInterval); // autoScrollInterval: for desktop 60ms and for touch 5ms
            }
        };

        var scrollFn = function (amount) {
            if ($scroller != null) {
                $scroller.$scroller.scrollTop($scroller.$scroller.scrollTop() - amount);
                direction = -amount;
                positionMoveBarFn();
            }
        };
        var withinRowsetFn = function (rowsetToCheck) {
            var values = {};
            values.top = rowsetToCheck.offset().top - offsets.grid;
            values.height = rowsetToCheck.height();
            values.bottom = values.top + values.height;
            values.fits = positions.pointer < values.bottom && positions.pointer > values.top;
            values.rowset = rowsetToCheck.data('id');
            return values;
        };

        var positionRowsetDrop = function () {
            var rowsetToCheck,
                dropRowset;
            // If mouse is still in the current rowset don't check
            if (currentDropRowset != null && positions.pointer < currentDropRowset.bottom && positions.pointer > currentDropRowset.top) {
                return;
            }
            // Check to find the mouse in rowsets above or below.
            if (positions.pointer < offsets.rowsetTop) {
                rowsetToCheck = rowsetConfig.prev; // Rowsets above
                while (rowsetToCheck != null) {
                    dropRowset = withinRowsetFn(grid.$rowsets[rowsetToCheck.id]);
                    if (dropRowset.fits) {
                        break;
                    } else {
                        dropRowset = null;
                        rowsetToCheck = rowsetToCheck.prev;
                    }
                }
            } else {
                rowsetToCheck = rowsetConfig.next; // Rowsets below
                while (rowsetToCheck != null) {
                    dropRowset = withinRowsetFn(grid.$rowsets[rowsetToCheck.id]);
                    if (dropRowset.fits) {
                        break;
                    } else {
                        dropRowset = null;
                        rowsetToCheck = rowsetToCheck.next;
                    }
                }
            }
            // Clear if none was found
            if (dropRowset == null) {
                $rowsetDrop.css('height', 0);
                $rowsetDrop.css('top', 0);
                currentDropRowset = null;
                $rowsetDrop.hide();
            } else if (currentDropRowset !== dropRowset) {
                    // Check if they can drop the row into this viewport
                if (dropRowset.rowset === 'headers' || dropRowset.rowset === 'columnFilterRowsetId' || grid.widget._trigger('before-drop-row', null, { currentRowset : rowsetId, dropRowset : dropRowset.rowset, rows : rows }) === false) {
                    return;
                } else {
                    dropRowset.permission = true;
                }
                $rowsetDrop.css('height', dropRowset.height);
                $rowsetDrop.css('top', dropRowset.top);
                currentDropRowset = dropRowset;
                $rowsetDrop.show();
                $rowMoveBar.hide();
                $rowMoveShade.hide();
            }
        };

        var positionMoveBarFn = function () {
            var amountToAdd = heights.row;
            if (direction < 0) {
                amountToAdd = 0;
            }
            whereToGo = parseInt(((positions.pointer - offsets.rowsetTop + amountToAdd) / heights.row), 10);

            prevPointer = (prevPointer === undefined) ? positions.pointer : prevPointer;
            if (grid.options.tree && $scroller.$scroller.scrollTop() === 0 && whereToGo <= 0) { // auto scrolled to top
                whereToGo = (grid.options.tree) ? 1 : 0;
            } else if (Math.abs(prevPointer - positions.pointer) < 5) {  // do nothing, avoid flickery $rowMoveBar
                return false;
            } else {
                prevPointer = positions.pointer;  // allow normal flow
            }

            var linePosition = (heights.row * whereToGo) + offsets.rowsetTop;
            if (linePosition < offsets.rowsetTop) {
                linePosition = offsets.rowsetTop;
            } else if (linePosition > offsets.rowsetBottom) {
                linePosition = offsets.rowsetBottom;
            }

            $rowMoveBar.css('top', linePosition);
            $rowMoveBar.show();
        };
        // TODO: multiple row selection, dont have time for it
        return {
            /**
             * Trigger column resize
             * @param rowRowset - location of the row
             * @param $selection - selection containing the row
             * @method initMoveRow
             * @private
             */
            initMoveRow : function (rowRowset, $selection) {
                var count = grid.widget.getSelection().length;
                $rowMoveShade.find('.count span').text(formatString(grid.widget.options.locale.rowsSelected, count));
                rowsetConfig = rowRowset.data('rowset');
                rowsetId = rowRowset.data('id');
                $scroller = grid.pseudoScrollers.vertical[rowRowset.data('id')];
                offsets.grid = grid.$grid.offset().top;
                offsets.rowsetTop = rowRowset.offset().top - offsets.grid;
                offsets.rowsetBottom = offsets.rowsetTop + rowRowset.height();
                heights.row = grid.widget.options.layout.rowHeight;
                heights.halfrow = heights.row / 2;
                heights.rows = heights.row * $selection.length;

                if (rowsetId === 'headers') {
                    offsets.rowsetTop += heights.row;
                }

                currentDropRowset = null;
                whereToGo = null;
                $rows = $selection;
                $rows.each(function () {
                    var recordId = grid.fixRecordIdType($(this).attr('data-id'));
                    rows.push(recordId);
                });
                $rowMoveShade.show();
                $rows.addClass('row-moving');
                $rowMoveShade.css('height', heights.rows);
            },
            mousemoveFn: mousemoveFn,
            mouseupFn: mouseupFn
        };
    };
});
