define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/constants'
], function ($, _, constants) {
    'use strict';


    return function (grid) {
        if (grid == null) {
            throw new Error('SelectionModel - grid param not set');
        }
        var focus = {
            recordId : null,
            column : null,
            recordIndex : null,
            rowsetId : null,
            groupId : null,
            viewport : null,
            columnIndex : null,
            pivotId : null,
            pivotIndex : null,
            $td : $()
        };
        var cellFocusedClass = grid.options.classes.cellFocused || 'cell-selected';
        var _cell = {
            _adjustPivotIndex : function (appendToSelection) {
                if (appendToSelection) {
                    if (focus.pivotIndex == null) {
                        focus.pivotIndex = focus.recordIndex;
                        focus.pivotId = focus.recordId;
                    }
                } else {
                    focus.pivotIndex = null;
                    focus.pivotId = null;
                }
            },

            _updateShiftSelection : function () {
                var lowerBounds, upperBounds,
                    clickedIndex = focus.recordIndex;

                if (focus.pivotIndex > clickedIndex) {
                    lowerBounds = clickedIndex;
                    upperBounds = focus.pivotIndex;
                } else {
                    lowerBounds = focus.pivotIndex;
                    upperBounds = clickedIndex;
                }

                var selection = grid.virtualRows[focus.rowsetId].records.slice(lowerBounds, upperBounds + 1);


                grid.widget.selectRows(selection, false, null, focus.columnId, { byPassBeforeTrigger : true });
            },
            moveUp : function (appendToSelection, moveOnlyFocus) {
                this._adjustPivotIndex(appendToSelection);
                if (focus.rowsetId != null) {
                    var renderLocation = 'top';
                    if (focus.subRow != null && focus.subRowPosition > 0) {
                        focus.subRowPosition--;
                    } else {
                        if (focus.subRow != null) {
                            focus.subRowPosition = focus.viewport.groupConfig.stackedCells.length - 1;
                        }
                        // Standard move up case
                        if (focus.recordIndex > 0) {
                            focus.recordIndex--;
                            // At the top of a rowset
                        } else {
                            var newRowset = focus.viewport.rowsetConfig.prev;

                            if (newRowset != null && !(newRowset.id === 'headers' && focus.columnId === 'menu-button-column')) {
                                focus.recordIndex = grid.virtualRows[newRowset.id].records.length - 1;
                                focus.rowsetId = newRowset.id;
                                focus.viewport = grid.viewports[newRowset.id][focus.groupId];
                                renderLocation = 'bottom';

                                if (newRowset.id === 'headers') {
                                    grid.requestFocus(); // LUX-506 give focus back to scrollyGrid moveUp_outOf
                                    grid.widget.selectHeaderCell(focus.columnId);
                                    return;
                                }
                                if (focus.rowsetId === constants.COLUMNFILTER_ROWSET_ID) { // LUX-506
                                    grid.widget.selectColumnFilterCell(focus.columnId); // selecting a cell without recordId
                                    grid.widget.view.columnDefinitions[focus.columnId].columnFilterRenderer.moveUp_into({ 'colId': focus.columnId });
                                    return;
                                }
                                if (focus.rowsetId === constants.ADVANCED_FILTER_ROWSET_ID) {
                                    grid.widget.selectAdvancedFilterCell(focus.columnId); // selecting a cell without recordId
                                    return;
                                }
                                // Nothing to go to above leave selection
                            } else {
                                return;
                            }
                        }
                    }
                    var newFocusId = grid.virtualRows[focus.rowsetId].records[focus.recordIndex];
                    focus.recordId = newFocusId;
                    if (focus.subRow != null) {
                        focus.subRow = focus.viewport.groupConfig.stackedCells[focus.subRowPosition];
                    }

                    if (moveOnlyFocus) {
                        grid.widget.focus.record = grid.widget._getRecord(focus.recordId);
                        grid.widget.focus.recordId = focus.recordId;
                        grid.renderFocus(focus.recordId, focus.rowsetId, focus.columnId, 'top');
                        grid.scrollFocusIntoView();
                        grid.focusRow(focus.recordId, focus.rowsetId, focus.columnId);
                    } else {
                        if (appendToSelection) {
                            this._updateShiftSelection();
                        }
                        grid.widget.selectCell(newFocusId, focus.columnId, renderLocation, appendToSelection, focus.subRow, undefined, { byPassBeforeTrigger : appendToSelection });
                    }
                }
            },

            moveDown : function (appendToSelection, moveOnlyFocus) {
                this._adjustPivotIndex(appendToSelection);
                if (focus.rowsetId != null) {
                    var renderLocation = 'bottom';
                    if (focus.subRow != null && focus.subRowPosition < focus.viewport.groupConfig.stackedCells.length - 1) {
                        focus.subRowPosition++;
                    } else {
                        if (focus.subRow != null) {
                            focus.subRowPosition = 0;
                        }
                        // standard move down case
                        if (focus.recordIndex < grid.virtualRows[focus.rowsetId].records.length - 1) {
                            focus.recordIndex++;
                            // At the bottom move to the next rowset
                        } else {
                            var newRowset = focus.viewport.rowsetConfig.next;
                            if (focus.rowsetId === constants.COLUMNFILTER_ROWSET_ID) { // LUX-506  moveDown_outof
                                grid.requestFocus(); // LUX-506 give focus back to scrollyGrid
                            } else if (focus.rowsetId === constants.ADVANCED_FILTER_ROWSET_ID) {
                                grid.requestFocus();
                            }
                            if (newRowset != null) {
                                focus.recordIndex = 0;
                                focus.rowsetId = newRowset.id;
                                focus.viewport = grid.viewports[newRowset.id][focus.groupId];
                                renderLocation = 'top';
                                // Nothing to go to below, leave selection
                            } else {
                                return false;
                            }
                        }
                    }
                    var newFocusId = grid.virtualRows[focus.rowsetId].records[focus.recordIndex];
                    focus.recordId = newFocusId;
                    if (focus.subRow != null) {
                        focus.subRow = focus.viewport.groupConfig.stackedCells[focus.subRowPosition];
                    }
                    if (moveOnlyFocus) {
                        grid.widget.focus.record = grid.widget._getRecord(focus.recordId);
                        grid.widget.focus.recordId = focus.recordId;
                        grid.scrollFocusIntoView();
                        grid.renderFocus(focus.recordId, focus.rowsetId, focus.columnId, 'bottom');
                        grid.focusRow(focus.recordId, focus.rowsetId, focus.columnId);
                    } else {
                        if (appendToSelection) {
                            this._updateShiftSelection();
                        }
                        if (focus.rowsetId === constants.COLUMNFILTER_ROWSET_ID) { // LUX-506
                            grid.widget.selectColumnFilterCell(focus.columnId);// selecting a cell without recordId
                            grid.widget.view.columnDefinitions[focus.columnId].columnFilterRenderer.moveDown_into({ 'colId': focus.columnId });
                            return;
                        }
                        if (focus.rowsetId === constants.ADVANCED_FILTER_ROWSET_ID) {
                            grid.widget.selectAdvancedFilterCell(focus.columnId);
                            return;
                        }
                        grid.widget.selectCell(newFocusId, focus.columnId, renderLocation, appendToSelection, focus.subRow);
                    }
                }
                return true;// only return false if it can not move down due to no row
            },

            moveLeft : function (fromTab, append) {
                if (focus.columnIndex != null) {
                    if (focus.columnIndex > 0) { // If you can move left do so
                        focus.columnIndex--;
                    } else { // Else find a group to the left.
                        var newGroup = focus.viewport.groupConfig.prev;
                        if (newGroup != null) { // If that group exists set the column to the last in the group.
                            if (newGroup.showStackedCells) {
                                if (focus.subRow == null) {
                                    focus.subRow = _.first(newGroup.stackedCells);
                                    focus.subRowPosition = 0;
                                }
                            } else {
                                delete focus.subRow;
                                delete focus.subRowPosition;
                            }
                            focus.columnIndex = newGroup.columns.length - 1;
                            focus.groupId = newGroup.id;
                            focus.viewport = grid.viewports[focus.rowsetId][focus.groupId];
                        } else if (grid.options.wrapSelection) {
                            if (focus.rowsetId !== 'headers') { // If there is no group move up and select the last
                                if (focus.viewport.groupConfig.showStackedCells) {
                                    if (focus.subRow != null) {
                                        focus.subRow = _.first(focus.viewport.groupConfig.stackedCells);
                                        focus.subRowPosition = 0;
                                    }
                                }
                                if (focus.recordIndex > 0 || focus.viewport.rowsetConfig.prev.id !== 'headers') { // But not for the top row as you must explictly tab up there
                                    focus.groupId = _.last(grid.config.groups).id;
                                    focus.viewport = grid.viewports[focus.rowsetId][focus.groupId];
                                    focus.columnIndex = focus.viewport.groupConfig.columns.length - 1;
                                    if (focus.viewport.groupConfig.columns[focus.columnIndex].dataIndex === 'empty-column') {
                                        this.moveLeft(fromTab, append);
                                    }
                                    focus.columnId = focus.viewport.groupConfig.columns[focus.columnIndex].dataIndex;
                                    if (focus.viewport.groupConfig.showStackedCells) {
                                        if (focus.subRow == null) {
                                            focus.subRow = _.first(focus.viewport.groupConfig.stackedCells);
                                            focus.subRowPosition = 0;
                                        }
                                    } else {
                                        delete focus.subRow;
                                        delete focus.subRowPosition;
                                    }
                                    this.moveUp();
                                    return;
                                } else if (fromTab && focus.viewport.rowsetConfig.prev.id === 'headers') { // if trying to move to the header row and you pressed tab allow it to happen
                                    this.moveUp();
                                    return;
                                }
                            } else if (fromTab) { // If you are in headers row and trying to shift tab out set the grid into escape mode
                                grid.setEscapeMode(true);
                                return;
                            }
                        } else {
                            return;
                        }
                    }
                    var newColumn = focus.viewport.groupConfig.columns[focus.columnIndex];
                    if (!newColumn.visible ||// If the column is not visible from the corrections above try again, easier than lots of loops up top
                        newColumn.dataIndex === 'empty-column' || (focus.rowsetId === 'headers' && newColumn.dataIndex === 'menu-button-column') || newColumn.removeFromTabOrder) {
                        this.moveLeft();
                    } else { // If it is visible update the column and select the cell or header cell accordingly
                        focus.columnId = newColumn.dataIndex;


                        if (focus.rowsetId === constants.COLUMNFILTER_ROWSET_ID) { // LUX-506
                            grid.widget.selectColumnFilterCell(focus.columnId);   // selecting a cell without recordId
                            grid.widget.view.columnDefinitions[focus.columnId].columnFilterRenderer.moveLeft({ 'colId': focus.columnId });
                            return;
                        }


                        if (focus.rowsetId === 'headers') {
                            grid.widget.selectHeaderCell(focus.columnId);
                        } else {
                            grid.widget.selectCell(focus.recordId, focus.columnId, null, append, focus.subRow);
                        }
                    }
                }
            },

            moveRight : function (fromTab, append) {
                if (focus.columnIndex != null) {
                    // If available column in this group to move to do so
                    if (focus.columnIndex < focus.viewport.groupConfig.columns.length - 1 &&
                        focus.viewport.groupConfig.columns[focus.columnIndex + 1].dataIndex !== 'empty-column') {
                        focus.columnIndex++;
                    } else {
                        // Else find the new group and set to the first column in that group
                        var newGroup = focus.viewport.groupConfig.next;
                        if (newGroup != null) { // first column in next group
                            if (newGroup.showStackedCells) {
                                if (focus.subRow == null) {
                                    focus.subRow = _.first(newGroup.stackedCells);
                                    focus.subRowPosition = 0;
                                }
                            } else {
                                delete focus.subRow;
                                delete focus.subRowPosition;
                            }
                            focus.columnIndex = 0;
                            focus.groupId = newGroup.id;
                            focus.viewport = grid.viewports[focus.rowsetId][focus.groupId];
                        } else if (grid.options.wrapSelection) {
                            if (focus.rowsetId !== 'headers' || fromTab) { // Else move down a row if you can for regular rows or tab in the header row
                                if (focus.viewport.groupConfig.showStackedCells) {
                                    if (focus.subRow != null) {
                                        focus.subRow = _.last(focus.viewport.groupConfig.stackedCells);
                                        focus.subRowPosition = focus.viewport.groupConfig.stackedCells.length - 1;
                                    }
                                }
                                if (!this.moveDown()) {
                                    if (fromTab) { // if you cant move down and tabbing set grid to escape mode
                                        grid.setEscapeMode(true);
                                    }
                                    return false; // else do not change the column selection leaving it on the last cell of the last row
                                } else { // You can move down so select the first column in the first group.
                                    focus.groupId = _.first(grid.config.groups).id;
                                    if (focus.groupId === 'menu-button-group') {
                                        focus.groupId = grid.config.groups[1].id;
                                    }
                                    focus.viewport = grid.viewports[focus.rowsetId][focus.groupId];
                                    focus.columnIndex = 0;
                                    focus.columnId = focus.viewport.groupConfig.columns[focus.columnIndex].dataIndex;
                                    if (focus.viewport.groupConfig.showStackedCells) {
                                        if (focus.subRow == null) {
                                            focus.subRow = _.first(focus.viewport.groupConfig.stackedCells);
                                            focus.subRowPosition = 0;
                                        }
                                    } else {
                                        delete focus.subRow;
                                        delete focus.subRowPosition;
                                    }
                                }
                            }
                        } else {
                            return;
                        }
                    }
                    if (!focus.viewport.groupConfig.columns[focus.columnIndex].visible ||
                        focus.viewport.groupConfig.columns[focus.columnIndex].dataIndex === 'empty-column' ||
                        focus.viewport.groupConfig.columns[focus.columnIndex].removeFromTabOrder ||
                        (focus.rowsetId !== 'headers' && grid.widget._isGroupRecordId(focus.recordId) && focus.viewport.groupConfig.columns[focus.columnIndex].hideGroupCell)) {
                        this.moveRight(fromTab, append);
                    } else {
                        focus.columnId = focus.viewport.groupConfig.columns[focus.columnIndex].dataIndex;

                        if (focus.rowsetId === constants.COLUMNFILTER_ROWSET_ID) { // LUX-506
                            grid.widget.selectColumnFilterCell(focus.columnId);    // selecting a cell without recordId
                            grid.widget.view.columnDefinitions[focus.columnId].columnFilterRenderer.moveRight({ 'colId': focus.columnId });
                            return;
                        }


                        if (focus.rowsetId === 'headers') {
                            grid.widget.selectHeaderCell(focus.columnId);
                        } else {
                            grid.widget.selectCell(focus.recordId, focus.columnId, null, append, focus.subRow);
                        }
                    }
                }
            },

            firstCellInGrid : function () {
                focus.columnId = grid.firstVisibleColumn;
                focus.recordId = _.first(grid.virtualRows[grid.firstRowsetId].records);
                grid.widget.selectCell(focus.recordId, focus.columnId);
            },

            lastCellInGrid : function () {
                focus.columnId = grid.lastVisibleColumn;
                focus.recordId = _.last(grid.virtualRows[_.last(grid.config.rowsets).id].records);
                grid.widget.selectCell(focus.recordId, focus.columnId);
            },

            firstCellInRow : function () {
                focus.columnId = grid.firstVisibleColumn;
                grid.widget.selectCell(focus.recordId, focus.columnId);
            },

            lastCellInRow : function () {
                focus.columnId = grid.lastVisibleColumn;
                grid.widget.selectCell(focus.recordId, focus.columnId);
            },

            // Called when a user clicks on a cell
            cellSelected : function ($td, options) {
                if (options == null) {
                    options = {};
                }
                if ($td != null && $td.length) {
                    // clear current selection
                    this.clear();

                    // cache
                    $td.addClass(cellFocusedClass);
                    var previousRow = focus.recordId,
                        previousCell = focus.columnId,
                        previousSubRow = focus.subRow;
                    this._updateFocusModel($td);
                    if (!options.skipPivotReset) {
                        focus.pivotId = null;
                        focus.pivotIndex = null;
                    }
                    var properFocus = _.clone(focus);
                    // close any other open editors
                    grid.closeEditor();

                    grid.widget.requestBrowserFocus(true, options.event);

                    // move focus
                    grid.widget.selectCell(properFocus.recordId, properFocus.columnId, 'bottom', options.append, properFocus.subRow);
                    if (properFocus.recordId === previousRow && properFocus.columnId === previousCell && properFocus.subRow === previousSubRow && grid.columnDefinitions[properFocus.columnId].displayType === 'select') {
                        setTimeout(function () {
                            grid.widget.editRow({
                                recordId : properFocus.recordId,
                                columnId : properFocus.columnId
                            });
                        }, 50);
                    }
                }
            },

            focusCell : function ($td) {
                if (focus.$td != null && focus.$td.length && focus.$td.is('.' + cellFocusedClass)) {
                    focus.$td.removeClass(cellFocusedClass);
                }

                this.clear();
                this._updateFocusModel($td);

                focus.$td = $td;
                focus.$td.addClass(cellFocusedClass);
                grid.widget.requestBrowserFocus();
                grid.widget.view.touchInteractions.resetDragHandles();
            },

            selectHeader : function (virtualRow, columnId) {
                if (focus.$td.length && focus.$td.is('.' + cellFocusedClass)) {
                    focus.$td.removeClass(cellFocusedClass);
                }
                focus.rowsetId = 'headers';
                focus.recordId = null;
                focus.recordIndex = -1;
                focus.columnId = columnId;
                focus.viewport = grid.viewports[focus.rowsetId][focus.groupId];
                if (focus.viewport && focus.viewport.groupConfig && focus.viewport.groupConfig.columns) {
                    for (var j = 0; j < focus.viewport.groupConfig.columns.length; j++) {
                        if (focus.viewport.groupConfig.columns[j].dataIndex === focus.columnId) {
                            focus.columnIndex = j;
                            break;
                        }
                    }
                }
                focus.$td = virtualRow.find('th[data-index="' + columnId + ['"]']);
                focus.$td.addClass(cellFocusedClass);
            },

            selectColumnFilter : function (virtualRow, columnId) {    // LUX-506
                if (focus.$td.length && focus.$td.is('.' + cellFocusedClass)) {
                    focus.$td.removeClass(cellFocusedClass);
                }
                focus.rowsetId = grid.options.advancedFilters.enabled ? constants.ADVANCED_FILTER_ROWSET_ID : constants.COLUMNFILTER_ROWSET_ID;
                focus.groupId = grid.getColumnGroupId(columnId);
                focus.recordId = null;
                focus.recordIndex = -1;
                focus.columnId = columnId;
                focus.viewport = grid.viewports[focus.rowsetId][focus.groupId];
                if (focus.viewport && focus.viewport.groupConfig && focus.viewport.groupConfig.columns) {
                    for (var j = 0; j < focus.viewport.groupConfig.columns.length; j++) {
                        if (focus.viewport.groupConfig.columns[j].dataIndex === focus.columnId) {
                            focus.columnIndex = j;
                            break;
                        }
                    }
                }
                focus.$td = virtualRow.find('th[data-index="' + columnId + ['"]']);
                focus.$td.addClass(cellFocusedClass);
            },

            _updateFocusModel : function ($td) {
                focus.columnId = $td.attr('data-index');

                focus.recordId = grid.fixRecordIdType($td.parent().attr('data-id'));

                focus.viewport = $td.closest('div.viewport').data().viewport;
                focus.rowsetId = focus.viewport.rowsetConfig.id;
                focus.groupId = focus.viewport.groupConfig.id;
                focus.recordIndex = grid.getRecordIndex(focus.recordId, focus.rowsetId);

                if (focus.viewport.groupConfig.showStackedCells) {
                    focus.subRow = $td.attr('data-sub-row');
                    focus.subRowPosition = focus.viewport.groupConfig.stackedCells.indexOf(focus.subRow);
                } else {
                    delete focus.subRow;
                    delete focus.subRowPosition;
                }

                for (var j = 0; j < focus.viewport.groupConfig.columns.length; j++) {
                    if (focus.viewport.groupConfig.columns[j].dataIndex === focus.columnId) {
                        focus.columnIndex = j;
                        break;
                    }
                }

                focus.$td = $td;
            },

            _updateFocusModelFromIds : function (recordId, columnId, subRow) {
                focus.columnId = columnId;
                focus.recordId = recordId;
                focus.rowsetId = grid.widget._getRecordRowset(recordId);
                focus.groupId = grid.getColumnGroupId(columnId);
                focus.viewport = grid.viewports[focus.rowsetId][focus.groupId];
                focus.recordIndex = grid.getRecordIndex(focus.recordId, focus.rowsetId);

                if (focus.viewport != null) {
                    for (var j = 0; j < focus.viewport.groupConfig.columns.length; j++) {
                        if (focus.viewport.groupConfig.columns[j].dataIndex === focus.columnId) {
                            focus.columnIndex = j;
                            break;
                        }
                    }
                    if (focus.viewport.groupConfig.showStackedCells) {
                        focus.subRow = subRow;
                        focus.subRowPosition = focus.viewport.groupConfig.stackedCells.indexOf(focus.subRow);
                    } else {
                        delete focus.subRow;
                        delete focus.subRowPosition;
                    }
                }
            },

            clear : function () {
                grid.$grid.find('.' + cellFocusedClass).removeClass(cellFocusedClass);
            }
        };
        var that = {
            focus : focus,

            clearFocus : function () {
                focus.rowsetId = null;
                focus.recordId = null;
                focus.viewport = null;
                focus.columnId = null;
                focus.groupId = null;
                focus.pivotIndex = null;
                focus.pivotId = null;
                focus.pivotTopId = null;
                focus.pivotBottomId = null;
            },

            updateFocus : function (record) {
                focus.recordId = grid.widget._getRecordId(record);
                focus.rowsetId = grid.widget._getRecordPlacement(record).rowsetId;
                focus.recordIndex = grid.getRecordIndex(focus.recordId, focus.rowsetId);
            },

            cell : _cell,

            getShiftSelection : function (rowsetId, recordId) {
                var selectedRowset = focus.rowsetId;
                var selection = [];

                if (selectedRowset === rowsetId) {
                    var lowerBounds, upperBounds,
                        currentIndex = focus.pivotIndex != null ? focus.pivotIndex : focus.recordIndex;

                    var clickedIndex = grid.getRecordIndex(recordId, rowsetId);

                    if (currentIndex > clickedIndex) {
                        lowerBounds = clickedIndex;
                        upperBounds = currentIndex;
                    } else {
                        lowerBounds = currentIndex;
                        upperBounds = clickedIndex;
                    }

                    focus.pivotIndex = currentIndex;
                    focus.pivotId = grid.virtualRows[focus.rowsetId].records[focus.pivotIndex];


                    selection = grid.virtualRows[focus.rowsetId].records.slice(lowerBounds, upperBounds + 1);
                    focus.pivotTopId = _.first(selection);
                    focus.pivotBottomId = _.last(selection);
                }

                return selection;
            },
            setPivotIndex : function () {
                if (focus.pivotIndex == null) {
                    focus.pivotId = focus.recordId;
                    focus.pivotIndex = focus.recordIndex;
                }
            }

        };
        return that;
    };
});
