define([
    'gridapi.jquery',
    'gridapi.lodash'
], function ($, _) {
    'use strict';


    var TIMEOUT = {  // ms
        SINGLE_TAP : 451,
        PRESS : 351,
        MOVE_THROTTLE : 50
    };

    var Gestures = function ($ele, loggerInstance) {
        var that = this;

        var timeoutSingletap, timeoutDoubletap, timeoutPress;
        that.tapCount = 0;
        var prev, next;
        that.registrableFns = ['touchStart', 'singleTap', 'doubleTap', 'touchMoveAll', 'touchMoveThrottle', 'touchEnd', 'allComplete', 'press'];

        that.logger = loggerInstance;

        $ele.on('touchstart.gridTouchHandler', function (event) {
            prev = {
                x : event.originalEvent.touches[0].pageX,
                y : event.originalEvent.touches[0].screenY
            };

            that.run('touchStart', [event]);

            that.tapCount++;
            if (that.tapCount > 1) { /* it's a doubleTap*/
                that.tapCount = 0;
                /*cancel singleTap*/
                clearTimeout(timeoutSingletap); clearTimeout(timeoutPress);
                /*STO-1 doubleTap probable*/
                timeoutDoubletap = setTimeout(function () { that.run('doubleTap', [event]); }, 1);
            } else {
                /*STO-351 press probable*/
                timeoutPress = setTimeout(function () { that.run('press', [event]); }, TIMEOUT.PRESS);
                /*attach singleTap only the first time; STO-451 singleTap probable*/
                timeoutSingletap = setTimeout(function () { that.run('singleTap', [event]); that.tapCount = 0; }, TIMEOUT.SINGLE_TAP);
            }
        });

        that.touchMoveThrottleFn = _.throttle(function (event) {
            var that = this;
            var direction;
            //------- calcualte direction ---------------
            next = {
                x : event.originalEvent.touches[0].pageX,
                y : event.originalEvent.touches[0].screenY
            };
            direction = (next.y > prev.y) ? 'down' : 'up';

            prev = next;
            //-----------------------
            that.run('touchMoveThrottle', [event, direction]);
        }, TIMEOUT.MOVE_THROTTLE, { 'leading': false, 'trailing': true });

        $ele.on('touchmove.gridTouchHandler', function (event) {
            // dragHandler calls 'preventDefault' so following code will not execute and singleTap will be fired, however this is not limitaion of this code.
            clearTimeout(timeoutSingletap);
            clearTimeout(timeoutDoubletap);

            that.tapCount = 0;

            that.touchMoveThrottleFn.apply(that, [event]);
            that.run('touchMoveAll', [event]);
        });

        $ele.on('touchend.gridTouchHandler touchcancel.gridTouchHandler', function (event) {
            prev = {}; next = {};
            that.run('touchEnd', [event]);
            var maxTimeout = _.max(_.values(TIMEOUT));
            setTimeout(function () { that.run('allComplete', [event]); }, maxTimeout);
            clearTimeout(timeoutPress);
        });
    };


    Gestures.prototype = {
        register : function (fnName, fnToRegister) {
            var that = this;
            if (_.isFunction(fnToRegister) && that.registrableFns.indexOf(fnName) > -1) {
                that[fnName] = fnToRegister;
            } else {
                that.logger.log(fnName + ' is not a function Or not registrable fn');
            }
        },

        run : function (fnName, args) {
            var that = this;
            if (that.registrableFns.indexOf(fnName) > -1 && _.isFunction(that[fnName])) {
                that[fnName].apply(null, args);
            }
        }
    };

    return Gestures;
});
