define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.moment',
    'gridapi.grid/utils/strings'
], function ($, _, moment, strings) {
    'use strict';

    return {
        hasOwn : function (obj, prop) {
            return Object.hasOwnProperty.call(obj, prop);
        },

        startOfWeek : function (date) {
            if (!moment.isMoment(date)) {
                return date;
            }
            return date.clone().startOf('day').subtract(date.day(), 'days');
        },
        endOfWeek : function (date) {
            if (!moment.isMoment(date)) {
                return date;
            }
            return date.clone().endOf('day').add(6 - date.day(), 'days');
        },

        remapColumns: function (colArray, renameMap) {
            if (!renameMap) {
                return colArray;
            } else if (_.isArray(colArray) && colArray.length) {
                return _.map(colArray, function (col) {
                    if (renameMap[col]) {
                        return renameMap[col];
                    }
                    return col;
                });
            } else if (!_.isArray(colArray) && _.isObject(colArray)) {
                return _.reduce(colArray, function (memo, col, index) {
                    if (renameMap[index]) {
                        memo[renameMap[index]] = col;
                    } else {
                        memo[index] = col;
                    }
                    return memo;
                }, {});
            } else {
                return colArray;
            }
        },

        timeoutFactory : function () {
            var timeouts = [];
            var intervals = [];
            return {
                setTimeout : function (fn, delay) {
                    timeouts.push(window.setTimeout(fn, delay));
                },
                setInterval : function (fn, delay) {
                    intervals.push(window.setInterval(fn, delay));
                },
                clear : function () {
                    _.forEach(timeouts, function (id) {
                        window.clearTimeout(id);
                    });
                    _.forEach(intervals, function (id) {
                        window.clearInterval(id);
                    });
                }
            };
        },

        calculateScrollbarWidth : function calculateScrollbarWidth() {
            var parent, child, width;
            parent = $('<div style="width:50px;height:50px;overflow:auto"><div/></div>').appendTo('body');
            child = parent.children();
            width = child.innerWidth() - child.height(99).innerWidth();
            parent.remove();
            return width;
        },

        basicTemplate : strings.template,
        formatString : strings.formatString
    };
});
