define(['gridapi.lodash'], function (_) {
    'use strict';

    function searchEnabled(groupIds, column, searchHidden) {
        var groupId = column.groupId;
        var visible = column.visible && _.includes(groupIds, groupId);
        var enabled = visible || searchHidden;
        return enabled && !column.internalDisplayOnly && column.searchable;
    }

    function isText(column) {
        return column.dataType === 'string' && _.includes(['text', 'textArea'], column.displayType);
    }

    function searchableType(column) {
        return _.includes(['select', 'link', 'number', 'picker'], column.displayType);
    }

    function customSearch(column) {
        return column.formattedSearch || _.isFunction(column.getSearchValue) || _.isFunction(column.generateSearchFn);
    }

    function hasSearchValue(column) {
        return isText(column) || searchableType(column) || customSearch(column);
    }

    /**
     * @method generateSearchableColumns
     * @param {Object} args Should have groups, columns and searchHidden
     */
    return function generateSearchableColumns(args) {
        var groupIds = _.map(args.groups, function (group) {
            return group.id;
        });

        return _.reduce(args.columns, function (searchableColumns, column) {
            if (searchEnabled(groupIds, column, args.searchHidden) && hasSearchValue(column)) {
                searchableColumns.push(column);
            }
            return searchableColumns;
        }, []);
    };
});

