
define([], function () {
    'use strict';

    var logger;

    function getTimeStr(part) {
        return part < 10 ? '0' + part : part;
    }

    function messageStamp(method) {
        var now = new Date();
        var datestamp = getTimeStr(now.getHours()) +
            ':' + getTimeStr(now.getMinutes()) + ':' + getTimeStr(now.getSeconds());

        return '[' + datestamp + ' ' + method.toUpperCase() + ']';
    }

    function log(method, str) {
        var message = messageStamp(method) + ' ' + str;

        if (window.console && window.console[method]) {
            window.console[method](message);
        }
        // Normalize 'fatal' to console.error; more likely to exist in browser console
        else if (method === 'fatal' && window.console && window.console.error) {
            window.console.error(message);
        } else if (window.console && window.console.log) {
            window.console.log(message);
        }

        try {
            var err = new Error();
            if (err.stack) {
                var fileLoc = err.stack.match(/(http:\/\/.*\.js.*\d+(?::\d+)?)/g)[2];
                if (window.console.log) {
                    window.console.log('\tat: ' + fileLoc);
                }
            }
        } catch (e) {}
    }

    logger = {
        trace: function (str) {}, // Only used for PGBU internal libraries, not pages project
        debug: function (str) {
            log('debug', str);
        },
        info: function (str) {
            log('info', str);
        },
        warn: function (str) {
            log('warn', str);
        },
        error: function (str) {
            log('error', str);
        },
        fatal: function (str) {
            log('fatal', str);
        }
    };

    logger.assert = function assert(val, message) {
        if (!val) {
            logger.fatal('ASSERTION FAILED: ' + message);
            throw new Error('ASSERTION FAILED: ' + message);
        }
    };

    return logger;
});
