
define(['primutils.log4javascript', 'primutils.lodash'], function (log4j, _) {
    'use strict';

    var logger;

    var eventQueue = [],
        debugLogs = [],
        appender = new log4j.Appender(),
        subAppender = new log4j.PopUpAppender(),
        hasFired = false;

    logger = log4j.getDefaultLogger();

    logger.removeAllAppenders();

    function inQueue (event) {
        return _.find(eventQueue, function (item) {
            return _.first(item.event.messages) === _.first(event.messages);
        });
    }

    function fireEvent(event) {
        var queueItem = inQueue(event);

        if (!hasFired) {
            subAppender.append(new log4j.LoggingEvent(logger, new Date(), log4j.Level.INFO, ['Some of the following DEBUG/INFO logs were hidden until this ' + event.level.toString() + ':']));
            _.each(debugLogs, function (debugEvent) {
                subAppender.append(debugEvent);
            });
            debugLogs = [];
        }

        hasFired = true;

        if (_.isUndefined(queueItem)) {
            eventQueue.push({
                event:     event,
                timeStamp: event.timeStampInMilliseconds,
                count:     0
            });

            subAppender.append(event);
        } else {
            queueItem.count += 1;
        }
    }

    function storeEvent(event) {
        if (hasFired) {
            fireEvent(event);
        } else {
            debugLogs.push(event);
        }
    }

    appender.append = function (loggingEvent) {
        switch (loggingEvent.level) {
        case log4j.Level.INFO:
        case log4j.Level.DEBUG:
        case log4j.Level.TRACE:
            storeEvent(loggingEvent);
            break;
        case log4j.Level.ERROR:
        case log4j.Level.WARN:
        case log4j.Level.FATAL:
            fireEvent(loggingEvent);
            break;
        default:
            break;
        }
    };

    setInterval(function() {
        var time = Date.now(),
            leaving = _.filter(eventQueue, function(event) {
                var eventTime = event.timeStamp;
                return Math.abs(time - eventTime) > 10;
            });

        _.each(leaving, function(event) {
            if (event.count > 0) {
                event.event.messages = ['[Occured ' + event.count + ' time(s)]'].concat(event.event.messages);
                subAppender.append(event.event);
            }
        });

        eventQueue = _.difference(eventQueue, leaving);
    }, 2000);

    logger.addAppender(appender);
    logger.addAppender(new log4j.BrowserConsoleAppender());

    logger.assert = function assert(val, message) {
        if (!val) {
            logger.fatal('ASSERTION FAILED: ' + message);
            throw new Error('ASSERTION FAILED: ' + message);
        }
    };

    // Usable functions:
    // logger.debug('msg');
    // logger.info('msg');
    // logger.warn('msg');
    // logger.error('msg');
    // logger.fatal('msg');

    return logger;
});
