/*jshint loopfunc: true */
/*global define */
/**
 * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * luxlogininfo module provides very simple API. It has two getter methods, two setter methods, and it triggers
 * two events. luxlogininfo module can be used to centrally organise your login related data which is accessible
 * to all other modules without the need of login widget.

 * @author Shailesh Zirpe
 */
//  Dependencies:  require.js ,jQuery events

/**
 * luxlogininfo
 * @module luxlogininfo
 */
define(function() {

    'use strict';
    /**
     * @alias module:luxlogininfo
     */
    var Singleton = function() {
        var loggedIn = false;
        var username = 'Anonymous';
        //getter
        /**
         * You can use the getLoginStatus method to retrieve the ‘loggedIn’ status stored inside the luxlogininfo module.
         * This method does not accept any arguments.
         * @returns {Boolean} ‘loggedIn’ status stored inside the luxlogininfo module
         * @example
         * <caption>use this getter method to retrive the loggedIn status from the module</caption>
         * var status = luxlogininfo.getLoginStatus();
         */
        var getLoginStatus = function() {
            return loggedIn;
        };
        /**
         * You can use the getUsername method to retrieve the ‘username’ stored inside the luxlogininfo module.
         * This method does not accept any arguments.
         * @returns {String} username stored inside the luxlogininfo module
         * @example
         * <caption>use this getter method to retrive the username from the module</caption>
         * var username  = luxlogininfo.getUsername();
         */
        var getUsername = function() {
            return username;
        };
        //setter
        /**
         * You can use the setLoginStatus method to set the ‘loggedIn’ status stored inside the luxlogininfo module.
         * @param {Boolean} loggedIn status to be stored inside the luxlogininfo module
         * @fires loginsuccess
         * @fires logoutsuccess
         * @example
         * <caption>use this setter method to set the loggedIn status in the luxlogininfo module</caption>
         * luxlogininfo.setLoginStatus(true);
         */
        var setLoginStatus = function(TorF) {
            if (typeof TorF === 'boolean') {
                var data = {};
                if (TorF) {
                    loggedIn = true;
                    data = {loggedIn: true, username: username};
                    /**
                     * This event is triggered on document node when the loggedIn status is set to true using the
                     * luxlogininfo.setLoginStatus() method. jQuery event object and data object are passed as parameters
                     * to the event handler. The data object has the two properties ‘loggedIn’ and ‘username’.
                     * For loginsuccess event the loggedIn property is always true.
                     * @event module:luxlogininfo#loginsuccess
                     * @param {jqEvent} event
                     * @param {object} data
                     * @param {Boolean} data.loggedIn loggedIn property is alwasys true, if this event is fired
                     * @param {string} data.username username stored in the module
                     * @example
                     * <caption>  Bind an event listener to the event:</caption>
                     *   $(document).on({'loginsuccess', function() {
                     *    console.log("printing loginStatus and Username");
                     *    console.log(luxlogininfo.getLoginStatus());
                     *    console.log(luxlogininfo.getUsername());
                     *   }});
                     */
                    $(document).trigger('loginsuccess', data);
                } else {
                    loggedIn = false;
                    data = {loggedIn: false, username: username};
                    /**
                     * This event is triggered on document node when the loggedIn status is set to false using the
                     * luxlogininfo.setLoginStatus() method. jQuery event object and data object are passed as parameters
                     * to the event handler. The data object has the two properties ‘loggedIn’ and ‘username’.
                     * For logoutsuccess event the loggedIn property is always false.
                     * @event module:luxlogininfo#logoutsuccess
                     * @param {jqEvent} event
                     * @param {object} data
                     * @param {Boolean} data.loggedIn loggedIn property is alwasys false, if this event is fired
                     * @param {string} data.username username stored in the module
                     * @example
                     * <caption>  Bind an event listener to the event:</caption>
                     *   $(document).on({'logoutsuccess', function() {
                     *        console.log("printing loginStatus and Username");
                     *        console.log(luxlogininfo.getLoginStatus());
                     *        console.log(luxlogininfo.getUsername());
                     *    }});
                     */
                    $(document).trigger('logoutsuccess', data);
                }
            }
        };
        /**
         * You can use the setUsername method to set the username stored inside the luxlogininfo module.
         * @param {String} username to be stored inside the luxlogininfo module
         * @example
         * <caption>use this setter method to set the username in the luxlogininfo module</caption>
         * luxlogininfo.setUsername('John Doe');
         */
        var setUsername = function(stringVal) {
            if (typeof stringVal === 'string') {
                username = stringVal;
            }
        };
        return {
            getLoginStatus: getLoginStatus,
            setLoginStatus: setLoginStatus,
            getUsername:    getUsername,
            setUsername:    setUsername
        };
    };
    return new Singleton();
});