
define('markup', ['gridapi.jquery', 'gridapi.lodash'],

    function($, _) {

        'use strict';

        var buildMap = {};
        var isBuild = false;

        function getText(fileName, url, callback, errback) {
            if (!isBuild) {
                require(['text!' + fileName], callback);
            } else {
                try {
                    var fs = require.nodeRequire('fs');
                    var file = fs.readFileSync(url, 'utf8');
                    //Remove BOM (Byte Mark Order) from utf8 files if it is there.
                    if (file.indexOf('\uFEFF') === 0) {
                        file = file.substring(1);
                    }
                    callback(file);
                } catch (e) {
                    errback(e);
                }
            }
        }

        function template(text) {
            var underscore = _;
            if (isBuild) {
            // TODO minify HTML
                underscore = require.nodeRequire('lodash');
            }
            return underscore.template(text.trim());
        }

        function filter(text, filterOn) {
            if (isBuild) {
                var cheerio = require.nodeRequire('cheerio');
                var jq = cheerio.load(text);
                return jq('script' + filterOn).html().trim();
            } else {
                return $(text).filter(filterOn).html().trim();
            }
        }

        return {

            load: function(name, req, onLoad, config) {
                if (config.isBuild) {
                    isBuild = true;
                }

                var fileName = name.split(':')[0];
                var optionalFilter = name.split(':')[1];

                getText(fileName, req.toUrl(fileName), function (text) {
                    var usableText = text;

                    if (optionalFilter) {
                        usableText = filter(usableText, optionalFilter);
                    }

                    buildMap[name] = usableText;
                    onLoad(template(usableText));
                }, function (e) {
                    onLoad.error(e);
                });
            },

            write: function(pluginName, moduleName, write) {
                if (buildMap[moduleName]) {
                    var source = template(buildMap[moduleName]).source;
                    write('define(\'' + pluginName + '!' + moduleName +
                                    '\', function () { return ' + source + ';});\n');
                }
            }

        };
    });

