/**
 * @module oj-lux-dashboard
 */

/**
 * Copyright (c) 2018 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * luxDashboard is a jQuery widget.
 *
 * To create the luxDashboard pattern, call luxDashboard({}) on any div element
 *
 */

var obj = /** @lends module:oj-lux-dashboard */{
/**
         * Options
         * <dl>

<dt>meta-store</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: The meta store defines the meta information about the tile, such as     <ul>
                                                                                                                        <li><b>tileId</b>: unique identifier for a tile</li>
                                                                                                                        <li><b>collapsedSizeClass</b>: css class that represents the tile content in collapsed state</li>
                                                                                                                        <li><b>expandedSizeClass</b>: css class that represents the tile content in expanded state</li>
                                                                                                                        <li><b>order</b>: positive natural number (1-based index) </li>
                                                                                                                        <li><b>inserted</b>: boolean value representing whether a tile is currently inserted in the Dashboard </li>
                                                                                                                        <li><b>flipped</b>: boolean value representing whether a tile is currently flipped in the Dashboard</li>
                                                                                                                        <li><b>expanded</b>: boolean value representing whether a tile is currently expanded in the Dashboard</li>
                                                                                                                </ul>
                    meta-store property allows you to specify either the default store (predefined RESTful store based on ojCollection) or use your custom store.
                </li>
                <li><i>Sub-properties</i>:
                    <ul>
                        <li><b>location</b>:  If you pass in an object, it would be used as a custom store. For a default store, you pass in a string representing the URL of the RESTful resource.
                            <ul>
                                <li><i>Example</i> :
<pre class="prettyprint" >
define([
    'ojs/ojcore',
    'knockout',
    'jquery',
    './customStore/customMetaStore',
    './customStore/metaStoreAdapterOverrides',
    'ojs/ojknockout',
    'oj-lux/oj-lux-dashboard/loader'
], function(oj, ko, $, customMetaStore, metaStoreAdapterOverrides) {

    var viewModel = function() {
        var self = this;

        self.metaStore = {
            location: customMetaStore,
            adapterOverrides: metaStoreAdapterOverrides
        };

        self.ctrlStore = {
            location: 'js/persistenceLayer/i18n/ctrlURL.json'
        }
    };
    return viewModel;
});
}</pre>
                                </li>
                            </ul>
                        </li>
                        <li><b>defaultStoreExtender</b>: This option allows for extending the default meta store.
                            <ul>
                                <li><b>object with adapter methods</b>:
                                    <ul>
                                        <li><i>default value</i> : null
                                        </li>
                                        <li><i>Example</i> : This example demonstrates how the default store can be extended to parse incoming data.
                                                            In this example the default model (model used in the default store) is extended with parse method.
                                                            The data coming from server does not have collapsedSizeClass and expandedSizeClass properties on it.
                                                            The example demonstrates how this properties can be added into the store during parse of the incoming data from the server.
        <pre class="prettyprint" >
        define(['ojs/ojcore', 'knockout', 'jquery', 'ojs/ojmodel'], function(oj, ko, $) {

            var parseData = function(data) {
                return {
                    "tileId": data.tileId,
                    "collapsedSizeClass": "oj-masonrylayout-tile-1x1",
                    "expandedSizeClass": "oj-masonrylayout-tile-3x1",
                    "order": data.order,
                    "inserted": data.inserted,
                    "flipped": data.flipped,
                    "expanded": data.expanded
                }
            };

            var MetaModel = oj.Model.extend({
                idAttribute: 'tileId',
                parse: parseData
            });

            return MetaModel;
        });
        </pre>
        Finally in the ViewModel the property is passed as Dashboard option.
        <pre class="prettyprint" >
        self.metaStore.defaultStoreExtender = {model: metaModel};
        </pre>
                                        </li>
                                    </ul>
                                </li>
                            </ul>
                        </li>
                        <li><b>adapterOverrides</b>:
                        adapterOverrides allows developers to override default adapter functionality or insert new functionality in the adapter. Developers would also need
                 to override adapter in case of implementing custom stores. Please refer to 'Overriding Adapter' demo and 'Custom store' demo for relevant use-cases.
                            <ul>
                                <li><b>object with adapter methods</b>:
                                    <ul>
                                        <li><i>default value</i> : null
                                        </li>
                                        <li><i>Example</i> : This example shows how adapter overriding can be used for custom cleanup when a certain tile is removed from the dashboard.
        <pre class="prettyprint" >
        var metaStoreAdapterOverrides = {
            'update': function(tileId, key, newVal) {
                var self = this;
                var model = self.metaStore.get(tileId);
                if (key === 'inserted' && newVal === false) {
                    console.log(tileId + ' removed custom cleanup after tile is removed');
                }
                model.set(key, newVal);
                model.save();
            }
        };
        </pre>
                                        </li>
                                    </ul>
                                </li>
                            </ul>
                        </li>
                        <li><b>persist</b>: This option decides whether changes made in UI (such as flipping a tile or removing a tile) updates the store or not. This is typically important during
                logging in and logging out process. The application typically would not want to persist changes made by a user who has not yet been logged-in, however as soon as the user logs in
                it is important to save the changes made by the user.
                            <ul>
                                <li><b>Boolean value</b>
                                    <ul>
                                        <li><i>default value</i> : true (boolean)
                                        </li>
                                    </ul>
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>ctrl-store</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: The ctrl store defines the control information about the tile, such as     <ul>
                                                                                                                        <li><b>tileId</b>: unique identifier for a tile</li>
                                                                                                                        <li><b>allow_insertRemove</b>: Defines whether a tile can be removed from the dashboard</li>
                                                                                                                        <li><b>allow_flip</b>:Defines whether a tile can be flipped</li>
                                                                                                                        <li><b>allow_reorder</b>:Defines whether a tile can be reordered</li>
                                                                                                                        <li><b>allow_resize</b>:Defines whether a tile can be expanded/collapsed</li>
                                                                                                                        <li><b>allow_flipExpand</b>:Defines whether a tile can be flipped and expanded at the same time</li>
                                                                                                                </ul>
                    ctrl-store is the client side persistence layer that defines the admin control exerted on the tile. This defines which actions can be performed on the tile by the user.
                    ctrl-store property allows you to specify either the default store (predefined RESTful store based on ojCollection) or use your custom store.
                </li>
                <li><i>Sub-properties</i>:
                    <ul>
                        <li><b>location</b>:  If you pass in an object, it would be used as a custom store. For a default store, you pass in a string representing the URL of the RESTful resource.
                        </li>
                        <li><b>defaultStoreExtender</b>: This option allows for extending the default control store.
                        </li>
                        <li><b>adapterOverrides</b>: This allows developers to override default adapter functionality or insert new functionality in the adapter. Developers would
                also need to override adapter in case of implementing custom stores. Please refer to 'Overriding Adapter' demo and 'Custom store' demo for relevant use-cases.
                        </li>
                        <li><b>admin</b>: This option should typically be only used for the administrator. After setting this option to true, hovering over the upper left corner of the tile
                shows a gear icon which opens a admin control dialog upon clicking.
                            <ul>
                                <li><b>Boolean value</b>
                                    <ul>
                                        <li><i>default value</i> : false (boolean)
                                        </li>
                                    </ul>
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>order-observer</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: This option allows for turning off persistence of reordering information. This option is only relevant when metaStore's persist option is set to 'true', ignored otherwise.
                Use-case: Sometime developers might choose to turn-off persistence of ordering information for performance reasons.
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>Boolean value</b>
                            <ul>
                                <li><i>default value</i> : true (boolean)
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>translations</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: This option allows for both ‘init’ time as well as run time ‘setter’ (overridden) or ‘getter’ of the i18n
                strings on the component (both batch or granular level, meaning set all strings at once or one at a time).
                LUX Dashboard uses internationalized strings with following priority:
                    <ol>
                        <li><b>translations option</b></li>
                        <li><b>i18n resource bundle</b></li>
                        <li><b>widget defaults</b></li>
                    </ol>
                </li>
                <li><i>Arguments</i>:
                    The translations option accepts object with following keys
                    <ul>
                        <li><b>removeTile</b></li>
                        <li><b>configureTile</b></li>
                        <li><b>flipBack</b></li>
                        <li><b>flipForward</b></li>
                        <li><b>reorderTitle</b></li>
                        <li><b>reorderLabel</b></li>
                        <li><b>resizeCollapse</b></li>
                        <li><b>resizeExpand</b></li>
                        <li><b>tileDialogTitle</b></li>
                        <li><b>addDashboardTilesButtonLabel</b></li>
                        <li><b>dashboardTileLabel</b></li>
                    </ul>
                </li>
                <li><i>Example</i>:

                    <ul>
                        <li><b>Overriding strings during initializing the widget</b>:
                             Initialize the component, overriding some translated resources. One or more strings can be overridden. If you override one string then, the other translations remain intact.
<pre class="prettyprint" >
    $('#divId_searchCol').luxDashboard({
        metaStore: 'useDefault',
        defaultMetaStoreURL: 'js/persistenceLayer/i18n/admin-metaURL.json',
        metaWriteBack: false,
        orderObserver: true,

        ctrlStore: 'useDefault',
        defaultCtrlStoreURL: 'js/persistenceLayer/i18n/admin-ctrlURL.json',
        ctrlWriteBack: false,
        translations: {"removeTile": "RemoveTileAppDuringInit"}  // set one during init.
    });
</pre>
                        </li>

                        <li><b>Set the translations option, after initialization</b>:
                             Use standard jQueryUI widget factory 'option' method as follows:
<pre class="prettyprint" >
    $('#divId_searchCol').luxDashboard("option", "translations.removeTile", "RemoveTileAppUsingMethod");
</pre>
                        </li>

                        <li><b>Set the translations option, after initialization</b>:
                             Use standard jQueryUI widget factory 'option' method as follows:
<pre class="prettyprint" >
    var str = $('#divId_searchCol').luxDashboard("option", "translations.removeTile");
</pre>
                        </li>
                    </ul>

                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>titleDescriptionInMetaStore</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: This option allows for alternate way of implementing internationalization in the luxDashboard. When using this option the 'title' and 'description'
                        strings must be defined in the metaStore. When locale is changed, the store must also be changed to reflect the translated versions
                        of the 'title' and 'description' strings for the new language. To update all the strings (generic and tile specific) call the
                        'refreshLocaleAndMetaStore' method on the luxDashboard by passing the new 'metaURL'. ('metaURL' is only required when using
                        'defaultMetaStoreURL' option. Not required for custom stores.)
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>Boolean value</b>
                            <ul>
                                <li><i>default value</i> : false (boolean)
                                </li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

<dt>viewInfoAll</dt>
<dd>
    <div class='row-fluid'>
        <div class='span6'>
            <ul>
                <li><i>Description</i>: Current state of tile can be accessed inside the application
                using the viewInfoAll property of the dashboard. viewInfoAll property is a 'tileId' keyed object,
                 each of these object has following four sub-properties. Any user action resulting in change of a tile
                 state will update the value of <code>viewInfoAll</code> property. Tile developers can setup different
                  contents inside each state of a tile and respond to <code>viewInfo</code> to represent different
                   contents inside the tile depending upon user actions.
                </li>
                <li><i>Arguments</i>:
                    <ul>
                        <li><b>tileId</b>
                            <ul>
                                <li><b>normalView</b>: neither flipped nor expanded</li>
                                <li><b>flippedView</b>: only flipped</li>
                                <li><b>expandedView</b>: only expanded</li>
                                <li><b>flippedExpandedView</b>: both flipped and expanded at the same time</li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </div>
    </div>
</dd>

* </dl>
*/
    options: {

    }
};

/* TBD
 * The method can be used to update values of strings representing text in multiple languages from the nls resource bundle when locale is changed.
 *
 * @method module:oj-lux-dashboard#refreshLocale
 * @public
 */

/* TBD/TBR
 * luxDashboard allows for swiftly replacing the data stores used. When using defaultMetaStore simply updating the value of
 * 'defaultMetaStoreURL' observable option is sufficient. However when using custom stores or overriding stores/adapters developers would need to call
 * this method to refresh the contents of the Dashboard to reflect the changed store. When updating both (meta and control) stores simultaneously,
 * developers instead should use 'reloadBothStores' option to avoid two refreshes on the screen.
 *
 * @method module:oj-lux-dashboard#reloadMetaStore
 * @param {string} defaultMetaStoreURL - (required only in case of using defaultMetaStore)
 * @public
 */

/* TBD
 * When using 'titleDescriptionInMetaStore' option on the luxDashboard; 'title' and 'description' properties must be defined inside the metaStore.
 * However when the locale of the application is changed, the developers need to both refresh the generalized translated strings and tile specific
 * translated strings (title and description). This can be achieved together using 'refreshLocaleAndMetaStore' method
 *
 * @method module:oj-lux-dashboard#refreshLocaleAndMetaStore
 * @param {string} defaultMetaStoreURL - (required only in case of using defaultMetaStore)
 * @public
 */

/* TBD/TBR
 * luxDashboard allows for swiftly replacing the data stores used. When using defaultCtrlStore simply updating the value of
 * 'defaultCtrlStoreURL' observable option is sufficient. However when using custom stores or overriding stores/adapters developers would need to call
 * this method to refresh the contents of the Dashboard to reflect the changed store. When updating both (meta and control) stores simultaneously,
 * developers instead should use 'reloadBothStores' option to avoid two refreshes on the screen.
 *
 * @method module:oj-lux-dashboard#reloadCtrlStore
 * @param {string} defaultCtrlStoreURL - (required only in case of using defaultCtrlStore)
 * @public
 */

/**
 * Call this method reloads the data stores if you make any changes to the component property metaStore's/ctrlStore's location and/or adapterOverrides (e.g. when you want to change the store location URL, or switch to use a custom store object).
 *
 * @method module:oj-lux-dashboard#reloadStores
 * @public
 */

/* TBD
 * insertTile method allows developers to programmatically insert a tile into the dashboard.
 * Note that a tile to be inserted must already be configured (have valid metaStore and ctrlStore information associated with it.)
 * and metaStore's inserted property must be false ("inserted": false). If the tile is already inserted, no action is taken.
 * Fires 'tileInserted' upon successful insertion of a tile.
 *
 * @method module:oj-lux-dashboard#insertTile
 * @param {object} object
 * @param {array} object.tileId - tileId of the tile to be inserted as defined in the metaStore and ctrlStore
 * @param {array} object.index - zero based index for the position where the tile would be inserted
 * @public
 */

/**
 * Returns an array of all the tiles currently present in the Dashboard (including removed tiles but not the once not loaded in the store)
 *
 * @method module:oj-lux-dashboard#getAllTileIds
 * @returns {array} array of tileId
 * @public
 */

/* TBD
 * Returns default meta store (oj.Collection) constructor for use with 'defaultMetaStoreExtender'. This is useful if you are overriding a method in 'defaultMetaStoreExtender'
 * and wants to refer to the 'super' of the same method.
 *
 * @method module:oj-lux-dashboard#getDefaultMetaStore
 * @returns {object} parent store
 * @public
 */

/* TBD
 * Returns default ctrl store (oj.Collection) constructor for use with 'defaultCtrlStoreExtender'. This is useful if you are overriding a method in 'defaultCtrlStoreExtender'
 * and wants to refer to the 'super' of the same method.
 *
 * @method module:oj-lux-dashboard#getDefaultCtrlStore
 * @returns {object} parent store
 * @public
 */

/* TBD
 * Returns an array of all the inserted tiles.
 *
 * @method module:oj-lux-dashboard#getInsertedTileIds
 * @returns {array} array of tileId
 * @public
 */

/**
 * This event is triggered when promises are resolved asynchronously for both stores.
 *
 * @event module:oj-lux-dashboard#dataReady
 * @param {jqEvent} event -  The jQuery event for this event
 * @param {object} data
 * @param {object} data.uuid uuid of the jQueryUI widget
 */

/**
 * This event is triggered when a tile is removed form the luxDashboard.
 *
 * @event module:oj-lux-dashboard#tileRemoved
 * @param {jqEvent} event -  The jQuery event for this event
 * @param {object} data
 * @param {object} data.uuid uuid of the jQueryUI widget
 * @param {object} data.tileId tileId of the tile that is removed.
 */

/**
 * This event is triggered when a tile is tileInserted into the luxDashboard.
 *
 * @event module:oj-lux-dashboard#tileInserted
 * @param {jqEvent} event -  The jQuery event for this event
 * @param {object} data
 * @param {object} data.uuid uuid of the jQueryUI widget
 * @param {object} data.tileId tileId of the tile that is inserted.
 */

/**
 * This event is triggered when a tile is resized in luxDashboard
 *
 * @event module:oj-lux-dashboard#tileResized
 * @param {jqEvent} event -  The jQuery event for this event
 * @param {object} data
 * @param {object} data.uuid uuid of the jQueryUI widget
 * @param {object} data.tileId tileId of the tile that is resized.
 */

/**
 * This event is triggered when a tile is flipped in luxDashboard.
 *
 * @event module:oj-lux-dashboard#tileFlipped
 * @param {jqEvent} event -  The jQuery event for this event
 * @param {object} data
 * @param {object} data.uuid uuid of the jQueryUI widget
 * @param {object} data.tileId tileId of the tile that is flipped.
 * @param {object} data.flipDirection direction in which tile is flipped.
 */

/**
 * This event is triggered when tile state is changed based on the user action such when user clicks on the 'expand-tile' button to expand the tile.
 *
 * @event module:oj-lux-dashboard#viewInfoAllUpdate
 * @param {jqEvent} event -  The jQuery event for this event
 * @param {object} data
 * @param {object} data.viewInfo updated viewInfo object for the tile.
 * @param {object} data.tileId tileId of the tile that is updated.
 */
