/*
 * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * Implements a default adapter for lux Dashboard pattern.
 * storeAdapter is the only module in luxDashboard pattern who  knows about the store (ojCollection),
 *  and instantiates the store and orchestrates the store.
 * @author Shailesh Zirpe
 *
 */
define(['ojs/ojcore',
    'knockout',
    'jquery',
    'underscore',
    '../defaultStore/metaCollection',
    'ojs/ojknockout'
], function(oj, ko, $, _, MetaCollection) {
    'use strict';

    var metaStoreAdapter = function(widgetOptions) {
        var self = this;
        var DeveloperExtendedMetaCollection;
        // use customStore provided in widgetOptions.metaStore if don't want to use the defalutStore.
        if ((widgetOptions.metaStore === 'useDefault') || _.isUndefined(widgetOptions.metaStore)) {
            DeveloperExtendedMetaCollection = MetaCollection.extend(widgetOptions.defaultMetaStoreExtender);
            self.metaStore = new DeveloperExtendedMetaCollection();
        } else {
            self.metaStore = widgetOptions.metaStore;
        }

        //adapt the adapter using metaStoreAdapterOverrides if using customStore
        _.each(widgetOptions.metaStoreAdapterOverrides, function(value, key) {
            self[key] = value || self[key];
        });

    };

    var prototypeObj = {
        'initAdapter': function($deferred, defaultMetaStoreURL) {
            var self = this;
            var $dfd = $.Deferred();
            var metaArray = ko.observableArray();
            $dfd.done(function(resolveWithCollection) {
                metaArray = oj.KnockoutUtils.map(self.metaStore, null, true);
                $deferred.resolve(metaArray);
            });

            // store kick-starter progresses from VM->adapter->store
            self.metaStore.setURL_and_fetch(defaultMetaStoreURL, $dfd);
        },

        'update': function(tileId, key, newVal) {
            var self = this;
            var model = self.metaStore.get(tileId);
            model.set(key, newVal);
            model.save();
        }

    };

    $.extend(metaStoreAdapter.prototype, prototypeObj);
    return metaStoreAdapter;
});
