/**
 * @module oj-lux-grid-basic
 */
/* jshint ignore:start */
var obj = /** @lends module:oj-lux-grid-basic */
{
   /**
    * Options
<dl>
   <dt>data</dt>
   <dd>The data to bind to the element. Must be of type oj.CollectionPagingDataSource for grid with pagination option, or oj.CollectionTableDataSource for grid without pagination.
               <ul>
                  <li><b>required</b>: true</li>
                  <li><b>type</b>: object</li>
               </ul>
   </dd>
</dl>
<dl>
   <dt>columns</dt>
   <dd>Array of column objects specifying each column's data index name and definitions.
               <ul>
                  <li><b>required</b>: true</li>
                  <li><b>type</b>: Array&lt;object&gt;</li>
               </ul>
      <div class="row-fluid"><div class="span6">
         <ul>
            <li><b>dataIndex</b>: A string used as a unique key for that column.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>title</b>: A string used as a title for that column.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>dataType</b>: A string used to identify the type of data used in the column. The default 'string' dataType supports 'text', 'textArea', 'link' displayType. 'date' dataType supports 'date' displayType. 'boolean' dataType supports 'check' displayType. 'float' dataType supports 'number', 'cost', 'percent' displayType.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>enumValues</b>: string,date,boolean,float</li>
               </ul>
            </li>
            <li><b>displayType</b>: A string used to identify the column type. Default value is 'text'.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>enumValues</b>: text,textArea,date,check,link,number,cost,percent,select</li>
               </ul>
            </li>
            <li><b>headerAlign</b>: A string used to identify the orientation of column header text. Default value is 'left'.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>enumValues</b>: left,right,center</li>
               </ul>
            </li>
            <li><b>align</b>: A string used to identify the orientation of column data text. Default value is 'left'.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>enumValues</b>: left,right,center</li>
               </ul>
            </li>
            <li><b>spinner</b>: Determine if spinner control should be enabled for 'number', 'cost', 'percent' display types. Default value is false.
               <ul>
                  <li><b>type</b>: boolean</li>
               </ul>
            </li>
            <li><b>width</b>: A string used to set width in the format of '100px'. Default value is '100px'.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>sortable</b>: A boolean flag used to determine if the column is sortable. Default value is false.
               <ul>
                  <li><b>type</b>: boolean</li>
               </ul>
            </li>
            <li><b>resizable</b>: A boolean flag used to determine if the column is resizable. Default value is false.
               <ul>
                  <li><b>type</b>: boolean</li>
               </ul>
            </li>
            <li><b>tooltip</b>: A string used as a tooltip that is displayed when the mouse hovers over the column header. Default value is empty string.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>readOnly</b>:  A boolean flag used to prevent the column from being editable. Default value is false.
               <ul>
                  <li><b>type</b>: boolean</li>
               </ul>
            </li>
            <li><b>rowHeader</b>: A boolean flag used to determine if the this column should be used as ARIA row-header. If none of the columns is set to have rowHeader, then first column is used by default. Default value is false.
               <ul>
                  <li><b>type</b>: boolean</li>
               </ul>
            </li>
            <li><b>showIcon</b>: Show an icon in the column's cell. Default value is false. If 'true', this option also requires dataDisplay option.
               <ul>
                  <li><b>type</b>: boolean</li>
               </ul>
            </li>
            <li><b>dataDisplay</b>: Specify the order in which icon and value are displayed. This option is required if showIcon option is set to true.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>enumValues</b>: icon,icon-value,value-icon</li>
               </ul>
            </li>
            <li><b>bigNumber</b>: Specify if the column supports big number.
               <ul>
                  <li><b>type</b>: boolean</li>
                  <li><b>default value</b>: false</li>
               </ul>
            </li>
            <li><b>bigNumberMin</b>: Option to specify big-number's minimum value. Default value is undefined.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>bigNumberMax</b>: Option to specify big-number's maximum value. Default value is undefined.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>bigNumberStep</b>: Specify big-number's step value for the spinner. Default is 1.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>getFormatter</b>: Callback to return which big-number currency format to use for a given record (based on the record's 'currency' attribute, if exists, the currency value should be a ISO 4217 alphabetic currency code).
               <ul>
                  <li><b>type</b>: function(record)</li>
               </ul>
            </li>
            <li><b>getSortValue</b>: Callback to customize big-number sorting.
               <ul>
                  <li><b>type</b>: function(value, args)</li>
               </ul>
            </li>
            <li><b>getSearchValue</b>: Callback to format big-number for search.
               <ul>
                  <li><b>type</b>: function(record, dataIndex, column)</li>
               </ul>
            </li>
         </ul>
      </div></div>
   </dd>
</dl>
<dl>
   <dt>recordKeys</dt>
   <dd>Specify the value to override the default record keys.
               <ul>
               </ul>
      <div class="row-fluid"><div class="span6">
         <ul>
            <li><b>id</b>: Specify a value for the grid's record key 'id'.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: id</li>
               </ul>
            </li>
         </ul>
      </div></div>
   </dd>
</dl>
<dl>
   <dt>width</dt>
   <dd>The width of the grid, measured either in pixels or in percentage of the parent container. Make sure the grid width is two pixels more than the sum of the individual column widths, to account for the grid border width.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: 500px</li>
               </ul>
   </dd>
</dl>
<dl>
   <dt>height</dt>
   <dd> The height of the grid, measured either in pixels or in percentage of the parent container.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: 500px</li>
               </ul>
   </dd>
</dl>
<dl>
   <dt>editable</dt>
   <dd>Determines the read-only status of the grid. 'true' for editable, 'false' for read only.
               <ul>
                  <li><b>type</b>: boolean</li>
                  <li><b>default value</b>: false</li>
               </ul>
   </dd>
</dl>
<dl>
   <dt>selectionMode</dt>
   <dd>Used to enable the row selector column. 'check' will enable the row selector column, 'default' will use default row selection.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>enumValues</b>: default,check</li>
                  <li><b>default value</b>: default</li>
               </ul>
   </dd>
</dl>
<dl>
   <dt>multiSelect</dt>
   <dd>Allows the user to select multiple rows simultaneously. 'true' will allow users to select multiple rows, 'false' will allow users to select only one row.
               <ul>
                  <li><b>type</b>: boolean</li>
                  <li><b>default value</b>: false</li>
               </ul>
   </dd>
</dl>
<dl>
   <dt>columnFilters</dt>
   <dd>Option to add filter fields (text input fields) under each of the column headers. This option is not supported if 'selectionMode' and/or 'pagination' enabled.
               <ul>
                  <li><b>type</b>: boolean</li>
                  <li><b>default value</b>: false</li>
               </ul>
   </dd>
</dl>
<dl>
   <dt>focusOnSelection</dt>
   <dd>Determines whether the grid automatically gets browser focus when it's selected. Note that the page will automatically jump to the grid after it's loaded if this is set to true.
               <ul>
                  <li><b>type</b>: boolean</li>
                  <li><b>default value</b>: false</li>
               </ul>
   </dd>
</dl>
<dl>
   <dt>getColumnIconMarkupCallback</dt>
   <dd>Callback function to query for an icon markup string to render in the specified grid cell.
               <ul>
                  <li><b>type</b>: function(dataIndex, record): &lt;icon markup string&gt;</li>
               </ul>
   </dd>
</dl>
<dl>
   <dt>toolbar</dt>
   <dd>Specific toolbar options for the grid. If not defined, no toolbar component would be instantiated.
               <ul>
               </ul>
      <div class="row-fluid"><div class="span6">
         <ul>
            <li><b>border</b>: Decorate toolbar with top and/or bottom border
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>enumValues</b>: top,bottom,both</li>
               </ul>
            </li>
            <li><b>spec</b>: Toolbar's layout specification.
               <ul>
                  <li><b>type</b>: {buttons: [{label:'Button1', icons:{start: 'icon spec'}, display:'all'},...], menuItems: [{label:'Item1', icon:'icon spec'}, ...]}</li>
               </ul>
            </li>
            <li><b>isElementDisabledStateCallback</b>: Callback function used by the component to query if toolbar's button/menu item element should be disabled.
               <ul>
                  <li><b>type</b>: function(elementId, checkedRows): boolean</li>
               </ul>
            </li>
            <li><b>shouldIncludeElementCallback</b>: Callback function used by the component to query if button/menu item element should be included in the toolbar.
               <ul>
                  <li><b>type</b>: function(elementId): boolean</li>
               </ul>
            </li>
         </ul>
      </div></div>
   </dd>
</dl>
<dl>
   <dt>pagination</dt>
   <dd>Specific pagination options for the grid. If not defined, no paging control component would be instantiated.
               <ul>
               </ul>
      <div class="row-fluid"><div class="span6">
         <ul>
            <li><b>pageSize</b>: The page size required for the paging control.
               <ul>
                  <li><b>type</b>: number</li>
               </ul>
            </li>
         </ul>
      </div></div>
   </dd>
</dl>
<dl>
   <dt>translations</dt>
   <dd>Provide a collection of instance-level translation strings for the underlying embedded components.
               <ul>
                  <li><b>type</b>: Object</li>
                  <li><b>default value</b>: null</li>
               </ul>
      <div class="row-fluid"><div class="span6">
         <ul>
            <li><b>ojLuxGrid</b>: Collection of instance-level translation strings for the LUX data grid component.

For application-level translation overrides, please see oj-lux-grid's translations property (oj-lux-grid's component.json).
               <ul>
                  <li><b>type</b>: Object</li>
         <ul>
            <li><b>notSorted</b>: Click to sort ascending
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>sort</b>: Sort
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>sortAscending</b>: Sort Ascending
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>sortDescending</b>: Sort Descending
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>sortedAscending</b>: Sorted ascending, click to sort descending
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>sortedDescending</b>: Sorted descending, click to clear sort
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>openColumnGroupMenu</b>: Open Menu
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>columnAdjustToContent</b>: Adjust to fit content
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>columnHideColumn</b>: Hide column
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>columnToggle</b>: Columns
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>invalidFormatError</b>: Format not correct for cell.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>uniqueError</b>: {0}: Already exists. Enter a unique value.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>loading</b>: Loading...
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>required</b>: Required
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>autoNumbered</b>: Auto-Number
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>selectDeselect</b>: Select / Deselect
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>selectDeselectAll</b>: Selector Column
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>noResultsFound</b>: No rows match the current filter(s).
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>selected</b>: Selected
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>unselected</b>: Unselected
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>selectedColumns</b>: Selected Columns
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>unselectedColumns</b>: Unselected Columns
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>gridSummary</b>: Brief description about this grid.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>gridUsage</b>: Describe the grid's usage.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>gridLabel</b>: Specify an unique label that is used by screen readers to describe the grid to users.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>rowsSelected</b>: {0} Rows Selected
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>yes</b>: Yes
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>no</b>: No
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>quickAddHint</b>: Start typing...
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>quickAddHintType</b>: record
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>escapeModeText</b>: Press tab to skip over this grid widget. Press enter, escape or click to enter the grid.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>noActionsAvailable</b>: No Actions Available
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>noGroupValue</b>: None
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>popoverTooltip</b>: More Info
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>rowSelectorCol</b>: Rows can be selected with the check boxes in the first column of the grid.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>cellToolbar_newRow</b>: Add Row
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>cellToolbar_newChildRow</b>: Add Child Row
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>cellToolbar_deleteRows</b>: Delete Row
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>columnFilterPlaceholderText</b>: Filter
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>cellState</b>: Grid cell state's translation strings.
               <ul>
         <ul>
            <li><b>groupRowHeaderLabel</b>: Group Row
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>rowHeaderLabel</b>: Row
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>columnHeaderLabel</b>: Column
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>cellLabel</b>: Cell Value
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>editableCell</b>: Press f2 or enter to open editor.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>checkboxCell</b>: Press space to toggle checkbox
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>selectRowCell</b>: Press space to toggle row selection
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>linkCell</b>: Press space to activate link.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>errorOnCell</b>: Opening the editor will explain why the entry is invalid
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>menuLabel</b>: Row menu
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>menuButtonDescription</b>: Press space to open row menu.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>quickAddButtonDescription</b>: Press space to add a new row
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>errorsLabel</b>: Cell Errors
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>columnSort</b>: Press space to sort by column.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>selected</b>: Selected
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>unselected</b>: Unselected
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>mixed</b>: Partially selected
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>checkboxCellAll</b>: Press space to toggle checkbox and select or deselect all rows
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>submenuLabel</b>: sub menu
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>columnMenu</b>: Press space to open column menu.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>collapsed</b>: Press Control Right Arrow to expand
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>expanded</b>: Press Control Left Arrow to collapse
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>collapsedState</b>: collapsed
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>expandedState</b>: expanded
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>columnFilter</b>: Filter the data grid by column 
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>disableColumnFilter</b>: disabled
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
         </ul>
               </ul>
            </li>
            <li><b>columnEditors</b>: Grid column editor's translation strings.
               <ul>
         <ul>
            <li><b>text</b>: Standard text editor input, enter text for this column.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>number</b>: Enter a numerical value.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>select</b>: Combo box editor, press up and down to go through the list to select a value. Select value with enter.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>date</b>: Enter a date as text or press Down Arrow to open date picker.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>picker</b>: Press f2 to open a picker popup to select items.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>textarea</b>: Standard text area editor input, enter text for this column.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>cost</b>: Enter amount
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>percent</b>: Enter percentage
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>num_spin</b>:  or press up and down arrows to increment and decrement the value.
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
         </ul>
               </ul>
            </li>
            <li><b>export</b>: Grid export's translation strings.
               <ul>
         <ul>
            <li><b>fileName</b>: Download
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>worksheetName</b>: Sheet1
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>user</b>: A User
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
         </ul>
               </ul>
            </li>
            <li><b>statusBar</b>: Grid status bar's translation strings.
               <ul>
         <ul>
            <li><b>countLabel</b>: Total Records:
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>enable</b>: Enable
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>disable</b>: Disable
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>readOnlyCellStyle</b>: Highlight read-only cells
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
            <li><b>exportToExcel</b>: Download
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
         </ul>
               </ul>
            </li>
            <li><b>advancedFilters</b>: Grid advanced filter's translation strings.
               <ul>
         <ul>
            <li><b>placeholder</b>: Filter
               <ul>
                  <li><b>type</b>: string</li>
               </ul>
            </li>
         </ul>
               </ul>
            </li>
         </ul>
               </ul>
            </li>
            <li><b>ojPagingControl</b>: Collection of instance-level translation strings for the JET oj-paging-control component.

For application-level translation overrides, please see oj-paging-control's translations property (http://jet.us.oracle.com/5.1.0/jsdocs/oj.ojPagingControl.html).
               <ul>
                  <li><b>type</b>: Object</li>
         <ul>
            <li><b>labelAccNavFirstPage</b>: First page label for screen readers.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: First Page</li>
               </ul>
            </li>
            <li><b>labelAccNavLastPage</b>: Last page label for screen readers.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: Last Page</li>
               </ul>
            </li>
            <li><b>labelAccNavNextPage</b>: Next page label for screen readers.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: Next Page</li>
               </ul>
            </li>
            <li><b>labelAccNavPage</b>: Current page label for screen readers.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: Page</li>
               </ul>
            </li>
            <li><b>labelAccNavPreviousPage</b>: Previous page label for screen readers.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: Previous Page</li>
               </ul>
            </li>
            <li><b>labelAccPaging</b>: Paging control label for screen readers.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: Pagination</li>
               </ul>
            </li>
            <li><b>labelLoadMore</b>: Load more link text.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: Show More...</li>
               </ul>
            </li>
            <li><b>labelLoadMoreMaxRows</b>: Label for when load more has reached the maximum limit of rows to fetch.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: Reached Maximum Limit of {maxRows} rows</li>
               </ul>
            </li>
            <li><b>labelNavInputPage</b>: Current page label.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: Page</li>
               </ul>
            </li>
            <li><b>labelNavInputPageMax</b>: Maximum page label.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: of {pageMax}</li>
               </ul>
            </li>
            <li><b>maxPageLinksInvalid</b>: Invalid max page links value error.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: Value for maxPageLinks is invalid.</li>
               </ul>
            </li>
            <li><b>msgItemRangeCurrent</b>: Item range text for known row count.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: {pageFrom}-{pageTo}</li>
               </ul>
            </li>
            <li><b>msgItemRangeCurrentSingle</b>: Item range text for single page range.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: {pageFrom}</li>
               </ul>
            </li>
            <li><b>msgItemRangeItems</b>: Item range items text.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: items</li>
               </ul>
            </li>
            <li><b>msgItemRangeOf</b>: Item range "of" text.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: of</li>
               </ul>
            </li>
            <li><b>msgItemRangeOfApprox</b>: Item range "of approx" text.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: approx.</li>
               </ul>
            </li>
            <li><b>msgItemRangeOfAtLeast</b>: Item range "of at least" text.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: at least</li>
               </ul>
            </li>
            <li><b>pageInvalid</b>: Invalid page error.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: The page value entered is invalid.</li>
               </ul>
            </li>
            <li><b>tipNavFirstPage</b>: First page button tip.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: First</li>
               </ul>
            </li>
            <li><b>tipNavInputPage</b>: Current page tip.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: Go To Page</li>
               </ul>
            </li>
            <li><b>tipNavLastPage</b>: Last page button tip.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: Last</li>
               </ul>
            </li>
            <li><b>tipNavNextPage</b>: Next page button tip.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: Next</li>
               </ul>
            </li>
            <li><b>tipNavPageLink</b>: Current page link tip.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: Go To Page {pageNum}</li>
               </ul>
            </li>
            <li><b>tipNavPreviousPage</b>: Previous page button tip.

See the translations attribute for usage examples.
               <ul>
                  <li><b>type</b>: string</li>
                  <li><b>default value</b>: Previous</li>
               </ul>
            </li>
         </ul>
               </ul>
            </li>
         </ul>
      </div></div>
   </dd>
</dl>
    */
   options: {
   }
};
/**
 * Call this method to refresh the grid.
 *
 * @method module:oj-lux-grid-basic#refresh
 * @public
 */
/**
 * This event is triggered when a toolbar's action item is clicked or selected.
 *
 * @event module:oj-lux-grid-basic#toolbarActionEvent
 * @param {string} actionId  ID of the item that triggers the action.
 * @param {Array} selectedRows  Row selected for the action event.
 */
/**
 * This event is triggered when a grid cell's value is updated.
 *
 * @event module:oj-lux-grid-basic#cellUpdated
 * @param {string} dataIndex  Column data index.
 * @param {object} record  Row data record.
 */
/**
 * This event is triggered when a grid cell's link is clicked.
 *
 * @event module:oj-lux-grid-basic#linkClicked
 * @param {string} dataIndex  Column data index.
 * @param {object} record  Row data record.
 */
/**
 * This event is triggered when a single row or multiple rows are selected.
 *
 * @event module:oj-lux-grid-basic#rowsSelected
 * @param {Array} selectedRecords  Row(s) selected.
 */
/* jshint ignore:end */
