/**
 * Copyright (c) 2018 Oracle and/or its affiliates. All rights reserved.
 * @author Daniel Hoang
 */
define([
    'ojs/ojcore',
    'knockout',
    'jquery',
    'lodash',
    'text!./component.json',
    'ojL10n!./resources/nls/oj-lux-toolbar-grid-strings',
    'AbstractCCAWidget'
],
function(oj, ko, $, _, metadata, toolbarStrings, AbstractCCAWidget) {
    'use strict';

    var ViewModel = function(context) {
        var self = this;

        // Initialize CCA widget
        self.widgetName = 'luxToolbarGrid';
        AbstractCCAWidget.call(self, context, metadata);

        // Creata element ID for the widnget container element
        var widgetContainerId = self._uniqueId('oj-lux-toolbar-grid-id-');

        // Register the widget container ID
        self.registerWidgetContainerId(widgetContainerId);

        // Initialize toolbar translation strings
        self.translations = toolbarStrings.luxToolbarGrid;

        // Save the resolved properties for later access
        self.properties = context.properties;
    };

    // Make LuxGridModel a subclass of AbstractCCAWidget
    ViewModel.prototype = Object.create(AbstractCCAWidget.prototype);
    ViewModel.prototype.constructor = ViewModel;

    // Overriding abstract class' bindingsApplied() life-cycle in order to handle the toolbar's translations config option.
    ViewModel.prototype.bindingsApplied = function(context) {
        var self = this;

        // Pick up per instance's translation overrides. We do this by checking for explicit overrides supplied
        // on the tag instance and apply those. They are the most specific.
        var instanceTranslations = (self.properties.translations) ? self.properties.translations : {};

        // Update the grid's translations by merging this grid instance's translations with the global translations.
        oj.Logger.info('Updating toolbar locale=' + oj.Config.getLocale());

        // options is defined by the abstract class to hold grid widget's options.
        // So here we update the toolbar's "translations" option with the translations.
        // Also, we need to **CLONE** the translations object (by merge({})) so as to decouple it from other instances.
        self.setWidgetOption('translations', _.merge({}, self.translations, instanceTranslations));
    };

    return ViewModel;
});
